@@ -1,3 +1,10 @@
+/*@-compdef@*/
+/*@-compmempass@*/
+/*@-nullstate@*/
+/*@-shiftimplementation@*/
+/*@-sizeoftype@*/
+/*@-temptrans@*/ 
+/*@-usereleased@*/
 /*
  * Copyright (c) Ian F. Darwin 1986-1995.
  * Software written by Ian F. Darwin and others;
@@ -43,9 +50,10 @@
 #ifdef QUICK
 #include <sys/mman.h>
 #endif
+#include <assert.h>
 
 #ifndef	lint
-FILE_RCSID("@(#)$Id: apprentice.c,v 1.84 2005/03/25 18:03:18 christos Exp $")
+FILE_RCSID("@(#)$Id: apprentice.c,v 1.85 2005/10/17 15:31:10 christos Exp $")
 #endif	/* lint */
 
 #define	EATAB {while (isascii((unsigned char) *l) && \
@@ -85,13 +93,13 @@ private size_t maxmagic = 0;
 /*@unchecked@*/
 private size_t magicsize = sizeof(struct magic);
 
-private int getvalue(struct magic_set *ms, struct magic *m, char **p)
+private int getvalue(struct magic_set *ms, struct magic *m, /*@out@*/ char **p)
 	/*@globals fileSystem @*/
 	/*@modifies ms, m, *p, fileSystem @*/;
 private int hextoint(int)
 	/*@*/;
 /*@null@*/
-private char *getstr(struct magic_set *ms, /*@returned@*/ char *s, char *p, int plen, int *slen)
+private char *getstr(struct magic_set *ms, /*@returned@*/ /*@out@*/ char *s, char *p, int plen, int *slen)
 	/*@modifies ms, *p, *slen @*/;
 private int parse(struct magic_set *ms, struct magic **magicp, uint32_t *nmagicp, char *l, int action)
 	/*@globals maxmagic, fileSystem @*/
@@ -280,6 +288,7 @@ file_apprentice(struct magic_set *ms, const char *fn, int action)
 			*p++ = '\0';
 		if (*fn == '\0')
 			break;
+/*@-branchstate@*/
 		if (ms->flags & MAGIC_MIME) {
 			if ((afn = malloc(strlen(fn) + 5 + 1)) == NULL) {
 				free(mfn);
@@ -291,6 +300,7 @@ file_apprentice(struct magic_set *ms, const char *fn, int action)
 			(void)strcat(afn, ".mime");
 			fn = afn;
 		}
+/*@=branchstate@*/
 		file_err = apprentice_1(ms, fn, action, mlist);
 		if (file_err > errs)
 			errs = file_err;
@@ -841,7 +851,7 @@ check_format(struct magic_set *ms, struct magic *m)
 private int
 getvalue(struct magic_set *ms, struct magic *m, char **p)
 {
-	int slen;
+	int slen = 0;
 
 	switch (m->type) {
 	case FILE_BESTRING16:
@@ -1091,7 +1101,7 @@ apprentice_map(struct magic_set *ms, struct magic **magicp, uint32_t *nmagicp,
 	if (dbname == NULL)
 		return -1;
 
-	if ((fd = open(dbname, O_RDONLY)) == -1)
+	if ((fd = open(dbname, O_RDONLY|O_BINARY)) == -1)
 		return -1;
 
 	if (fstat(fd, &st) == -1) {
@@ -1121,6 +1131,7 @@ apprentice_map(struct magic_set *ms, struct magic **magicp, uint32_t *nmagicp,
 	}
 #define RET	1
 #endif
+assert(mm != NULL);
 	*magicp = mm;
 	(void)close(fd);
 	fd = -1;
@@ -1182,7 +1193,7 @@ apprentice_compile(struct magic_set *ms, struct magic **magicp,
 	if (dbname == NULL) 
 		return -1;
 
-	if ((fd = open(dbname, O_WRONLY|O_CREAT|O_TRUNC, 0644)) == -1) {
+	if ((fd = open(dbname, O_WRONLY|O_CREAT|O_TRUNC|O_BINARY, 0644)) == -1) {
 		file_error(ms, errno, "cannot open `%s'", dbname);
 		return -1;
 	}
@@ -1280,3 +1291,10 @@ bs1(struct magic *m)
 		m->value.l = swap4(m->value.l);
 	m->mask = swap4(m->mask);
 }
+/*@=usereleased@*/
+/*@=temptrans@*/
+/*@=sizeoftype@*/
+/*@=shiftimplementation@*/
+/*@=nullstate@*/
+/*@=compmempass@*/
+/*@=compdef@*/
