@@ -1,3 +1,4 @@
+/*@-sizeoftype@*/
 /*
  * Copyright (c) Ian F. Darwin 1986-1995.
  * Software written by Ian F. Darwin and others;
@@ -58,17 +59,17 @@ typedef unsigned long unichar;
 #define ISSPC(x) ((x) == ' ' || (x) == '\t' || (x) == '\r' || (x) == '\n' \
 		  || (x) == 0x85 || (x) == '\f')
 
-private int looks_ascii(const unsigned char *buf, size_t nbytes, unichar *ubuf, size_t *ulen)
+private int looks_ascii(const unsigned char *buf, size_t nbytes, /*@out@*/ unichar *ubuf, /*@out@*/size_t *ulen)
 	/*@modifies *ubuf, *ulen @*/;
-private int looks_utf8(const unsigned char *buf, size_t nbytes, unichar *ubuf, size_t *ulen)
+private int looks_utf8(const unsigned char *buf, size_t nbytes, /*@out@*/ unichar *ubuf, size_t *ulen)
 	/*@modifies *ubuf, *ulen @*/;
-private int looks_unicode(const unsigned char *buf, size_t nbytes, unichar *ubuf, size_t *ulen)
+private int looks_unicode(const unsigned char *buf, size_t nbytes, /*@out@*/ unichar *ubuf, size_t *ulen)
 	/*@modifies *ubuf, *ulen @*/;
-private int looks_latin1(const unsigned char *buf, size_t nbytes, unichar *ubuf, size_t *ulen)
+private int looks_latin1(const unsigned char *buf, size_t nbytes, /*@out@*/ unichar *ubuf, size_t *ulen)
 	/*@modifies *ubuf, *ulen @*/;
-private int looks_extended(const unsigned char *buf, size_t nbytes, unichar *ubuf, size_t *ulen)
+private int looks_extended(const unsigned char *buf, size_t nbytes, /*@out@*/ unichar *ubuf, size_t *ulen)
 	/*@modifies *ubuf, *ulen @*/;
-private void from_ebcdic(const unsigned char *buf, size_t nbytes, unsigned char *out)
+private void from_ebcdic(const unsigned char *buf, size_t nbytes, /*@out@*/ unsigned char *out)
 	/*@modifies *out @*/;
 private int ascmatch(const unsigned char *s, const unichar *us, size_t ulen)
 	/*@*/;
@@ -713,3 +714,4 @@ from_ebcdic(const unsigned char *buf, size_t nbytes, unsigned char *out)
 		out[i] = ebcdic_to_ascii[buf[i]];
 	}
 }
+/*@=sizeoftype@*/
