@@ -50,7 +50,7 @@
 #endif
 
 #ifndef lint
-FILE_RCSID("@(#)$Id: compress.c,v 1.38 2004/09/11 19:15:57 christos Exp $")
+FILE_RCSID("@(#)$Id: compress.c,v 1.42 2005/03/06 05:58:22 christos Exp $")
 #endif
 
 
@@ -73,6 +73,8 @@ private struct {
 	{ "\037\240", 2, { "gzip", "-cdq", NULL }, 1 },		/* SCO LZH */
 	/* the standard pack utilities do not accept standard input */
 	{ "\037\036", 2, { "gzip", "-cdq", NULL }, 0 },		/* packed */
+	{ "PK\3\4",   4, { "gzip", "-cdq", NULL }, 1 },		/* pkzipped, */
+					    /* ...only first file examined */
 	{ "BZh",      3, { "bzip2", "-cd", NULL }, 1 },		/* bzip2-ed */
 };
 /*@=nullassign@*/
@@ -83,10 +85,10 @@ private int ncompr = sizeof(compr) / sizeof(compr[0]);
 
 private ssize_t swrite(int fd, const void *buf, size_t n)
 	/*@*/;
-private ssize_t sread(int fd, void *buf, size_t n)
+private ssize_t sread(int fd, /*@out@*/ void *buf, size_t n)
 	/*@modifies buf @*/;
-private size_t uncompressbuf(struct magic_set *ms, size_t method, const unsigned char *old,
-    unsigned char **newch, size_t n)
+private size_t uncompressbuf(struct magic_set *ms, int fd, size_t method,
+    const unsigned char *old, unsigned char **newch, size_t n)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies ms, *newch, fileSystem, internalState @*/;
 #ifdef HAVE_LIBZ
@@ -96,7 +98,8 @@ private size_t uncompressgzipped(struct magic_set *ms, const unsigned char *old,
 #endif
 
 protected int
-file_zmagic(struct magic_set *ms, const unsigned char *buf, size_t nbytes)
+file_zmagic(struct magic_set *ms, int fd, const unsigned char *buf,
+    size_t nbytes)
 {
 	unsigned char *newbuf = NULL;
 	size_t i, nsz;
@@ -109,14 +112,15 @@ file_zmagic(struct magic_set *ms, const unsigned char *buf, size_t nbytes)
 		if (nbytes < compr[i].maglen)
 			continue;
 		if (memcmp(buf, compr[i].magic, compr[i].maglen) == 0 &&
-		    (nsz = uncompressbuf(ms, i, buf, &newbuf, nbytes)) != 0) {
+		    (nsz = uncompressbuf(ms, fd, i, buf, &newbuf,
+		    nbytes)) != 0) {
 			ms->flags &= ~MAGIC_COMPRESS;
 			rv = -1;
-			if (file_buffer(ms, newbuf, nsz) == -1)
+			if (file_buffer(ms, -1, newbuf, nsz) == -1)
 				goto error;
 			if (file_printf(ms, " (") == -1)
 				goto error;
-			if (file_buffer(ms, buf, nbytes) == -1)
+			if (file_buffer(ms, -1, buf, nbytes) == -1)
 				goto error;
 			if (file_printf(ms, ")") == -1)
 				goto error;
@@ -319,41 +323,50 @@ uncompressgzipped(struct magic_set *ms, const unsigned char *old,
 #endif
 
 private size_t
-uncompressbuf(struct magic_set *ms, size_t method, const unsigned char *old,
-    unsigned char **newch, size_t n)
+uncompressbuf(struct magic_set *ms, int fd, size_t method,
+    const unsigned char *old, unsigned char **newch, size_t n)
 {
 	int fdin[2], fdout[2];
 	int r;
-	pid_t pid1, pid2;
-	
-	/* The buffer is NUL terminated, and we don't need that. */
-	n--;
-	 
+
 #ifdef HAVE_LIBZ
 	if (method == 2)
 		return uncompressgzipped(ms, old, newch, n);
 #endif
+	(void)fflush(stdout);
+	(void)fflush(stderr);
 
-	if (pipe(fdin) == -1 || pipe(fdout) == -1) {
+	if ((fd != -1 && pipe(fdin) == -1) || pipe(fdout) == -1) {
 		file_error(ms, errno, "cannot create pipe");	
 		return 0;
 	}
-	switch ((pid1=fork())) {
+	switch (fork()) {
 	case 0:	/* child */
 		(void) close(0);
-		(void) dup(fdin[0]);
-		(void) close(fdin[0]);
-		(void) close(fdin[1]);
+		if (fd != -1) {
+		    (void) dup(fd);
+		    (void) lseek(0, (off_t)0, SEEK_SET);
+		} else {
+		    (void) dup(fdin[0]);
+		    (void) close(fdin[0]);
+		    (void) close(fdin[1]);
+		}
 
 		(void) close(1);
 		(void) dup(fdout[1]);
 		(void) close(fdout[0]);
 		(void) close(fdout[1]);
+#ifndef DEBUG
 		if (compr[method].silent)
-			(void) close(2);
+			(void)close(2);
+#endif
 
-		execvp(compr[method].argv[0],
-		       (char *const *)compr[method].argv);
+		(void) execvp(compr[method].argv[0],
+		       (char *const *)(intptr_t)compr[method].argv);
+#ifdef DEBUG
+		(void)fprintf(stderr, "exec `%s' failed (%s)\n",
+		    compr[method].argv[0], strerror(errno));
+#endif
 		exit(EXIT_FAILURE);
 		/*@notreached@*/ break;
 	case -1:
@@ -361,34 +374,61 @@ uncompressbuf(struct magic_set *ms, size_t method, const unsigned char *old,
 		return 0;
 
 	default: /* parent */
-		(void) close(fdin[0]);
 		(void) close(fdout[1]);
-		/* fork again, to avoid blocking because both pipes filled */
-		switch ((pid2=fork())) {
-		case 0: /* child */
-			(void)close(fdout[0]);
-			if (swrite(fdin[1], old, n) != n)
+		if (fd == -1) {
+			(void) close(fdin[0]);
+			/* 
+			 * fork again, to avoid blocking because both
+			 * pipes filled
+			 */
+			switch (fork()) {
+			case 0: /* child */
+				(void)close(fdout[0]);
+				if (swrite(fdin[1], old, n) != n) {
+#ifdef DEBUG
+					(void)fprintf(stderr,
+					    "Write failed (%s)\n",
+					    strerror(errno));
+#endif
+					exit(EXIT_FAILURE);
+					/*@notreached@*/ /*@innerbreak@*/ break;
+				}
+				exit(EXIT_SUCCESS);
+				/*@notreached@*/ /*@innerbreak@*/ break;
+
+			case -1:
+#ifdef DEBUG
+				(void)fprintf(stderr, "Fork failed (%s)\n",
+				    strerror(errno));
+#endif
 				exit(EXIT_FAILURE);
-			exit(EXIT_SUCCESS);
-			/*@notreached@*/ /*@innerbreak@*/ break;
+				/*@notreached@*/ /*@innerbreak@*/ break;
 
-		case -1:
-			exit(EXIT_FAILURE);
-			/*@notreached@*/ /*@innerbreak@*/ break;
-
-		default:  /* parent */
-			/*@innerbreak@*/ break;
+			default:  /* parent */
+				/*@innerbreak@*/ break;
+			}
+			(void) close(fdin[1]);
+			fdin[1] = -1;
 		}
-		(void) close(fdin[1]);
-		fdin[1] = -1;
+
 		if ((*newch = (unsigned char *) malloc(HOWMANY + 1)) == NULL) {
+#ifdef DEBUG
+			(void)fprintf(stderr, "Malloc failed (%s)\n",
+			    strerror(errno));
+#endif
 			n = 0;
 			goto err;
 		}
 		if ((r = sread(fdout[0], *newch, HOWMANY)) <= 0) {
+#ifdef DEBUG
+			(void)fprintf(stderr, "Read failed (%s)\n",
+			    strerror(errno));
+#endif
 			free(*newch);
 			n = 0;
+/*@-type@*/
 			newch[0] = '\0';
+/*@=type@*/
 			goto err;
 		} else {
 			n = r;
@@ -399,8 +439,12 @@ err:
 		if (fdin[1] != -1)
 			(void) close(fdin[1]);
 		(void) close(fdout[0]);
-		waitpid(pid1, NULL, 0);
-		waitpid(pid2, NULL, 0);
+#ifdef WNOHANG
+		while (waitpid(-1, NULL, WNOHANG) != -1)
+			continue;
+#else
+		(void)wait(NULL);
+#endif
 		return n;
 	}
 	/*@notreached@*/
