@@ -39,9 +39,10 @@
 #include <unistd.h>
 #endif
 #include <time.h>
+#include <assert.h>
 
 #ifndef lint
-FILE_RCSID("@(#)$Id: print.c,v 1.46 2004/11/13 08:11:39 christos Exp $")
+FILE_RCSID("@(#)$Id: print.c,v 1.48 2005/10/12 19:29:42 christos Exp $")
 #endif  /* lint */
 
 #define SZOF(a)	(sizeof(a) / sizeof(a[0]))
@@ -59,8 +60,10 @@ file_mdump(struct magic *m)
 	private const char optyp[] = { '@', '&', '|', '^', '+', '-', 
 				      '*', '/', '%' };
 	(void) fputc('[', stderr);
+/*@-formatconst@*/
 	(void) fprintf(stderr, ">>>>>>>> %d" + 8 - (m->cont_level & 7),
 		       m->offset);
+/*@=formatconst@*/
 
 	if (m->flag & INDIR) {
 		(void) fprintf(stderr, "(%s,",
@@ -81,9 +84,9 @@ file_mdump(struct magic *m)
 		(void) fputc('~', stderr);
 	if (m->mask) {
 		if ((m->mask_op & 0x7F) < SZOF(optyp)) 
-			fputc(optyp[m->mask_op&0x7F], stderr);
+			(void) fputc(optyp[m->mask_op&0x7F], stderr);
 		else
-			fputc('?', stderr);
+			(void) fputc('?', stderr);
 		if(FILE_STRING != m->type || FILE_PSTRING != m->type)
 			(void) fprintf(stderr, "%.8x", m->mask);
 		else {
@@ -141,6 +144,7 @@ protected void
 file_magwarn(struct magic_set *ms, const char *f, ...)
 {
 	va_list va;
+assert(f != NULL);
 	va_start(va, f);
 
 	/* cuz we use stdout for most, stderr here */
@@ -150,10 +154,10 @@ file_magwarn(struct magic_set *ms, const char *f, ...)
 	    (unsigned long)ms->line);
 	(void) vfprintf(stderr, f, va);
 	va_end(va);
-	fputc('\n', stderr);
+	(void) fputc('\n', stderr);
 }
 
-protected char *
+protected const char *
 file_fmttime(uint32_t v, int local)
 {
 	char *pp, *rt;
@@ -172,6 +176,8 @@ file_fmttime(uint32_t v, int local)
 			struct tm *tm1;
 			(void)time(&now);
 			tm1 = localtime(&now);
+			if (tm1 == NULL)
+				return "*Invalid time*";
 			daylight = tm1->tm_isdst;
 		}
 #endif /* HAVE_TM_ISDST */
@@ -179,10 +185,14 @@ file_fmttime(uint32_t v, int local)
 		if (daylight)
 			t += 3600;
 		tm = gmtime(&t);
+		if (tm == NULL)
+			return "*Invalid time*";
 		pp = asctime(tm);
 	}
 
+/*@-modobserver@*/
 	if ((rt = strchr(pp, '\n')) != NULL)
 		*rt = '\0';
+/*@=modobserver@*/
 	return pp;
 }
