@@ -22,10 +22,9 @@
 #include "debug.h"
 
 /*@access tsortInfo @*/
+/*@access rpmte @*/		/* XXX for install <-> erase associate. */
 /*@access rpmts @*/
 
-/*@access dbiIndex @*/		/* XXX for dbi->dbi_txnid */
-
 /*@access alKey @*/	/* XXX for reordering and RPMAL_NOMATCH assign */
 
 /**
@@ -73,22 +72,30 @@ static int intcmp(const void * a, const void * b)
  * @param ts		transaction set
  * @param h		header
  * @param dboffset	rpm database instance
+ * @retval *indexp	removed element index (if not NULL)
  * @param depends	installed package of pair (or RPMAL_NOMATCH on erase)
  * @return		0 on success
  */
 static int removePackage(rpmts ts, Header h, int dboffset,
+		/*@null@*/ int * indexp,
 		/*@exposed@*/ /*@dependent@*/ /*@null@*/ alKey depends)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
-	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@modifies ts, h, *indexp, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
     rpmte p;
 
     /* Filter out duplicate erasures. */
     if (ts->numRemovedPackages > 0 && ts->removedPackages != NULL) {
+	int * needle = NULL;
 /*@-boundswrite@*/
-	if (bsearch(&dboffset, ts->removedPackages, ts->numRemovedPackages,
-			sizeof(*ts->removedPackages), intcmp) != NULL)
+	needle = bsearch(&dboffset, ts->removedPackages, ts->numRemovedPackages,
+			sizeof(*ts->removedPackages), intcmp);
+	if (needle != NULL) {
+	    /* XXX lastx should be per-call, not per-ts. */
+	    if (indexp != NULL)
+	        *indexp = needle - ts->removedPackages;
 	    return 0;
+	}
 /*@=boundswrite@*/
     }
 
@@ -118,14 +125,44 @@ static int removePackage(rpmts ts, Header h, int dboffset,
     p = rpmteNew(ts, h, TR_REMOVED, NULL, NULL, dboffset, depends);
 /*@-boundswrite@*/
     ts->order[ts->orderCount] = p;
+    if (indexp != NULL)
+	*indexp = ts->orderCount;
     ts->orderCount++;
 /*@=boundswrite@*/
 
-    return 0;
+/*@-nullstate@*/	/* XXX FIX: ts->order[] can be NULL. */
+   return 0;
+/*@=nullstate@*/
+}
+
+/**
+ * Are two headers identical?
+ * @param first		first header
+ * @param second	second header
+ * @return		1 if headers are identical, 0 otherwise
+ */
+static int rpmHeadersIdentical(Header first, Header second)
+	/*@*/
+{
+    const char * one, * two;
+
+    if (!headerGetEntry(first, RPMTAG_HDRID, NULL, (void **) &one, NULL))
+	one = NULL;
+    if (!headerGetEntry(second, RPMTAG_HDRID, NULL, (void **) &two, NULL))
+	two = NULL;
+
+    if (one && two)
+	return ((strcmp(one, two) == 0) ? 1 : 0);
+    if (one && !two)
+	return 0;
+    if (!one && two)
+	return 0;
+    /* XXX Headers w/o digests case devolves to NEVR comparison. */
+    return ((rpmVersionCompare(first, second) == 0) ? 1 : 0);
 }
 
 int rpmtsAddInstallElement(rpmts ts, Header h,
-			fnpyKey key, int upgrade, rpmRelocation * relocs)
+			fnpyKey key, int upgrade, rpmRelocation relocs)
 {
     uint_32 tscolor = rpmtsColor(ts);
     uint_32 dscolor;
@@ -149,7 +186,6 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 
     /*
      * Check for previously added versions with the same name and arch/os.
-     * FIXME: only catches previously added, older packages.
      */
     arch = NULL;
     xx = hge(h, RPMTAG_ARCH, NULL, (void **)&arch, NULL);
@@ -158,14 +194,8 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
     hcolor = hGetColor(h);
     pkgKey = RPMAL_NOMATCH;
 
-    /* Check for supported payload format if it's a package */
-    if (key && headerCheckPayloadFormat(h) != RPMRC_OK) {
-	ec = 1;
-	goto exit;
-    }
-
     /* XXX Always add source headers. */
-    isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
+    isSource = (headerIsEntry(h, RPMTAG_SOURCERPM) == 0) ;
     if (isSource) {
 	oc = ts->orderCount;
 	goto addheader;
@@ -201,12 +231,9 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 	if ((this = rpmteDS(p, RPMTAG_NAME)) == NULL)
 	    continue;	/* XXX can't happen */
 
-	/* 
- 	 * On upgrade, if newer NEVR was previously added, 
- 	 * then skip adding older. 
- 	 */
+	/* If newer NEVR was previously added, then skip adding older. */
 	rc = rpmdsCompare(newChk, this);
-	if (upgrade && rc != 0) {
+	if (rc != 0) {
 	    const char * pkgNEVR = rpmdsDNEVR(this);
 	    const char * addNEVR = rpmdsDNEVR(oldChk);
 	    if (rpmIsVerbose())
@@ -214,16 +241,13 @@ int rpmtsAddInstallElement(rpmts ts, Header h,
 		    _("package %s was already added, skipping %s\n"),
 		    (pkgNEVR ? pkgNEVR + 2 : "?pkgNEVR?"),
 		    (addNEVR ? addNEVR + 2 : "?addNEVR?"));
-	    ec = 0;
-	    goto exit;
+	    ec = 1;
+	    break;
 	}
 
-	/*
- 	 * On upgrade, if older NEVR was previously added, 
- 	 * then replace old with new. 
- 	 */
+	/* If older NEVR was previously added, then replace old with new. */
 	rc = rpmdsCompare(oldChk, this);
-	if (upgrade && rc != 0) {
+	if (rc != 0) {
 	    const char * pkgNEVR = rpmdsDNEVR(this);
 	    const char * addNEVR = rpmdsDNEVR(newChk);
 	    if (rpmIsVerbose())
@@ -253,6 +277,7 @@ addheader:
     }
 
     p = rpmteNew(ts, h, TR_ADDED, key, relocs, -1, pkgKey);
+assert(p != NULL);
 
     if (duplicate && oc < ts->orderCount) {
 /*@-type -unqualifiedtrans@*/
@@ -277,6 +302,7 @@ addheader:
 /*@-boundswrite@*/
 	ts->order[oc] = rpmteFree(ts->order[oc]);
 /*@=boundswrite@*/
+	ts->teInstall = NULL;
 	ec = 1;
 	goto exit;
     }
@@ -286,6 +312,8 @@ addheader:
 	ts->numAddedPackages++;
     }
 
+    ts->teInstall = ts->order[oc];
+
     /* XXX rpmgi hack: Save header in transaction element if requested. */
     if (upgrade & 0x2)
 	(void) rpmteSetHeader(p, h);
@@ -306,19 +334,33 @@ addheader:
 
     /* On upgrade, erase older packages of same color (if any). */
 
-    mi = rpmtsInitIterator(ts, RPMTAG_NAME, rpmteN(p), 0);
+    mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, rpmteN(p), 0);
     while((oh = rpmdbNextIterator(mi)) != NULL) {
+	int lastx;
+	rpmte q;
 
 	/* Ignore colored packages not in our rainbow. */
 	ohcolor = hGetColor(oh);
 	if (tscolor && hcolor && ohcolor && !(hcolor & ohcolor))
 	    continue;
 
-	/* Skip packages that contain identical NEVR. */
-	if (rpmVersionCompare(h, oh) == 0)
+	/* Skip identical packages. */
+	if (rpmHeadersIdentical(h, oh))
 	    continue;
 
-	xx = removePackage(ts, oh, rpmdbGetIteratorOffset(mi), pkgKey);
+	/* Create an erasure element. */
+	lastx = -1;
+	xx = removePackage(ts, oh, rpmdbGetIteratorOffset(mi), &lastx, pkgKey);
+assert(lastx >= 0 && lastx < ts->orderCount);
+	q = ts->order[lastx];
+
+	/* Chain through upgrade flink. */
+	xx = rpmteChain(p, q, oh, "Upgrades");
+
+/*@-nullptrarith@*/
+	rpmMessage(RPMMESS_DEBUG, _("   upgrade erases %s\n"), rpmteNEVRA(q));
+/*@=nullptrarith@*/
+
     }
     mi = rpmdbFreeIterator(mi);
 
@@ -345,17 +387,22 @@ addheader:
 	if (!strcmp(rpmteN(p), Name))
 	    continue;
 
+	/* Obsolete containing package if given a file, otherwise provide. */
 	if (Name[0] == '/')
 	    mi = rpmtsInitIterator(ts, RPMTAG_BASENAMES, Name, 0);
 	else
-	    mi = rpmtsInitIterator(ts, RPMTAG_NAME, Name, 0);
+	    mi = rpmtsInitIterator(ts, RPMTAG_PROVIDENAME, Name, 0);
 
 	xx = rpmdbPruneIterator(mi,
 	    ts->removedPackages, ts->numRemovedPackages, 1);
 
 	while((oh = rpmdbNextIterator(mi)) != NULL) {
+	    int lastx;
+	    rpmte q;
+
 	    /* Ignore colored packages not in our rainbow. */
 	    ohcolor = hGetColor(oh);
+
 	    /* XXX provides *are* colored, effectively limiting Obsoletes:
 		to matching only colored Provides: based on pkg coloring. */
 	    if (tscolor && hcolor && ohcolor && !(hcolor & ohcolor))
@@ -365,19 +412,23 @@ addheader:
 	     * Rpm prior to 3.0.3 does not have versioned obsoletes.
 	     * If no obsoletes version info is available, match all names.
 	     */
-	    if (rpmdsEVR(obsoletes) == NULL
-	     || rpmdsAnyMatchesDep(oh, obsoletes, _rpmds_nopromote)) {
-		const char * ohNEVRA = hGetNEVRA(oh, NULL);
-#ifdef	DYING	/* XXX see http://bugzilla.redhat.com #134497 */
-		if (rpmVersionCompare(h, oh))
-#endif
-		    xx = removePackage(ts, oh, rpmdbGetIteratorOffset(mi), pkgKey);
+	    if (!(rpmdsEVR(obsoletes) == NULL
+	     || rpmdsAnyMatchesDep(oh, obsoletes, _rpmds_nopromote)))
+		/*@innercontinue@*/ continue;
+
+	    /* Create an erasure element. */
+	    lastx = -1;
+	    xx = removePackage(ts, oh, rpmdbGetIteratorOffset(mi), &lastx, pkgKey);
+assert(lastx >= 0 && lastx < ts->orderCount);
+	    q = ts->order[lastx];
+
+	    /* Chain through obsoletes flink. */
+	    xx = rpmteChain(p, q, oh, "Obsoletes");
+
 /*@-nullptrarith@*/
-		rpmMessage(RPMMESS_DEBUG, _("  Obsoletes: %s\t\terases %s\n"),
-			rpmdsDNEVR(obsoletes)+2, ohNEVRA);
+	    rpmMessage(RPMMESS_DEBUG, _("  Obsoletes: %s\t\terases %s\n"),
+			rpmdsDNEVR(obsoletes)+2, rpmteNEVRA(q));
 /*@=nullptrarith@*/
-		ohNEVRA = _free(ohNEVRA);
-	    }
 	}
 	mi = rpmdbFreeIterator(mi);
     }
@@ -392,7 +443,13 @@ exit:
 
 int rpmtsAddEraseElement(rpmts ts, Header h, int dboffset)
 {
-    return removePackage(ts, h, dboffset, RPMAL_NOMATCH);
+    int oc = -1;
+    int rc = removePackage(ts, h, dboffset, &oc, RPMAL_NOMATCH);
+    if (rc == 0 && oc >= 0 && oc < ts->orderCount)
+	ts->teErase = ts->order[oc];
+    else
+	ts->teErase = NULL;
+    return rc;
 }
 
 /**
@@ -405,7 +462,7 @@ int rpmtsAddEraseElement(rpmts ts, Header h, int dboffset)
 static int unsatisfiedDepend(rpmts ts, rpmds dep, int adding)
 	/*@globals _cacheDependsRC, rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
-	/*@modifies ts, _cacheDependsRC, rpmGlobalMacroContext,
+	/*@modifies ts, dep, _cacheDependsRC, rpmGlobalMacroContext,
 		fileSystem, internalState @*/
 {
     DBT * key = alloca(sizeof(*key));
@@ -440,7 +497,7 @@ static int unsatisfiedDepend(rpmts ts, rpmds dep, int adding)
 		size_t datalen = 0;
 		size_t DNEVRlen = strlen(DNEVR);
 
-		xx = dbiCopen(dbi, dbi->dbi_txnid, &dbcursor, 0);
+		xx = dbiCopen(dbi, dbiTxnid(dbi), &dbcursor, 0);
 
 		memset(key, 0, sizeof(*key));
 /*@i@*/		key->data = (void *) DNEVR;
@@ -474,33 +531,50 @@ static int unsatisfiedDepend(rpmts ts, rpmds dep, int adding)
 retry:
     rc = 0;	/* assume dependency is satisfied */
 
-#if defined(DYING) || defined(__LCLINT__)
-  { static /*@observer@*/ const char noProvidesString[] = "nada";
-    static /*@observer@*/ const char * rcProvidesString = noProvidesString;
-    int_32 Flags = rpmdsFlags(dep);
-    const char * start;
-    int i;
+    /* Expand macro probe dependencies. */
+    if (Name[0] == '%' && Name[1] == '{' && Name[strlen(Name)-1] == '}') {
+	xx = rpmExpandNumeric(Name);
+	/* XXX how should macro values be mapped? */
+	rc = (xx ? 0 : 1);
+	if (rpmdsFlags(dep) & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(macro probe)"), rc);
+	goto exit;
+    }
 
-    if (rcProvidesString == noProvidesString)
-	rcProvidesString = rpmGetVar(RPMVAR_PROVIDES);
+    /* Evaluate access(2) probe dependencies. */
+    if (strlen(Name) > 5 && Name[strlen(Name)-1] == ')'
+     && (	(  strchr("Rr_", Name[0]) != NULL
+		&& strchr("Ww_", Name[1]) != NULL
+		&& strchr("Xx_", Name[2]) != NULL
+		&& Name[3] == '(')
+	  ||	!strncmp(Name, "exists(", sizeof("exists(")-1)
+	  ||	!strncmp(Name, "executable(", sizeof("executable(")-1)
+	  ||	!strncmp(Name, "readable(", sizeof("readable(")-1)
+	  ||	!strncmp(Name, "writable(", sizeof("writable(")-1)
+	))
+    {
+	rc = rpmioAccess(Name, NULL, X_OK);
+	if (rpmdsFlags(dep) & RPMSENSE_MISSINGOK)
+	    goto unsatisfied;
+	rpmdsNotify(dep, _("(access probe)"), rc);
+	goto exit;
+    }
 
-    if (rcProvidesString != NULL && !(Flags & RPMSENSE_SENSEMASK)) {
+    /* Search system configured provides. */
+    if (!rpmioAccess("/etc/rpm/sysinfo", NULL, R_OK)) {
+	static rpmds sysinfoP = NULL;
+	static int oneshot = -1;
 
-	i = strlen(Name);
-	/*@-observertrans -mayaliasunique@*/
-	while ((start = strstr(rcProvidesString, Name))) {
-	/*@=observertrans =mayaliasunique@*/
-/*@-boundsread@*/
-	    if (xisspace(start[i]) || start[i] == '\0' || start[i] == ',') {
-		rpmdsNotify(dep, _("(rpmrc provides)"), rc);
+	if (oneshot)
+	    oneshot = rpmdsSysinfo(&sysinfoP, NULL);
+	if (sysinfoP != NULL) {
+	    if (rpmdsSearch(sysinfoP, dep) >= 0) {
+		rpmdsNotify(dep, _("(sysinfo provides)"), rc);
 		goto exit;
 	    }
-/*@=boundsread@*/
-	    rcProvidesString = start + 1;
 	}
     }
-  }
-#endif
 
     /*
      * New features in rpm packaging implicitly add versioned dependencies
@@ -508,13 +582,104 @@ retry:
      * Check those dependencies now.
      */
     if (!strncmp(Name, "rpmlib(", sizeof("rpmlib(")-1)) {
-	if (rpmCheckRpmlibProvides(dep)) {
+	static rpmds rpmlibP = NULL;
+	static int oneshot = -1;
+
+	if (oneshot)
+	    oneshot = rpmdsRpmlib(&rpmlibP, NULL);
+	if (rpmlibP == NULL)
+	    goto unsatisfied;
+
+	if (rpmdsSearch(rpmlibP, dep) >= 0) {
 	    rpmdsNotify(dep, _("(rpmlib provides)"), rc);
 	    goto exit;
 	}
 	goto unsatisfied;
     }
 
+    if (!strncmp(Name, "cpuinfo(", sizeof("cpuinfo(")-1)) {
+	static rpmds cpuinfoP = NULL;
+	static int oneshot = -1;
+
+	if (oneshot)
+	    oneshot = rpmdsCpuinfo(&cpuinfoP, NULL);
+	if (cpuinfoP == NULL)
+	    goto unsatisfied;
+
+	if (rpmdsSearch(cpuinfoP, dep) >= 0) {
+	    rpmdsNotify(dep, _("(cpuinfo provides)"), rc);
+	    goto exit;
+	}
+	goto unsatisfied;
+    }
+
+    if (!strncmp(Name, "getconf(", sizeof("getconf(")-1)) {
+	static rpmds getconfP = NULL;
+	static int oneshot = -1;
+
+	if (oneshot)
+	    oneshot = rpmdsGetconf(&getconfP, NULL);
+	if (getconfP == NULL)
+	    goto unsatisfied;
+
+	if (rpmdsSearch(getconfP, dep) >= 0) {
+	    rpmdsNotify(dep, _("(getconf provides)"), rc);
+	    goto exit;
+	}
+	goto unsatisfied;
+    }
+
+    if (!strncmp(Name, "uname(", sizeof("uname(")-1)) {
+	static rpmds unameP = NULL;
+	static int oneshot = -1;
+
+	if (oneshot)
+	    oneshot = rpmdsUname(&unameP, NULL);
+	if (unameP == NULL)
+	    goto unsatisfied;
+
+	if (rpmdsSearch(unameP, dep) >= 0) {
+	    rpmdsNotify(dep, _("(uname provides)"), rc);
+	    goto exit;
+	}
+	goto unsatisfied;
+    }
+
+    if (!strncmp(Name, "soname(", sizeof("soname(")-1)) {
+	rpmds sonameP = NULL;
+	rpmPRCO PRCO = memset(alloca(sizeof(*PRCO)), 0, sizeof(*PRCO));
+	char * fn = strcpy(alloca(strlen(Name)+1), Name);
+	int flags = 0;	/* XXX RPMELF_FLAG_SKIPREQUIRES? */
+	rpmds ds;
+
+	/* Strip the soname(/path/to/dso) wrapper. */
+	fn += sizeof("soname(")-1;
+
+	/* XXX Only absolute paths for now. */
+	if (*fn != '/')
+	    goto unsatisfied;
+	fn[strlen(fn)-1] = '\0';
+
+	/* Extract ELF Provides: from /path/to/DSO. */
+	PRCO->Pdsp = &sonameP;
+	xx = rpmdsELF(fn, flags, rpmdsMergePRCO, PRCO);
+	if (!(xx == 0 && sonameP != NULL))
+	    goto unsatisfied;
+
+	/* Search using the original {EVR,"",Flags} from the dep set. */
+	ds = rpmdsSingle(rpmdsTagN(dep), rpmdsEVR(dep), "", rpmdsFlags(dep));
+	xx = rpmdsSearch(sonameP, ds);
+	ds = rpmdsFree(ds);
+	sonameP = rpmdsFree(sonameP);
+
+	/* Was the dependency satisfied? */
+	if (xx >= 0) {
+	    rpmdsNotify(dep, _("(soname provides)"), rc);
+	    goto exit;
+	}
+	goto unsatisfied;
+    }
+
     /* Search added packages for the dependency. */
     if (rpmalSatisfiesDepend(ts->addedPackages, dep, NULL) != NULL) {
 	/*
@@ -558,20 +723,6 @@ retry:
 	}
 	mi = rpmdbFreeIterator(mi);
 
-#if defined(DYING) || defined(__LCLINT__)
-	mi = rpmtsInitIterator(ts, RPMTAG_NAME, Name, 0);
-	(void) rpmdbPruneIterator(mi,
-			ts->removedPackages, ts->numRemovedPackages, 1);
-	while ((h = rpmdbNextIterator(mi)) != NULL) {
-	    if (rpmdsAnyMatchesDep(h, dep, _rpmds_nopromote)) {
-		rpmdsNotify(dep, _("(db package)"), rc);
-		mi = rpmdbFreeIterator(mi);
-		goto exit;
-	    }
-	}
-	mi = rpmdbFreeIterator(mi);
-#endif
-
     }
 
     /*
@@ -593,8 +744,13 @@ retry:
 /*@=boundsread@*/
 
 unsatisfied:
-    rc = 1;	/* dependency is unsatisfied */
-    rpmdsNotify(dep, NULL, rc);
+    if (rpmdsFlags(dep) & RPMSENSE_MISSINGOK) {
+	rc = 0;	/* dependency is unsatisfied, but just a hint. */
+	rpmdsNotify(dep, _("(hint skipped)"), rc);
+    } else {
+	rc = 1;	/* dependency is unsatisfied */
+	rpmdsNotify(dep, NULL, rc);
+    }
 
 exit:
     /*
@@ -613,7 +769,7 @@ exit:
 		DBC * dbcursor = NULL;
 		size_t DNEVRlen = strlen(DNEVR);
 
-		xx = dbiCopen(dbi, dbi->dbi_txnid, &dbcursor, DB_WRITECURSOR);
+		xx = dbiCopen(dbi, dbiTxnid(dbi), &dbcursor, DB_WRITECURSOR);
 
 		memset(key, 0, sizeof(*key));
 /*@i@*/		key->data = (void *) DNEVR;
@@ -641,13 +797,16 @@ exit:
  * @param pkgNEVRA	package name-version-release.arch
  * @param requires	Requires: dependencies (or NULL)
  * @param conflicts	Conflicts: dependencies (or NULL)
+ * @param dirnames	Dirnames: dependencies (or NULL)
  * @param depName	dependency name to filter (or NULL)
  * @param tscolor	color bits for transaction set (0 disables)
  * @param adding	dependency is from added package set?
  * @return		0 no problems found
  */
 static int checkPackageDeps(rpmts ts, const char * pkgNEVRA,
-		/*@null@*/ rpmds requires, /*@null@*/ rpmds conflicts,
+		/*@null@*/ rpmds requires,
+		/*@null@*/ rpmds conflicts,
+		/*@null@*/ rpmds dirnames,
 		/*@null@*/ const char * depName, uint_32 tscolor, int adding)
 	/*@globals rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
@@ -733,6 +892,48 @@ static int checkPackageDeps(rpmts ts, const char * pkgNEVRA,
 	}
     }
 
+    dirnames = rpmdsInit(dirnames);
+    if (dirnames != NULL)
+    while (!ourrc && rpmdsNext(dirnames) >= 0) {
+
+	if ((Name = rpmdsN(dirnames)) == NULL)
+	    continue;	/* XXX can't happen */
+
+	/* Filter out dirnames that came along for the ride. */
+	if (depName != NULL && strcmp(depName, Name))
+	    continue;
+
+	/* Ignore colored dirnames not in our rainbow. */
+	dscolor = rpmdsColor(dirnames);
+	if (tscolor && dscolor && !(tscolor & dscolor))
+	    continue;
+
+	rc = unsatisfiedDepend(ts, dirnames, adding);
+
+	switch (rc) {
+	case 0:		/* requirements are satisfied. */
+	    /*@switchbreak@*/ break;
+	case 1:		/* requirements are not satisfied. */
+	{   fnpyKey * suggestedKeys = NULL;
+
+	    /*@-branchstate@*/
+	    if (ts->availablePackages != NULL) {
+		suggestedKeys = rpmalAllSatisfiesDepend(ts->availablePackages,
+				dirnames, NULL);
+	    }
+	    /*@=branchstate@*/
+
+	    rpmdsProblem(ts->probs, pkgNEVRA, dirnames, suggestedKeys, adding);
+
+	}
+	    /*@switchbreak@*/ break;
+	case 2:		/* something went wrong! */
+	default:
+	    ourrc = 1;
+	    /*@switchbreak@*/ break;
+	}
+    }
+
     return ourrc;
 }
 
@@ -759,7 +960,7 @@ static int checkPackageSet(rpmts ts, const char * dep,
 		ts->removedPackages, ts->numRemovedPackages, 1);
     while ((h = rpmdbNextIterator(mi)) != NULL) {
 	const char * pkgNEVRA;
-	rpmds requires, conflicts;
+	rpmds requires, conflicts, dirnames;
 	int rc;
 
 	pkgNEVRA = hGetNEVRA(h, NULL);
@@ -767,7 +968,11 @@ static int checkPackageSet(rpmts ts, const char * dep,
 	(void) rpmdsSetNoPromote(requires, _rpmds_nopromote);
 	conflicts = rpmdsNew(h, RPMTAG_CONFLICTNAME, scareMem);
 	(void) rpmdsSetNoPromote(conflicts, _rpmds_nopromote);
-	rc = checkPackageDeps(ts, pkgNEVRA, requires, conflicts, dep, 0, adding);
+	dirnames = rpmdsNew(h, RPMTAG_DIRNAMES, scareMem);
+	(void) rpmdsSetNoPromote(dirnames, _rpmds_nopromote);
+	rc = checkPackageDeps(ts, pkgNEVRA, requires, conflicts, dirnames,
+		dep, 0, adding);
+	dirnames = rpmdsFree(dirnames);
 	conflicts = rpmdsFree(conflicts);
 	requires = rpmdsFree(requires);
 	pkgNEVRA = _free(pkgNEVRA);
@@ -792,9 +997,15 @@ static int checkDependentPackages(rpmts ts, const char * dep)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/
 {
-    rpmdbMatchIterator mi;
-    mi = rpmtsInitIterator(ts, RPMTAG_REQUIRENAME, dep, 0);
-    return checkPackageSet(ts, dep, mi, 0);
+    int rc = 0;
+
+    /* XXX rpmdb can be closed here, avoid error msg. */
+    if (rpmtsGetRdb(ts) != NULL) {
+	rpmdbMatchIterator mi;
+	mi = rpmtsInitIterator(ts, RPMTAG_REQUIRENAME, dep, 0);
+	rc = checkPackageSet(ts, dep, mi, 0);
+    }
+    return rc;
 }
 
 /**
@@ -809,7 +1020,8 @@ static int checkDependentConflicts(rpmts ts, const char * dep)
 {
     int rc = 0;
 
-    if (rpmtsGetRdb(ts) != NULL) {	/* XXX is this necessary? */
+    /* XXX rpmdb can be closed here, avoid error msg. */
+    if (rpmtsGetRdb(ts) != NULL) {
 	rpmdbMatchIterator mi;
 	mi = rpmtsInitIterator(ts, RPMTAG_CONFLICTNAME, dep, 0);
 	rc = checkPackageSet(ts, dep, mi, 1);
@@ -830,6 +1042,28 @@ static struct badDeps_s {
 /*@observer@*/ /*@null@*/ const char * pname;
 /*@observer@*/ /*@null@*/ const char * qname;
 } badDeps[] = {
+    { "libtermcap", "bash" },
+    { "modutils", "vixie-cron" },
+    { "ypbind", "yp-tools" },
+    { "ghostscript-fonts", "ghostscript" },
+    /* 7.2 only */
+    { "libgnomeprint15", "gnome-print" },
+    { "nautilus", "nautilus-mozilla" },
+    /* 7.1 only */
+    { "arts", "kdelibs-sound" },
+    /* 7.0 only */
+    { "pango-gtkbeta-devel", "pango-gtkbeta" },
+    { "XFree86", "Mesa" },
+    { "compat-glibc", "db2" },
+    { "compat-glibc", "db1" },
+    { "pam", "initscripts" },
+    { "initscripts", "sysklogd" },
+    /* 6.2 */
+    { "egcs-c++", "libstdc++" },
+    /* 6.1 */
+    { "pilot-link-devel", "pilot-link" },
+    /* 5.2 */
+    { "pam", "pamconfig" },
     { NULL, NULL }
 };
 #else
@@ -949,11 +1183,9 @@ static void markLoop(/*@special@*/ tsortInfo tsi, rpmte q)
     /*@=branchstate@*/
 }
 
-static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
+static inline /*@observer@*/ const char * identifyDepend(int_32 f)
 	/*@*/
 {
-    if (isLegacyPreReq(f))
-	return "PreReq:";
     f = _notpre(f);
     if (f & RPMSENSE_SCRIPT_PRE)
 	return "Requires(pre):";
@@ -965,6 +1197,8 @@ static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
 	return "Requires(postun):";
     if (f & RPMSENSE_SCRIPT_VERIFY)
 	return "Requires(verify):";
+    if (f & RPMSENSE_MISSINGOK)
+	return "Requires(hint):";
     if (f & RPMSENSE_FIND_REQUIRES)
 	return "Requires(auto):";
     return "Requires:";
@@ -977,7 +1211,6 @@ static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
  * pure Requires: dependencies) successor node(s).
  * @param q		sucessor (i.e. package required by p)
  * @param p		predecessor (i.e. package that "Requires: q")
- * @param requires	relation
  * @param zap		max. no. of co-requisites to remove (-1 is all)?
  * @retval nzaps	address of no. of relations removed
  * @param msglvl	message level at which to spew
@@ -987,10 +1220,10 @@ static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
 /*@-mustmod@*/ /* FIX: hack modifies, but -type disables */
 static /*@owned@*/ /*@null@*/ const char *
 zapRelation(rpmte q, rpmte p,
-		/*@null@*/ rpmds requires,
 		int zap, /*@in@*/ /*@out@*/ int * nzaps, int msglvl)
 	/*@modifies q, p, requires, *nzaps @*/
 {
+    rpmds requires;
     tsortInfo tsi_prev;
     tsortInfo tsi;
     const char *dp = NULL;
@@ -1009,6 +1242,7 @@ zapRelation(rpmte q, rpmte p,
 	    continue;
 	/*@=abstractcompare@*/
 
+	requires = rpmteDS(p, tsi->tsi_tagn);
 	if (requires == NULL) continue;		/* XXX can't happen */
 
 	(void) rpmdsSetIx(requires, tsi->tsi_reqx);
@@ -1021,7 +1255,7 @@ zapRelation(rpmte q, rpmte p,
 	 * Attempt to unravel a dependency loop by eliminating Requires's.
 	 */
 	/*@-branchstate@*/
-	if (zap && !(Flags & RPMSENSE_PREREQ)) {
+	if (zap) {
 	    rpmMessage(msglvl,
 			_("removing %s \"%s\" from tsort relations.\n"),
 			(rpmteNEVRA(p) ?  rpmteNEVRA(p) : "???"), dp);
@@ -1065,8 +1299,10 @@ static inline int addRelation(rpmts ts,
     tsortInfo tsi;
     const char * Name;
     fnpyKey key;
+    int teType = rpmteType(p);
     alKey pkgKey;
     int i = 0;
+    rpmal al = (teType == TR_ADDED ? ts->addedPackages : ts->erasedPackages);
 
     if ((Name = rpmdsN(requires)) == NULL)
 	return 0;
@@ -1080,33 +1316,30 @@ static inline int addRelation(rpmts ts,
 	return 0;
 
     pkgKey = RPMAL_NOMATCH;
-    key = rpmalSatisfiesDepend(ts->addedPackages, requires, &pkgKey);
+    key = rpmalSatisfiesDepend(al, requires, &pkgKey);
 
-    /* Ordering depends only on added package relations. */
+    /* Ordering depends only on added/erased package relations. */
     if (pkgKey == RPMAL_NOMATCH)
 	return 0;
 
-/* XXX Set q to the added package that has pkgKey == q->u.addedKey */
-/* XXX FIXME: bsearch is possible/needed here */
+/* XXX Set q to the added/removed package that was found. */
+    /* XXX pretend erasedPackages are just appended to addedPackages. */
+    if (teType == TR_REMOVED)
+	pkgKey = (alKey)(((long)pkgKey) + ts->numAddedPackages);
+
     for (qi = rpmtsiInit(ts), i = 0; (q = rpmtsiNext(qi, 0)) != NULL; i++) {
-
-	/* XXX Only added packages need be checked for matches. */
-	if (rpmteType(q) == TR_REMOVED)
-	    continue;
-
 	if (pkgKey == rpmteAddedKey(q))
 	    break;
     }
     qi = rpmtsiFree(qi);
-    if (q == NULL || i == ts->orderCount)
+    if (q == NULL || i >= ts->orderCount)
 	return 0;
 
     /* Avoid certain dependency relations. */
-    if (ignoreDep(ts, p, q))
+    if (teType == TR_ADDED && ignoreDep(ts, p, q))
 	return 0;
 
     /* Avoid redundant relations. */
-    /* XXX TODO: add control bit. */
 /*@-boundsread@*/
     if (selected[i] != 0)
 	return 0;
@@ -1126,6 +1359,7 @@ static inline int addRelation(rpmts ts,
     tsi = xcalloc(1, sizeof(*tsi));
     tsi->tsi_suc = p;
 
+    tsi->tsi_tagn = rpmdsTagN(requires);
     tsi->tsi_reqx = rpmdsIx(requires);
 
     tsi->tsi_next = rpmteTSI(q)->tsi_next;
@@ -1160,14 +1394,13 @@ static int orderListIndexCmp(const void * one, const void * two)	/*@*/
 /*@-mustmod@*/
 static void addQ(/*@dependent@*/ rpmte p,
 		/*@in@*/ /*@out@*/ rpmte * qp,
-		/*@in@*/ /*@out@*/ rpmte * rp,
-		uint_32 prefcolor)
+		/*@in@*/ /*@out@*/ rpmte * rp)
 	/*@modifies p, *qp, *rp @*/
 {
     rpmte q, qprev;
 
     /* Mark the package as queued. */
-    rpmteTSI(p)->tsi_reqx = 1;
+    rpmteTSI(p)->tsi_queued = 1;
 
     if ((*rp) == NULL) {	/* 1st element */
 	/*@-dependenttrans@*/ /* FIX: double indirection */
@@ -1181,10 +1414,9 @@ static void addQ(/*@dependent@*/ rpmte p,
 	 q != NULL;
 	 qprev = q, q = rpmteTSI(q)->tsi_suc)
     {
-	/* XXX Insure preferred color first. */
-	if (rpmteColor(p) != prefcolor && rpmteColor(p) != rpmteColor(q))
+	/* XXX Insure removed after added. */
+	if (rpmteType(p) == TR_REMOVED && rpmteType(p) != rpmteType(q))
 	    continue;
-
 	if (rpmteTSI(q)->tsi_qcnt <= rpmteTSI(p)->tsi_qcnt)
 	    break;
     }
@@ -1213,7 +1445,6 @@ int rpmtsOrder(rpmts ts)
     rpmds requires;
     int_32 Flags;
     int anaconda = rpmtsFlags(ts) & RPMTRANS_FLAG_ANACONDA;
-    uint_32 prefcolor = rpmtsPrefColor(ts);
     rpmtsi pi; rpmte p;
     rpmtsi qi; rpmte q;
     rpmtsi ri; rpmte r;
@@ -1240,11 +1471,29 @@ int rpmtsOrder(rpmts ts)
     int qlen;
     int i, j;
 
-    /*
-     * XXX FIXME: this gets needlesly called twice on normal usage patterns,
-     * should track the need for generating the index somewhere
-     */
+#ifdef	DYING
     rpmalMakeIndex(ts->addedPackages);
+#endif
+
+    /* Create erased package index. */
+    pi = rpmtsiInit(ts);
+    while ((p = rpmtsiNext(pi, TR_REMOVED)) != NULL) {
+	alKey pkgKey;
+	fnpyKey key;
+	int tscolor = 0x3;
+	pkgKey = RPMAL_NOMATCH;
+/*@-abstract@*/
+	key = (fnpyKey) p;
+/*@=abstract@*/
+	pkgKey = rpmalAdd(&ts->erasedPackages, pkgKey, key,
+			rpmteDS(p, RPMTAG_PROVIDENAME),
+			rpmteFI(p, RPMTAG_BASENAMES), tscolor);
+	/* XXX pretend erasedPackages are just appended to addedPackages. */
+	pkgKey = (alKey)(((long)pkgKey) + ts->numAddedPackages);
+	rpmteSetAddedKey(p, pkgKey);
+    }
+    pi = rpmtsiFree(pi);
+    rpmalMakeIndex(ts->erasedPackages);
 
     (void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_ORDER), 0);
 
@@ -1272,8 +1521,7 @@ int rpmtsOrder(rpmts ts)
     pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, oType)) != NULL) {
 
-	if ((requires = rpmteDS(p, RPMTAG_REQUIRENAME)) == NULL)
-	    continue;
+      if ((requires = rpmteDS(p, RPMTAG_REQUIRENAME)) != NULL) {
 
 	memset(selected, 0, sizeof(*selected) * ts->orderCount);
 
@@ -1291,13 +1539,13 @@ int rpmtsOrder(rpmts ts)
 
 	    switch (rpmteType(p)) {
 	    case TR_REMOVED:
-		/* Skip if not %preun/%postun requires or legacy prereq. */
-		if (!( isErasePreReq(Flags) || isLegacyPreReq(Flags) ) )
+		/* Skip if not %preun/%postun requires. */
+		if (!isErasePreReq(Flags))
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    case TR_ADDED:
-		/* Skip if not %pre/%post requires or legacy prereq. */
-		if (!( isInstallPreReq(Flags) || isLegacyPreReq(Flags) ) )
+		/* Skip if not %pre/%post requires. */
+		if (!isInstallPreReq(Flags))
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    }
@@ -1316,15 +1564,13 @@ int rpmtsOrder(rpmts ts)
 
 	    switch (rpmteType(p)) {
 	    case TR_REMOVED:
-		/* Skip if %preun/%postun requires or legacy prereq. */
-		if (isInstallPreReq(Flags)
-		 ||  ( isErasePreReq(Flags) || isLegacyPreReq(Flags) ) )
+		/* Skip if %preun/%postun requires. */
+		if (isErasePreReq(Flags))
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    case TR_ADDED:
-		/* Skip if %pre/%post requires or legacy prereq. */
-		if (isErasePreReq(Flags)
-		 ||  ( isInstallPreReq(Flags) || isLegacyPreReq(Flags) ) )
+		/* Skip if %pre/%post requires. */
+		if (isInstallPreReq(Flags))
 		    /*@innercontinue@*/ continue;
 		/*@switchbreak@*/ break;
 	    }
@@ -1333,6 +1579,30 @@ int rpmtsOrder(rpmts ts)
 	    (void) addRelation(ts, p, selected, requires);
 
 	}
+      }
+
+#ifdef	NOTYET
+      {	const char * Name = rpmteN(p);
+	if (Name == NULL || !strcmp(Name, "setup") || !strcmp(Name, "filesystem"))
+	    continue;
+
+	/* Finally synthesize a requirement on parent directories. */
+	if ((requires = rpmteDS(p, RPMTAG_DIRNAMES)) == NULL)
+	    continue;
+	requires = rpmdsInit(requires);
+	if (requires != NULL)
+	while (rpmdsNext(requires) >= 0) {
+
+	    /* T3. Record next "q <- p" relation (i.e. "p" requires "q"). */
+	    (void) addRelation(ts, p, selected, requires);
+
+	    /* XXX only add the 1st dirname to avoid loops. */
+	    break;
+
+	}
+      }
+#endif
+
     }
     pi = rpmtsiFree(pi);
 
@@ -1345,11 +1615,13 @@ int rpmtsOrder(rpmts ts)
 	npreds = rpmteTSI(p)->tsi_count;
 
 	(void) rpmteSetNpreds(p, npreds);
-	(void) rpmteSetDepth(p, 1);
+	(void) rpmteSetDepth(p, 0);
 
-	if (npreds == 0)
-	    (void) rpmteSetTree(p, treex++);
-	else
+	if (npreds == 0) {
+	    treex++;
+	    (void) rpmteSetTree(p, treex);
+	    (void) rpmteSetBreadth(p, treex);
+	} else
 	    (void) rpmteSetTree(p, -1);
 #ifdef	UNNECESSARY
 	(void) rpmteSetParent(p, NULL);
@@ -1360,7 +1632,7 @@ int rpmtsOrder(rpmts ts)
     ts->ntrees = treex;
 
     /* T4. Scan for zeroes. */
-    rpmMessage(RPMMESS_DEBUG, _("========== tsorting packages (order, #predecessors, #succesors, tree, depth, breadth)\n"));
+    rpmMessage(RPMMESS_DEBUG, _("========== tsorting packages (order, #predecessors, #succesors, tree, Ldepth, Rbreadth)\n"));
 
 rescan:
     if (pi != NULL) pi = rpmtsiFree(pi);
@@ -1376,7 +1648,7 @@ rescan:
 	if (rpmteTSI(p)->tsi_count != 0)
 	    continue;
 	rpmteTSI(p)->tsi_suc = NULL;
-	addQ(p, &q, &r, prefcolor);
+	addQ(p, &q, &r);
 	qlen++;
     }
     pi = rpmtsiFree(pi);
@@ -1385,7 +1657,7 @@ rescan:
     for (; q != NULL; q = rpmteTSI(q)->tsi_suc) {
 
 	/* Mark the package as unqueued. */
-	rpmteTSI(q)->tsi_reqx = 0;
+	rpmteTSI(q)->tsi_queued = 0;
 
 	if (oType != 0)
 	switch (rpmteType(q)) {
@@ -1419,14 +1691,7 @@ rescan:
 	(void) rpmteSetDegree(q, 0);
 	tsbytes += rpmtePkgFileSize(q);
 
-	switch (rpmteType(q)) {
-	case TR_ADDED:
-            ordering[orderingCount] = rpmteAddedKey(q);
-            /*@switchbreak@*/ break;
-	case TR_REMOVED:
-            ordering[orderingCount] = RPMAL_NOMATCH;
-            /*@switchbreak@*/ break;
-        }
+	ordering[orderingCount] = rpmteAddedKey(q);
 	orderingCount++;
 	qlen--;
 	loopcheck--;
@@ -1447,7 +1712,9 @@ rescan:
 
 		/* XXX TODO: add control bit. */
 		rpmteTSI(p)->tsi_suc = NULL;
-		addQ(p, &rpmteTSI(q)->tsi_suc, &r, prefcolor);
+/*@-nullstate@*/	/* XXX FIX: rpmteTSI(q)->tsi_suc can be NULL. */
+		addQ(p, &rpmteTSI(q)->tsi_suc, &r);
+/*@=nullstate@*/
 		qlen++;
 	    }
 	    tsi = _free(tsi);
@@ -1463,7 +1730,7 @@ rescan:
 	    pi = rpmtsiInit(ts);
 	    while ((p = rpmtsiNext(pi, oType)) != NULL) {
 		/* Is this element in the queue? */
-		if (rpmteTSI(p)->tsi_reqx == 0)
+		if (rpmteTSI(p)->tsi_queued == 0)
 		    /*@innercontinue@*/ continue;
 		tsi->tsi_suc = p;
 		tsi = rpmteTSI(p);
@@ -1482,7 +1749,7 @@ rescan:
 	qi = rpmtsiInit(ts);
 	while ((q = rpmtsiNext(qi, oType)) != NULL) {
 	    rpmteTSI(q)->tsi_chain = NULL;
-	    rpmteTSI(q)->tsi_reqx = 0;
+	    rpmteTSI(q)->tsi_queued = 0;
 	    /* Mark packages already sorted. */
 	    if (rpmteTSI(q)->tsi_count == 0)
 		rpmteTSI(q)->tsi_count = -1;
@@ -1512,9 +1779,9 @@ rescan:
 	    for (q = rpmteTSI(r)->tsi_chain; q != NULL;
 		 q = rpmteTSI(q)->tsi_chain)
 	    {
-		if (rpmteTSI(q)->tsi_reqx)
+		if (rpmteTSI(q)->tsi_queued)
 		    /*@innerbreak@*/ break;
-		rpmteTSI(q)->tsi_reqx = 1;
+		rpmteTSI(q)->tsi_queued = 1;
 	    }
 
 	    /* T13. Print predecessor chain from start of loop. */
@@ -1534,11 +1801,7 @@ rescan:
 		}
 
 		/* Find (and destroy if co-requisite) "q <- p" relation. */
-		requires = rpmteDS(p, RPMTAG_REQUIRENAME);
-		requires = rpmdsInit(requires);
-		if (requires == NULL)
-		    /*@innercontinue@*/ continue;	/* XXX can't happen */
-		dp = zapRelation(q, p, requires, 1, &nzaps, msglvl);
+		dp = zapRelation(q, p, 1, &nzaps, msglvl);
 
 		/* Print next member of loop. */
 		buf[0] = '\0';
@@ -1556,7 +1819,7 @@ rescan:
 	    {
 		/* Unchain linear part of predecessor loop. */
 		rpmteTSI(p)->tsi_chain = NULL;
-		rpmteTSI(p)->tsi_reqx = 0;
+		rpmteTSI(p)->tsi_queued = 0;
 	    }
 	}
 	ri = rpmtsiFree(ri);
@@ -1571,6 +1834,10 @@ rescan:
 	/* Return no. of packages that could not be ordered. */
 	rpmMessage(RPMMESS_ERROR, _("rpmtsOrder failed, %d elements remain\n"),
 			loopcheck);
+
+	/* Do autorollback goal since we could not sort this transaction properly. */
+	(void) rpmtsDoARBGoal(ts, NULL, RPMPROB_FILTER_NONE);
+
 	return loopcheck;
     }
 
@@ -1581,24 +1848,14 @@ rescan:
     pi = rpmtsiFree(pi);
 
     /*
-     * The order ends up as installed packages followed by removed packages,
-     * with removes for upgrades immediately following the installation of
-     * the new package. This would be easier if we could sort the
-     * addedPackages array, but we store indexes into it in various places.
+     * The order ends up as installed packages followed by removed packages.
      */
     orderList = xcalloc(numOrderList, sizeof(*orderList));
     j = 0;
     pi = rpmtsiInit(ts);
     while ((p = rpmtsiNext(pi, oType)) != NULL) {
-	/* Prepare added package ordering permutation. */
-	switch (rpmteType(p)) {
-	case TR_ADDED:
-	    orderList[j].pkgKey = rpmteAddedKey(p);
-	    /*@switchbreak@*/ break;
-	case TR_REMOVED:
-	    orderList[j].pkgKey = RPMAL_NOMATCH;
-	    /*@switchbreak@*/ break;
-	}
+	/* Prepare added/erased package ordering permutation. */
+	orderList[j].pkgKey = rpmteAddedKey(p);
 	orderList[j].orIndex = rpmtsiOc(pi);
 	j++;
     }
@@ -1618,8 +1875,7 @@ rescan:
 	key.pkgKey = ordering[i];
 	needle = bsearch(&key, orderList, numOrderList,
 				sizeof(key), orderListIndexCmp);
-	/* bsearch should never, ever fail */
-	if (needle == NULL)
+	if (needle == NULL)	/* XXX can't happen */
 	    continue;
 
 	j = needle->orIndex;
@@ -1628,27 +1884,9 @@ rescan:
 
 	newOrder[newOrderCount++] = q;
 	ts->order[j] = NULL;
-	if (anaconda)
-	for (j = needle->orIndex + 1; j < ts->orderCount; j++) {
-	    if ((q = ts->order[j]) == NULL)
-		/*@innerbreak@*/ break;
-	    if (rpmteType(q) == TR_REMOVED
-	     && rpmteDependsOnKey(q) == needle->pkgKey)
-	    {
-		newOrder[newOrderCount++] = q;
-		ts->order[j] = NULL;
-	    } else
-		/*@innerbreak@*/ break;
-	}
     }
     /*@=branchstate@*/
 
-    for (j = 0; j < ts->orderCount; j++) {
-	if ((p = ts->order[j]) == NULL)
-	    continue;
-	newOrder[newOrderCount++] = p;
-	ts->order[j] = NULL;
-    }
 assert(newOrderCount == ts->orderCount);
 
 /*@+voidabstract@*/
@@ -1707,6 +1945,7 @@ int rpmtsCheck(rpmts ts)
 	rc = checkPackageDeps(ts, rpmteNEVRA(p),
 			rpmteDS(p, RPMTAG_REQUIRENAME),
 			rpmteDS(p, RPMTAG_CONFLICTNAME),
+			rpmteDS(p, RPMTAG_DIRNAMES),
 			NULL,
 			tscolor, 1);
 	if (rc)
@@ -1804,5 +2043,10 @@ exit:
     else if (_cacheDependsRC)
 	xx = rpmdbCloseDBI(rpmtsGetRdb(ts), RPMDBI_DEPENDS);
     /*@=branchstate@*/
+
+     /* If we have a failure we need to do the autorollback goal if specified.  */
+    if (rc || rpmpsNumProblems(rpmtsProblems(ts)) > 0)
+	(void) rpmtsDoARBGoal(ts, NULL, RPMPROB_FILTER_NONE);
+
     return rc;
 }
