@@ -3,15 +3,19 @@
  */
 
 #include "system.h"
+
 #include "rpmio_internal.h"
 #include <rpmlib.h>
 #include <rpmmacro.h>	/* XXX for %_i18ndomains */
 
+#include <rpmds.h>
 #include <rpmfi.h>
 
 #include "legacy.h"
 #include "manifest.h"
+#include "argv.h"
 #include "misc.h"
+#include "fs.h"
 
 #include "debug.h"
 
@@ -27,7 +31,7 @@
  * @param element	(unused)
  * @return		formatted string
  */
-static /*@only@*/ char * triggertypeFormat(int_32 type, const void * data, 
+static /*@only@*/ char * triggertypeFormat(int_32 type, const void * data,
 		/*@unused@*/ char * formatPrefix, /*@unused@*/ int padding,
 		/*@unused@*/ int element)
 	/*@requires maxRead(data) >= 0 @*/
@@ -36,7 +40,7 @@ static /*@only@*/ char * triggertypeFormat(int_32 type, const void * data,
     char * val;
 
     if (type != RPM_INT32_TYPE)
-	val = xstrdup(_("(not a number)"));
+	val = xstrdup(_("(invalid type)"));
     else if (*item & RPMSENSE_TRIGGERPREIN)
 	val = xstrdup("prein");
     else if (*item & RPMSENSE_TRIGGERIN)
@@ -68,7 +72,7 @@ static /*@only@*/ char * permsFormat(int_32 type, const void * data,
     char * buf;
 
     if (type != RPM_INT32_TYPE) {
-	val = xstrdup(_("(not a number)"));
+	val = xstrdup(_("(invalid type)"));
     } else {
 	val = xmalloc(15 + padding);
 /*@-boundswrite@*/
@@ -93,7 +97,7 @@ static /*@only@*/ char * permsFormat(int_32 type, const void * data,
  * @param element	(unused)
  * @return		formatted string
  */
-static /*@only@*/ char * fflagsFormat(int_32 type, const void * data, 
+static /*@only@*/ char * fflagsFormat(int_32 type, const void * data,
 		char * formatPrefix, int padding, /*@unused@*/ int element)
 	/*@modifies formatPrefix @*/
 	/*@requires maxRead(data) >= 0 @*/
@@ -103,7 +107,7 @@ static /*@only@*/ char * fflagsFormat(int_32 type, const void * data,
     int anint = *((int_32 *) data);
 
     if (type != RPM_INT32_TYPE) {
-	val = xstrdup(_("(not a number)"));
+	val = xstrdup(_("(invalid type)"));
     } else {
 	buf[0] = '\0';
 /*@-boundswrite@*/
@@ -147,7 +151,7 @@ static /*@only@*/ char * fflagsFormat(int_32 type, const void * data,
  * @param element	no. bytes of binary data
  * @return		formatted string
  */
-static /*@only@*/ char * armorFormat(int_32 type, const void * data, 
+static /*@only@*/ char * armorFormat(int_32 type, const void * data,
 		/*@unused@*/ char * formatPrefix, /*@unused@*/ int padding,
 		int element)
 	/*@*/
@@ -158,6 +162,8 @@ static /*@only@*/ char * armorFormat(int_32 type, const void * data,
     int atype;
 
     switch (type) {
+    case RPM_OPENPGP_TYPE:
+    case RPM_ASN1_TYPE:		/* XXX WRONG */
     case RPM_BIN_TYPE:
 	s = data;
 	/* XXX HACK ALERT: element field abused as no. bytes of binary data. */
@@ -176,6 +182,7 @@ static /*@only@*/ char * armorFormat(int_32 type, const void * data,
     case RPM_INT8_TYPE:
     case RPM_INT16_TYPE:
     case RPM_INT32_TYPE:
+    case RPM_INT64_TYPE:
     case RPM_I18NSTRING_TYPE:
     default:
 	return xstrdup(_("(invalid type)"));
@@ -196,13 +203,13 @@ static /*@only@*/ char * armorFormat(int_32 type, const void * data,
  * @param element
  * @return		formatted string
  */
-static /*@only@*/ char * base64Format(int_32 type, const void * data, 
+static /*@only@*/ char * base64Format(int_32 type, const void * data,
 		/*@unused@*/ char * formatPrefix, int padding, int element)
 	/*@*/
 {
     char * val;
 
-    if (type != RPM_BIN_TYPE) {
+    if (!(type == RPM_BIN_TYPE || type == RPM_ASN1_TYPE || type == RPM_OPENPGP_TYPE)) {
 	val = xstrdup(_("(not a blob)"));
     } else {
 	const char * enc;
@@ -217,8 +224,8 @@ static /*@only@*/ char * base64Format(int_32 type, const void * data,
 	/* Add additional bytes necessary for eol string(s). */
 	if (b64encode_chars_per_line > 0 && b64encode_eolstr != NULL) {
 	    lc = (nt + b64encode_chars_per_line - 1) / b64encode_chars_per_line;
-        if (((nt + b64encode_chars_per_line - 1) % b64encode_chars_per_line) != 0)
-            ++lc;
+	if (((nt + b64encode_chars_per_line - 1) % b64encode_chars_per_line) != 0)
+	    ++lc;
 	    nt += lc * strlen(b64encode_eolstr);
 	}
 	/*@=globs@*/
@@ -237,6 +244,9 @@ static /*@only@*/ char * base64Format(int_32 type, const void * data,
 }
 
 /**
+ * Return length of string represented with xml characters substituted.
+ * @param s		string
+ * @return		length of xml string
  */
 static size_t xmlstrlen(const char * s)
 	/*@*/
@@ -249,15 +259,20 @@ static size_t xmlstrlen(const char * s)
 /*@=boundsread@*/
     {
 	switch (c) {
-	case '<': case '>':	len += 4;	/*@switchbreak@*/ break;
-	case '&':		len += 5;	/*@switchbreak@*/ break;
-	default:		len += 1;	/*@switchbreak@*/ break;
+	case '<':
+	case '>':	len += sizeof("&lt;") - 1;	/*@switchbreak@*/ break;
+	case '&':	len += sizeof("&amp;") - 1;	/*@switchbreak@*/ break;
+	default:	len += 1;			/*@switchbreak@*/ break;
 	}
     }
     return len;
 }
 
 /**
+ * Copy source string to target, substituting for  xml characters.
+ * @param t		target xml string
+ * @param s		source string
+ * @return		target xml string
  */
 static char * xmlstrcpy(/*@returned@*/ char * t, const char * s)
 	/*@modifies t @*/
@@ -289,7 +304,7 @@ static char * xmlstrcpy(/*@returned@*/ char * t, const char * s)
  * @return		formatted string
  */
 /*@-bounds@*/
-static /*@only@*/ char * xmlFormat(int_32 type, const void * data, 
+static /*@only@*/ char * xmlFormat(int_32 type, const void * data,
 		char * formatPrefix, int padding,
 		/*@unused@*/ int element)
 	/*@modifies formatPrefix @*/
@@ -299,7 +314,8 @@ static /*@only@*/ char * xmlFormat(int_32 type, const void * data,
     char * val;
     const char * s = NULL;
     char * t, * te;
-    unsigned long anint = 0;
+    unsigned long long anint = 0;
+    int freeit = 0;
     int xx;
 
 /*@-branchstate@*/
@@ -308,7 +324,13 @@ static /*@only@*/ char * xmlFormat(int_32 type, const void * data,
     case RPM_STRING_TYPE:
 	s = data;
 	xtag = "string";
+	/* XXX Force utf8 strings. */
+	s = xstrdup(s);
+	s = xstrtolocale(s);
+	freeit = 1;
 	break;
+    case RPM_OPENPGP_TYPE:
+    case RPM_ASN1_TYPE:
     case RPM_BIN_TYPE:
     {	int cpl = b64encode_chars_per_line;
 /*@-mods@*/
@@ -321,6 +343,7 @@ static /*@only@*/ char * xmlFormat(int_32 type, const void * data,
 	b64encode_chars_per_line = cpl;
 /*@=mods@*/
 	xtag = "base64";
+	freeit = 1;
     }	break;
     case RPM_CHAR_TYPE:
     case RPM_INT8_TYPE:
@@ -332,6 +355,9 @@ static /*@only@*/ char * xmlFormat(int_32 type, const void * data,
     case RPM_INT32_TYPE:
 	anint = *((uint_32 *) data);
 	break;
+    case RPM_INT64_TYPE:
+	anint = *((uint_64 *) data);
+	break;
     case RPM_NULL_TYPE:
     case RPM_STRING_ARRAY_TYPE:
     default:
@@ -342,10 +368,12 @@ static /*@only@*/ char * xmlFormat(int_32 type, const void * data,
 
 /*@-branchstate@*/
     if (s == NULL) {
-	int tlen = 32;
+	int tlen = 64;
 	t = memset(alloca(tlen+1), 0, tlen+1);
+/*@-duplicatequals@*/
 	if (anint != 0)
-	    xx = snprintf(t, tlen, "%lu", anint);
+	    xx = snprintf(t, tlen, "%llu", anint);
+/*@=duplicatequals@*/
 	s = t;
 	xtag = "integer";
     }
@@ -365,9 +393,237 @@ static /*@only@*/ char * xmlFormat(int_32 type, const void * data,
 	te = stpcpy( stpcpy( stpcpy(te, "</"), xtag), ">");
     }
 
+/*@-branchstate@*/
+    if (freeit)
+	s = _free(s);
+/*@=branchstate@*/
+
+    nb += padding;
+    val = xmalloc(nb+1);
+/*@-boundswrite@*/
+    strcat(formatPrefix, "s");
+/*@=boundswrite@*/
+/*@-formatconst@*/
+    xx = snprintf(val, nb, formatPrefix, t);
+/*@=formatconst@*/
+    val[nb] = '\0';
+
+    return val;
+}
+/*@=bounds@*/
+
+/**
+ * Return length of string represented with yaml indentation.
+ * @param s		string
+ * @param lvl		indentation level
+ * @return		length of yaml string
+ */
+static size_t yamlstrlen(const char * s, int lvl)
+	/*@*/
+{
+    size_t len = 0;
+    int indent = (lvl > 0);
+    int c;
+
+/*@-boundsread@*/
+    while ((c = *s++) != '\0')
+/*@=boundsread@*/
+    {
+	if (indent) {
+	    len += 2 * lvl;
+	    indent = 0;
+	}
+	if (c == '\n')
+	    indent = (lvl > 0);
+	len++;
+    }
+    return len;
+}
+
+/**
+ * Copy source string to target, indenting for yaml.
+ * @param t		target yaml string
+ * @param s		source string
+ * @param lvl		indentation level
+ * @return		target yaml string
+ */
+static char * yamlstrcpy(/*@out@*/ /*@returned@*/ char * t, const char * s, int lvl)
+	/*@modifies t @*/
+{
+    char * te = t;
+    int indent = (lvl > 0);
+    int c;
+
+/*@-bounds@*/
+    while ((c = *s++) != '\0') {
+	if (indent) {
+	    int i;
+	    for (i = 0; i < lvl; i++) {
+		*te++ = ' ';
+		*te++ = ' ';
+	    }
+	    indent = 0;
+	}
+	if (c == '\n')
+	    indent = (lvl > 0);
+	*te++ = c;
+    }
+    *te = '\0';
+/*@=bounds@*/
+    return t;
+}
+
+/**
+ * Wrap tag data in simple header yaml markup.
+ * @param type		tag type
+ * @param data		tag value
+ * @param formatPrefix
+ * @param padding
+ * @param element	element index (or -1 for non-array).
+ * @return		formatted string
+ */
+/*@-bounds@*/
+static /*@only@*/ char * yamlFormat(int_32 type, const void * data,
+		char * formatPrefix, int padding,
+		int element)
+	/*@modifies formatPrefix @*/
+{
+    const char * xtag = NULL;
+    const char * ytag = NULL;
+    size_t nb;
+    char * val;
+    const char * s = NULL;
+    char * t, * te;
+    unsigned long long anint = 0;
+    int freeit = 0;
+    int lvl = 0;
+    int xx;
+    int c;
+
+/*@-branchstate@*/
+    switch (type) {
+    case RPM_I18NSTRING_TYPE:
+    case RPM_STRING_TYPE:
+	xx = 0;
+	s = data;
+	if (strchr("[", s[0]))	/* leading [ */
+	    xx = 1;
+	if (xx == 0)
+	while ((c = *s++) != '\0') {
+	    switch (c) {
+	    default:
+		continue;
+	    case '\n':	/* multiline */
+		xx = 1;
+		/*@switchbreak@*/ break;
+	    case '-':	/* leading "- \"" */
+	    case ':':	/* embedded ": " or ":" at EOL */
+		if (s[0] != ' ' && s[0] != '\0' && s[1] != '"')
+		    continue;
+		xx = 1;
+		/*@switchbreak@*/ break;
+	    }
+	    /*@loopbreak@*/ break;
+	}
+	if (xx) {
+	    if (element >= 0) {
+		xtag = "- |-\n";
+		lvl = 3;
+	    } else {
+		xtag = "|-\n";
+		lvl = 2;
+	    }
+	} else {
+	    xtag = (element >= 0 ? "- " : NULL);
+	}
+
+	/* XXX Force utf8 strings. */
+	s = xstrdup(data);
+	s = xstrtolocale(s);
+	freeit = 1;
+	break;
+    case RPM_OPENPGP_TYPE:
+    case RPM_ASN1_TYPE:
+    case RPM_BIN_TYPE:
+    {	int cpl = b64encode_chars_per_line;
+/*@-mods@*/
+	b64encode_chars_per_line = 0;
+/*@=mods@*/
+/*@-formatconst@*/
+	s = base64Format(type, data, formatPrefix, padding, element);
+	element = -element;	/* XXX skip "    " indent. */
+/*@=formatconst@*/
+/*@-mods@*/
+	b64encode_chars_per_line = cpl;
+/*@=mods@*/
+	xtag = "!!binary ";
+	freeit = 1;
+    }	break;
+    case RPM_CHAR_TYPE:
+    case RPM_INT8_TYPE:
+	anint = *((uint_8 *) data);
+	break;
+    case RPM_INT16_TYPE:
+	anint = *((uint_16 *) data);
+	break;
+    case RPM_INT32_TYPE:
+	anint = *((uint_32 *) data);
+	break;
+    case RPM_INT64_TYPE:
+	anint = *((uint_64 *) data);
+	break;
+    case RPM_NULL_TYPE:
+    case RPM_STRING_ARRAY_TYPE:
+    default:
+	return xstrdup(_("(invalid yaml type)"));
+	/*@notreached@*/ break;
+    }
+/*@=branchstate@*/
+
+/*@-branchstate@*/
+    if (s == NULL) {
+	int tlen = 64;
+	t = memset(alloca(tlen+1), 0, tlen+1);
+/*@-duplicatequals@*/
+	xx = snprintf(t, tlen, "%llu", anint);
+/*@=duplicatequals@*/
+	s = t;
+	xtag = (element >= 0 ? "- " : NULL);
+    }
+/*@=branchstate@*/
+
+    nb = yamlstrlen(s, lvl);
+    if (nb == 0) {
+	if (element >= 0)
+	    nb += sizeof("    ") - 1;
+	nb += sizeof("- ~") - 1;
+	nb++;
+	te = t = alloca(nb);
+	if (element >= 0)
+	    te = stpcpy(te, "    ");
+	te = stpcpy(te, "- ~");
+    } else {
+	if (element >= 0)
+	    nb += sizeof("    ") - 1;
+	if (xtag)
+	    nb += strlen(xtag);
+	if (ytag)
+	    nb += strlen(ytag);
+	nb++;
+	te = t = alloca(nb);
+	if (element >= 0)
+	    te = stpcpy(te, "    ");
+	if (xtag)
+	    te = stpcpy(te, xtag);
+	te = yamlstrcpy(te, s, lvl);
+	te += strlen(te);
+	if (ytag)
+	    te = stpcpy(te, ytag);
+    }
+
     /* XXX s was malloc'd */
 /*@-branchstate@*/
-    if (!strcmp(xtag, "base64"))
+    if (freeit)
 	s = _free(s);
 /*@=branchstate@*/
 
@@ -394,7 +650,7 @@ static /*@only@*/ char * xmlFormat(int_32 type, const void * data,
  * @param element	(unused)
  * @return		formatted string
  */
-static /*@only@*/ char * pgpsigFormat(int_32 type, const void * data, 
+static /*@only@*/ char * pgpsigFormat(int_32 type, const void * data,
 		/*@unused@*/ char * formatPrefix, /*@unused@*/ int padding,
 		/*@unused@*/ int element)
 	/*@globals fileSystem, internalState @*/
@@ -402,7 +658,7 @@ static /*@only@*/ char * pgpsigFormat(int_32 type, const void * data,
 {
     char * val, * t;
 
-    if (type != RPM_BIN_TYPE) {
+    if (!(type == RPM_BIN_TYPE || type == RPM_ASN1_TYPE || type == RPM_OPENPGP_TYPE)) {
 	val = xstrdup(_("(not a blob)"));
     } else {
 	unsigned char * pkt = (byte *) data;
@@ -507,7 +763,7 @@ static /*@only@*/ char * pgpsigFormat(int_32 type, const void * data,
  * @param element	(unused)
  * @return		formatted string
  */
-static /*@only@*/ char * depflagsFormat(int_32 type, const void * data, 
+static /*@only@*/ char * depflagsFormat(int_32 type, const void * data,
 		char * formatPrefix, int padding, /*@unused@*/ int element)
 	/*@modifies formatPrefix @*/
 	/*@requires maxRead(data) >= 0 @*/
@@ -517,13 +773,23 @@ static /*@only@*/ char * depflagsFormat(int_32 type, const void * data,
     int anint;
 
     if (type != RPM_INT32_TYPE) {
-	val = xstrdup(_("(not a number)"));
+	val = xstrdup(_("(invalid type)"));
     } else {
 	anint = *((int_32 *) data);
 	buf[0] = '\0';
 
 /*@-boundswrite@*/
-	if (anint & RPMSENSE_LESS) 
+#ifdef	NOTYET	/* XXX appending markers breaks :depflags format. */
+	if (anint & RPMSENSE_SCRIPT_PRE)
+	    strcat(buf,"(pre)");
+	if (anint & RPMSENSE_SCRIPT_POST)
+	    strcat(buf,"(post)");
+	if (anint & RPMSENSE_SCRIPT_PREUN)
+	    strcat(buf,"(preun)");
+	if (anint & RPMSENSE_SCRIPT_POSTUN)
+	    strcat(buf,"(postun)");
+#endif
+	if (anint & RPMSENSE_LESS)
 	    strcat(buf, "<");
 	if (anint & RPMSENSE_GREATER)
 	    strcat(buf, ">");
@@ -572,7 +838,7 @@ static int fsnamesTag( /*@unused@*/ Header h, /*@out@*/ int_32 * type,
     if (data) *((const char ***) data) = list;
     if (freeData) *freeData = 0;
 
-    return 0; 
+    return 0;
 }
 
 /**
@@ -635,7 +901,7 @@ static int fssizesTag(Header h, /*@out@*/ rpmTagType * type,
     HGE_t hge = (HGE_t)headerGetEntryMinMemory;
     const char ** filenames;
     int_32 * filesizes;
-    uint_32 * usages;
+    uint_64 * usages;
     int numFiles;
 
     if (!hge(h, RPMTAG_FILESIZES, NULL, (void **) &filesizes, &numFiles)) {
@@ -651,11 +917,11 @@ static int fssizesTag(Header h, /*@out@*/ rpmTagType * type,
 	return 1;
 /*@=boundswrite@*/
 
-    *type = RPM_INT32_TYPE;
+    *type = RPM_INT64_TYPE;
     *freeData = 1;
 
     if (filenames == NULL) {
-	usages = xcalloc((*count), sizeof(usages));
+	usages = xcalloc((*count), sizeof(*usages));
 	*data = usages;
 
 	return 0;
@@ -831,7 +1097,7 @@ static int filenamesTag(Header h, /*@out@*/ rpmTagType * type,
     *type = RPM_STRING_ARRAY_TYPE;
     rpmfiBuildFNames(h, RPMTAG_BASENAMES, (const char ***) data, count);
     *freeData = 1;
-    return 0; 
+    return 0;
 }
 
 /**
@@ -855,7 +1121,7 @@ static int fileclassTag(Header h, /*@out@*/ rpmTagType * type,
     *type = RPM_STRING_ARRAY_TYPE;
     rpmfiBuildFClasses(h, (const char ***) data, count);
     *freeData = 1;
-    return 0; 
+    return 0;
 }
 
 /**
@@ -879,7 +1145,7 @@ static int filecontextsTag(Header h, /*@out@*/ rpmTagType * type,
     *type = RPM_STRING_ARRAY_TYPE;
     rpmfiBuildFContexts(h, (const char ***) data, count);
     *freeData = 1;
-    return 0; 
+    return 0;
 }
 
 /**
@@ -903,7 +1169,7 @@ static int fscontextsTag(Header h, /*@out@*/ rpmTagType * type,
     *type = RPM_STRING_ARRAY_TYPE;
     rpmfiBuildFSContexts(h, (const char ***) data, count);
     *freeData = 1;
-    return 0; 
+    return 0;
 }
 
 /**
@@ -927,7 +1193,7 @@ static int recontextsTag(Header h, /*@out@*/ rpmTagType * type,
     *type = RPM_STRING_ARRAY_TYPE;
     rpmfiBuildREContexts(h, (const char ***) data, count);
     *freeData = 1;
-    return 0; 
+    return 0;
 }
 
 /**
@@ -951,7 +1217,7 @@ static int fileprovideTag(Header h, /*@out@*/ rpmTagType * type,
     *type = RPM_STRING_ARRAY_TYPE;
     rpmfiBuildFDeps(h, RPMTAG_PROVIDENAME, (const char ***) data, count);
     *freeData = 1;
-    return 0; 
+    return 0;
 }
 
 /**
@@ -975,7 +1241,71 @@ static int filerequireTag(Header h, /*@out@*/ rpmTagType * type,
     *type = RPM_STRING_ARRAY_TYPE;
     rpmfiBuildFDeps(h, RPMTAG_REQUIRENAME, (const char ***) data, count);
     *freeData = 1;
-    return 0; 
+    return 0;
+}
+
+/**
+ * Retrieve Requires(missingok): array for Suggests: or Enhances:.
+ * @param h		header
+ * @retval *type	tag type
+ * @retval *data	tag value
+ * @retval *count	no. of data items
+ * @retval *freeData	data-was-malloc'ed indicator
+ * @return		0 on success
+ */
+static int missingokTag(Header h, /*@out@*/ rpmTagType * type,
+		/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
+		/*@out@*/ int * freeData)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies h, *type, *data, *count, *freeData,
+		rpmGlobalMacroContext, fileSystem, internalState @*/
+	/*@requires maxSet(type) >= 0 /\ maxSet(data) >= 0
+		/\ maxSet(count) >= 0 /\ maxSet(freeData) >= 0 @*/
+{
+    rpmds ds = rpmdsNew(h, RPMTAG_REQUIRENAME, 0);
+    ARGV_t av = NULL;
+    ARGV_t argv;
+    int argc = 0;
+    char * t;
+    size_t nb = 0;
+    int i;
+
+assert(ds != NULL);
+    /* Collect dependencies marked as hints. */
+    ds = rpmdsInit(ds);
+    if (ds != NULL)
+    while (rpmdsNext(ds) >= 0) {
+	int Flags = rpmdsFlags(ds);
+	const char * DNEVR;
+	if (!(Flags & RPMSENSE_MISSINGOK))
+	    continue;
+	DNEVR = rpmdsDNEVR(ds);
+	if (DNEVR == NULL)
+	    continue;
+	nb += sizeof(*argv) + strlen(DNEVR+2) + 1;
+	(void) argvAdd(&av, DNEVR+2);
+	argc++;
+    }
+    nb += sizeof(*argv);	/* final argv NULL */
+
+    /* Create contiguous header string array. */
+    argv = (ARGV_t) xcalloc(nb, 1);
+    t = (char *)(argv + argc);
+    for (i = 0; i < argc; i++) {
+	argv[i] = t;
+	t = stpcpy(t, av[i]);
+	*t++ = '\0';
+    }
+    av = argvFree(av);
+    ds = rpmdsFree(ds);
+
+    /* XXX perhaps return "(none)" inband if no suggests/enhances <shrug>. */
+
+    *type = RPM_STRING_ARRAY_TYPE;
+    *data = argv;
+    *count = argc;
+    *freeData = 1;
+    return 0;
 }
 
 /* I18N look aside diversions */
@@ -1028,11 +1358,8 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ rpmTagType * type,
 	{   const char * tn = tagName(tag);
 	    const char * n;
 	    char * mk;
-	    size_t nb = sizeof("()");
-	    int xx = headerNVR(h, &n, NULL, NULL);
-	    if (tn)	nb += strlen(tn);
-	    if (n)	nb += strlen(n);
-	    mk = alloca(nb);
+	    (void) headerNVR(h, &n, NULL, NULL);
+	    mk = alloca(strlen(n) + strlen(tn) + sizeof("()"));
 	    sprintf(mk, "%s(%s)", n, tn);
 	    msgkey = mk;
 	}
@@ -1080,6 +1407,7 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ rpmTagType * type,
 
     if (rc && (*data) != NULL) {
 	*data = xstrdup(*data);
+	*data = xstrtolocale(*data);
 	*freeData = 1;
 	return 0;
     }
@@ -1090,6 +1418,57 @@ static int i18nTag(Header h, int_32 tag, /*@out@*/ rpmTagType * type,
     return 1;
 }
 
+/**
+ * Retrieve text and convert to locale.
+ */
+static int localeTag(Header h, int_32 tag, /*@out@*/ rpmTagType * type,
+		/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
+		/*@out@*/ int * freeData)
+	/*@modifies *type, *data, *count, *freeData @*/
+{
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    rpmTagType t;
+    char **d, **d2, *dp;
+    int rc, i, l;
+
+    rc = hge(h, tag, &t, (void **)&d, count);
+    if (!rc || d == NULL || *count == 0) {
+	*freeData = 0;
+	*data = NULL;
+	*count = 0;
+	return 1;
+    }
+    if (type)
+	*type = t;
+    if (t == RPM_STRING_TYPE) {
+	d = (char **)xstrdup((char *)d);
+	d = (char **)xstrtolocale((char *)d);
+	*freeData = 1;
+    } else if (t == RPM_STRING_ARRAY_TYPE) {
+	l = 0;
+	for (i = 0; i < *count; i++) {
+	    d[i] = xstrdup(d[i]);
+	    d[i] = (char *)xstrtolocale(d[i]);
+assert(d[i] != NULL);
+	    l += strlen(d[i]) + 1;
+	}
+	d2 = xmalloc(*count * sizeof(*d2) + l);
+	dp = (char *)(d2 + *count);
+	for (i = 0; i < *count; i++) {
+	    d2[i] = dp;
+	    strcpy(dp, d[i]);
+	    dp += strlen(dp) + 1;
+	    d[i] = _free(d[i]);
+	}
+	d = _free(d);
+	d = d2;
+	*freeData = 1;
+    } else
+	*freeData = 0;
+    *data = (void **)d;
+    return 0;
+}
+
 /**
  * Retrieve summary text.
  * @param h		header
@@ -1130,6 +1509,26 @@ static int descriptionTag(Header h, /*@out@*/ rpmTagType * type,
     return i18nTag(h, RPMTAG_DESCRIPTION, type, data, count, freeData);
 }
 
+static int changelognameTag(Header h, /*@out@*/ rpmTagType * type,
+		/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
+		/*@out@*/ int * freeData)
+	/*@modifies *type, *data, *count, *freeData @*/
+	/*@requires maxSet(type) >= 0 /\ maxSet(data) >= 0
+		/\ maxSet(count) >= 0 /\ maxSet(freeData) >= 0 @*/
+{
+    return localeTag(h, RPMTAG_CHANGELOGNAME, type, data, count, freeData);
+}
+
+static int changelogtextTag(Header h, /*@out@*/ rpmTagType * type,
+		/*@out@*/ const void ** data, /*@out@*/ int_32 * count,
+		/*@out@*/ int * freeData)
+	/*@modifies *type, *data, *count, *freeData @*/
+	/*@requires maxSet(type) >= 0 /\ maxSet(data) >= 0
+		/\ maxSet(count) >= 0 /\ maxSet(freeData) >= 0 @*/
+{
+    return localeTag(h, RPMTAG_CHANGELOGTEXT, type, data, count, freeData);
+}
+
 /**
  * Retrieve group text.
  * @param h		header
@@ -1152,9 +1551,10 @@ static int groupTag(Header h, /*@out@*/ rpmTagType * type,
 
 /*@-type@*/ /* FIX: cast? */
 const struct headerSprintfExtension_s rpmHeaderFormats[] = {
-    { HEADER_EXT_TAG, "RPMTAG_GROUP",		{ groupTag } },
+    { HEADER_EXT_TAG, "RPMTAG_CHANGELOGNAME",	{ changelognameTag } },
+    { HEADER_EXT_TAG, "RPMTAG_CHANGELOGTEXT",	{ changelogtextTag } },
     { HEADER_EXT_TAG, "RPMTAG_DESCRIPTION",	{ descriptionTag } },
-    { HEADER_EXT_TAG, "RPMTAG_SUMMARY",		{ summaryTag } },
+    { HEADER_EXT_TAG, "RPMTAG_ENHANCES",	{ missingokTag } },
     { HEADER_EXT_TAG, "RPMTAG_FILECLASS",	{ fileclassTag } },
     { HEADER_EXT_TAG, "RPMTAG_FILECONTEXTS",	{ filecontextsTag } },
     { HEADER_EXT_TAG, "RPMTAG_FILENAMES",	{ filenamesTag } },
@@ -1163,19 +1563,23 @@ const struct headerSprintfExtension_s rpmHeaderFormats[] = {
     { HEADER_EXT_TAG, "RPMTAG_FSCONTEXTS",	{ fscontextsTag } },
     { HEADER_EXT_TAG, "RPMTAG_FSNAMES",		{ fsnamesTag } },
     { HEADER_EXT_TAG, "RPMTAG_FSSIZES",		{ fssizesTag } },
+    { HEADER_EXT_TAG, "RPMTAG_GROUP",		{ groupTag } },
     { HEADER_EXT_TAG, "RPMTAG_INSTALLPREFIX",	{ instprefixTag } },
     { HEADER_EXT_TAG, "RPMTAG_RECONTEXTS",	{ recontextsTag } },
+    { HEADER_EXT_TAG, "RPMTAG_SUGGESTS",	{ missingokTag } },
+    { HEADER_EXT_TAG, "RPMTAG_SUMMARY",		{ summaryTag } },
     { HEADER_EXT_TAG, "RPMTAG_TRIGGERCONDS",	{ triggercondsTag } },
     { HEADER_EXT_TAG, "RPMTAG_TRIGGERTYPE",	{ triggertypeTag } },
     { HEADER_EXT_FORMAT, "armor",		{ armorFormat } },
     { HEADER_EXT_FORMAT, "base64",		{ base64Format } },
-    { HEADER_EXT_FORMAT, "pgpsig",		{ pgpsigFormat } },
     { HEADER_EXT_FORMAT, "depflags",		{ depflagsFormat } },
     { HEADER_EXT_FORMAT, "fflags",		{ fflagsFormat } },
     { HEADER_EXT_FORMAT, "perms",		{ permsFormat } },
     { HEADER_EXT_FORMAT, "permissions",		{ permsFormat } },
+    { HEADER_EXT_FORMAT, "pgpsig",		{ pgpsigFormat } },
     { HEADER_EXT_FORMAT, "triggertype",		{ triggertypeFormat } },
     { HEADER_EXT_FORMAT, "xml",			{ xmlFormat } },
+    { HEADER_EXT_FORMAT, "yaml",		{ yamlFormat } },
     { HEADER_EXT_MORE, NULL,		{ (void *) headerDefaultFormats } }
 } ;
 /*@=type@*/
