@@ -6,6 +6,7 @@
 
 #include <rpmio_internal.h>
 #include <rpmlib.h>
+#include <rpmmacro.h>
 
 #include "stringbuf.h"
 #include "manifest.h"
@@ -14,22 +15,29 @@
 
 /*@access StringBuf @*/
 
+/*@-boundswrite@*/
 char * rpmPermsString(int mode)
 {
     char *perms = xstrdup("----------");
    
-    if (S_ISDIR(mode)) 
+    if (S_ISREG(mode)) 
+	perms[0] = '-';
+    else if (S_ISDIR(mode)) 
 	perms[0] = 'd';
     else if (S_ISLNK(mode))
 	perms[0] = 'l';
     else if (S_ISFIFO(mode)) 
 	perms[0] = 'p';
+    /*@-unrecog@*/
     else if (S_ISSOCK(mode)) 
 	perms[0] = 's';
+    /*@=unrecog@*/
     else if (S_ISCHR(mode))
 	perms[0] = 'c';
     else if (S_ISBLK(mode))
 	perms[0] = 'b';
+    else
+	perms[0] = '?';
 
     if (mode & S_IRUSR) perms[1] = 'r';
     if (mode & S_IWUSR) perms[2] = 'w';
@@ -54,9 +62,11 @@ char * rpmPermsString(int mode)
 
     return perms;
 }
+/*@=boundswrite@*/
 
 /**@todo Infinite loops through manifest files exist, operator error for now. */
-int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
+/*@-boundsread@*/
+rpmRC rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
 {
     StringBuf sb = newStringBuf();
     char * s = NULL;
@@ -65,11 +75,26 @@ int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
     const char ** av = NULL;
     int argc = (argcPtr ? *argcPtr : 0);
     const char ** argv = (argvPtr ? *argvPtr : NULL);
-    FILE * f = fdGetFp(fd);
-    int rc = 0;
-    int i;
+    FD_t xfd;
+    FILE * f;
+    rpmRC rpmrc = RPMRC_OK;
+    int i, j, next, npre;
+
+/*@-boundswrite@*/
+/*@-branchstate@*/
+    if (fdGetFp(fd) == NULL)
+	xfd = Fdopen(fd, "r.fpio");
+    else
+	xfd = fd;
+/*@=branchstate@*/
+
+/*@+voidabstract@*/
+    if ((f = (FILE *) fdGetFp(xfd)) == NULL) {
+/*@=voidabstract@*/
+	rpmrc = RPMRC_NOTFOUND;
+	goto exit;
+    }
 
-    if (f != NULL)
     while (1) {
 	char line[BUFSIZ];
 
@@ -80,6 +105,13 @@ int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
 	    break;
 	}
 
+	/* XXX stop processing manifest if HTML is found. */
+#define	DOCTYPE_HTML_PUBLIC	"<!DOCTYPE HTML PUBLIC"
+	if (!strncmp(line, DOCTYPE_HTML_PUBLIC, sizeof(DOCTYPE_HTML_PUBLIC)-1)) {
+	    rpmrc = RPMRC_NOTFOUND;
+	    goto exit;
+	}
+
 	/* Skip comments. */
 	if ((se = strchr(s, '#')) != NULL) *se = '\0';
 
@@ -93,7 +125,7 @@ int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
 
 	/* Insure that file contains only ASCII */
 	if (*s < 32) {
-	    rc = 1;
+	    rpmrc = RPMRC_NOTFOUND;
 	    goto exit;
 	}
 
@@ -103,31 +135,47 @@ int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
 	appendStringBuf(sb, s);
     }
 
+    /*@-branchstate@*/
     if (s == NULL)		/* XXX always true */
 	s = getStringBuf(sb);
+    /*@=branchstate@*/
 
     if (!(s && *s)) {
-	rc = 1;
+	rpmrc = RPMRC_NOTFOUND;
 	goto exit;
     }
 
     /* Glob manifest items. */
-    rc = rpmGlob(s, &ac, &av);
-    if (rc) goto exit;
+    rpmrc = rpmGlob(s, &ac, &av);
+    if (rpmrc != RPMRC_OK) goto exit;
 
-    /* Find 1st existing unprocessed arg. */
-    for (i = 0; i < argc; i++)
-	if (argv && argv[i]) break;
+    rpmMessage(RPMMESS_DEBUG, _("adding %d args from manifest.\n"), ac);
 
-    /* Concatenate existing unprocessed args after manifest contents. */
-    if (argv && i < argc) {
-	int nac = ac + (argc - i);
+    /* Count non-NULL args, keeping track of 1st arg after last NULL. */
+    npre = 0;
+    next = 0;
+    if (argv != NULL)
+    for (i = 0; i < argc; i++) {
+	if (argv[i] != NULL)
+	    npre++;
+	else if (i >= next)
+	    next = i + 1;
+    }
+
+    /* Copy old arg list, inserting manifest before argv[next]. */
+    if (argv != NULL) {
+	int nac = npre + ac;
 	const char ** nav = xcalloc((nac + 1), sizeof(*nav));
 
+	for (i = 0, j = 0; i < next; i++) {
+	    if (argv[i] != NULL)
+		nav[j++] = argv[i];
+	}
+
 	if (ac)
-	    memcpy(nav, av, ac * sizeof(*nav));
-	if ((argc - i) > 0)
-	    memcpy(nav + ac, argv + i, (argc - i) * sizeof(*nav));
+	    memcpy(nav + j, av, ac * sizeof(*nav));
+	if ((argc - next) > 0)
+	    memcpy(nav + j + ac, argv + next, (argc - next) * sizeof(*nav));
 	nav[nac] = NULL;
 
 	if (argvPtr)
@@ -144,16 +192,22 @@ int rpmReadPackageManifest(FD_t fd, int * argcPtr, const char *** argvPtr)
     }
     if (argcPtr)
 	*argcPtr = ac;
+/*@=boundswrite@*/
 
 exit:
-    if (argvPtr == NULL || (rc != 0 && av)) {
+    /*@-branchstate@*/
+    if (argvPtr == NULL || (rpmrc != RPMRC_OK && av)) {
 	if (av)
+/*@-boundswrite@*/
 	for (i = 0; i < ac; i++)
 	    /*@-unqualifiedtrans@*/av[i] = _free(av[i]); /*@=unqualifiedtrans@*/
+/*@=boundswrite@*/
 	/*@-dependenttrans@*/ av = _free(av); /*@=dependenttrans@*/
     }
+    /*@=branchstate@*/
     sb = freeStringBuf(sb);
-    /*@-nullstate@*/
-    return rc;
+    /*@-nullstate@*/ /* FIX: *argvPtr may be NULL. */
+    return rpmrc;
     /*@=nullstate@*/
 }
+/*@=boundsread@*/
