@@ -26,6 +26,9 @@ static int _debug = 0;
 /*@unchecked@*/
 extern int _rpmds_nopromote;
 
+/*@unchecked@*/
+extern int _cpio_debug;
+
 /*@unchecked@*/
 extern int _fps_debug;
 
@@ -84,6 +87,9 @@ extern int _rpmts_debug;
 /*@unchecked@*/
 extern int _rpmts_stats;
 
+/*@unchecked@*/
+extern int _tar_debug;
+
 /*@unchecked@*/
 extern int noLibio;
 /*@=exportheadervar@*/
@@ -173,24 +179,15 @@ static void rpmcliAllArgCallback( /*@unused@*/ poptContext con,
 	(void) rpmDefineMacro(NULL, arg, RMIL_CMDLINE);
 	break;
     case 'D':
-    {   char *s, *t;
-	/* XXX Convert '-' in macro name to underscore, skip leading %. */
-	s = t = xstrdup(arg);
-	while (*t && !xisspace(*t)) {
-	    if (*t == '-') *t = '_';
-	    t++;
-	}
-	t = s;
-	if (*t == '%') t++;
 	/* XXX Predefine macro if not initialized yet. */
 	if (rpmcliInitialized < 0)
-	    (void) rpmDefineMacro(NULL, t, RMIL_CMDLINE);
+	    (void) rpmDefineMacro(NULL, arg, RMIL_CMDLINE);
 	rpmcliConfigured();
-	(void) rpmDefineMacro(NULL, t, RMIL_CMDLINE);
-	(void) rpmDefineMacro(rpmCLIMacroContext, t, RMIL_CMDLINE);
-	s = _free(s);
+/*@-type@*/
+	(void) rpmDefineMacro(NULL, arg, RMIL_CMDLINE);
+	(void) rpmDefineMacro(rpmCLIMacroContext, arg, RMIL_CMDLINE);
+/*@=type@*/
 	break;
-    }
     case 'E':
 	rpmcliConfigured();
 	{   const char *val = rpmExpand(arg, NULL);
@@ -265,7 +262,7 @@ struct poptOption rpmcliAllPoptTable[] = {
  { "debug", 'd', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_debug, -1,
         NULL, NULL },
 
- { "predefine", 'D', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_PREDEFINE,
+ { "predefine", '\0', POPT_ARG_STRING|POPT_ARGFLAG_DOC_HIDDEN, 0, POPT_PREDEFINE,
 	N_("predefine MACRO with value EXPR"),
 	N_("'MACRO EXPR'") },
  { "define", 'D', POPT_ARG_STRING, 0, 'D',
@@ -324,6 +321,8 @@ struct poptOption rpmcliAllPoptTable[] = {
  { "promoteepoch", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmds_nopromote, 0,
 	NULL, NULL},
 
+ { "cpiodebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_cpio_debug, -1,
+	N_("debug cpio payloads"), NULL},
  { "fpsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_fps_debug, -1,
 	NULL, NULL},
  { "fsmdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_fsm_debug, -1,
@@ -372,6 +371,8 @@ struct poptOption rpmcliAllPoptTable[] = {
 	NULL, NULL},
  { "rpmtsdebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmts_debug, -1,
 	NULL, NULL},
+ { "tardebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_tar_debug, -1,
+	N_("debug tar payloads"), NULL},
  { "stats", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_rpmts_stats, -1,
 	NULL, NULL},
  { "urldebug", '\0', POPT_ARG_VAL|POPT_ARGFLAG_DOC_HIDDEN, &_url_debug, -1,
