@@ -39,14 +39,6 @@ int _psm_debug = _PSM_DEBUG;
 /*@unchecked@*/
 int _psm_threads = 0;
 
-/* Give access to the rpmte global tracking the last instance added
- * to the database.
- */
-/*@-exportheadervar@*/
-/*@unchecked@*/
-extern unsigned int myinstall_instance;
-/*@=exportheadervar@*/
-
 /*@access FD_t @*/		/* XXX void ptr args */
 /*@access rpmpsm @*/
 
@@ -55,27 +47,30 @@ extern unsigned int myinstall_instance;
 /*@access rpmts @*/	/* XXX ts->notify */
 
 /*@access rpmluav @*/
-/*@access rpmtsScore @*/
-/*@access rpmtsScoreEntry @*/
 
 int rpmVersionCompare(Header first, Header second)
 {
     const char * one, * two;
     int_32 * epochOne, * epochTwo;
-    static int_32 zero = 0;
     int rc;
 
     if (!headerGetEntry(first, RPMTAG_EPOCH, NULL, (void **) &epochOne, NULL))
-	epochOne = &zero;
+	epochOne = NULL;
     if (!headerGetEntry(second, RPMTAG_EPOCH, NULL, (void **) &epochTwo, NULL))
-	epochTwo = &zero;
+	epochTwo = NULL;
 
+    if (epochOne != NULL && epochTwo == NULL)
+	return 1;
+    else if (epochOne == NULL && epochTwo != NULL)
+	return -1;
+    else if (epochOne != NULL && epochTwo != NULL) {
 /*@-boundsread@*/
 	if (*epochOne < *epochTwo)
 	    return -1;
 	else if (*epochOne > *epochTwo)
 	    return 1;
 /*@=boundsread@*/
+    }
 
     rc = headerGetEntry(first, RPMTAG_VERSION, NULL, (void **) &one, NULL);
     rc = headerGetEntry(second, RPMTAG_VERSION, NULL, (void **) &two, NULL);
@@ -270,7 +265,7 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 
     rpmrc = RPMRC_OK;
 
-    isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
+    isSource = (headerIsEntry(h, RPMTAG_SOURCERPM) == 0);
 
     if (!isSource) {
 	rpmError(RPMERR_NOTSRPM, _("source package expected, binary found\n"));
@@ -278,10 +273,7 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	goto exit;
     }
 
-    if (rpmtsAddInstallElement(ts, h, NULL, 0, NULL)) {
-	rpmrc = RPMRC_FAIL;
-	goto exit;
-    }
+    (void) rpmtsAddInstallElement(ts, h, NULL, 0, NULL);
 
     fi = rpmfiNew(ts, h, RPMTAG_BASENAMES, scareMem);
     h = headerFree(h);
@@ -319,9 +311,6 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	    *cookie = xstrdup(*cookie);
     }
 
-    /* XXX FIXME: can't do endian neutral MD5 verification yet. */
-/*@i@*/ fi->fmd5s = hfd(fi->fmd5s, -1);
-
     /* XXX FIXME: don't do per-file mapping, force global flags. */
     fi->fmapflags = _free(fi->fmapflags);
     fi->mapflags = CPIO_MAP_PATH | CPIO_MAP_MODE | CPIO_MAP_UID | CPIO_MAP_GID;
@@ -450,24 +439,49 @@ static char * SCRIPT_PATH = "PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/X11R6/bin";
  * @param tag		scriptlet tag
  * @return		name of scriptlet
  */
-static /*@observer@*/ const char * const tag2sln(int tag)
+static /*@observer@*/ const char * tag2sln(int tag)
 	/*@*/
 {
     switch (tag) {
-    case RPMTAG_PRETRANS:       return "%pretrans";
-    case RPMTAG_PREIN:          return "%pre";
-    case RPMTAG_POSTIN:         return "%post";
-    case RPMTAG_TRIGGERIN:      return "%triggerin";
-    case RPMTAG_TRIGGERUN:      return "%triggerun";
-    case RPMTAG_PREUN:          return "%preun";
-    case RPMTAG_POSTUN:         return "%postun";
-    case RPMTAG_POSTTRANS:      return "%posttrans";
-    case RPMTAG_TRIGGERPOSTUN:  return "%triggerpostun";
-    case RPMTAG_VERIFYSCRIPT:   return "%verify";
+    case RPMTAG_PRETRANS:	return "%pretrans";
+    case RPMTAG_TRIGGERPREIN:	return "%triggerprein";
+    case RPMTAG_PREIN:		return "%pre";
+    case RPMTAG_POSTIN:		return "%post";
+    case RPMTAG_TRIGGERIN:	return "%triggerin";
+    case RPMTAG_TRIGGERUN:	return "%triggerun";
+    case RPMTAG_PREUN:		return "%preun";
+    case RPMTAG_POSTUN:		return "%postun";
+    case RPMTAG_POSTTRANS:	return "%posttrans";
+    case RPMTAG_TRIGGERPOSTUN:	return "%triggerpostun";
+    case RPMTAG_VERIFYSCRIPT:	return "%verify";
     }
     return "%unknownscript";
 }
 
+/**
+ * Return scriptlet id from tag.
+ * @param tag		scriptlet tag
+ * @return		id of scriptlet
+ */
+static rpmScriptID tag2slx(int tag)
+	/*@*/
+{
+    switch (tag) {
+    case RPMTAG_PRETRANS:	return RPMSCRIPT_PRETRANS;
+    case RPMTAG_TRIGGERPREIN:	return RPMSCRIPT_TRIGGERPREIN;
+    case RPMTAG_PREIN:		return RPMSCRIPT_PREIN;
+    case RPMTAG_POSTIN:		return RPMSCRIPT_POSTIN;
+    case RPMTAG_TRIGGERIN:	return RPMSCRIPT_TRIGGERIN;
+    case RPMTAG_TRIGGERUN:	return RPMSCRIPT_TRIGGERUN;
+    case RPMTAG_PREUN:		return RPMSCRIPT_PREUN;
+    case RPMTAG_POSTUN:		return RPMSCRIPT_POSTUN;
+    case RPMTAG_POSTTRANS:	return RPMSCRIPT_POSTTRANS;
+    case RPMTAG_TRIGGERPOSTUN:	return RPMSCRIPT_TRIGGERPOSTUN;
+    case RPMTAG_VERIFYSCRIPT:	return RPMSCRIPT_VERIFY;
+    }
+    return RPMSCRIPT_UNKNOWN;
+}
+
 /**
  * Wait for child process to be reaped.
  * @param psm		package state machine data
@@ -490,6 +504,13 @@ static pid_t psmWait(rpmpsm psm)
 	(unsigned)psm->sq.reaped, psm->sq.status,
 	(unsigned)msecs/1000, (unsigned)msecs%1000);
 
+    if (psm->sstates != NULL)
+    {	int * ssp = psm->sstates + tag2slx(psm->scriptTag);
+	*ssp &= ~0xffff;
+	*ssp |= (psm->sq.status & 0xffff);
+	*ssp |= RPMSCRIPT_STATE_REAPED;
+    }
+
     return psm->sq.reaped;
 }
 
@@ -511,6 +532,12 @@ static rpmRC runLuaScript(rpmpsm psm, Header h, const char *sln,
     int xx;
     rpmlua lua = NULL; /* Global state. */
     rpmluav var;
+    int * ssp = NULL;
+
+    if (psm->sstates != NULL)
+	ssp = psm->sstates + tag2slx(psm->scriptTag);
+    if (ssp != NULL)
+	*ssp |= (RPMSCRIPT_STATE_LUA|RPMSCRIPT_STATE_EXEC);
 
     xx = headerNVR(h, &n, &v, &r);
 
@@ -557,8 +584,14 @@ static rpmRC runLuaScript(rpmpsm psm, Header h, const char *sln,
     {
 	char buf[BUFSIZ];
 	xx = snprintf(buf, BUFSIZ, "%s(%s-%s-%s)", sln, n, v, r);
-	if (rpmluaRunScript(lua, script, buf) == -1)
+	xx = rpmluaRunScript(lua, script, buf);
+	if (xx == -1)
 	    rc = RPMRC_FAIL;
+	if (ssp != NULL) {
+	    *ssp &= ~0xffff;
+	    *ssp |= (xx & 0xffff);
+	    *ssp |= RPMSCRIPT_STATE_REAPED;
+	}
     }
 
     rpmluaDelVar(lua, "arg");
@@ -633,6 +666,12 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     FD_t out;
     rpmRC rc = RPMRC_OK;
     const char *n, *v, *r, *a;
+    int * ssp = NULL;
+
+    if (psm->sstates != NULL)
+	ssp = psm->sstates + tag2slx(psm->scriptTag);
+    if (ssp != NULL)
+	*ssp = RPMSCRIPT_STATE_UNKNOWN;
 
     if (progArgv == NULL && script == NULL)
 	return rc;
@@ -655,6 +694,10 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 
     psm->sq.reaper = 1;
 
+    /* XXX bash must have functional libtermcap.so.2 */
+    if (!strcmp(n, "libtermcap"))
+	ldconfig_done = 0;
+
     /*
      * If a successor node, and ldconfig was just run, don't bother.
      */
@@ -832,8 +875,13 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	    /* make sure we don't close stdin/stderr/stdout by mistake! */
 	    if (ofdno > STDERR_FILENO && ofdno != sfdno)
 		xx = Fclose (out);
-	    if (sfdno > STDERR_FILENO && ofdno != sfdno)
+	    if (sfdno > STDERR_FILENO)
 		xx = Fclose (scriptFd);
+	    else {
+/*@-usereleased@*/
+		xx = Fclose(out);
+/*@=usereleased@*/
+	    }
 	}
 
 	{   const char *ipath = rpmExpand("PATH=%{_install_script_path}", NULL);
@@ -883,8 +931,13 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	    /* XXX Don't mtrace into children. */
 	    unsetenv("MALLOC_CHECK_");
 
+	    if (ssp != NULL)
+		*ssp |= RPMSCRIPT_STATE_EXEC;
+
 	    /* Permit libselinux to do the scriptlet exec. */
 	    if (rpmtsSELinuxEnabled(ts) == 1) {	
+		if (ssp != NULL)
+		    *ssp |= RPMSCRIPT_STATE_SELINUX;
 /*@-moduncon@*/
 		xx = rpm_execcon(0, argv[0], argv, environ);
 /*@=moduncon@*/
@@ -905,17 +958,14 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
 	    break;
 	}
 
+	if (ssp != NULL)
+	    *ssp &= ~RPMSCRIPT_STATE_EXEC;
+
  	_exit(-1);
 	/*@notreached@*/
     }
     /*@=branchstate@*/
 
-    if (psm->sq.child == (pid_t)-1) {
-	rpmError(RPMERR_FORK, _("Couldn't fork %s: %s\n"), sln, strerror(errno));
-	rc = RPMRC_FAIL;
-	goto exit;
-    }
-
     (void) psmWait(psm);
 
   /* XXX filter order dependent multilib "other" arch helper error. */
@@ -940,7 +990,6 @@ static rpmRC runScript(rpmpsm psm, Header h, const char * sln,
     }
   }
 
-exit:
     if (freePrefixes) prefixes = hfd(prefixes, ipt);
 
     xx = Fclose(out);	/* XXX dup'd STDOUT_FILENO */
@@ -1130,7 +1179,7 @@ static rpmRC runTriggers(rpmpsm psm)
 
     if (psm->te) 	/* XXX can't happen */
 	N = rpmteN(psm->te);
-/* XXX: Might need to adjust instance counts four autorollback. */
+/* XXX: Might need to adjust instance counts for autorollback. */
     if (N) 		/* XXX can't happen */
 	numPackage = rpmdbCountPackages(rpmtsGetRdb(ts), N)
 				+ psm->countCorrection;
@@ -1212,7 +1261,7 @@ static rpmRC runImmedTriggers(rpmpsm psm)
     return rc;
 }
 
-/*@observer@*/ static const char *const pkgStageString(pkgStage a)
+/*@observer@*/ static const char * pkgStageString(pkgStage a)
 	/*@*/
 {
     switch(a) {
@@ -1296,6 +1345,8 @@ rpmpsm rpmpsmFree(rpmpsm psm)
     psm->ts = rpmtsFree(psm->ts);
 /*@=internalglobs@*/
 
+    psm->sstates = _free(psm->sstates);
+
     (void) rpmpsmUnlink(psm, msg);
 
     /*@-refcounttrans -usereleased@*/
@@ -1324,6 +1375,8 @@ rpmpsm rpmpsmNew(rpmts ts, rpmte te, rpmfi fi)
 #endif
     if (fi)	psm->fi = rpmfiLink(fi, msg);
 
+    psm->sstates = xcalloc(RPMSCRIPT_MAX, sizeof(*psm->sstates));
+
     return rpmpsmLink(psm, msg);
 }
 
@@ -1358,12 +1411,13 @@ rpmRC rpmpsmStage(rpmpsm psm, pkgStage stage)
     uint_32 tscolor = rpmtsColor(ts);
     rpmfi fi = psm->fi;
     HGE_t hge = fi->hge;
+    HAE_t hae = fi->hae;
     HFD_t hfd = (fi->hfd ? fi->hfd : headerFreeData);
     rpmRC rc = psm->rc;
     int saveerrno;
     int xx;
 
-    /*@-branchstate@*/
+/*@-branchstate@*/
     switch (stage) {
     case PSM_UNKNOWN:
 	break;
@@ -1383,66 +1437,40 @@ rpmRC rpmpsmStage(rpmpsm psm, pkgStage stage)
 	    break;
 	}
 
-	/* If we have a score then autorollback is enabled.  If autorollback is
- 	 * enabled, and this is an autorollback transaction, then we may need to
-	 * adjust the pkgs installed count.
-	 *
-	 * If all this is true, this adjustment should only be made if the PSM goal
-	 * is an install.  No need to make this adjustment on the erase
-	 * component of the upgrade, or even more absurd to do this when doing a
-	 * PKGSAVE.
-	 */
-	if (rpmtsGetScore(ts) != NULL &&
-	    rpmtsGetType(ts) == RPMTRANS_TYPE_AUTOROLLBACK &&
-	    (psm->goal & ~(PSM_PKGSAVE|PSM_PKGERASE))) {
-	    /* Get the score, if its not NULL, get the appropriate
- 	     * score entry.
-	     */
-	    rpmtsScore score = rpmtsGetScore(ts);
-	    if (score != NULL) {
-		/* OK, we got a real score so lets get the appropriate
-		 * score entry.
-		 */
-		rpmtsScoreEntry se;
-		se = rpmtsScoreGetEntry(score, rpmteN(psm->te));
-
-		/* IF the header for the install element has been installed,
-		 * but the header for the erase element has not been erased,
-		 * then decrement the instance count.  This is because in an
-		 * autorollback, if the header was added in the initial transaction
-		 * then in the case of an upgrade the instance count will be
-		 * 2 instead of one when re-installing the old package, and 3 when
-		 * erasing the new package.
-		 *
-		 * Another wrinkle is we only want to make this adjustement
-		 * if the thing we are rollback was an upgrade of package.  A pure
-		 * install or erase does not need the adjustment
-		 */
-		if (se && se->installed &&
-	 	    !se->erased &&
-		    (se->te_types & (TR_ADDED|TR_REMOVED)))
-		    psm->npkgs_installed--;
-	   }
+	/* Adjust package count on rollback downgrade. */
+assert(psm->te != NULL);
+	if (rpmtsType(ts) == RPMTRANS_TYPE_AUTOROLLBACK &&
+	    (psm->goal & ~(PSM_PKGSAVE|PSM_PKGERASE)))
+	{
+	    if (psm->te->downgrade)
+		psm->npkgs_installed--;
 	}
 
 	if (psm->goal == PSM_PKGINSTALL) {
 	    int fc = rpmfiFC(fi);
+	    const char * hdrid;
 
 	    psm->scriptArg = psm->npkgs_installed + 1;
 
 assert(psm->mi == NULL);
-	    psm->mi = rpmtsInitIterator(ts, RPMTAG_NAME, rpmteN(psm->te), 0);
-	    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_EPOCH, RPMMIRE_STRCMP,
+	    hdrid = rpmteHdrid(psm->te);
+	    if (hdrid != NULL) {
+		/* XXX should use RPMTAG_HDRID not RPMTAG_SHA1HEADER */
+		psm->mi = rpmtsInitIterator(ts, RPMTAG_SHA1HEADER, hdrid, 0);
+	    } else {
+		psm->mi = rpmtsInitIterator(ts, RPMTAG_NAME, rpmteN(psm->te),0);
+		xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_EPOCH, RPMMIRE_STRCMP,
 			rpmteE(psm->te));
-	    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_VERSION, RPMMIRE_STRCMP,
+		xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_VERSION, RPMMIRE_STRCMP,
 			rpmteV(psm->te));
-	    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_RELEASE, RPMMIRE_STRCMP,
+		xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_RELEASE, RPMMIRE_STRCMP,
 			rpmteR(psm->te));
-	    if (tscolor) {
-		xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_ARCH, RPMMIRE_STRCMP,
+		if (tscolor) {
+		    xx = rpmdbSetIteratorRE(psm->mi,RPMTAG_ARCH, RPMMIRE_STRCMP,
 			rpmteA(psm->te));
-		xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_OS, RPMMIRE_STRCMP,
+		    xx = rpmdbSetIteratorRE(psm->mi, RPMTAG_OS, RPMMIRE_STRCMP,
 			rpmteO(psm->te));
+		}
 	    }
 
 	    while ((psm->oh = rpmdbNextIterator(psm->mi)) != NULL) {
@@ -1451,6 +1479,7 @@ assert(psm->mi == NULL);
 		/*@loopbreak@*/ break;
 	    }
 	    psm->mi = rpmdbFreeIterator(psm->mi);
+
 	    rc = RPMRC_OK;
 
 	    /* XXX lazy alloc here may need to be done elsewhere. */
@@ -1536,9 +1565,18 @@ psm->te->h = headerLink(fi->h);
 	if (psm->goal == PSM_PKGINSTALL) {
 	    psm->scriptTag = RPMTAG_PREIN;
 	    psm->progTag = RPMTAG_PREINPROG;
+	    psm->sense = RPMSENSE_TRIGGERPREIN;
+	    psm->countCorrection = 0;	/* XXX is this correct?!? */
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOTRIGGERPREIN)) {
-		/* XXX FIXME: implement %triggerprein. */
+
+		/* Run triggers in other package(s) this package sets off. */
+		rc = rpmpsmNext(psm, PSM_TRIGGERS);
+		if (rc) break;
+
+		/* Run triggers in this package other package(s) set off. */
+		rc = rpmpsmNext(psm, PSM_IMMED_TRIGGERS);
+		if (rc) break;
 	    }
 
 	    if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_NOPRE)) {
@@ -1574,11 +1612,18 @@ psm->te->h = headerLink(fi->h);
 	}
 	if (psm->goal == PSM_PKGSAVE) {
 	    int noArchiveSize = 0;
+	    const char * origin;
 
 	    /* Regenerate original header. */
 	    {	void * uh = NULL;
 		int_32 uht, uhc;
 
+		/* Save originnal header's origin (i.e. URL) */
+		origin = NULL;
+		xx = headerGetEntry(fi->h, RPMTAG_PACKAGEORIGIN, NULL,
+			(void **)&origin, NULL);
+
+		/* Retrieve original header blob. */
 		if (headerGetEntry(fi->h, RPMTAG_HEADERIMMUTABLE, &uht, &uh, &uhc)) {
 		    psm->oh = headerCopyLoad(uh);
 		    uh = hfd(uh, uht);
@@ -1603,7 +1648,7 @@ psm->te->h = headerLink(fi->h);
 		    {
 			if (tag == RPMTAG_ARCHIVESIZE)
 			    noArchiveSize = 1;
-		        if (ptr) (void) headerAddEntry(psm->oh, tag, type, ptr, count);
+		        if (ptr) xx = hae(psm->oh, tag, type, ptr, count);
 		    }
 		    hi = headerFreeIterator(hi);
 		    /*@=branchstate@*/
@@ -1666,8 +1711,30 @@ psm->te->h = headerLink(fi->h);
 	    /* Add remove transaction id to header. */
 	    if (psm->oh != NULL)
 	    {	int_32 tid = rpmtsGetTid(ts);
-		xx = headerAddEntry(psm->oh, RPMTAG_REMOVETID,
-			RPM_INT32_TYPE, &tid, 1);
+		int ac;
+
+		xx = hae(psm->oh, RPMTAG_REMOVETID, RPM_INT32_TYPE,
+				&tid, 1);
+
+		/* Add original header's origin (i.e. URL) */
+		if (origin != NULL)
+		    xx = hae(psm->oh, RPMTAG_PACKAGEORIGIN, RPM_STRING_TYPE,
+				origin, 1);
+
+assert(psm->te != NULL);
+		ac = argvCount(psm->te->flink.NEVRA);
+		if (ac > 0)
+		    xx = hae(psm->oh, RPMTAG_FLINKNEVRA, RPM_STRING_ARRAY_TYPE,
+				argvData(psm->te->flink.NEVRA), ac);
+		ac = argvCount(psm->te->flink.Pkgid);
+		if (ac > 0)
+		    xx = hae(psm->oh, RPMTAG_FLINKPKGID, RPM_STRING_ARRAY_TYPE,
+				argvData(psm->te->flink.Pkgid), ac);
+		ac = argvCount(psm->te->flink.Hdrid);
+		if (ac > 0)
+		    xx = hae(psm->oh, RPMTAG_FLINKHDRID, RPM_STRING_ARRAY_TYPE,
+				argvData(psm->te->flink.Hdrid), ac);
+
 	    }
 
 	    /* Write the metadata section into the package. */
@@ -1706,7 +1773,7 @@ psm->te->h = headerLink(fi->h);
 		break;
 	    }
 
-	    rc = fsmSetup(fi->fsm, FSM_PKGINSTALL, ts, fi,
+	    rc = fsmSetup(fi->fsm, FSM_PKGINSTALL, psm->payload_format, ts, fi,
 			psm->cfd, NULL, &psm->failedFile);
 	    (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_UNCOMPRESS),
 			fdstat_op(psm->cfd, FDSTAT_READ));
@@ -1752,21 +1819,14 @@ psm->te->h = headerLink(fi->h);
 
 	    if (rpmtsFlags(ts) & RPMTRANS_FLAG_JUSTDB)	break;
 	    if (rpmtsFlags(ts) & RPMTRANS_FLAG_APPLYONLY)	break;
-
-	    /* XXX Synthesize callbacks for packages with no files. */
-	    if (rpmfiFC(fi) <= 0) {
-		void * ptr;
-		ptr = rpmtsNotify(ts, fi->te, RPMCALLBACK_UNINST_START, 0, 100);
-		ptr = rpmtsNotify(ts, fi->te, RPMCALLBACK_UNINST_STOP, 0, 100);
-		break;
-	    }
+	    if (fc <= 0)				break;
 
 	    psm->what = RPMCALLBACK_UNINST_START;
 	    psm->amount = fc;		/* XXX W2DO? looks wrong. */
 	    psm->total = fc;
 	    xx = rpmpsmNext(psm, PSM_NOTIFY);
 
-	    rc = fsmSetup(fi->fsm, FSM_PKGERASE, ts, fi,
+	    rc = fsmSetup(fi->fsm, FSM_PKGERASE, psm->payload_format, ts, fi,
 			NULL, NULL, &psm->failedFile);
 	    xx = fsmTeardown(fi->fsm);
 
@@ -1796,8 +1856,8 @@ psm->te->h = headerLink(fi->h);
 		break;
 	    }
 
-	    rc = fsmSetup(fi->fsm, FSM_PKGBUILD, ts, fi, psm->cfd,
-			NULL, &psm->failedFile);
+	    rc = fsmSetup(fi->fsm, FSM_PKGBUILD, psm->payload_format, ts, fi,
+			psm->cfd, NULL, &psm->failedFile);
 	    (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_COMPRESS),
 			fdstat_op(psm->cfd, FDSTAT_WRITE));
 	    (void) rpmswAdd(rpmtsOp(ts, RPMTS_OP_DIGEST),
@@ -1827,18 +1887,40 @@ psm->te->h = headerLink(fi->h);
 	if (psm->goal == PSM_PKGINSTALL) {
 	    int_32 installTime = (int_32) time(NULL);
 	    int fc = rpmfiFC(fi);
+	    const char * origin;
+	    int ac;
 
 	    if (fi->h == NULL) break;	/* XXX can't happen */
 	    if (fi->fstates != NULL && fc > 0)
-		xx = headerAddEntry(fi->h, RPMTAG_FILESTATES, RPM_CHAR_TYPE,
+		xx = hae(fi->h, RPMTAG_FILESTATES, RPM_CHAR_TYPE,
 				fi->fstates, fc);
 
-	    xx = headerAddEntry(fi->h, RPMTAG_INSTALLTIME, RPM_INT32_TYPE,
+	    xx = hae(fi->h, RPMTAG_INSTALLTIME, RPM_INT32_TYPE,
 				&installTime, 1);
 
-	    xx = headerAddEntry(fi->h, RPMTAG_INSTALLCOLOR, RPM_INT32_TYPE,
+	    xx = hae(fi->h, RPMTAG_INSTALLCOLOR, RPM_INT32_TYPE,
 				&tscolor, 1);
 
+	    /* Add the header's origin (i.e. URL) */
+	    origin = headerGetOrigin(fi->h);
+	    if (origin != NULL)
+		xx = hae(fi->h, RPMTAG_PACKAGEORIGIN, RPM_STRING_TYPE,
+				origin, 1);
+	
+assert(psm->te != NULL);
+	    ac = argvCount(psm->te->blink.NEVRA);
+	    if (ac > 0)
+		xx = hae(fi->h, RPMTAG_BLINKNEVRA, RPM_STRING_ARRAY_TYPE,
+				argvData(psm->te->blink.NEVRA), ac);
+	    ac = argvCount(psm->te->blink.Pkgid);
+	    if (ac > 0)
+		xx = hae(fi->h, RPMTAG_BLINKPKGID, RPM_STRING_ARRAY_TYPE,
+				argvData(psm->te->blink.Pkgid), ac);
+	    ac = argvCount(psm->te->blink.Hdrid);
+	    if (ac > 0)
+		xx = hae(fi->h, RPMTAG_BLINKHDRID, RPM_STRING_ARRAY_TYPE,
+				argvData(psm->te->blink.Hdrid), ac);
+
 	    /*
 	     * If this package has already been installed, remove it from
 	     * the database before adding the new one.
@@ -1953,6 +2035,7 @@ psm->te->h = headerFree(psm->te->h);
 	psm->oh = headerFree(psm->oh);
 	psm->pkgURL = _free(psm->pkgURL);
 	psm->rpmio_flags = _free(psm->rpmio_flags);
+	psm->payload_format = _free(psm->payload_format);
 	psm->failedFile = _free(psm->failedFile);
 
 	fi->fgroup = hfd(fi->fgroup, -1);
@@ -1991,7 +2074,7 @@ psm->te->h = headerFree(psm->te->h);
 	if (!(rpmtsFlags(ts) & RPMTRANS_FLAG_PKGCOMMIT)) break;
 	if (rpmtsFlags(ts) & RPMTRANS_FLAG_APPLYONLY) break;
 
-	rc = fsmSetup(fi->fsm, FSM_PKGCOMMIT, ts, fi,
+	rc = fsmSetup(fi->fsm, FSM_PKGCOMMIT, psm->payload_format, ts, fi,
 			NULL, NULL, &psm->failedFile);
 	xx = fsmTeardown(fi->fsm);
 	break;
@@ -2045,15 +2128,18 @@ psm->te->h = headerFree(psm->te->h);
 	break;
     case PSM_TRIGGERS:
 	/* Run triggers in other package(s) this package sets off. */
+	if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)	break;
 	rc = runTriggers(psm);
 	break;
     case PSM_IMMED_TRIGGERS:
 	/* Run triggers in this package other package(s) set off. */
+	if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)	break;
 	rc = runImmedTriggers(psm);
 	break;
 
     case PSM_RPMIO_FLAGS:
     {	const char * payload_compressor = NULL;
+	const char * payload_format = NULL;
 	char * t;
 
 	/*@-branchstate@*/
@@ -2068,6 +2154,14 @@ psm->te->h = headerFree(psm->te->h);
 	    t = stpcpy(t, ".gzdio");
 	if (!strcmp(payload_compressor, "bzip2"))
 	    t = stpcpy(t, ".bzdio");
+
+	/*@-branchstate@*/
+	if (!hge(fi->h, RPMTAG_PAYLOADFORMAT, NULL,
+			    (void **) &payload_format, NULL)
+	 || !(!strcmp(payload_format, "tar") || !strcmp(payload_format, "ustar")))
+	    payload_format = "cpio";
+	/*@=branchstate@*/
+	psm->payload_format = xstrdup(payload_format);
 	rc = RPMRC_OK;
     }	break;
 
@@ -2075,17 +2169,22 @@ psm->te->h = headerFree(psm->te->h);
 assert(psm->mi == NULL);
 	psm->mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES,
 				&fi->record, sizeof(fi->record));
-
 	fi->h = rpmdbNextIterator(psm->mi);
 	if (fi->h != NULL)
 	    fi->h = headerLink(fi->h);
-
 	psm->mi = rpmdbFreeIterator(psm->mi);
-	rc = (fi->h != NULL ? RPMRC_OK : RPMRC_FAIL);
+
+	if (fi->h != NULL) {
+	    (void) headerSetInstance(fi->h, fi->record);
+	    rc = RPMRC_OK;
+	} else
+	    rc = RPMRC_FAIL;
 	break;
     case PSM_RPMDB_ADD:
 	if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)	break;
 	if (fi->h == NULL)	break;	/* XXX can't happen */
+
+	/* Add header to db, doing header check if requested */
 	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DBADD), 0);
 	if (!(rpmtsVSFlags(ts) & RPMVSF_NOHDRCHK))
 	    rc = rpmdbAdd(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->h,
@@ -2093,80 +2192,44 @@ assert(psm->mi == NULL);
 	else
 	    rc = rpmdbAdd(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->h,
 				NULL, NULL);
-
-	/* Set the database instance so consumers (i.e. rpmtsRun())
-	 * can add this to a rollback transaction.
-	 */
-	rpmteSetDBInstance(psm->te, myinstall_instance);
-
-	/*
-	 * If the score exists and this is not a rollback or autorollback
-	 * then lets check off installed for this package.
-	 */
-	if (rpmtsGetScore(ts) != NULL &&
-	    rpmtsGetType(ts) != RPMTRANS_TYPE_ROLLBACK &&
-	    rpmtsGetType(ts) != RPMTRANS_TYPE_AUTOROLLBACK)
-	{
-	    /* Get the score, if its not NULL, get the appropriate
- 	     * score entry.
-	     */
-	    rpmtsScore score = rpmtsGetScore(ts);
-	    if (score != NULL) {
-		rpmtsScoreEntry se;
-		/* OK, we got a real score so lets get the appropriate
-		 * score entry.
-		 */
-		rpmMessage(RPMMESS_DEBUG,
-		    _("Attempting to mark %s as installed in score board(%u).\n"),
-		    rpmteN(psm->te), (unsigned) score);
-		se = rpmtsScoreGetEntry(score, rpmteN(psm->te));
-		if (se != NULL) se->installed = 1;
-	    }
-	}
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DBADD), 0);
+
+	if (rc != RPMRC_OK) break;
+
+assert(psm->te != NULL);
+	/* Mark non-rollback elements as installed. */
+	if (
+	    rpmtsType(ts) != RPMTRANS_TYPE_ROLLBACK &&
+	    rpmtsType(ts) != RPMTRANS_TYPE_AUTOROLLBACK)
+	{
+	    psm->te->installed = 1;
+	}
+
+	/* Set the database instance for (possible) rollbacks. */
+	rpmteSetDBInstance(psm->te, headerGetInstance(fi->h));
+
 	break;
     case PSM_RPMDB_REMOVE:
+    {	
+
 	if (rpmtsFlags(ts) & RPMTRANS_FLAG_TEST)	break;
+
 	(void) rpmswEnter(rpmtsOp(ts, RPMTS_OP_DBREMOVE), 0);
 	rc = rpmdbRemove(rpmtsGetRdb(ts), rpmtsGetTid(ts), fi->record,
 				NULL, NULL);
-
-	/*
-	 * If the score exists and this is not a rollback or autorollback
-	 * then lets check off erased for this package.
-	 */
-	if (rpmtsGetScore(ts) != NULL &&
-	   rpmtsGetType(ts) != RPMTRANS_TYPE_ROLLBACK &&
-	   rpmtsGetType(ts) != RPMTRANS_TYPE_AUTOROLLBACK)
-	{
-	    /* Get the score, if its not NULL, get the appropriate
-	     * score entry.
-	     */
-	    rpmtsScore score = rpmtsGetScore(ts);
-
-	    if (score != NULL) { /* XXX: Can't happen */
-		rpmtsScoreEntry se;
-		/* OK, we got a real score so lets get the appropriate
-		 * score entry.
-		 */
-		rpmMessage(RPMMESS_DEBUG,
-		    _("Attempting to mark %s as erased in score board(0x%x).\n"),
-		    rpmteN(psm->te), (unsigned) score);
-		se = rpmtsScoreGetEntry(score, rpmteN(psm->te));
-		if (se != NULL) se->erased = 1;
-	    }
-	}
-
 	(void) rpmswExit(rpmtsOp(ts, RPMTS_OP_DBREMOVE), 0);
-	break;
+
+	if (rc != RPMRC_OK) break;
+
+    }	break;
 
     default:
 	break;
 /*@i@*/    }
-    /*@=branchstate@*/
+/*@=branchstate@*/
 
-    /*@-nullstate@*/	/* FIX: psm->oh and psm->fi->h may be NULL. */
+/*@-nullstate@*/	/* FIX: psm->oh and psm->fi->h may be NULL. */
     return rc;
-    /*@=nullstate@*/
+/*@=nullstate@*/
 }
 /*@=bounds =nullpass@*/
