@@ -76,17 +76,21 @@ struct rpmpsm_s {
     FD_t fd;			/*!< Repackage file handle. */
     Header oh;			/*!< Repackage header. */
 /*@null@*/
-    rpmdbMatchIterator mi;
+    rpmdbMatchIterator mi;	/*!< An rpmdb iterator for this psm's use. */
 /*@observer@*/
-    const char * stepName;
+    const char * stepName;	/*!< The current PSM step (for display). */
 /*@only@*/ /*@null@*/
-    const char * rpmio_flags;
+    const char * rpmio_flags;	/*!< Payload compression type/flags. */
+/*@only@*/ /*@null@*/
+    const char * payload_format;/*!< Payload archive format. */
 /*@only@*/ /*@null@*/
     const char * failedFile;
 /*@only@*/ /*@null@*/
     const char * pkgURL;	/*!< Repackage URL. */
 /*@dependent@*/
     const char * pkgfn;		/*!< Repackage file name. */
+/*@only@*/ /*@null@*/
+    int *sstates;		/*!< Scriptlet states and return codes. */
     int scriptTag;		/*!< Scriptlet data tag. */
     int progTag;		/*!< Scriptlet interpreter tag. */
     int npkgs_installed;	/*!< No. of installed instances. */
@@ -96,8 +100,8 @@ struct rpmpsm_s {
     int chrootDone;		/*!< Was chroot(2) done by pkgStage? */
     int unorderedSuccessor;	/*!< Can the PSM be run asynchronously? */
     rpmCallbackType what;	/*!< Callback type. */
-    unsigned long amount;	/*!< Callback amount. */
-    unsigned long total;	/*!< Callback total. */
+    unsigned long long amount;	/*!< Callback amount. */
+    unsigned long long total;	/*!< Callback total. */
     rpmRC rc;
     pkgStage goal;
 /*@unused@*/
