@@ -103,7 +103,7 @@ static void printFileInfo(char * te, const char * name,
 	(void)strftime(timefield, sizeof(timefield) - 1, fmt, tm);
     }
 
-    sprintf(te, "%s %4d %-8s%-8s %10s %s %s", perms,
+    sprintf(te, "%s %4d %-7s %-8s %10s %s %s", perms,
 	(int)nlink, ownerfield, groupfield, sizefield, timefield, namefield);
     perms = _free(perms);
 }
@@ -124,55 +124,32 @@ static inline /*@null@*/ const char * queryHeader(Header h, const char * qfmt)
     return str;
 }
 
-/**
- */
-static void flushBuffer(char ** tp, char ** tep, int nonewline)
-	/*@ modifies *tp, *tep @*/
-{
-    char *t, *te;
-
-    t = *tp;
-    te = *tep;
-    if (te > t) {
-	if (!nonewline) {
-	    *te++ = '\n';
-	    *te = '\0';
-	}
-	rpmMessage(RPMMESS_NORMAL, "%s", t);
-	te = t;
-	*t = '\0';
-    }
-    *tp = t;
-    *tep = te;
-}
-
 int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 {
     int scareMem = 0;
     rpmfi fi = NULL;
-    size_t tb = 2 * BUFSIZ;
-    size_t sb;
     char * t, * te;
     char * prefix = NULL;
     int rc = 0;		/* XXX FIXME: need real return code */
+    int nonewline = 0;
     int i;
 
-    te = t = xmalloc(tb);
+    te = t = xmalloc(BUFSIZ);
 /*@-boundswrite@*/
     *te = '\0';
 /*@=boundswrite@*/
 
     if (qva->qva_queryFormat != NULL) {
 	const char * str = queryHeader(h, qva->qva_queryFormat);
+	nonewline = 1;
 	/*@-branchstate@*/
 	if (str) {
-	    size_t tx = (te - t);
+	    size_t tb = (te - t);
+	    size_t sb = strlen(str);
 
-	    sb = strlen(str);
-	    if (sb) {
-		tb += sb;
-		t = xrealloc(t, tb);
-		te = t + tx;
+	    if (sb >= (BUFSIZ - tb)) {
+		t = xrealloc(t, BUFSIZ+sb);
+		te = t + tb;
 	    }
 /*@-boundswrite@*/
 	    /*@-usereleased@*/
@@ -180,7 +157,6 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	    /*@=usereleased@*/
 /*@=boundswrite@*/
 	    str = _free(str);
-	    flushBuffer(&t, &te, 1);
 	}
 	/*@=branchstate@*/
     }
@@ -245,19 +221,18 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 	if ((qva->qva_flags & QUERY_FOR_CONFIG) && !(fflags & RPMFILE_CONFIG))
 	    continue;
 
+	/* If not querying %config, skip config files. */
+	if ((qva->qva_fflags & RPMFILE_CONFIG) && (fflags & RPMFILE_CONFIG))
+	    continue;
+
+	/* If not querying %doc, skip doc files. */
+	if ((qva->qva_fflags & RPMFILE_DOC) && (fflags & RPMFILE_DOC))
+	    continue;
+
 	/* If not querying %ghost, skip ghost files. */
 	if ((qva->qva_fflags & RPMFILE_GHOST) && (fflags & RPMFILE_GHOST))
 	    continue;
 
-	/* Insure space for header derived data */
-	sb = strlen(fn) + strlen(fmd5) + strlen(fuser) + strlen(fgroup) + strlen(flink);
-	if ((sb + BUFSIZ) > tb) {
-	    size_t tx = (te - t);
-	    tb += sb + BUFSIZ;
-	    t = xrealloc(t, tb);
-	    te = t + tx;
-	}
-
 /*@-boundswrite@*/
 	if (!rpmIsVerbose() && prefix)
 	    te = stpcpy(te, prefix);
@@ -337,13 +312,31 @@ int showQueryPackage(QVA_t qva, rpmts ts, Header h)
 			_("package has neither file owner or id lists\n"));
 	    }
 	}
-	flushBuffer(&t, &te, 0);
+/*@-branchstate@*/
+	if (te > t) {
+/*@-boundswrite@*/
+	    *te++ = '\n';
+	    *te = '\0';
+	    rpmMessage(RPMMESS_NORMAL, "%s", t);
+	    te = t;
+	    *t = '\0';
+/*@=boundswrite@*/
+	}
+/*@=branchstate@*/
     }
 	    
     rc = 0;
 
 exit:
-    flushBuffer(&t, &te, 0);
+    if (te > t) {
+	if (!nonewline) {
+/*@-boundswrite@*/
+	    *te++ = '\n';
+	    *te = '\0';
+/*@=boundswrite@*/
+	}
+	rpmMessage(RPMMESS_NORMAL, "%s", t);
+    }
     t = _free(t);
 
     fi = rpmfiFree(fi);
@@ -356,8 +349,22 @@ void rpmDisplayQueryTags(FILE * fp)
     int i;
     const struct headerSprintfExtension_s * ext = rpmHeaderFormats;
 
-    for (i = 0, t = rpmTagTable; i < rpmTagTableSize; i++, t++)
-	if (t->name) fprintf(fp, "%s\n", t->name + 7);
+    for (i = 0, t = rpmTagTable; i < rpmTagTableSize; i++, t++) {
+	if (t->name == NULL)
+	    continue;
+	fprintf(fp, "%-20s", t->name + 7);
+	if (rpmIsVerbose()) {
+	    /*@observer@*/
+	    static const char * tagtypes[] = {
+		"", "char", "int8", "int16", "int32", "int64",
+		"string", "blob", "argv", "i18nstring"
+	    };
+	    fprintf(fp, " %6d", t->val);
+	    if (t->type > RPM_NULL_TYPE && t->type <= RPM_MAX_TYPE)
+		fprintf(fp, " %s", tagtypes[t->type]);
+	}
+	fprintf(fp, "\n");
+    }
 
     while (ext->name != NULL) {
 	if (ext->type == HEADER_EXT_MORE) {
@@ -432,7 +439,6 @@ static inline unsigned char nibble(char c)
     return 0;
 }
 
-/*@-bounds@*/ /* LCL: segfault (realpath annotation?) */
 int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 {
     int res = 0;
@@ -543,7 +549,7 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
         for (i = 0, t = MD5, s = arg; i < 16; i++, t++, s += 2)
             *t = (nibble(s[0]) << 4) | nibble(s[1]);
 
-	qva->qva_mi = rpmtsInitIterator(ts, RPMTAG_FILEMD5S, MD5, sizeof(MD5));
+	qva->qva_mi = rpmtsInitIterator(ts, RPMTAG_FILEDIGESTS, MD5, sizeof(MD5));
 	if (qva->qva_mi == NULL) {
 	    rpmError(RPMERR_QUERYINFO, _("no package matches %s: %s\n"),
 			"fileid", arg);
@@ -581,6 +587,7 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
 	    res = rpmcliShowMatches(qva, ts);
     }	break;
 
+    case RPMQV_WHATNEEDS:
     case RPMQV_WHATREQUIRES:
 	qva->qva_mi = rpmtsInitIterator(ts, RPMTAG_REQUIRENAME, arg, 0);
 	if (qva->qva_mi == NULL) {
@@ -678,30 +685,19 @@ int rpmQueryVerify(QVA_t qva, rpmts ts, const char * arg)
     }	break;
 
     case RPMQV_PACKAGE:
-    {
-	int matches = 0;
-	rpmdbMatchIterator mi;
-	mi = rpmtsInitIterator(ts, RPMDBI_LABEL, arg, 0);
-	while (rpmdbNextIterator(mi) != NULL) {
-	    matches++;
-	}
-	rpmdbFreeIterator(mi);
-	if (! matches) {
+	/* XXX HACK to get rpmdbFindByLabel out of the API */
+	qva->qva_mi = rpmtsInitIterator(ts, RPMDBI_LABEL, arg, 0);
+	if (qva->qva_mi == NULL) {
 	    rpmError(RPMERR_QUERYINFO, _("package %s is not installed\n"), arg);
 	    res = 1;
-	} else {
-	    qva->qva_mi = rpmtsInitIterator(ts, RPMDBI_LABEL, arg, 0);
+	} else
 	    res = rpmcliShowMatches(qva, ts);
-	}
 	break;
     }
-    
-    }
     /*@=branchstate@*/
    
     return res;
 }
-/*@=bounds@*/
 
 int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
 {
@@ -776,7 +772,10 @@ int rpmcliArgIter(rpmts ts, QVA_t qva, ARGV_t argv)
 	qva->qva_rc = rpmgiSetArgs(qva->qva_gi, argv, ftsOpts,
 		(giFlags | (RPMGI_NOGLOB|RPMGI_NOHEADER)));
 	while (rpmgiNext(qva->qva_gi) == RPMRC_OK) {
-	    ec += rpmQueryVerify(qva, ts, rpmgiHdrPath(qva->qva_gi));
+	    const char * path;
+	    path = rpmgiHdrPath(qva->qva_gi);
+assert(path != NULL);
+	    ec += rpmQueryVerify(qva, ts, path);
 	    rpmtsEmpty(ts);
 	}
 	break;
@@ -812,6 +811,22 @@ int rpmcliQuery(rpmts ts, QVA_t qva, const char ** argv)
     if (qva->qva_flags & VERIFY_HDRCHK)
 	vsflags |= RPMVSF_NOHDRCHK;
 
+#ifdef	NOTYET
+    /* Initialize security context patterns (if not already done). */
+    if (!(qva->qva_flags & VERIFY_CONTEXTS)) {
+	rpmsx sx = rpmtsREContext(ts);
+	if (sx == NULL) {
+	    arg = rpmGetPath("%{?_verify_file_context_path}", NULL);
+	    if (arg != NULL && *arg != '\0') {
+		sx = rpmsxNew(arg);
+		(void) rpmtsSetREContext(ts, sx);
+	    }
+	    arg = _free(arg);
+	}
+	sx = rpmsxFree(sx);
+    }
+#endif
+
     ovsflags = rpmtsSetVSFlags(ts, vsflags);
     ec = rpmcliArgIter(ts, qva, argv);
     vsflags = rpmtsSetVSFlags(ts, ovsflags);
