@@ -116,10 +116,6 @@ static int copyFile(FD_t *sfdp, const char **sfnp,
 	rpmError(RPMERR_FREAD, _("%s: Fread failed: %s\n"), *sfnp, Fstrerror(*sfdp));
 	goto exit;
     }
-    if (Fflush(*tfdp) != 0) {
-	rpmError(RPMERR_FWRITE, _("%s: Fflush failed: %s\n"), *tfnp,
-	    Fstrerror(*tfdp));
-    }
 
     rc = 0;
 
@@ -698,6 +694,7 @@ assert(dig->md5ctx == NULL);
 	    fddig->hashctx = NULL;
 	    /*@switchbreak@*/ break;
 	case PGPHASHALGO_SHA1:
+	case PGPHASHALGO_RIPEMD160:
 #if HAVE_BEECRYPT_API_H
 	case PGPHASHALGO_SHA256:
 	case PGPHASHALGO_SHA384:
@@ -802,11 +799,12 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
 	}
 
 	dig = rpmtsDig(ts);
+assert(dig != NULL);
 	sigp = rpmtsSignature(ts);
 
 	/* XXX RSA needs the hash_algo, so decode early. */
 	if (sigtag == RPMSIGTAG_RSA || sigtag == RPMSIGTAG_PGP) {
-	    xx = headerGetEntry(sigh, sigtag, &sigtype, &sig, &siglen);
+	    xx = headerGetEntry(sigh, sigtag, &sigtype, (void **)&sig, &siglen);
 	    xx = pgpPrtPkts(sig, siglen, dig, 0);
 	    sig = headerFreeData(sig, sigtype);
 	    /* XXX assume same hash_algo in header-only and header+payload */
@@ -862,10 +860,9 @@ int rpmVerifySignatures(QVA_t qva, rpmts ts, FD_t fd,
 
 		if (sigp->version != 3 && sigp->version != 4) {
 		    rpmError(RPMERR_SIGVFY,
-		_("skipping package %s with unverifiable V%u signature\n"),
-			fn, sigp->version);
-		    res++;
-		    goto exit;
+		_("only V3 or V4 signatures can be verified, skipping V%u signature\n"),
+			sigp->version);
+		    continue;
 		}
 		/*@switchbreak@*/ break;
 	    case RPMSIGTAG_SHA1:
@@ -1089,14 +1086,10 @@ int rpmcliSign(rpmts ts, QVA_t qva, const char ** argv)
     while ((arg = *argv++) != NULL) {
 	FD_t fd;
 
-	fd = Fopen(arg, "r.ufdio");
-	if (fd == NULL || Ferror(fd)) {
-	    rpmError(RPMERR_OPEN, _("%s: open failed: %s\n"), 
-		     arg, Fstrerror(fd));
+	if ((fd = Fopen(arg, "r.ufdio")) == NULL
+	 || Ferror(fd)
+	 || rpmVerifySignatures(qva, ts, fd, arg))
 	    res++;
-	} else if (rpmVerifySignatures(qva, ts, fd, arg)) {
-	    res++;
-	}
 
 	if (fd != NULL) xx = Fclose(fd);
     }
