@@ -1,4 +1,4 @@
-/** \ingroup rpmdep
+/** \ingroup rpmfi
  * \file lib/rpmfi.c
  * Routines to handle file info tag sets.
  */
@@ -22,13 +22,13 @@
 #include "rpmte.h"
 #include "rpmts.h"
 
-#include "legacy.h"     /* XXX domd5 */
 #include "misc.h"	/* XXX stripTrailingChar */
 #include "rpmmacro.h"	/* XXX rpmCleanPath */
 
 #include "debug.h"
 
 /*@access rpmte @*/
+/*@access FSM_t @*/	/* XXX fsm->repackaged */
 
 /*@unchecked@*/
 int _rpmfi_debug = 0;
@@ -204,17 +204,36 @@ rpmfileState rpmfiFState(rpmfi fi)
     return fstate;
 }
 
-const unsigned char * rpmfiMD5(rpmfi fi)
+const unsigned char * rpmfiDigest(rpmfi fi, int * algop, size_t * lenp)
 {
-    unsigned char * MD5 = NULL;
+    unsigned char * digest = NULL;
 
     if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
 /*@-boundsread@*/
-	if (fi->md5s != NULL)
-	    MD5 = fi->md5s + (16 * fi->i);
+	if (fi->digests != NULL) {
+	    digest = fi->digests + (fi->digestlen * fi->i);
+	    if (algop != NULL)
+		*algop = (fi->fdigestalgos
+			? fi->fdigestalgos[fi->i] : fi->digestalgo);
+	    if (lenp != NULL)
+		*lenp = fi->digestlen;
+	}
 /*@=boundsread@*/
     }
-    return MD5;
+    return digest;
+}
+
+const unsigned char * rpmfiMD5(rpmfi fi)
+{
+    unsigned char * digest = NULL;
+
+    if (fi != NULL && fi->i >= 0 && fi->i < fi->fc) {
+/*@-boundsread@*/
+	if (fi->digests != NULL)
+	    digest = fi->digests + (fi->digestlen * fi->i);
+/*@=boundsread@*/
+    }
+    return digest;
 }
 
 const char * rpmfiFLink(rpmfi fi)
@@ -485,7 +504,7 @@ rpmfi rpmfiInitD(rpmfi fi, int dx)
  * @return		string to identify a file type
  */
 static /*@observer@*/
-const char *const ftstring (fileTypes ft)
+const char * ftstring (fileTypes ft)
 	/*@*/
 {
     switch (ft) {
@@ -531,12 +550,19 @@ int rpmfiCompare(const rpmfi afi, const rpmfi bfi)
 	if (blink == NULL) return -1;
 	return strcmp(alink, blink);
     } else if (awhat == REG) {
-	const unsigned char * amd5 = rpmfiMD5(afi);
-	const unsigned char * bmd5 = rpmfiMD5(bfi);
-	if (amd5 == bmd5) return 0;
-	if (amd5 == NULL) return 1;
-	if (bmd5 == NULL) return -1;
-	return memcmp(amd5, bmd5, 16);
+	int aalgo = 0;
+	size_t alen = 0;
+	const unsigned char * adigest = rpmfiDigest(afi, &aalgo, &alen);
+	int balgo = 0;
+	size_t blen = 0;
+	const unsigned char * bdigest = rpmfiDigest(bfi, &balgo, &blen);
+	/* XXX W2DO? changing file digest algo may break rpmfiCompare. */
+	if (!(aalgo == balgo && alen == blen))
+	    return -1;
+	if (adigest == bdigest) return 0;
+	if (adigest == NULL) return 1;
+	if (bdigest == NULL) return -1;
+	return memcmp(adigest, bdigest, alen);
     }
 
     return 0;
@@ -578,7 +604,7 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
     if (newWhat == XDIR)
 	return FA_CREATE;
 
-    if (diskWhat != newWhat && dbWhat != REG && dbWhat != LINK)
+    if (diskWhat != newWhat)
 	return save;
     else if (newWhat != dbWhat && diskWhat != dbWhat)
 	return save;
@@ -593,34 +619,32 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
      */
     memset(buffer, 0, sizeof(buffer));
     if (dbWhat == REG) {
-	const unsigned char * omd5, * nmd5;
-	omd5 = rpmfiMD5(ofi);
-	if (diskWhat == REG) {
-	    if (domd5(fn, (unsigned char *)buffer, 0, NULL))
-		return FA_CREATE;       /* assume file has been removed */
-	    if (omd5 && !memcmp(omd5, buffer, 16))
-		return FA_CREATE;       /* unmodified config file, replace. */
-	}
-
-	if (domd5(fn, buffer, 0, NULL))
+	int oalgo = 0;
+	size_t olen = 0;
+	const unsigned char * odigest;
+	int nalgo = 0;
+	size_t nlen = 0;
+	const unsigned char * ndigest;
+	odigest = rpmfiDigest(ofi, &oalgo, &olen);
+	/* XXX avoid digest on sparse /var/log/lastlog file. */
+	if (strcmp(fn, "/var/log/lastlog"))
+	if (dodigest(oalgo, fn, buffer, 0, NULL))
 	    return FA_CREATE;	/* assume file has been removed */
-	omd5 = rpmfiMD5(ofi);
-	if (omd5 && !memcmp(omd5, buffer, 16))
+	if (odigest && !memcmp(odigest, buffer, olen))
 	    return FA_CREATE;	/* unmodified config file, replace. */
-	nmd5 = rpmfiMD5(nfi);
+	ndigest = rpmfiDigest(nfi, &nalgo, &nlen);
 /*@-nullpass@*/
-	if (omd5 && nmd5 && !memcmp(omd5, nmd5, 16))
+	if (odigest && ndigest && oalgo == nalgo && olen == nlen
+	 && !memcmp(odigest, ndigest, nlen))
 	    return FA_SKIP;	/* identical file, don't bother. */
 /*@=nullpass@*/
     } else /* dbWhat == LINK */ {
 	const char * oFLink, * nFLink;
-	oFLink = rpmfiFLink(ofi);
-	if (diskWhat == LINK) {
 	if (readlink(fn, buffer, sizeof(buffer) - 1) == -1)
 	    return FA_CREATE;	/* assume file has been removed */
+	oFLink = rpmfiFLink(ofi);
 	if (oFLink && !strcmp(oFLink, buffer))
 	    return FA_CREATE;	/* unmodified config file, replace. */
-	}
 	nFLink = rpmfiFLink(nfi);
 /*@-nullpass@*/
 	if (oFLink && nFLink && !strcmp(oFLink, nFLink))
@@ -638,51 +662,8 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
 }
 /*@=boundsread@*/
 
-/*@-boundsread@*/
-int rpmfiConfigConflict(const rpmfi fi)
-{
-    const char * fn = rpmfiFN(fi);
-    int flags = rpmfiFFlags(fi);
-    char buffer[1024];
-    fileTypes newWhat, diskWhat;
-    struct stat sb;
-
-    if (!(flags & RPMFILE_CONFIG) || lstat(fn, &sb)) {
-	return 0;
-    }
-
-    diskWhat = whatis((int_16)sb.st_mode);
-    newWhat = whatis(rpmfiFMode(fi));
-
-    if (newWhat != LINK && newWhat != REG)
-	return 1;
-
-    if (diskWhat != newWhat)
-	return 1;
-    
-    memset(buffer, 0, sizeof(buffer));
-    if (newWhat == REG) {
-	const unsigned char * nmd5;
-	if (domd5(fn, (unsigned char *)buffer, 0, NULL))
-	    return 0;	/* assume file has been removed */
-	nmd5 = rpmfiMD5(fi);
-	if (nmd5 && !memcmp(nmd5, buffer, 16))
-	    return 0;	/* unmodified config file */
-    } else /* newWhat == LINK */ {
-	const char * nFLink;
-	if (readlink(fn, buffer, sizeof(buffer) - 1) == -1)
-	    return 0;	/* assume file has been removed */
-	nFLink = rpmfiFLink(fi);
-	if (nFLink && !strcmp(nFLink, buffer))
-	    return 0;	/* unmodified config file */
-    }
-
-    return 1;
-}
-/*@=boundsread@*/
-
 /*@observer@*/
-const char *const rpmfiTypeString(rpmfi fi)
+const char * rpmfiTypeString(rpmfi fi)
 {
     switch(rpmteType(fi->te)) {
     case TR_ADDED:	return " install";
@@ -717,7 +698,7 @@ Header relocateFileList(const rpmts ts, rpmfi fi,
     HFD_t hfd = (fi->hfd ? fi->hfd : headerFreeData);
     static int _printed = 0;
     int allowBadRelocate = (rpmtsFilterFlags(ts) & RPMPROB_FILTER_FORCERELOCATE);
-    rpmRelocation * relocations = NULL;
+    rpmRelocation relocations = NULL;
     int numRelocations;
     const char ** validRelocations;
     rpmTagType validType;
@@ -835,7 +816,7 @@ assert(p != NULL);
 	int madeSwap;
 	madeSwap = 0;
 	for (j = 1; j < numRelocations; j++) {
-	    rpmRelocation tmpReloc;
+	    struct rpmRelocation_s tmpReloc;
 	    if (relocations[j - 1].oldPath == NULL || /* XXX can't happen */
 		relocations[j    ].oldPath == NULL || /* XXX can't happen */
 	strcmp(relocations[j - 1].oldPath, relocations[j].oldPath) <= 0)
@@ -1175,8 +1156,8 @@ fprintf(stderr, "*** fi %p\t%s[%d]\n", fi, fi->Type, fi->fc);
 
 	fi->flinks = hfd(fi->flinks, -1);
 	fi->flangs = hfd(fi->flangs, -1);
-	fi->fmd5s = hfd(fi->fmd5s, -1);
-	fi->md5s = _free(fi->md5s);
+	fi->fdigests = hfd(fi->fdigests, -1);
+	fi->digests = _free(fi->digests);
 
 	fi->cdict = hfd(fi->cdict, -1);
 
@@ -1297,11 +1278,18 @@ rpmfi rpmfiNew(const rpmts ts, Header h, rpmTag tagN, int scareMem)
     fi->hre = (HRE_t) headerRemoveEntry;
     fi->hfd = headerFreeData;
 
-    fi->h = (scareMem ? headerLink(h) : NULL);
+    fi->h = (h != NULL && scareMem ? headerLink(h) : NULL);
 
     if (fi->fsm == NULL)
 	fi->fsm = newFSM();
 
+    {	int_32 rtid = 0;
+	if (hge(h, RPMTAG_REMOVETID, NULL, (void **) &rtid, NULL) && rtid > 0)
+	    fi->fsm->repackaged = 1;
+	else
+	    fi->fsm->repackaged = 0;
+    }
+
     /* 0 means unknown */
     xx = hge(h, RPMTAG_ARCHIVESIZE, NULL, (void **) &uip, NULL);
     fi->archivePos = 0;
@@ -1358,27 +1346,56 @@ if (fi->actions == NULL)
     xx = hge(h, RPMTAG_FILELINKTOS, NULL, (void **) &fi->flinks, NULL);
     xx = hge(h, RPMTAG_FILELANGS, NULL, (void **) &fi->flangs, NULL);
 
-    fi->fmd5s = NULL;
-    xx = hge(h, RPMTAG_FILEMD5S, NULL, (void **) &fi->fmd5s, NULL);
-
-    fi->md5s = NULL;
-    if (fi->fmd5s) {
-	t = xmalloc(fi->fc * 16);
-	fi->md5s = t;
+    fi->digestalgo = PGPHASHALGO_MD5;
+    fi->digestlen = 16;
+    fi->fdigestalgos = NULL;
+    xx = hge(h, RPMTAG_FILEDIGESTALGOS, NULL, (void **) &fi->fdigestalgos, NULL);
+    if (fi->fdigestalgos) {
+	int dalgo = 0;
+	/* XXX Insure that all algorithms are either 0 or constant. */
 	for (i = 0; i < fi->fc; i++) {
-	    const char * fmd5;
+	    if (fi->fdigestalgos[i] == 0)
+		continue;
+	    if (dalgo == 0)
+		dalgo = fi->fdigestalgos[i];
+	    else
+assert(dalgo == fi->fdigestalgos[i]);
+	}
+	fi->digestalgo = dalgo;
+	switch (dalgo) {
+	case PGPHASHALGO_MD5:		fi->digestlen = 128/8;	break;
+	case PGPHASHALGO_SHA1:		fi->digestlen = 160/8;	break;
+	case PGPHASHALGO_RIPEMD128:	fi->digestlen = 128/8;	break;
+	case PGPHASHALGO_RIPEMD160:	fi->digestlen = 160/8;	break;
+	case PGPHASHALGO_SHA256:	fi->digestlen = 256/8;	break;
+	case PGPHASHALGO_SHA384:	fi->digestlen = 384/8;	break;
+	case PGPHASHALGO_SHA512:	fi->digestlen = 512/8;	break;
+	case PGPHASHALGO_CRC32:		fi->digestlen = 32/8;	break;
+	}
+	fi->fdigestalgos = NULL;
+    }
+
+    fi->fdigests = NULL;
+    xx = hge(h, RPMTAG_FILEDIGESTS, NULL, (void **) &fi->fdigests, NULL);
+
+    fi->digests = NULL;
+    if (fi->fdigests) {
+	t = xmalloc(fi->fc * fi->digestlen);
+	fi->digests = t;
+	for (i = 0; i < fi->fc; i++) {
+	    const char * fdigests;
 	    int j;
 
-	    fmd5 = fi->fmd5s[i];
-	    if (!(fmd5 && *fmd5 != '\0')) {
-		memset(t, 0, 16);
-		t += 16;
+	    fdigests = fi->fdigests[i];
+	    if (!(fdigests && *fdigests != '\0')) {
+		memset(t, 0, fi->digestlen);
+		t += fi->digestlen;
 		continue;
 	    }
-	    for (j = 0; j < 16; j++, t++, fmd5 += 2)
-		*t = (nibble(fmd5[0]) << 4) | nibble(fmd5[1]);
+	    for (j = 0; j < fi->digestlen; j++, t++, fdigests += 2)
+		*t = (nibble(fdigests[0]) << 4) | nibble(fdigests[1]);
 	}
-	fi->fmd5s = hfd(fi->fmd5s, -1);
+	fi->fdigests = hfd(fi->fdigests, -1);
     }
 
     /* XXX TR_REMOVED doesn;t need fmtimes, frdevs, finodes, or fcontexts */
@@ -1395,7 +1412,7 @@ if (fi->actions == NULL)
     if (ts != NULL)
     if (fi != NULL)
     if ((p = rpmtsRelocateElement(ts)) != NULL && rpmteType(p) == TR_ADDED
-     && !headerIsEntry(h, RPMTAG_SOURCEPACKAGE)
+     && headerIsEntry(h, RPMTAG_SOURCERPM)
      && !headerIsEntry(h, RPMTAG_ORIGBASENAMES))
     {
 	const char * fmt = rpmGetPath("%{?_autorelocate_path}", NULL);
@@ -1453,7 +1470,7 @@ if (fi->actions == NULL)
 /* XXX DYING */
 if (fi->actions == NULL)
 	fi->actions = xcalloc(fi->fc, sizeof(*fi->actions));
-	/*@-compdef@*/ /* FIX: fi-md5s undefined */
+	/*@-compdef@*/ /* FIX: fi->digests undefined */
 	foo = relocateFileList(ts, fi, h, fi->actions);
 	/*@=compdef@*/
 	fi->h = headerFree(fi->h);
