@@ -1,7 +1,7 @@
 #ifndef H_RPMFI
 #define H_RPMFI
 
-/** \ingroup rpmdep rpmtrans
+/** \ingroup rpmfi
  * \file lib/rpmfi.h
  * Structure(s) used for file info tag sets.
  */
@@ -25,7 +25,7 @@ struct sharedFileInfo_s {
     int isRemoved;
 };
 
-/**
+/** \ingroup rpmfi
  * A package filename set.
  */
 struct rpmfi_s {
@@ -45,7 +45,9 @@ struct rpmfi_s {
     const char ** dnl;		/*!< Directory name(s) (from header) */
 
 /*@only@*/ /*@relnull@*/
-    const char ** fmd5s;	/*!< File MD5 sum(s) (from header) */
+    const char ** fdigests;	/*!< File digest(s) (from header) */
+/*@only@*/ /*@null@*/
+    uint_32 * fdigestalgos;	/*!< File digest algorithm(s) (from header) */
 /*@only@*/ /*@relnull@*/
     const char ** flinks;	/*!< File link(s) (from header) */
 /*@only@*/ /*@null@*/
@@ -126,7 +128,18 @@ struct rpmfi_s {
     int_32 * odil;		/*!< Original dirindex(s) (from header) */
 
 /*@only@*/ /*@relnull@*/
-    unsigned char * md5s;	/*!< File md5 sums in binary. */
+    unsigned char * digests;	/*!< File digest(s) in binary. */
+    uint_32 digestalgo;		/*!< File digest algorithm. */
+    uint_32 digestlen;		/*!< No. bytes in binary digest. */
+
+/*@only@*/ /*@relnull@*/
+    const char * pretrans;
+/*@only@*/ /*@relnull@*/
+    const char * pretransprog;
+/*@only@*/ /*@relnull@*/
+    const char * posttrans;
+/*@only@*/ /*@relnull@*/
+    const char * posttransprog;
 
 /*@only@*/ /*@null@*/
     char * fn;			/*!< File name buffer. */
@@ -134,8 +147,8 @@ struct rpmfi_s {
 
     int astriplen;
     int striplen;
-    unsigned int archivePos;
-    unsigned int archiveSize;
+    unsigned long long archivePos;
+    unsigned long long archiveSize;
     mode_t dperms;		/*!< Directory perms (0755) if not mapped. */
     mode_t fperms;		/*!< File perms (0644) if not mapped. */
 /*@only@*/ /*@null@*/
@@ -165,6 +178,9 @@ struct rpmfi_s {
 extern "C" {
 #endif
 
+/** \name RPMFI */
+/*@{*/
+
 /**
  * Unreference a file info set instance.
  * @param fi		file info set
@@ -325,8 +341,22 @@ int_16 rpmfiFMode(/*@null@*/ rpmfi fi)
 rpmfileState rpmfiFState(/*@null@*/ rpmfi fi)
 	/*@*/;
 
+/**
+ * Return current file (binary) digest from file info set.
+ * @param fi		file info set
+ * @retval *algop	digest algorithm
+ * @retval *lenp	digest length (in bytes)
+ * @return		current file digest, NULL on invalid
+ */
+/*@observer@*/ /*@null@*/
+extern const unsigned char * rpmfiDigest(/*@null@*/ rpmfi fi,
+		/*@out@*/ /*@null@*/ int * algop,
+		/*@out@*/ /*@null@*/ size_t * lenp)
+	/*@modifies *algop, *lenp @*/;
+
 /**
  * Return current file (binary) md5 digest from file info set.
+ * @deprecated Use rpmfiDigest instead.
  * @param fi		file info set
  * @return		current file md5 digest, NULL on invalid
  */
@@ -498,6 +528,7 @@ rpmfi rpmfiFree(/*@killref@*/ /*@only@*/ /*@null@*/ rpmfi fi)
 
 /**
  * Create and load a file info set.
+ * @deprecated Only scareMem = 0 will be permitted.
  * @param ts		transaction set (NULL skips path relocation)
  * @param h		header
  * @param tagN		RPMTAG_BASENAMES
@@ -577,8 +608,9 @@ void rpmfiBuildREContexts(Header h,
  */
 void rpmfiBuildFDeps(Header h, rpmTag tagN,
 		/*@out@*/ const char *** fdepsp, /*@out@*/ int * fcp)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
-	/*@modifies h, *fdepsp, *fcp, rpmGlobalMacroContext, fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies h, *fdepsp, *fcp,
+		rpmGlobalMacroContext, fileSystem, internalState @*/;
 
 /**
  * Return file type from mode_t.
@@ -615,10 +647,12 @@ fileAction rpmfiDecideFate(const rpmfi ofi, rpmfi nfi, int skipMissing)
  */
 /*@-redef@*/
 /*@observer@*/
-const char *const rpmfiTypeString(rpmfi fi)
+const char * rpmfiTypeString(rpmfi fi)
 	/*@*/;
 /*@=redef@*/
 
+/*@}*/
+
 #ifdef __cplusplus
 }
 #endif
