@@ -1,6 +1,6 @@
 /*@-modfilesys@*/
-/** \ingroup rpmio
- * \file rpmio/rpmio.c
+/** \ingroup rpmgi
+ * \file lib/rpmgi.c
  */
 #include "system.h"
 
@@ -22,15 +22,23 @@
 /*@access rpmts @*/
 /*@access rpmps @*/
 
+/**
+ */
 /*@unchecked@*/
 int _rpmgi_debug = 0;
 
+/**
+ */
 /*@unchecked@*/
 rpmgiFlags giFlags = RPMGI_NONE;
 
+/**
+ */
 /*@unchecked@*/
 static int indent = 2;
 
+/**
+ */
 /*@unchecked@*/ /*@observer@*/
 static const char * ftsInfoStrings[] = {
     "UNKNOWN",
@@ -50,6 +58,8 @@ static const char * ftsInfoStrings[] = {
     "W",
 };
 
+/**
+ */
 /*@observer@*/
 static const char * ftsInfoStr(int fts_info)
 	/*@*/
@@ -147,6 +157,7 @@ static Header rpmgiReadHeader(rpmgi gi, const char * path)
 /**
  * Read next header from package, lazily expanding manifests as found.
  * @todo An empty file read as manifest truncates argv returning RPMRC_NOTFOUND.
+ * @todo Errors, e.g. non-existent path in manifest, will terminate iteration.
  * @todo Chained manifests lose an arg someplace.
  * @param gi		generalized iterator
  * @return		RPMRC_OK on success
@@ -173,9 +184,6 @@ static rpmRC rpmgiLoadReadHeader(rpmgi gi)
 
 	if (rpmrc == RPMRC_OK || gi->flags & RPMGI_NOMANIFEST)
 	    break;
-	if (errno == ENOENT) {
-	    break;
-	}
 
 	/* Not a header, so try for a manifest. */
 	gi->argv[gi->i] = NULL;		/* Mark the insertion point */
@@ -310,14 +318,12 @@ static rpmRC rpmgiGlobArgv(rpmgi gi, /*@null@*/ ARGV_t argv)
 
     if (argv != NULL)
     while ((arg = *argv++) != NULL) {
-	char * t = rpmEscapeSpaces(arg);
 	ARGV_t av = NULL;
 
-	xx = rpmGlob(t, &ac, &av);
+	xx = rpmGlob(arg, &ac, &av);
 	xx = argvAppend(&gi->argv, av);
 	gi->argc += ac;
 	av = argvFree(av);
-	t = _free(t);
 	ac = 0;
     }
     return rpmrc;
@@ -365,7 +371,7 @@ fprintf(stderr, "*** gi %p\tmi %p\n", gi, gi->mi);
 
 	if (!res) {
 if (_rpmgi_debug  < 0)
-fprintf(stderr, "\tav %p[%d]: \"%s\" -> %s ~= \"%s\"\n", gi->argv, (av - gi->argv), *av, tagName(tag), pat);
+fprintf(stderr, "\tav %p[%d]: \"%s\" -> %s ~= \"%s\"\n", gi->argv, (int)(av - gi->argv), *av, tagName(tag), pat);
 	    res = rpmdbSetIteratorRE(gi->mi, tag, RPMMIRE_DEFAULT, pat);
 	}
 	a = _free(a);
@@ -450,6 +456,7 @@ rpmgi rpmgiNew(rpmts ts, int tag, const void * keyp, size_t keylen)
 	return NULL;
 
     gi->ts = rpmtsLink(ts, __FUNCTION__);
+    gi->tsOrder = rpmtsOrder;
     gi->tag = tag;
 /*@-assignexpose@*/
     gi->keyp = keyp;
@@ -459,7 +466,6 @@ rpmgi rpmgiNew(rpmts ts, int tag, const void * keyp, size_t keylen)
     gi->flags = 0;
     gi->active = 0;
     gi->i = -1;
-    gi->errors = 0;
     gi->hdrPath = NULL;
     gi->h = NULL;
 
@@ -486,6 +492,9 @@ rpmRC rpmgiNext(/*@null@*/ rpmgi gi)
     if (gi == NULL)
 	return rpmrc;
 
+if (_rpmgi_debug)
+fprintf(stderr, "*** %s(%p) tag %s\n", __FUNCTION__, gi, tagName(gi->tag));
+
     /* Free header from previous iteration. */
     gi->h = headerFree(gi->h);
     gi->hdrPath = _free(gi->hdrPath);
@@ -521,20 +530,27 @@ rpmRC rpmgiNext(/*@null@*/ rpmgi gi)
 	    goto enditer;
 	}
 	break;
+    case RPMDBI_REMOVED:
     case RPMDBI_ADDED:
     {	rpmte p;
+	int teType = 0;
+	const char * teTypeString = NULL;
 
 	if (!gi->active) {
 	    gi->tsi = rpmtsiInit(gi->ts);
 	    gi->active = 1;
 	}
-	if ((p = rpmtsiNext(gi->tsi, TR_ADDED)) != NULL) {
+	if ((p = rpmtsiNext(gi->tsi, teType)) != NULL) {
 	    Header h = rpmteHeader(p);
 	    if (h != NULL)
 		if (!(gi->flags & RPMGI_NOHEADER)) {
 		    gi->h = headerLink(h);
+		switch(rpmteType(p)) {
+		case TR_ADDED:	teTypeString = "+++";	/*@switchbreak@*/break;
+		case TR_REMOVED: teTypeString = "---";	/*@switchbreak@*/break;
+		}
 		sprintf(hnum, "%u", (unsigned)gi->i);
-		gi->hdrPath = rpmExpand("added h# ", hnum, NULL);
+		gi->hdrPath = rpmExpand("%s h# ", teTypeString, hnum, NULL);
 		rpmrc = RPMRC_OK;
 		h = headerFree(h);
 	    }
@@ -567,22 +583,12 @@ rpmRC rpmgiNext(/*@null@*/ rpmgi gi)
 	    goto enditer;
 	}
 	break;
-    case RPMDBI_ARGLIST: 
+    case RPMDBI_ARGLIST:
 	/* XXX gi->active initialize? */
 if (_rpmgi_debug  < 0)
 fprintf(stderr, "*** gi %p\t%p[%d]: %s\n", gi, gi->argv, gi->i, gi->argv[gi->i]);
-
-	/* 
- 	 * Read next header, lazily expanding manifests as found,
- 	 * count + skip errors.
- 	 */
-	rpmrc = RPMRC_NOTFOUND;	
-	while (gi->i < gi->argc) {
-	    if ((rpmrc = rpmgiLoadReadHeader(gi)) == RPMRC_OK) 
-		break;
-	    gi->errors++;
-	    gi->i++;
-        }
+	/* Read next header, lazily expanding manifests as found. */
+	rpmrc = rpmgiLoadReadHeader(gi);
 
 	if (rpmrc != RPMRC_OK)	/* XXX check this */
 	    goto enditer;
@@ -590,7 +596,7 @@ fprintf(stderr, "*** gi %p\t%p[%d]: %s\n", gi, gi->argv, gi->i, gi->argv[gi->i])
 	gi->hdrPath = xstrdup(gi->argv[gi->i]);
 	break;
     case RPMDBI_FTSWALK:
-	if (gi->argv == NULL)		/* HACK */
+	if (gi->argv == NULL || gi->argv[0] == NULL)		/* HACK */
 	    goto enditer;
 
 	if (!gi->active) {
@@ -616,7 +622,14 @@ fprintf(stderr, "*** gi %p\t%p[%d]: %s\n", gi, gi->argv, gi->i, gi->argv[gi->i])
 
     if ((gi->flags & RPMGI_TSADD) && gi->h != NULL) {
 	/* XXX rpmgi hack: Save header in transaction element. */
-	xx = rpmtsAddInstallElement(gi->ts, gi->h, (fnpyKey)gi->hdrPath, 2, NULL);
+	if (gi->flags & RPMGI_ERASING) {
+	    static int hdrx = 0;
+	    int dboffset = headerGetInstance(gi->h);
+	    if (dboffset <= 0)
+		dboffset = --hdrx;
+	    xx = rpmtsAddEraseElement(gi->ts, gi->h, dboffset);
+	} else
+	    xx = rpmtsAddInstallElement(gi->ts, gi->h, (fnpyKey)gi->hdrPath, 2, NULL);
     }
 
     return rpmrc;
@@ -627,9 +640,11 @@ enditer:
 	rpmps ps;
 	int i;
 
-	/* XXX installed database needs close here. */
-	xx = rpmtsCloseDB(ts);
-	ts->dbmode = -1;	/* XXX disable lazy opens */
+	/* XXX database needs close for added package closure check. */
+	if (!(gi->flags & RPMGI_ERASING)) {
+	    xx = rpmtsCloseDB(ts);
+	    ts->dbmode = -1;	/* XXX disable lazy opens */
+	}
 
 	xx = rpmtsCheck(ts);
 
@@ -663,9 +678,14 @@ enditer:
 	ps = rpmpsFree(ps);
 	ts->probs = rpmpsFree(ts->probs);	/* XXX hackery */
 
-	xx = rpmtsOrder(ts);
+	/* XXX Display dependency loops with rpm -qvT. */
+	if (rpmIsVerbose())
+	    (void) rpmtsSetFlags(ts, (rpmtsFlags(ts) | RPMTRANS_FLAG_DEPLOOPS));
 
-	gi->tag = RPMDBI_ADDED;			/* XXX hackery */
+	xx = (*gi->tsOrder) (ts);
+
+	/* XXX hackery alert! */
+	gi->tag = (!(gi->flags & RPMGI_ERASING) ? RPMDBI_ADDED : RPMDBI_REMOVED);
 	gi->flags &= ~(RPMGI_TSADD|RPMGI_TSORDER);
 
     }
@@ -698,14 +718,10 @@ rpmts rpmgiTs(rpmgi gi)
 
 rpmRC rpmgiSetArgs(rpmgi gi, ARGV_t argv, int ftsOpts, rpmgiFlags flags)
 {
+    if (gi == NULL) return RPMRC_FAIL;
     gi->ftsOpts = ftsOpts;
     gi->flags = flags;
     return rpmgiGlobArgv(gi, argv);
 }
 
-int rpmgiNumErrors(rpmgi gi)
-{
-    return (gi != NULL ? gi->errors : -1);
-}
-
 /*@=modfilesys@*/
