@@ -9,7 +9,8 @@
 #include "rpmdb.h"
 #include "rpmds.h"
 
-#include "rpmte.h"	/* XXX: rpmts.h needs this for rpmtsScoreEntries */
+#define	_RPMTE_INTERNAL		/* XXX findErases needs rpmte internals. */
+#include "rpmte.h"		/* XXX: rpmteChain */
 #define	_RPMTS_INTERNAL		/* ts->goal, ts->dbmode, ts->suggests */
 #include "rpmts.h"
 
@@ -18,6 +19,7 @@
 #include "debug.h"
 
 /*@access rpmts @*/	/* XXX ts->goal, ts->dbmode */
+/*@access rpmte @*/	/* XXX p->hdrid, p->pkgid, p->NEVRA */
 /*@access IDTX @*/
 /*@access IDT @*/
 
@@ -28,9 +30,9 @@ int rpmcliHashesCurrent = 0;
 /*@unchecked@*/
 int rpmcliHashesTotal = 0;
 /*@unchecked@*/
-int rpmcliProgressCurrent = 0;
+unsigned long long rpmcliProgressCurrent = 0;
 /*@unchecked@*/
-int rpmcliProgressTotal = 0;
+unsigned long long rpmcliProgressTotal = 0;
 
 /**
  * Print a CLI progress bar.
@@ -38,7 +40,7 @@ int rpmcliProgressTotal = 0;
  * @param amount	current
  * @param total		final
  */
-static void printHash(const unsigned long amount, const unsigned long total)
+static void printHash(const unsigned long long amount, const unsigned long long total)
 	/*@globals rpmcliHashesCurrent, rpmcliHashesTotal,
 		rpmcliProgressCurrent, fileSystem @*/
 	/*@modifies rpmcliHashesCurrent, rpmcliHashesTotal,
@@ -91,8 +93,8 @@ static void printHash(const unsigned long amount, const unsigned long total)
 
 void * rpmShowProgress(/*@null@*/ const void * arg,
 			const rpmCallbackType what,
-			const unsigned long amount,
-			const unsigned long total,
+			const unsigned long long amount,
+			const unsigned long long total,
 			/*@null@*/ fnpyKey key,
 			/*@null@*/ void * data)
 	/*@globals rpmcliHashesCurrent, rpmcliProgressCurrent, rpmcliProgressTotal,
@@ -154,7 +156,7 @@ void * rpmShowProgress(/*@null@*/ const void * arg,
 	    s = headerSprintf(h, "%{NAME}",
 				rpmTagTable, rpmHeaderFormats, NULL);
 	    if (isatty (STDOUT_FILENO))
-		fprintf(stdout, "%4d:%-23.23s", rpmcliProgressCurrent + 1, s);
+		fprintf(stdout, "%4d:%-23.23s", (int)rpmcliProgressCurrent + 1, s);
 	    else
 		fprintf(stdout, "%-28.28s", s);
 	    (void) fflush(stdout);
@@ -277,7 +279,7 @@ struct rpmEIU {
 /*@only@*/ /*@null@*/
     str_t * argv;
 /*@dependent@*/
-    rpmRelocation * relocations;
+    rpmRelocation relocations;
     rpmRC rpmrc;
 };
 
@@ -290,7 +292,7 @@ int rpmInstall(rpmts ts,
     struct rpmEIU * eiu = memset(alloca(sizeof(*eiu)), 0, sizeof(*eiu));
     rpmps ps;
     rpmprobFilterFlags probFilter;
-    rpmRelocation * relocations;
+    rpmRelocation relocations;
 /*@only@*/ /*@null@*/ const char * fileURL = NULL;
     int stopInstall = 0;
     const char ** av = NULL;
@@ -323,6 +325,16 @@ int rpmInstall(rpmts ts,
     }
     (void) rpmtsSetFlags(ts, ia->transFlags);
 
+    /* Display and set autorollback goal. */
+    if (rpmExpandNumeric("%{?_rollback_transaction_on_failure}")) {
+	if (ia->arbtid) {
+	    time_t ttid = (time_t)ia->arbtid;
+	    rpmMessage(RPMMESS_DEBUG, _("Autorollback Goal: %-24.24s (0x%08x)\n"), 
+		ctime(&ttid), ia->arbtid);
+	    rpmtsSetARBGoal(ts, ia->arbtid);
+	}	
+    }
+
     probFilter = ia->probFilter;
     relocations = ia->relocations;
 
@@ -510,7 +522,7 @@ if (fileURL[0] == '=') {
 	    /*@switchbreak@*/ break;
 	}
 
-	eiu->isSource = headerIsEntry(eiu->h, RPMTAG_SOURCEPACKAGE);
+	eiu->isSource = (headerIsEntry(eiu->h, RPMTAG_SOURCERPM) == 0);
 
 	if (eiu->isSource) {
 	    rpmMessage(RPMMESS_DEBUG, _("\tadded source package [%d]\n"),
@@ -599,10 +611,6 @@ if (fileURL[0] == '=') {
 	    eiu->numFailed++;
 	    goto exit;
 	    /*@notreached@*/ /*@switchbreak@*/ break;
-	default:
-	    eiu->numFailed++;
-	    goto exit;
-	    /*@notreached@*/ /*@switchbreak@*/ break;
 	}
 
 	eiu->numRPMS++;
@@ -781,11 +789,21 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 
     (void) rpmtsSetFlags(ts, ia->transFlags);
 
+    /* Display and set autorollback goal. */
+    if (rpmExpandNumeric("%{?_rollback_transaction_on_failure}")) {
+	if (ia->arbtid) {
+	    time_t ttid = (time_t)ia->arbtid;
+	    rpmMessage(RPMMESS_DEBUG, _("Autorollback Goal: %-24.24s (0x%08x)\n"), 
+		ctime(&ttid), ia->arbtid);
+	    rpmtsSetARBGoal(ts, ia->arbtid);
+	}	
+    }
+
 #ifdef	NOTYET	/* XXX no callbacks on erase yet */
     {	int notifyFlags;
 	notifyFlags = ia->eraseInterfaceFlags | (rpmIsVerbose() ? INSTALL_LABEL : 0 );
 	xx = rpmtsSetNotifyCallback(ts,
-			rpmShowProgress, (void *) ((long)notifyFlags)
+			rpmShowProgress, (void *) ((long)notifyFlags));
     }
 #endif
 
@@ -846,13 +864,13 @@ int rpmErase(rpmts ts, struct rpmInstallArguments_s * ia,
 	}
     }
 
-    if (numPackages && !stopUninstall) {
+    if (!stopUninstall) {
 	(void) rpmtsSetFlags(ts, (rpmtsFlags(ts) | RPMTRANS_FLAG_REVERSE));
 
 	/* Drop added/available package indices and dependency sets. */
 	rpmtsClean(ts);
 
-	numPackages = rpmtsRun(ts, NULL, ia->probFilter & (RPMPROB_FILTER_DISKSPACE|RPMPROB_FILTER_DISKNODES));
+	numPackages = rpmtsRun(ts, NULL, 0);
 	ps = rpmtsProblems(ts);
 	if (rpmpsNumProblems(ps) > 0)
 	    rpmpsPrint(NULL, ps);
@@ -967,7 +985,7 @@ IDTX IDTXsort(IDTX idtx)
     return idtx;
 }
 
-IDTX IDTXload(rpmts ts, rpmTag tag)
+IDTX IDTXload(rpmts ts, rpmTag tag, uint_32 rbtid)
 {
     IDTX idtx = NULL;
     rpmdbMatchIterator mi;
@@ -991,16 +1009,19 @@ IDTX IDTXload(rpmts ts, rpmTag tag)
 	if (type == RPM_INT32_TYPE && (*tidp == 0 || *tidp == -1))
 	    continue;
 
-	idtx = IDTXgrow(idtx, 1);
-	if (idtx == NULL)
+	/* Don't bother with headers installed prior to the rollback goal. */
+	if (*tidp < rbtid)
 	    continue;
-	if (idtx->idt == NULL)
+
+	idtx = IDTXgrow(idtx, 1);
+	if (idtx == NULL || idtx->idt == NULL)
 	    continue;
 
 	{   IDT idt;
 	    /*@-nullderef@*/
 	    idt = idtx->idt + idtx->nidt;
 	    /*@=nullderef@*/
+	    idt->done = 0;
 	    idt->h = headerLink(h);
 	    idt->key = NULL;
 	    idt->instance = rpmdbGetIteratorOffset(mi);
@@ -1014,10 +1035,10 @@ IDTX IDTXload(rpmts ts, rpmTag tag)
     return IDTXsort(idtx);
 }
 
-IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag)
+IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag, uint_32 rbtid)
 {
+    HGE_t hge = (HGE_t) headerGetEntry;		/* XXX MinMem? <shrug> */
     IDTX idtx = NULL;
-    HGE_t hge = (HGE_t) headerGetEntry;
     Header h;
     int_32 * tidp;
     FD_t fd;
@@ -1030,6 +1051,7 @@ IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag)
     av = NULL;	ac = 0;
     xx = rpmGlob(globstr, &ac, &av);
 
+/*@-branchstate@*/
     if (xx == 0)
     for (i = 0; i < ac; i++) {
 	rpmTagType type;
@@ -1038,8 +1060,8 @@ IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag)
 
 	fd = Fopen(av[i], "r.ufdio");
 	if (fd == NULL || Ferror(fd)) {
-            rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), av[i],
-                        Fstrerror(fd));
+	    rpmError(RPMERR_OPEN, _("open of %s failed: %s\n"), av[i],
+	    		Fstrerror(fd));
 	    if (fd != NULL) (void) Fclose(fd);
 	    continue;
 	}
@@ -1053,34 +1075,42 @@ IDTX IDTXglob(rpmts ts, const char * globstr, rpmTag tag)
 	case RPMRC_NOTTRUSTED:
 	case RPMRC_NOKEY:
 	case RPMRC_OK:
-	    isSource = headerIsEntry(h, RPMTAG_SOURCEPACKAGE);
+	    isSource = (headerIsEntry(h, RPMTAG_SOURCERPM) == 0);
 	    if (isSource)
 		goto bottom;
 	    /*@switchbreak@*/ break;
 	}
 
+{ const char * origin = headerGetOrigin(h);
+assert(origin != NULL);
+assert(!strcmp(av[i], origin));
+}
 	tidp = NULL;
-	/*@-branchstate@*/
-	if (hge(h, tag, &type, (void **) &tidp, &count) && tidp != NULL) {
+	if (!hge(h, tag, &type, (void **) &tidp, &count) || tidp == NULL)
+	    goto bottom;
 
-	    idtx = IDTXgrow(idtx, 1);
-	    if (idtx == NULL || idtx->idt == NULL)
-		goto bottom;
+	/* Don't bother with headers installed prior to the rollback goal. */
+	if (*tidp < rbtid)
+	    goto bottom;
 
-	    {	IDT idt;
-		idt = idtx->idt + idtx->nidt;
-		idt->h = headerLink(h);
-		idt->key = av[i];
-		av[i] = NULL;
-		idt->instance = 0;
-		idt->val.u32 = *tidp;
-	    }
-	    idtx->nidt++;
+	idtx = IDTXgrow(idtx, 1);
+	if (idtx == NULL || idtx->idt == NULL)
+	    goto bottom;
+
+	{   IDT idt;
+	    idt = idtx->idt + idtx->nidt;
+	    idt->done = 0;
+	    idt->h = headerLink(h);
+	    idt->key = av[i];
+	    av[i] = NULL;
+	    idt->instance = 0;
+	    idt->val.u32 = *tidp;
 	}
-	/*@=branchstate@*/
+	idtx->nidt++;
 bottom:
 	h = headerFree(h);
     }
+/*@=branchstate@*/
 
     for (i = 0; i < ac; i++)
 	av[i] = _free(av[i]);
@@ -1089,6 +1119,133 @@ bottom:
     return IDTXsort(idtx);
 }
 
+/**
+ * Search for string B in argv array AV.
+ * @param lname		type of link
+ * @param AV		argv array
+ * @param AC		no. of args
+ * @param B		string
+ * @return		1 if found, 0 not found, -1 error
+ */
+static int cmpArgvStr(const char *lname, const char ** AV, int AC,
+		/*@null@*/ const char * B)
+	/*@*/
+{
+    const char * A;
+    int i;
+
+    if (AV != NULL && AC > 0 && B == NULL) {
+	rpmError(RPMERR_OPEN, _("Missing repackaged package(s) detected:\n"));
+	for (i = 0; i < AC && (A = AV[i]) != NULL; i++)
+	    rpmError(RPMERR_OPEN, _("    %s[%d]: %s\n"), lname, i, A);
+	return -1;
+    }
+
+    if (AV != NULL && B != NULL)
+    for (i = 0; i < AC && (A = AV[i]) != NULL; i++) {
+	if (*A && *B && !strcmp(A, B))
+	    return 1;
+    }
+    return 0;
+}
+
+/**
+ * Find (and add to transaction set) all erase elements with matching blink.
+ * In addition, recreate any added transaction element linkages.
+ *
+ * XXX rp->h should have FLINK{HDRID,PKGID,NEVRA} populated.
+ * XXX ip->h should have BLINK{HDRID,PKGID,NEVRA} populated.
+ * XXX p = ts->teInstall is added transaction element from rp->h.
+ *
+ * @param ts		transaction set (ts->teInstall set to last added pkg)
+ * @param p		most recently added install element (NULL skips linking)
+ * @param thistid	current transaction id
+ * @param ip		currently installed package(s) to be erased
+ * @param niids		no. of currently installed package(s)
+ * @return		-1 on error, otherwise no. of erase elemnts added
+ */
+static int findErases(rpmts ts, /*@null@*/ rpmte p, unsigned thistid,
+		/*@null@*/ IDT ip, int niids)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, p, ip, rpmGlobalMacroContext, fileSystem, internalState @*/
+{
+    int rc = 0;
+    int xx;
+
+    /* Erase the previously installed packages for this transaction. 
+     * Provided this transaction is not excluded from the rollback.
+     */
+    while (ip != NULL && ip->val.u32 == thistid) {
+
+	if (ip->done)
+	    goto bottom;
+
+	{
+	    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+	    const char ** flinkPkgid = NULL;
+	    const char ** flinkHdrid = NULL;
+	    const char ** flinkNEVRA = NULL;
+	    rpmTagType pt, ht, nt;
+	    int_32 pn, hn, nn;
+	    int bingo;
+
+	    xx = hge(ip->h, RPMTAG_BLINKPKGID, &pt, (void **)&flinkPkgid, &pn);
+	    xx = hge(ip->h, RPMTAG_BLINKHDRID, &ht, (void **)&flinkHdrid, &hn);
+	    xx = hge(ip->h, RPMTAG_BLINKNEVRA, &nt, (void **)&flinkNEVRA, &nn);
+
+	    /*
+	     * Link data may be missing and can have multiple entries.
+	     * Try hdrid, pkgid, finally NEVRA, argv vs. str compares.
+	     */
+	    bingo = cmpArgvStr("Hdrid", flinkHdrid, hn, (p ? p->hdrid : NULL));
+	    if (!bingo)
+		bingo = cmpArgvStr("Pkgid", flinkPkgid, pn, (p ? p->pkgid : NULL));
+	    if (!bingo)
+		bingo = cmpArgvStr("NEVRA", flinkNEVRA, nn, (p ? p->NEVRA : NULL));
+	    flinkPkgid = headerFreeData(flinkPkgid, pt);
+	    flinkHdrid = headerFreeData(flinkHdrid, ht);
+	    flinkNEVRA = headerFreeData(flinkNEVRA, nt);
+
+	    if (bingo < 0) {
+		rc = -1;
+		goto exit;
+	    }
+
+	    if (!bingo)
+		goto bottom;
+	}
+
+	rpmMessage(RPMMESS_DEBUG, "\t--- erase h#%u\n", ip->instance);
+
+	rc = rpmtsAddEraseElement(ts, ip->h, ip->instance);
+	if (rc != 0)
+	    goto exit;
+
+	/* Cross link the transaction elements to mimic --upgrade. */
+	if (p != NULL) {
+	    rpmte q = ts->teErase;
+	    xx = rpmteChain(p, q, ip->h, "Rollback");
+	}
+
+#ifdef	NOTYET
+	ip->instance = 0;
+#endif
+	ip->done = 1;
+
+bottom:
+
+	/* Go to the next header in the rpmdb */
+	niids--;
+	if (niids > 0)
+	    ip++;
+	else
+	    ip = NULL;
+    }
+
+exit:
+    return rc;
+}
+
 /** @todo Transaction handling, more, needs work. */
 int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
 {
@@ -1106,6 +1263,7 @@ int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
     int vsflags, ovsflags;
     int numAdded;
     int numRemoved;
+    int excluded;
     rpmps ps;
     int _unsafe_rollbacks = 0;
     rpmtransFlags transFlags = ia->transFlags;
@@ -1134,7 +1292,7 @@ int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
      */
     rpmtsSetType(ts, RPMTRANS_TYPE_ROLLBACK);
 
-    itids = IDTXload(ts, RPMTAG_INSTALLTID);
+    itids = IDTXload(ts, RPMTAG_INSTALLTID, ia->rbtid);
     if (itids != NULL) {
 	ip = itids->idt;
 	niids = itids->nidt;
@@ -1149,7 +1307,7 @@ int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
 	    rc = -1;
 	    goto exit;
 	}
-	rtids = IDTXglob(ts, globstr, RPMTAG_REMOVETID);
+	rtids = IDTXglob(ts, globstr, RPMTAG_REMOVETID, ia->rbtid);
 
 	if (rtids != NULL) {
 	    rp = rtids->idt;
@@ -1191,30 +1349,59 @@ int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
 	if (_unsafe_rollbacks && thistid <= _unsafe_rollbacks)
 	    break;
 
+	/* Make sure this tid is not excluded */
+	excluded = 0;	/* Assume its not */
+	if (ia->rbtidExcludes != NULL && ia->numrbtidExcludes > 0)
+	{
+	    uint_32 *excludedTID;
+	    for(excludedTID = ia->rbtidExcludes; 
+		excludedTID < ia->rbtidExcludes + ia->numrbtidExcludes;
+		excludedTID++) {
+		if (thistid == *excludedTID) {
+		    time_t ttid = (time_t)thistid;
+		    rpmMessage(RPMMESS_NORMAL,
+			_("Excluding TID from rollback: %-24.24s (0x%08x)\n"),
+				ctime(&ttid), thistid);
+		    excluded = 1;
+		    /*@innerbreak@*/ break;	
+		}
+	    }	
+	}
+
 	rpmtsEmpty(ts);
 	(void) rpmtsSetFlags(ts, transFlags);
 
-	/* Install the previously erased packages for this transaction. */
+	/* Install the previously erased packages for this transaction. 
+	 * Provided this transaction is not excluded from the rollback.
+	 */
 	while (rp != NULL && rp->val.u32 == thistid) {
-
-	    rpmMessage(RPMMESS_DEBUG, "\t+++ install %s\n",
+	    if (!excluded && !rp->done) {
+		rpmMessage(RPMMESS_DEBUG, "\t+++ install %s\n",
 			(rp->key ? rp->key : "???"));
 
 /*@-abstract@*/
-	    rc = rpmtsAddInstallElement(ts, rp->h, (fnpyKey)rp->key,
+		rc = rpmtsAddInstallElement(ts, rp->h, (fnpyKey)rp->key,
 			       0, ia->relocations);
 /*@=abstract@*/
-	    if (rc != 0)
-		goto exit;
+		if (rc != 0)
+		    goto exit;
 
-	    numAdded++;
-	    rpmcliPackagesTotal++;
-	    if (!(ia->installInterfaceFlags & ifmask))
-		ia->installInterfaceFlags |= INSTALL_UPGRADE;
+		numAdded++;
+		rpmcliPackagesTotal++;
+		if (!(ia->installInterfaceFlags & ifmask))
+		    ia->installInterfaceFlags |= INSTALL_UPGRADE;
 
+		/* Re-add linked (i.e. from upgrade/obsoletes) erasures. */
+		rc = findErases(ts, ts->teInstall, thistid, ip, niids);
+		if (rc < 0)
+		    goto exit;
 #ifdef	NOTYET
-	    rp->h = headerFree(rp->h);
+	    	rp->h = headerFree(rp->h);
 #endif
+		rp->done = 1;
+	    }
+
+	    /* Go to the next repackaged package */
 	    nrids--;
 	    if (nrids > 0)
 		rp++;
@@ -1222,29 +1409,27 @@ int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
 		rp = NULL;
 	}
 
-	/* Erase the previously installed packages for this transaction. */
+	/* Re-add pure (i.e. not from upgrade/obsoletes) erasures. */
+	rc = findErases(ts, NULL, thistid, ip, niids);
+	if (rc < 0)
+	    goto exit;
+
+	/* Check that all erasures have been re-added. */
 	while (ip != NULL && ip->val.u32 == thistid) {
+assert(excluded || ip->done);
+	    if (!excluded) {
+		numRemoved++;
 
-	    rpmMessage(RPMMESS_DEBUG,
-			"\t--- erase h#%u\n", ip->instance);
+		if (_unsafe_rollbacks)
+		    rpmcliPackagesTotal++;
 
-	    rc = rpmtsAddEraseElement(ts, ip->h, ip->instance);
-	    if (rc != 0)
-		goto exit;
-
-	    numRemoved++;
-
-	    if (_unsafe_rollbacks)
-		rpmcliPackagesTotal++;
-
-	    if (!(ia->installInterfaceFlags & ifmask)) {
-		ia->installInterfaceFlags |= INSTALL_ERASE;
-		(void) rpmtsSetFlags(ts, (transFlags | RPMTRANS_FLAG_REVERSE));
+		if (!(ia->installInterfaceFlags & ifmask)) {
+		    ia->installInterfaceFlags |= INSTALL_ERASE;
+		    (void) rpmtsSetFlags(ts, (transFlags | RPMTRANS_FLAG_REVERSE));
+		}
 	    }
 
-#ifdef	NOTYET
-	    ip->instance = 0;
-#endif
+	    /* Go to the next header in the rpmdb */
 	    niids--;
 	    if (niids > 0)
 		ip++;
@@ -1252,6 +1437,10 @@ int rpmRollback(rpmts ts, struct rpmInstallArguments_s * ia, const char ** argv)
 		ip = NULL;
 	}
 
+	/* If this transaction is excluded then continue */
+	if (excluded)
+	    continue;
+
 	/* Anything to do? */
 	if (rpmcliPackagesTotal <= 0)
 	    break;
