@@ -1,7 +1,7 @@
 #ifndef H_RPMLIB
 #define	H_RPMLIB
 
-/** \ingroup rpmcli rpmrc rpmdep rpmtrans rpmdb lead signature header payload dbi
+/** \ingroup rpmcli rpmrc rpmts rpmte rpmds rpmfi rpmdb lead signature header payload dbi
  * \file lib/rpmlib.h
  *
  * In Memoriam: Steve Taylor <staylor@redhat.com> was here, now he's not.
@@ -49,6 +49,7 @@ extern int rpmFLAGS;
 extern "C" {
 #endif
 
+#if !defined(SWIG)
 /**
  * Wrapper to free(3), hides const compilation noise, permit NULL, return NULL.
  * @param p		memory to free
@@ -61,8 +62,9 @@ void * _free(/*@only@*/ /*@null@*/ /*@out@*/ const void * p)
     if (p != NULL)	free((void *)p);
     return NULL;
 }
+#endif
 
-/** \ingroup rpmtrans
+/** \ingroup rpmts
  * The RPM Transaction Set.
  * Transaction sets are inherently unordered! RPM may reorder transaction
  * sets to reduce errors. In general, installs/upgrades are done before
@@ -74,30 +76,30 @@ typedef /*@abstract@*/ /*@refcounted@*/ struct rpmts_s * rpmts;
  */
 typedef struct Spec_s * Spec;
 
-/** \ingroup rpmtrans
+/** \ingroup rpmts
  * An added/available package retrieval key.
  */
 typedef /*@abstract@*/ void * alKey;
 #define	RPMAL_NOMATCH	((alKey)-1L)
 
-/** \ingroup rpmtrans
+/** \ingroup rpmts
  * An added/available package retrieval index.
  */
 /*@-mutrep@*/
 typedef /*@abstract@*/ int alNum;
 /*@=mutrep@*/
 
-/** \ingroup rpmtrans
+/** \ingroup rpmts rpmds 
  * Dependency tag sets from a header, so that a header can be discarded early.
  */
 typedef /*@abstract@*/ /*@refcounted@*/ struct rpmds_s * rpmds;
 
-/** \ingroup rpmtrans
+/** \ingroup rpmts rpmfi
  * File info tag sets from a header, so that a header can be discarded early.
  */
 typedef /*@abstract@*/ /*@refcounted@*/ struct rpmfi_s * rpmfi;
 
-/** \ingroup rpmtrans
+/** \ingroup rpmts rpmte
  * An element of a transaction set, i.e. a TR_ADDED or TR_REMOVED package.
  */
 typedef /*@abstract@*/ struct rpmte_s * rpmte;
@@ -112,7 +114,7 @@ typedef /*@abstract@*/ /*@refcounted@*/ struct rpmdb_s * rpmdb;
  */
 typedef /*@abstract@*/ struct _rpmdbMatchIterator * rpmdbMatchIterator;
 
-/** \ingroup rpmcli
+/** \ingroup rpmcli rpmgi
  * Generalized iterator.
  */
 typedef /*@abstract@*/ /*@refcounted@*/ struct rpmgi_s * rpmgi;
@@ -168,24 +170,6 @@ void headerMergeLegacySigs(Header h, const Header sigh)
 Header headerRegenSigHeader(const Header h, int noArchiveSize)
 	/*@modifies h @*/;
 
-/** \ingroup header
- * Retrieve tag info from header.
- * This is a "dressed" entry to headerGetEntry to do:
- *	1) DIRNAME/BASENAME/DIRINDICES -> FILENAMES tag conversions.
- *	2) i18n lookaside (if enabled).
- *
- * @param h		header
- * @param tag		tag
- * @retval type		address of tag value data type
- * @retval p		address of pointer to tag value(s)
- * @retval c		address of number of values
- * @return		0 on success, 1 on bad magic, 2 on error
- */
-/*@unused@*/
-int rpmHeaderGetEntry(Header h, int_32 tag, /*@out@*/ int_32 *type,
-		/*@out@*/ void **p, /*@out@*/ int_32 *c)
-	/*@modifies *type, *p, *c @*/;
-
 /**
  * Automatically generated table of tag name/value pairs.
  */
@@ -202,6 +186,9 @@ extern const struct headerTagTableEntry_s * rpmTagTable;
 extern const int rpmTagTableSize;
 /*@=redecl@*/
 
+/*@unchecked@*/
+extern headerTagIndices rpmTags;
+
 /**
  * Table of query format extensions.
  * @note Chains to headerDefaultFormats[].
@@ -224,7 +211,7 @@ extern const struct headerSprintfExtension_s rpmHeaderFormats[];
 #define	RPMDBI_ARGLIST		7	/* (rpmgi) Argument list. */
 #define	RPMDBI_FTSWALK		8	/* (rpmgi) File tree  walk. */
 
-/**
+/** \ingroup header
  * Tags identify data in package headers.
  * @note tags should not have value 0!
  */
@@ -269,7 +256,6 @@ typedef enum rpmTag_e {
 #define	RPMTAG_R	RPMTAG_RELEASE	/* s */
     RPMTAG_EPOCH   		= 1003,	/* i */
 #define	RPMTAG_E	RPMTAG_EPOCH	/* i */
-#define	RPMTAG_SERIAL	RPMTAG_EPOCH	/* i backward comaptibility */
     RPMTAG_SUMMARY		= 1004,	/* s{} */
     RPMTAG_DESCRIPTION		= 1005,	/* s{} */
     RPMTAG_BUILDTIME		= 1006,	/* i */
@@ -281,7 +267,6 @@ typedef enum rpmTag_e {
     RPMTAG_GIF			= 1012,	/* x */
     RPMTAG_XPM			= 1013,	/* x */
     RPMTAG_LICENSE		= 1014,	/* s */
-#define	RPMTAG_COPYRIGHT RPMTAG_LICENSE	/* s backward comaptibility */
     RPMTAG_PACKAGER		= 1015,	/* s */
     RPMTAG_GROUP		= 1016,	/* s{} */
 /*@-enummemuse@*/
@@ -297,16 +282,17 @@ typedef enum rpmTag_e {
     RPMTAG_PREUN		= 1025,	/* s */
     RPMTAG_POSTUN		= 1026,	/* s */
     RPMTAG_OLDFILENAMES		= 1027, /* s[] obsolete */
-    RPMTAG_FILESIZES		= 1028,	/* i */
-    RPMTAG_FILESTATES		= 1029, /* c */
-    RPMTAG_FILEMODES		= 1030,	/* h */
+    RPMTAG_FILESIZES		= 1028,	/* i[] */
+    RPMTAG_FILESTATES		= 1029, /* c[] */
+    RPMTAG_FILEMODES		= 1030,	/* h[] */
     RPMTAG_FILEUIDS		= 1031, /*!< internal */
     RPMTAG_FILEGIDS		= 1032, /*!< internal */
-    RPMTAG_FILERDEVS		= 1033,	/* h */
-    RPMTAG_FILEMTIMES		= 1034, /* i */
-    RPMTAG_FILEMD5S		= 1035,	/* s[] */
+    RPMTAG_FILERDEVS		= 1033,	/* h[] */
+    RPMTAG_FILEMTIMES		= 1034, /* i[] */
+    RPMTAG_FILEDIGESTS		= 1035,	/* s[] */
+#define RPMTAG_FILEMD5S	RPMTAG_FILEDIGESTS /* s[] */
     RPMTAG_FILELINKTOS		= 1036,	/* s[] */
-    RPMTAG_FILEFLAGS		= 1037,	/* i */
+    RPMTAG_FILEFLAGS		= 1037,	/* i[] */
 /*@-enummemuse@*/
     RPMTAG_ROOT			= 1038, /*!< internal - obsolete */
 /*@=enummemuse@*/
@@ -316,21 +302,20 @@ typedef enum rpmTag_e {
     RPMTAG_EXCLUDE		= 1041, /*!< internal - obsolete */
     RPMTAG_EXCLUSIVE		= 1042, /*!< internal - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_ICON			= 1043,
+    RPMTAG_ICON			= 1043, /* internal - obsolete */
     RPMTAG_SOURCERPM		= 1044,	/* s */
-    RPMTAG_FILEVERIFYFLAGS	= 1045,	/* i */
+    RPMTAG_FILEVERIFYFLAGS	= 1045,	/* i[] */
     RPMTAG_ARCHIVESIZE		= 1046,	/* i */
     RPMTAG_PROVIDENAME		= 1047,	/* s[] */
 #define	RPMTAG_PROVIDES RPMTAG_PROVIDENAME	/* s[] */
 #define	RPMTAG_P	RPMTAG_PROVIDENAME	/* s[] */
-    RPMTAG_REQUIREFLAGS		= 1048,	/* i */
+    RPMTAG_REQUIREFLAGS		= 1048,	/* i[] */
     RPMTAG_REQUIRENAME		= 1049,	/* s[] */
 #define	RPMTAG_REQUIRES RPMTAG_REQUIRENAME	/* s[] */
-#define	RPMTAG_D	RPMTAG_REQUIRENAME	/* s[] */
     RPMTAG_REQUIREVERSION	= 1050,	/* s[] */
     RPMTAG_NOSOURCE		= 1051, /*!< internal */
     RPMTAG_NOPATCH		= 1052, /*!< internal */
-    RPMTAG_CONFLICTFLAGS	= 1053, /* i */
+    RPMTAG_CONFLICTFLAGS	= 1053, /* i[] */
     RPMTAG_CONFLICTNAME		= 1054,	/* s[] */
 #define	RPMTAG_CONFLICTS RPMTAG_CONFLICTNAME	/* s[] */
 #define	RPMTAG_C	RPMTAG_CONFLICTNAME	/* s[] */
@@ -347,10 +332,10 @@ typedef enum rpmTag_e {
     RPMTAG_TRIGGERSCRIPTS	= 1065,	/* s[] */
     RPMTAG_TRIGGERNAME		= 1066,	/* s[] */
     RPMTAG_TRIGGERVERSION	= 1067,	/* s[] */
-    RPMTAG_TRIGGERFLAGS		= 1068,	/* i */
-    RPMTAG_TRIGGERINDEX		= 1069,	/* i */
+    RPMTAG_TRIGGERFLAGS		= 1068,	/* i[] */
+    RPMTAG_TRIGGERINDEX		= 1069,	/* i[] */
     RPMTAG_VERIFYSCRIPT		= 1079,	/* s */
-    RPMTAG_CHANGELOGTIME	= 1080,	/* i */
+    RPMTAG_CHANGELOGTIME	= 1080,	/* i[] */
     RPMTAG_CHANGELOGNAME	= 1081,	/* s[] */
     RPMTAG_CHANGELOGTEXT	= 1082,	/* s[] */
 /*@-enummemuse@*/
@@ -366,11 +351,11 @@ typedef enum rpmTag_e {
 #define	RPMTAG_OBSOLETES RPMTAG_OBSOLETENAME	/* s[] */
 #define	RPMTAG_O	RPMTAG_OBSOLETENAME	/* s[] */
     RPMTAG_VERIFYSCRIPTPROG	= 1091,	/* s */
-    RPMTAG_TRIGGERSCRIPTPROG	= 1092,	/* s */
+    RPMTAG_TRIGGERSCRIPTPROG	= 1092,	/* s[] */
     RPMTAG_DOCDIR		= 1093, /*!< internal */
     RPMTAG_COOKIE		= 1094,	/* s */
-    RPMTAG_FILEDEVICES		= 1095,	/* i */
-    RPMTAG_FILEINODES		= 1096,	/* i */
+    RPMTAG_FILEDEVICES		= 1095,	/* i[] */
+    RPMTAG_FILEINODES		= 1096,	/* i[] */
     RPMTAG_FILELANGS		= 1097,	/* s[] */
     RPMTAG_PREFIXES		= 1098,	/* s[] */
     RPMTAG_INSTPREFIXES		= 1099,	/* s[] */
@@ -380,9 +365,9 @@ typedef enum rpmTag_e {
     RPMTAG_AUTOREQ		= 1103, /*!< internal */
     RPMTAG_AUTOPROV		= 1104, /*!< internal */
 /*@-enummemuse@*/
-    RPMTAG_CAPABILITY		= 1105, /*!< internal - obsolete */
+    RPMTAG_CAPABILITY		= 1105, /*!< i legacy - obsolete */
 /*@=enummemuse@*/
-    RPMTAG_SOURCEPACKAGE	= 1106, /*!< i src.rpm header marker */
+    RPMTAG_SOURCEPACKAGE	= 1106, /*!< i legacy - obsolete */
 /*@-enummemuse@*/
     RPMTAG_OLDORIGFILENAMES	= 1107, /*!< internal - obsolete */
 /*@=enummemuse@*/
@@ -392,16 +377,16 @@ typedef enum rpmTag_e {
 /*@-enummemuse@*/
     RPMTAG_BUILDMACROS		= 1111, /*!< internal - unused */
 /*@=enummemuse@*/
-    RPMTAG_PROVIDEFLAGS		= 1112,	/* i */
+    RPMTAG_PROVIDEFLAGS		= 1112,	/* i[] */
     RPMTAG_PROVIDEVERSION	= 1113,	/* s[] */
-    RPMTAG_OBSOLETEFLAGS	= 1114,	/* i */
+    RPMTAG_OBSOLETEFLAGS	= 1114,	/* i[] */
     RPMTAG_OBSOLETEVERSION	= 1115,	/* s[] */
-    RPMTAG_DIRINDEXES		= 1116,	/* i */
+    RPMTAG_DIRINDEXES		= 1116,	/* i[] */
     RPMTAG_BASENAMES		= 1117,	/* s[] */
     RPMTAG_DIRNAMES		= 1118,	/* s[] */
-    RPMTAG_ORIGDIRINDEXES	= 1119, /*!< internal */
-    RPMTAG_ORIGBASENAMES	= 1120, /*!< internal */
-    RPMTAG_ORIGDIRNAMES		= 1121, /*!< internal */
+    RPMTAG_ORIGDIRINDEXES	= 1119, /*!< i[] relocation */
+    RPMTAG_ORIGBASENAMES	= 1120, /*!< s[] relocation */
+    RPMTAG_ORIGDIRNAMES		= 1121, /*!< s[] relocation */
     RPMTAG_OPTFLAGS		= 1122,	/* s */
     RPMTAG_DISTURL		= 1123,	/* s */
     RPMTAG_PAYLOADFORMAT	= 1124,	/* s */
@@ -422,12 +407,12 @@ typedef enum rpmTag_e {
     RPMTAG_CACHEPKGPATH		= 1137,	/* s */
     RPMTAG_CACHEPKGSIZE		= 1138,	/* i */
     RPMTAG_CACHEPKGMTIME	= 1139,	/* i */
-    RPMTAG_FILECOLORS		= 1140,	/* i */
-    RPMTAG_FILECLASS		= 1141,	/* i */
+    RPMTAG_FILECOLORS		= 1140,	/* i[] */
+    RPMTAG_FILECLASS		= 1141,	/* i[] */
     RPMTAG_CLASSDICT		= 1142,	/* s[] */
-    RPMTAG_FILEDEPENDSX		= 1143,	/* i */
-    RPMTAG_FILEDEPENDSN		= 1144,	/* i */
-    RPMTAG_DEPENDSDICT		= 1145,	/* i */
+    RPMTAG_FILEDEPENDSX		= 1143,	/* i[] */
+    RPMTAG_FILEDEPENDSN		= 1144,	/* i[] */
+    RPMTAG_DEPENDSDICT		= 1145,	/* i[] */
     RPMTAG_SOURCEPKGID		= 1146,	/* x */
     RPMTAG_FILECONTEXTS		= 1147,	/* s[] */
     RPMTAG_FSCONTEXTS		= 1148,	/*!< s[] extension */
@@ -439,15 +424,33 @@ typedef enum rpmTag_e {
     RPMTAG_POSTTRANSPROG	= 1154,	/* s */
     RPMTAG_DISTTAG		= 1155,	/* s */
     RPMTAG_SUGGESTSNAME		= 1156,	/* s[] extension placeholder */
+#define	RPMTAG_SUGGESTS RPMTAG_SUGGESTSNAME	/* s[] */
     RPMTAG_SUGGESTSVERSION	= 1157,	/* s[] extension placeholder */
-    RPMTAG_SUGGESTSFLAGS	= 1158,	/* i   extension placeholder */
+    RPMTAG_SUGGESTSFLAGS	= 1158,	/* i[]   extension placeholder */
     RPMTAG_ENHANCESNAME		= 1159,	/* s[] extension placeholder */
+#define	RPMTAG_ENHANCES RPMTAG_ENHANCESNAME	/* s[] */
     RPMTAG_ENHANCESVERSION	= 1160,	/* s[] extension placeholder */
-    RPMTAG_ENHANCESFLAGS	= 1161,	/* i   extension placeholder */
-    RPMTAG_PRIORITY		= 1162, /* i   extension placeholder */
+    RPMTAG_ENHANCESFLAGS	= 1161,	/* i[]   extension placeholder */
+    RPMTAG_PRIORITY		= 1162, /* i[]   extension placeholder */
     RPMTAG_CVSID		= 1163, /* s */
 #define	RPMTAG_SVNID	RPMTAG_CVSID	/* s */
+    RPMTAG_BLINKPKGID		= 1164, /* s[] */
+    RPMTAG_BLINKHDRID		= 1165, /* s[] */
+    RPMTAG_BLINKNEVRA		= 1166, /* s[] */
+    RPMTAG_FLINKPKGID		= 1167, /* s[] */
+    RPMTAG_FLINKHDRID		= 1168, /* s[] */
+    RPMTAG_FLINKNEVRA		= 1169, /* s[] */
+    RPMTAG_PACKAGEORIGIN	= 1170, /* s */
     RPMTAG_TRIGGERPREIN		= 1171, /*!< internal */
+    RPMTAG_BUILDSUGGESTS	= 1172, /*!< internal */
+    RPMTAG_BUILDENHANCES	= 1173, /*!< internal */
+    RPMTAG_SCRIPTSTATES		= 1174, /*!< i[] scriptlet exit codes */
+    RPMTAG_SCRIPTMETRICS	= 1175, /*!< i[] scriptlet execution times */
+    RPMTAG_BUILDCPUCLOCK	= 1176, /*!< i */
+    RPMTAG_FILEDIGESTALGOS	= 1177, /*!< i[] */
+    RPMTAG_VARIANTS		= 1178, /*!< s[] */
+    RPMTAG_XMAJOR		= 1179, /*!< i */
+    RPMTAG_XMINOR		= 1180, /*!< i */
 
 /*@-enummemuse@*/
     RPMTAG_FIRSTFREE_TAG	/*!< internal */
@@ -456,6 +459,40 @@ typedef enum rpmTag_e {
 
 #define	RPMTAG_EXTERNAL_TAG		1000000
 
+/**
+ * Scriptlet identifiers.
+ */
+typedef enum rpmScriptID_e {
+    RPMSCRIPT_UNKNOWN		=  0,	/*!< unknown scriptlet */
+    RPMSCRIPT_PRETRANS		=  1,	/*!< %pretrans scriptlet */
+    RPMSCRIPT_TRIGGERPREIN	=  2,	/*!< %triggerprein scriptlet */
+    RPMSCRIPT_PREIN		=  3,	/*!< %pre scriptlet */
+    RPMSCRIPT_POSTIN		=  4,	/*!< %post scriptlet  */
+    RPMSCRIPT_TRIGGERIN		=  5,	/*!< %triggerin scriptlet  */
+    RPMSCRIPT_TRIGGERUN		=  6,	/*!< %triggerun scriptlet  */
+    RPMSCRIPT_PREUN		=  7,	/*!< %preun scriptlet  */
+    RPMSCRIPT_POSTUN		=  8,	/*!< %postun scriptlet  */
+    RPMSCRIPT_TRIGGERPOSTUN	=  9,	/*!< %triggerpostun scriptlet  */
+    RPMSCRIPT_POSTTRANS		= 10,	/*!< %posttrans scriptlet  */
+	/* 11-15 unused */
+    RPMSCRIPT_VERIFY		= 16,	/*!< %verify scriptlet  */
+    RPMSCRIPT_MAX		= 32
+} rpmScriptID;
+
+/**
+ * Scriptlet states (when installed).
+ */
+typedef enum rpmScriptState_e {
+    RPMSCRIPT_STATE_UNKNOWN	= 0,
+	/* 0-15 reserved for waitpid return. */
+    RPMSCRIPT_STATE_EXEC	= (1 << 16), /*!< scriptlet was exec'd */
+    RPMSCRIPT_STATE_REAPED	= (1 << 17), /*!< scriptlet was reaped */
+	/* 18-23 unused */
+    RPMSCRIPT_STATE_SELINUX	= (1 << 24), /*!< scriptlet exec by SELinux */
+    RPMSCRIPT_STATE_EMULATOR	= (1 << 25), /*!< scriptlet exec in emulator */
+    RPMSCRIPT_STATE_LUA		= (1 << 26)  /*!< scriptlet exec with lua */
+} rpmScriptState;
+
 /**
  * File States (when installed).
  */
@@ -498,24 +535,23 @@ typedef	enum rpmfileAttrs_e {
 typedef	enum rpmsenseFlags_e {
     RPMSENSE_ANY	= 0,
 /*@-enummemuse@*/
-    RPMSENSE_SERIAL	= (1 << 0),	/*!< @todo Legacy. */
+    RPMSENSE_SERIAL	= (1 << 0),	/*!< (obsolete). */
 /*@=enummemuse@*/
     RPMSENSE_LESS	= (1 << 1),
     RPMSENSE_GREATER	= (1 << 2),
     RPMSENSE_EQUAL	= (1 << 3),
     RPMSENSE_PROVIDES	= (1 << 4), /* only used internally by builds */
     RPMSENSE_CONFLICTS	= (1 << 5), /* only used internally by builds */
-	/* bit 6 used to be RPMSENSE_PREREQ */
-#define	RPMSENSE_PREREQ	RPMSENSE_ANY
+    RPMSENSE_PREREQ	= (1 << 6),	/*!< (obsolete). */
     RPMSENSE_OBSOLETES	= (1 << 7), /* only used internally by builds */
     RPMSENSE_INTERP	= (1 << 8),	/*!< Interpreter used by scriptlet. */
-    RPMSENSE_SCRIPT_PRE	= ((1 << 9)|RPMSENSE_PREREQ), /*!< %pre dependency. */
-    RPMSENSE_SCRIPT_POST = ((1 << 10)|RPMSENSE_PREREQ), /*!< %post dependency. */
-    RPMSENSE_SCRIPT_PREUN = ((1 << 11)|RPMSENSE_PREREQ), /*!< %preun dependency. */
-    RPMSENSE_SCRIPT_POSTUN = ((1 << 12)|RPMSENSE_PREREQ), /*!< %postun dependency. */
+    RPMSENSE_SCRIPT_PRE	= (1 << 9),	/*!< %pre dependency. */
+    RPMSENSE_SCRIPT_POST = (1 << 10),	/*!< %post dependency. */
+    RPMSENSE_SCRIPT_PREUN = (1 << 11),	/*!< %preun dependency. */
+    RPMSENSE_SCRIPT_POSTUN = (1 << 12), /*!< %postun dependency. */
     RPMSENSE_SCRIPT_VERIFY = (1 << 13),	/*!< %verify dependency. */
-    RPMSENSE_FIND_REQUIRES = (1 << 14), /*!< find-requires generated dependency. */
-    RPMSENSE_FIND_PROVIDES = (1 << 15), /*!< find-provides generated dependency. */
+    RPMSENSE_FIND_REQUIRES = (1 << 14), /*!< find-requires dependency. */
+    RPMSENSE_FIND_PROVIDES = (1 << 15), /*!< find-provides dependency. */
 
     RPMSENSE_TRIGGERIN	= (1 << 16),	/*!< %triggerin dependency. */
     RPMSENSE_TRIGGERUN	= (1 << 17),	/*!< %triggerun dependency. */
@@ -525,16 +561,16 @@ typedef	enum rpmsenseFlags_e {
     RPMSENSE_SCRIPT_BUILD = (1 << 21),	/*!< %build build dependency. */
     RPMSENSE_SCRIPT_INSTALL = (1 << 22),/*!< %install build dependency. */
     RPMSENSE_SCRIPT_CLEAN = (1 << 23),	/*!< %clean build dependency. */
-    RPMSENSE_RPMLIB = ((1 << 24) | RPMSENSE_PREREQ), /*!< rpmlib(feature) dependency. */
-/*@-enummemuse@*/
-    RPMSENSE_TRIGGERPREIN = (1 << 25),	/*!< @todo Implement %triggerprein. */
-/*@=enummemuse@*/
+    RPMSENSE_RPMLIB = (1 << 24),	/*!< rpmlib(feature) dependency. */
+    RPMSENSE_TRIGGERPREIN = (1 << 25),	/*!< %triggerprein dependency. */
     RPMSENSE_KEYRING	= (1 << 26),
     RPMSENSE_PATCHES	= (1 << 27),
-    RPMSENSE_CONFIG	= (1 << 28)
+    RPMSENSE_CONFIG	= (1 << 28),
+    RPMSENSE_PROBE	= (1 << 29),
+    RPMSENSE_PACKAGE	= (1 << 30)
 } rpmsenseFlags;
 
-#define	RPMSENSE_SENSEMASK	15	 /* Mask to get senses, ie serial, */
+#define	RPMSENSE_SENSEMASK	0x0e	 /* Mask to get senses, ie serial, */
                                          /* less, greater, equal.          */
 
 #define	RPMSENSE_TRIGGER	\
@@ -548,12 +584,14 @@ typedef	enum rpmsenseFlags_e {
     RPMSENSE_SCRIPT_POSTUN | \
     RPMSENSE_SCRIPT_VERIFY | \
     RPMSENSE_FIND_REQUIRES | \
+    RPMSENSE_MISSINGOK | \
     RPMSENSE_SCRIPT_PREP | \
     RPMSENSE_SCRIPT_BUILD | \
     RPMSENSE_SCRIPT_INSTALL | \
     RPMSENSE_SCRIPT_CLEAN | \
     RPMSENSE_RPMLIB | \
-    RPMSENSE_KEYRING )
+    RPMSENSE_KEYRING | \
+    RPMSENSE_PACKAGE )
 
 #define	_notpre(_x)		((_x) & ~RPMSENSE_PREREQ)
 #define	_INSTALL_ONLY_MASK \
@@ -561,7 +599,6 @@ typedef	enum rpmsenseFlags_e {
 #define	_ERASE_ONLY_MASK  \
     _notpre(RPMSENSE_SCRIPT_PREUN|RPMSENSE_SCRIPT_POSTUN)
 
-#define	isLegacyPreReq(_x)  (((_x) & _ALL_REQUIRES_MASK) == RPMSENSE_PREREQ)
 #define	isInstallPreReq(_x)	((_x) & _INSTALL_ONLY_MASK)
 #define	isErasePreReq(_x)	((_x) & _ERASE_ONLY_MASK)
 
@@ -571,7 +608,6 @@ typedef	enum rpmsenseFlags_e {
 
 /* Stuff for maintaining "variables" like SOURCEDIR, BUILDDIR, etc */
 #define	RPMVAR_OPTFLAGS			3
-#define	RPMVAR_PROVIDES			38
 #define	RPMVAR_INCLUDE			43
 #define	RPMVAR_MACROFILES		49
 
@@ -583,7 +619,8 @@ typedef	enum rpmsenseFlags_e {
  * @todo Eliminate from API.
  */
 /*@-redecl@*/
-/*@observer@*/ /*@null@*/ extern const char * rpmGetVar(int var)
+/*@observer@*/ /*@null@*/
+extern const char * rpmGetVar(int var)
 	/*@*/;
 /*@=redecl@*/
 
@@ -813,12 +850,15 @@ typedef enum rpmprobFilterFlags_e {
 /**
  * We pass these around as an array with a sentinel.
  */
-typedef struct rpmRelocation_s {
+typedef struct rpmRelocation_s * rpmRelocation;
+#if !defined(SWIG)
+struct rpmRelocation_s {
 /*@only@*/ /*@null@*/
     const char * oldPath;	/*!< NULL here evals to RPMTAG_DEFAULTPREFIX, */
 /*@only@*/ /*@null@*/
     const char * newPath;	/*!< NULL means to omit the file completely! */
-} rpmRelocation;
+};
+#endif
 
 /**
  * Compare headers to determine which header is "newer".
@@ -877,7 +917,7 @@ typedef /*@abstract@*/ struct fsmIterator_s * FSMI_t;
  */
 typedef /*@abstract@*/ struct fsm_s * FSM_t;
 
-/** \ingroup rpmtrans
+/** \ingroup rpmts
  * Package state machine data.
  */
 typedef /*@abstract@*/ /*@refcounted@*/ struct rpmpsm_s * rpmpsm;
@@ -894,13 +934,6 @@ typedef /*@abstract@*/ /*@refcounted@*/ struct rpmpsm_s * rpmpsm;
 int headerVerifyInfo(int il, int dl, const void * pev, void * iv, int negate)
 	/*@modifies *iv @*/;
 
-/**
- * Check for supported payload format in header.
- * @param h		header to check
- * @return		RPMRC_OK if supported, RPMRC_FAIL otherwise
- */
-rpmRC headerCheckPayloadFormat(Header h);
-
 /** 
  * Check header consistency, performing headerGetEntry() the hard way.
  *  
@@ -963,7 +996,7 @@ rpmRC rpmInstallSourcePackage(rpmts ts, FD_t fd,
 	/*@modifies ts, fd, *specFilePtr, *cookie, rpmGlobalMacroContext,
 		fileSystem, internalState @*/;
 
-/** \ingroup rpmtrans
+/** \ingroup rpmts
  * Bit(s) to control rpmtsRun() operation.
  */
 typedef enum rpmtransFlags_e {
@@ -1006,7 +1039,7 @@ typedef enum rpmtransFlags_e {
     RPMTRANS_FLAG_APPLYONLY	= (1 << 25),
 
     RPMTRANS_FLAG_ANACONDA	= (1 << 26),	/*!< from --anaconda */
-    RPMTRANS_FLAG_NOMD5		= (1 << 27),	/*!< from --nomd5 */
+    RPMTRANS_FLAG_NOFDIGESTS	= (1 << 27),	/*!< from --nofdigests */
     RPMTRANS_FLAG_NOSUGGEST	= (1 << 28),	/*!< from --nosuggest */
     RPMTRANS_FLAG_ADDINDEPS	= (1 << 29),	/*!< from --aid */
     RPMTRANS_FLAG_NOCONFIGS	= (1 << 30),	/*!< from --noconfigs */
@@ -1027,20 +1060,7 @@ typedef enum rpmtransFlags_e {
     RPMTRANS_FLAG_NOTRIGGERPOSTUN \
   )
 
-/** \ingroup rpmtrans
- * Return copy of rpmlib internal provides.
- * @retval provNames	address of array of rpmlib internal provide names
- * @retval provFlags	address of array of rpmlib internal provide flags
- * @retval provVersions	address of array of rpmlib internal provide versions
- * @return		no. of entries
- */
-/*@unused@*/
-int rpmGetRpmlibProvides(/*@null@*/ /*@out@*/ const char *** provNames,
-			/*@null@*/ /*@out@*/ int ** provFlags,
-			/*@null@*/ /*@out@*/ const char *** provVersions)
-	/*@modifies *provNames, *provFlags, *provVersions @*/;
-
-/** \ingroup rpmtrans
+/** \ingroup rpmts
  * Segmented string compare for version and/or release.
  *
  * @param a		1st string
@@ -1050,49 +1070,53 @@ int rpmGetRpmlibProvides(/*@null@*/ /*@out@*/ const char *** provNames,
 int rpmvercmp(const char * a, const char * b)
 	/*@*/;
 
-/** \ingroup rpmtrans
- * Check dependency against internal rpmlib feature provides.
- * @param key		dependency
- * @return		1 if dependency overlaps, 0 otherwise
- */
-int rpmCheckRpmlibProvides(const rpmds key)
-	/*@*/;
-
-/** \ingroup rpmcli
- * Display current rpmlib feature provides.
- * @param fp		output file handle
- */
-void rpmShowRpmlibProvides(FILE * fp)
-	/*@globals fileSystem @*/
-	/*@modifies *fp, fileSystem @*/;
-
 /*@}*/
 
-/**
- * Return  tag data type from value.
- * @param tag		tag value
- * @return		tag data type, RPM_NULL_TYPE on not found.
- */
-int tagType(int tag)
-	/*@*/;
-
+#if !defined(SWIG)
 /**
  * Return tag name from value.
  * @param tag		tag value
  * @return		tag name, "(unknown)" on not found
  */
 /*@-redecl@*/
-/*@observer@*/ extern const char *const tagName(int tag)
-	/*@*/;
+/*@unused@*/ static inline /*@observer@*/
+const char * tagName(int tag)
+	/*@*/
+{
+/*@-type@*/
+    return ((*rpmTags->tagName)(tag));
+/*@=type@*/
+}
 /*@=redecl@*/
 
+/**
+ * Return tag data type from value.
+ * @param tag		tag value
+ * @return		tag data type, RPM_NULL_TYPE on not found.
+ */
+/*@unused@*/ static inline
+int tagType(int tag)
+	/*@*/
+{
+/*@-type@*/
+    return ((*rpmTags->tagType)(tag));
+/*@=type@*/
+}
+
 /**
  * Return tag value from name.
  * @param tagstr	name of tag
  * @return		tag value, -1 on not found
  */
+/*@unused@*/ static inline
 int tagValue(const char * tagstr)
-	/*@*/;
+	/*@*/
+{
+/*@-type@*/
+    return ((*rpmTags->tagValue)(tagstr));
+/*@=type@*/
+}
+#endif
 
 #define	RPMLEAD_BINARY 0
 #define	RPMLEAD_SOURCE 1
@@ -1122,50 +1146,6 @@ struct rpmlead {
 /*@unused@*/ char reserved[16];	/*!< Pad to 96 bytes -- 8 byte aligned! */
 } ;
 
-/**
- * Release storage used by file system usage cache.
- */
-void freeFilesystems(void)
-	/*@globals internalState@*/
-	/*@modifies internalState@*/;
-
-/**
- * Return (cached) file system mount points.
- * @retval listptr		addess of file system names (or NULL)
- * @retval num			address of number of file systems (or NULL)
- * @return			0 on success, 1 on error
- */
-/*@-incondefs@*/
-int rpmGetFilesystemList( /*@null@*/ /*@out@*/ const char *** listptr,
-		/*@null@*/ /*@out@*/ int * num)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies *listptr, *num, fileSystem, internalState @*/
-	/*@requires maxSet(listptr) >= 0 /\ maxSet(num) >= 0 @*/
-	/*@ensures maxRead(num) == 0 @*/;
-/*@=incondefs@*/
-
-/**
- * Determine per-file system usage for a list of files.
- * @param fileList		array of absolute file names
- * @param fssizes		array of file sizes
- * @param numFiles		number of files in list
- * @retval usagesPtr		address of per-file system usage array (or NULL)
- * @param flags			(unused)
- * @return			0 on success, 1 on error
- */
-/*@-incondefs@*/
-int rpmGetFilesystemUsage(const char ** fileList, int_32 * fssizes,
-		int numFiles, /*@null@*/ /*@out@*/ uint_32 ** usagesPtr,
-		int flags)
-	/*@globals rpmGlobalMacroContext, h_errno,
-		fileSystem, internalState @*/
-	/*@modifies *usagesPtr, rpmGlobalMacroContext,
-		fileSystem, internalState @*/
-	/*@requires maxSet(fileList) >= 0 /\ maxSet(fssizes) == 0
-		/\ maxSet(usagesPtr) >= 0 @*/
-	/*@ensures maxRead(usagesPtr) == 0 @*/;
-/*@=incondefs@*/
-
 /* ==================================================================== */
 /** \name RPMEIU */
 /*@{*/
@@ -1242,11 +1222,11 @@ rpmRC rpmVerifySignature(const rpmts ts,
 
 /** \ingroup signature
  * Destroy signature header from package.
- * @param h		signature header
+ * @param sigh		signature header
  * @return		NULL always
  */
-/*@null@*/ Header rpmFreeSignature(/*@null@*/ /*@killref@*/ Header h)
-	/*@modifies h @*/;
+/*@null@*/ Header rpmFreeSignature(/*@null@*/ /*@killref@*/ Header sigh)
+	/*@modifies sigh @*/;
 
 /*@}*/
 
