@@ -17,6 +17,7 @@
 #include <rpmlib.h>
 #include <rpmmacro.h>
 #include <rpmlua.h>
+#include <rpmds.h>
 
 #include "misc.h"
 #include "debug.h"
@@ -119,7 +120,6 @@ static struct rpmOption optionTable[] = {
     { "include",		RPMVAR_INCLUDE,			0, 1,	0, 2 },
     { "macrofiles",		RPMVAR_MACROFILES,		0, 0,	0, 1 },
     { "optflags",		RPMVAR_OPTFLAGS,		1, 0,	1, 0 },
-    { "provides",               RPMVAR_PROVIDES,                0, 0,	0, 0 },
 };
 /*@=fullinitblock@*/
 
@@ -525,13 +525,13 @@ static void setDefaults(void)
     addMacro(NULL, "_preScriptEnvironment",NULL, prescriptenviron,RMIL_DEFAULT);
 
     setVarDefault(-1,			"_topdir",
-		"/usr/src/redhat",	"%{_usr}/src/redhat");
+		"/usr/src/rpm",		"%{_usr}/src/rpm");
     setVarDefault(-1,			"_tmppath",
 		"/var/tmp",		"%{_var}/tmp");
     setVarDefault(-1,			"_dbpath",
 		"/var/lib/rpm",		"%{_var}/lib/rpm");
     setVarDefault(-1,			"_defaultdocdir",
-		"/usr/doc",		"%{_usr}/doc");
+		"/usr/share/doc",	"%{_usr}/share/doc");
 
     setVarDefault(-1,			"_rpmfilename",
 	"%%{ARCH}/%%{NAME}-%%{VERSION}-%%{RELEASE}.%%{ARCH}.rpm",NULL);
@@ -674,17 +674,6 @@ static int doReadRC( /*@killref@*/ FD_t fd, const char * urlfn)
 		}
 		se = (char *)fn;
 		/*@switchbreak@*/ break;
-	    case RPMVAR_PROVIDES:
-	      {	char *t;
-		s = rpmGetVar(RPMVAR_PROVIDES);
-		if (s == NULL) s = "";
-		fn = t = xmalloc(strlen(s) + strlen(se) + 2);
-		while (*s != '\0') *t++ = *s++;
-		*t++ = ' ';
-		while (*se != '\0') *t++ = *se++;
-		*t++ = '\0';
-		se = (char *)fn;
-	      }	/*@switchbreak@*/ break;
 	    default:
 		/*@switchbreak@*/ break;
 	    }
@@ -887,7 +876,7 @@ exit:
 /*
  * Generic CPUID function
  */
-static inline void cpuid(unsigned int op, int *eax, int *ebx, int *ecx, int *edx)
+static inline void cpuid(unsigned int op, unsigned int *eax, unsigned int *ebx, unsigned int *ecx, unsigned int *edx)
 	/*@modifies *eax, *ebx, *ecx, *edx @*/
 {
 #ifdef	__LCLINT__
@@ -953,28 +942,20 @@ static inline int RPMClass(void)
 {
 	int cpu;
 	unsigned int tfms, junk, cap, capamd;
-	struct sigaction oldsa;
 	
-	sigaction(SIGILL, NULL, &oldsa);
 	signal(SIGILL, model3);
 	
-	if (sigsetjmp(jenv, 1)) {
-		sigaction(SIGILL, &oldsa, NULL);
+	if (sigsetjmp(jenv, 1))
 		return 3;
-	}
 		
-	if (cpuid_eax(0x000000000)==0) {
-		sigaction(SIGILL, &oldsa, NULL);
+	if (cpuid_eax(0x000000000)==0)
 		return 4;
-	}
 
 	cpuid(0x00000001, &tfms, &junk, &junk, &cap);
 	cpuid(0x80000001, &junk, &junk, &junk, &capamd);
 	
 	cpu = (tfms>>8)&15;
 	
-	sigaction(SIGILL, &oldsa, NULL);
-
 	if (cpu < 6)
 		return cpu;
 		
@@ -1015,7 +996,7 @@ static int is_athlon(void)
 	return 1;
 }
 
-static int is_pentium3()
+static int is_pentium3(void)
 {
     unsigned int eax, ebx, ecx, edx, family, model;
     char vendor[16];
@@ -1054,7 +1035,7 @@ static int is_pentium3()
     return 0;
 }
 
-static int is_pentium4()
+static int is_pentium4(void)
 {
     unsigned int eax, ebx, ecx, edx, family, model;
     char vendor[16];
@@ -1334,6 +1315,40 @@ static void defaultMachine(/*@out@*/ const char ** arch,
 	}
 #	endif
 
+#	if defined(__linux__) && defined(__powerpc__)
+	{
+	    unsigned pvr = 0;
+	    __sighandler_t oldh = signal(SIGILL, mfspr_ill);
+	    if (setjmp(mfspr_jmpbuf) == 0) {
+		__asm__ __volatile__ ("mfspr %0, 287" : "=r" (pvr));
+	    }
+	    signal(SIGILL, oldh);
+
+	    if ( pvr ) {
+		pvr >>= 16;
+		switch (pvr) {
+		/* IBM750FX, 7410, 7450,  7451, 7441, 7455, 7445 */ 
+		case 0x7000:
+		case 0x8000:
+		case 0x8001:
+		case 0x800c:
+		    strcpy(un.machine, "ppc"); 
+		    break;
+		case 0x36:
+		case 0x37:
+		    strcpy(un.machine, "ppciseries");
+		    break;
+		default:
+		    if ( pvr >= 0x40)
+			strcpy(un.machine, "ppcpseries");
+		    else
+			strcpy(un.machine, "ppc");
+		    break;
+		}
+	    }
+	}
+#	endif
+
 	/* the uname() result goes through the arch_canon table */
 	canon = lookupInCanonTable(un.machine,
 				   tables[RPM_MACHTABLE_INSTARCH].canons,
@@ -1440,8 +1455,7 @@ static void rpmSetVarArch(int var, const char * val, const char * arch)
 	}
     }
 
-    next->value = _free(next->value);
-    next->value = xstrdup(val);
+    next->value = xstrdup(val);		/* XXX memory leak, hard to plug */
     next->arch = (arch ? xstrdup(arch) : NULL);
 }
 
@@ -1558,11 +1572,6 @@ static void getMachineInfo(int type, /*@null@*/ /*@out@*/ const char ** name,
     } else {
 	if (num) *num = 255;
 	if (name) *name = current[type];
-
-	if (tables[currTables[type]].hasCanon) {
-	    rpmMessage(RPMMESS_WARNING, _("Unknown system: %s\n"), current[type]);
-	    rpmMessage(RPMMESS_WARNING, _("Please contact rpm-maint@lists.rpm.org\n"));
-	}
     }
 }
 
@@ -1880,9 +1889,11 @@ int rpmReadConfigFiles(const char * file, const char * target)
 
 int rpmShowRC(FILE * fp)
 {
+    rpmds ds = NULL;
     struct rpmOption *opt;
     int i;
     machEquivTable equivTable;
+    int xx;
 
     /* the caller may set the build arch which should be printed here */
     fprintf(fp, "ARCHITECTURE AND OS:\n");
@@ -1928,10 +1939,76 @@ int rpmShowRC(FILE * fp)
     }
     fprintf(fp, "\n");
 
-    fprintf(fp, "Features supported by rpmlib:\n");
-    rpmShowRpmlibProvides(fp);
+    if (rpmIsVerbose()) {
+	xx = rpmdsSysinfo(&ds, NULL);
+	if (ds != NULL) {
+	    fprintf(fp, _("Configured system provides (from /etc/rpm/sysinfo):\n"));
+	    ds = rpmdsInit(ds);
+	    while (rpmdsNext(ds) >= 0) {
+		const char * DNEVR = rpmdsDNEVR(ds);
+		if (DNEVR != NULL)
+		    fprintf(fp, "    %s\n", DNEVR+2);
+	    }
+	    ds = rpmdsFree(ds);
+	    fprintf(fp, "\n");
+	}
+    }
+
+    fprintf(fp, _("Features provided by rpmlib installer:\n"));
+    xx = rpmdsRpmlib(&ds, NULL);
+    ds = rpmdsInit(ds);
+    while (rpmdsNext(ds) >= 0) {
+	const char * DNEVR = rpmdsDNEVR(ds);
+	if (DNEVR != NULL)
+	    fprintf(fp, "    %s\n", DNEVR+2);
+    }
+    ds = rpmdsFree(ds);
     fprintf(fp, "\n");
 
+    if (rpmIsVerbose()) {
+	xx = rpmdsCpuinfo(&ds, NULL);
+	if (ds != NULL) {
+	    fprintf(fp,
+		_("Features provided by current cpuinfo (from /proc/cpuinfo):\n"));
+	    ds = rpmdsInit(ds);
+	    while (rpmdsNext(ds) >= 0) {
+		const char * DNEVR = rpmdsDNEVR(ds);
+		if (DNEVR != NULL)
+		    fprintf(fp, "    %s\n", DNEVR+2);
+	    }
+	    ds = rpmdsFree(ds);
+	    fprintf(fp, "\n");
+	}
+
+	xx = rpmdsGetconf(&ds, NULL);
+	if (ds != NULL) {
+	    fprintf(fp,
+		_("Features provided by current getconf:\n"));
+	    ds = rpmdsInit(ds);
+	    while (rpmdsNext(ds) >= 0) {
+		const char * DNEVR = rpmdsDNEVR(ds);
+		if (DNEVR != NULL)
+		    fprintf(fp, "    %s\n", DNEVR+2);
+	    }
+	    ds = rpmdsFree(ds);
+	    fprintf(fp, "\n");
+	}
+
+	xx = rpmdsUname(&ds, NULL);
+	if (ds != NULL) {
+	    fprintf(fp,
+		_("Features provided by current uname:\n"));
+	    ds = rpmdsInit(ds);
+	    while (rpmdsNext(ds) >= 0) {
+		const char * DNEVR = rpmdsDNEVR(ds);
+		if (DNEVR != NULL)
+		    fprintf(fp, "    %s\n", DNEVR+2);
+	    }
+	    ds = rpmdsFree(ds);
+	    fprintf(fp, "\n");
+	}
+    }
+
     rpmDumpMacroTable(NULL, fp);
 
     return 0;
