@@ -5,6 +5,7 @@
  * \file lib/rpmte.h
  * Structures used for an "rpmte" transaction element.
  */
+#include <argv.h>
 
 /**
  */
@@ -45,14 +46,28 @@ struct tsortInfo_s {
 #define	tsi_count	tsi_u.count
 #define	tsi_suc		tsi_u.suc
 /*@owned@*/ /*@null@*/
-    struct tsortInfo_s * tsi_next;
+    tsortInfo	tsi_next;
 /*@exposed@*/ /*@dependent@*/ /*@null@*/
-    rpmte tsi_chain;
+    rpmte	tsi_chain;
+    int		tsi_tagn;
     int		tsi_reqx;
+    int		tsi_queued;
     int		tsi_qcnt;
 };
 /*@=fielduse@*/
 
+/** \ingroup rpmte
+ * Info used to link transaction element upgrade/rollback side effects.
+ */
+struct rpmChainLink_s {
+/*@only@*/ /*@null@*/
+    ARGV_t Pkgid;		/*!< link element pkgid's. */
+/*@only@*/ /*@null@*/
+    ARGV_t Hdrid;		/*!< link element hdrid's. */
+/*@only@*/ /*@null@*/
+    ARGV_t NEVRA;		/*!< link element NEVRA's. */
+};
+
 /** \ingroup rpmte
  * A single package instance to be installed/removed atomically.
  */
@@ -65,6 +80,10 @@ struct rpmte_s {
     const char * NEVR;		/*!< Package name-version-release. */
 /*@only@*/
     const char * NEVRA;		/*!< Package name-version-release.arch. */
+/*@only@*/ /*@relnull@*/
+    const char * pkgid;		/*!< Package identifier (header+payload md5). */
+/*@only@*/ /*@relnull@*/
+    const char * hdrid;		/*!< Package header identifier (header sha1). */
 /*@owned@*/
     const char * name;		/*!< Name: */
 /*@only@*/ /*@null@*/
@@ -79,12 +98,15 @@ struct rpmte_s {
     const char * os;		/*!< Operating system hint. */
     int archScore;		/*!< (TR_ADDED) Arch score. */
     int osScore;		/*!< (TR_ADDED) Os score. */
+    int isSource;		/*!< (TR_ADDED) source rpm? */
 
     rpmte parent;		/*!< Parent transaction element. */
     int degree;			/*!< No. of immediate children. */
-    int depth;			/*!< Max. depth in dependency tree. */
     int npreds;			/*!< No. of predecessors. */
     int tree;			/*!< Tree index. */
+    int depth;			/*!< Depth in dependency tree. */
+    int breadth;		/*!< Breadth in dependency tree. */
+    unsigned int db_instance;   /*!< Database Instance after add */
 /*@owned@*/
     tsortInfo tsi;		/*!< Dependency ordering chains. */
 
@@ -98,6 +120,10 @@ struct rpmte_s {
     rpmds conflicts;		/*!< Conflicts: dependencies. */
 /*@refcounted@*/ /*@null@*/
     rpmds obsoletes;		/*!< Obsoletes: dependencies. */
+/*@refcounted@*/ /*@null@*/
+    rpmds triggers;		/*!< Triggers: dependencies. */
+/*@refcounted@*/ /*@null@*/
+    rpmds dirnames;		/*!< Dirnames: dependencies. */
 /*@refcounted@*/ /*@null@*/
     rpmfi fi;			/*!< File information. */
 
@@ -107,14 +133,21 @@ struct rpmte_s {
 /*@exposed@*/ /*@dependent@*/ /*@null@*/
     fnpyKey key;		/*!< (TR_ADDED) Retrieval key. */
 /*@owned@*/ /*@null@*/
-    rpmRelocation * relocs;	/*!< (TR_ADDED) Payload file relocations. */
+    rpmRelocation relocs;	/*!< (TR_ADDED) Payload file relocations. */
     int nrelocs;		/*!< (TR_ADDED) No. of relocations. */
     int autorelocatex;		/*!< (TR_ADDED) Auto relocation entry index. */
 /*@refcounted@*/ /*@null@*/	
     FD_t fd;			/*!< (TR_ADDED) Payload file descriptor. */
 
-/*@-fielduse@*/	/* LCL: confused by union? */
-    union { 
+    struct rpmChainLink_s blink;/*!< Backward link info to erased element. */
+    struct rpmChainLink_s flink;/*!< Forward link info to installed element. */
+    int linkFailed;		/*!< Did the linked element upgrade succeed? */
+    int done;			/*!< Has the element been installed/erased? */
+
+    int installed;		/*!< Was the header installed? */
+    int downgrade;		/*!< Adjust package count on downgrades. */
+
+    struct {
 /*@exposed@*/ /*@dependent@*/ /*@null@*/
 	alKey addedKey;
 	struct {
@@ -123,7 +156,6 @@ struct rpmte_s {
 	    int dboffset;
 	} removed;
     } u;
-/*@=fielduse@*/
 
 };
 
@@ -168,11 +200,28 @@ rpmte rpmteFree(/*@only@*/ /*@null@*/ rpmte te)
 /*@only@*/ /*@null@*/
 rpmte rpmteNew(const rpmts ts, Header h, rpmElementType type,
 		/*@exposed@*/ /*@dependent@*/ /*@null@*/ fnpyKey key,
-		/*@null@*/ rpmRelocation * relocs,
+		/*@null@*/ rpmRelocation relocs,
 		int dboffset,
 		/*@exposed@*/ /*@dependent@*/ /*@null@*/ alKey pkgKey)
-	/*@globals rpmGlobalMacroContext, h_errno, fileSystem @*/
-	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem @*/;
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/;
+
+/**
+ * Retrieve header from transaction element.
+ * @param te		transaction element
+ * @return		header
+ */
+extern Header rpmteHeader(rpmte te)
+	/*@modifies te @*/;
+
+/**
+ * Save header into transaction element.
+ * @param te		transaction element
+ * @param h		header
+ * @return		NULL always
+ */
+extern Header rpmteSetHeader(rpmte te, Header h)
+	/*@modifies te, h @*/;
 
 /**
  * Retrieve type of transaction element.
@@ -236,6 +285,14 @@ extern const char * rpmteA(rpmte te)
 extern const char * rpmteO(rpmte te)
 	/*@*/;
 
+/**
+ * Retrieve isSource attribute of transaction element.
+ * @param te		transaction element
+ * @return		isSource attribute
+ */
+extern int rpmteIsSource(rpmte te)
+	/*@*/;
+
 /**
  * Retrieve color bits of transaction element.
  * @param te		transaction element
@@ -253,6 +310,23 @@ uint_32 rpmteColor(rpmte te)
 uint_32 rpmteSetColor(rpmte te, uint_32 color)
 	/*@modifies te @*/;
 
+/**
+ * Retrieve last instance installed to the database.
+ * @param te		transaction element
+ * @return		last install instance.
+ */
+unsigned int rpmteDBInstance(rpmte te)
+	/*@*/;
+
+/**
+ * Set last instance installed to the database.
+ * @param te		transaction element
+ * @param instance	Database instance of last install element.
+ * @return		last install instance.
+ */
+void rpmteSetDBInstance(rpmte te, unsigned int instance)
+	/*@modifies te @*/;
+
 /**
  * Retrieve size in bytes of package file.
  * @todo Signature header is estimated at 256b.
@@ -263,7 +337,7 @@ uint_32 rpmtePkgFileSize(rpmte te)
 	/*@*/;
 
 /**
- * Retrieve tsort tree depth of transaction element.
+ * Retrieve dependency tree depth of transaction element.
  * @param te		transaction element
  * @return		depth
  */
@@ -271,7 +345,7 @@ int rpmteDepth(rpmte te)
 	/*@*/;
 
 /**
- * Set tsort tree depth of transaction element.
+ * Set dependency tree depth of transaction element.
  * @param te		transaction element
  * @param ndepth	new depth
  * @return		previous depth
@@ -279,6 +353,23 @@ int rpmteDepth(rpmte te)
 int rpmteSetDepth(rpmte te, int ndepth)
 	/*@modifies te @*/;
 
+/**
+ * Retrieve dependency tree breadth of transaction element.
+ * @param te		transaction element
+ * @return		breadth
+ */
+int rpmteBreadth(rpmte te)
+	/*@*/;
+
+/**
+ * Set dependency tree breadth of transaction element.
+ * @param te		transaction element
+ * @param nbreadth	new breadth
+ * @return		previous breadth
+ */
+int rpmteSetBreadth(rpmte te, int nbreadth)
+	/*@modifies te @*/;
+
 /**
  * Retrieve tsort no. of predecessors of transaction element.
  * @param te		transaction element
@@ -399,15 +490,6 @@ alKey rpmteSetAddedKey(rpmte te,
 		/*@exposed@*/ /*@dependent@*/ /*@null@*/ alKey npkgKey)
 	/*@modifies te @*/;
 
-/**
- * Retrieve dependent pkgKey of TR_REMOVED transaction element.
- * @param te		transaction element
- * @return		dependent pkgKey
- */
-/*@exposed@*/ /*@dependent@*/ /*@null@*/
-alKey rpmteDependsOnKey(rpmte te)
-	/*@*/;
-
 /**
  * Retrieve rpmdb instance of TR_REMOVED transaction element.
  * @param te		transaction element
@@ -436,6 +518,28 @@ extern const char * rpmteNEVRA(rpmte te)
 	/*@*/;
 /*@=exportlocal@*/
 
+/**
+ * Retrieve pkgid string from transaction element.
+ * @param te		transaction element
+ * @return		pkgid string
+ */
+/*@-exportlocal@*/
+/*@observer@*/ /*@null@*/
+extern const char * rpmtePkgid(rpmte te)
+	/*@*/;
+/*@=exportlocal@*/
+
+/**
+ * Retrieve hdrid string from transaction element.
+ * @param te		transaction element
+ * @return		hdrid string
+ */
+/*@-exportlocal@*/
+/*@observer@*/ /*@null@*/
+extern const char * rpmteHdrid(rpmte te)
+	/*@*/;
+/*@=exportlocal@*/
+
 /**
  * Retrieve file handle from transaction element.
  * @param te		transaction element
@@ -481,6 +585,17 @@ void rpmteColorDS(rpmte te, rpmTag tag)
         /*@modifies te @*/;
 /*@=exportlocal@*/
 
+/**
+ * Chain p <-> q forward/backward transaction element links.
+ * @param p		installed element (needs backward link)
+ * @param q		erased element (needs forward link)
+ * @param oh		erased element header
+ * @param msg		operation identifier for debugging (NULL uses "")
+ * @return		0 on success
+ */
+int rpmteChain(rpmte p, rpmte q, Header oh, /*@null@*/ const char * msg)
+	/*@modifies p, q, oh @*/;
+
 /**
  * Return transaction element index.
  * @param tsi		transaction element iterator
@@ -545,6 +660,59 @@ rpmtsi XrpmtsiInit(rpmts ts,
 rpmte rpmtsiNext(rpmtsi tsi, rpmElementType type)
         /*@modifies tsi @*/;
 
+#if !defined(SWIG)
+#if	defined(_RPMTE_INTERNAL)
+static inline void rpmtePrintID(rpmte p)
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/
+{
+    if (p != NULL) {
+	if (p->blink.Pkgid) argvPrint("blink.Pkgid", p->blink.Pkgid, NULL);
+	if (p->blink.Hdrid) argvPrint("blink.Hdrid", p->blink.Hdrid, NULL);
+	if (p->blink.NEVRA) argvPrint("blink.NEVRA", p->blink.NEVRA, NULL);
+	if (p->flink.Pkgid) argvPrint("flink.Pkgid", p->flink.Pkgid, NULL);
+	if (p->flink.Hdrid) argvPrint("flink.Hdrid", p->flink.Hdrid, NULL);
+	if (p->flink.NEVRA) argvPrint("flink.NEVRA", p->flink.NEVRA, NULL);
+    }
+};
+#endif
+
+static inline void hdrPrintInstalled(Header h)
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/
+{
+    const char * qfmt = "[%{erasednevra} O:%{packageorigin} P:%{erasedpkgid} H:%{erasedhdrid}\n]";
+    const char * errstr = "(unknown error)";
+/*@-modobserver@*/
+    const char * str = headerSprintf(h, qfmt, rpmTagTable, rpmHeaderFormats, &errstr);
+/*@=modobserver@*/
+
+    if (str == NULL)
+	fprintf(stderr, "error: %s\n", errstr);
+    else {
+	fprintf(stderr, "%s", str);
+	str = _free(str);
+    }
+}
+
+static inline void hdrPrintErased(Header h)
+	/*@globals fileSystem @*/
+	/*@modifies fileSystem @*/
+{
+    const char * qfmt = "[%{installednevra} O:%{packageorigin} P:%{installedpkgid} H:%{installedhdrid}\n]";
+    const char * errstr = "(unknown error)";
+/*@-modobserver@*/
+    const char * str = headerSprintf(h, qfmt, rpmTagTable, rpmHeaderFormats, &errstr);
+/*@=modobserver@*/
+    if (str == NULL)
+	fprintf(stderr, "error: %s\n", errstr);
+    else {
+	fprintf(stderr, "%s", str);
+	str = _free(str);
+    }
+}
+#endif
+
 #ifdef __cplusplus
 }
 #endif
