@@ -8,6 +8,7 @@
 #include <rpmlib.h>
 #include <rpmmacro.h>		/* XXX rpmtsOpenDB() needs rpmGetPath */
 
+#define	_RPMDB_INTERNAL		/* XXX almost opaque sigh */
 #include "rpmdb.h"		/* XXX stealing db->db_mode. */
 
 #include "rpmal.h"
@@ -21,6 +22,8 @@
 #define	_RPMTS_INTERNAL
 #include "rpmts.h"
 
+#include "fs.h"
+
 /* XXX FIXME: merge with existing (broken?) tests in system.h */
 /* portability fiddles */
 #if STATFS_IN_SYS_STATVFS
@@ -91,10 +94,14 @@ char * hGetNEVRA(Header h, const char ** np)
 {
     const char * n, * v, * r, * a;
     char * NVRA, * t;
-    int xx;
 
     (void) headerNVR(h, &n, &v, &r);
-    xx = headerGetEntry(h, RPMTAG_ARCH, NULL, (void **) &a, NULL);
+    /* XXX pubkeys have no arch. */
+/*@-branchstate@*/
+    a = NULL;
+    if (!headerGetEntry(h, RPMTAG_ARCH, NULL, (void **) &a, NULL) || a == NULL)
+	a = "pubkey";
+/*@=branchstate@*/
     NVRA = t = xcalloc(1, strlen(n) + strlen(v) + strlen(r) + strlen(a) + sizeof("--."));
 /*@-boundswrite@*/
     t = stpcpy(t, n);
@@ -193,23 +200,20 @@ int rpmtsOpenDB(rpmts ts, int dbmode)
 int rpmtsInitDB(rpmts ts, int dbmode)
 {
     void *lock = rpmtsAcquireLock(ts);
-    int rc = -1;
-    if (lock)
-	    rc = rpmdbInit(ts->rootDir, dbmode);
-    rpmtsFreeLock(lock);
+    int rc = rpmdbInit(ts->rootDir, dbmode);
+    lock = rpmtsFreeLock(lock);
     return rc;
 }
 
 int rpmtsRebuildDB(rpmts ts)
 {
-    int rc;
     void *lock = rpmtsAcquireLock(ts);
-    if (!lock) return -1;
+    int rc;
     if (!(ts->vsflags & RPMVSF_NOHDRCHK))
 	rc = rpmdbRebuild(ts->rootDir, ts, headerCheck);
     else
 	rc = rpmdbRebuild(ts->rootDir, NULL, NULL);
-    rpmtsFreeLock(lock);
+    lock = rpmtsFreeLock(lock);
     return rc;
 }
 
@@ -228,15 +232,16 @@ static int isArch(const char * arch)
     static const char *arches[] = {
 	"i386", "i486", "i586", "i686", "athlon", "pentium3", "pentium4", "x86_64", "amd64", "ia32e",
 	"alpha", "alphaev5", "alphaev56", "alphapca56", "alphaev6", "alphaev67",
-	"sparc", "sun4", "sun4m", "sun4c", "sun4d", "sparcv8", "sparcv9", "sparcv9v",
-	"sparc64", "sparc64v", "sun4u",
+	"sparc", "sun4", "sun4m", "sun4c", "sun4d", "sparcv8", "sparcv9",
+	"sparc64", "sun4u",
 	"mips", "mipsel", "IP",
 	"ppc", "ppciseries", "ppcpseries",
 	"ppc64", "ppc64iseries", "ppc64pseries",
 	"m68k",
 	"rs6000",
 	"ia64",
-	"armv3l", "armv4b", "armv4l", "armv4tl", "armv5tel", "armv5tejl", "armv6l",
+	"armv3l", "armv4b", "armv4l",
+	"armv5teb", "armv5tel",
 	"s390", "i370", "s390x",
 	"sh", "xtensa",
 	"noarch",
@@ -521,6 +526,8 @@ int rpmtsOpenSDB(rpmts ts, int dbmode)
 	rpmMessage(RPMMESS_WARNING,
 			_("cannot open Solve database in %s\n"), dn);
 	dn = _free(dn);
+	/* XXX only try to open the solvedb once. */
+	has_sdbpath = 0;
     }
     delMacro(NULL, "_dbpath");
 
@@ -773,6 +780,9 @@ void rpmtsClean(rpmts ts)
     ts->addedPackages = rpmalFree(ts->addedPackages);
     ts->numAddedPackages = 0;
 
+    ts->erasedPackages = rpmalFree(ts->erasedPackages);
+    ts->numErasedPackages = 0;
+
     ts->suggests = _free(ts->suggests);
     ts->nsuggests = 0;
 
@@ -816,7 +826,7 @@ static void rpmtsPrintStat(const char * name, /*@null@*/ struct rpmop_s * op)
 {
     static unsigned int scale = (1000 * 1000);
     if (op != NULL && op->count > 0)
-	fprintf(stderr, "   %s %6d %6lu.%06lu MB %6lu.%06lu secs\n",
+	fprintf(stderr, "   %s %8d %6lu.%06lu MB %6lu.%06lu secs\n",
 		name, op->count,
 		(unsigned long)op->bytes/scale, (unsigned long)op->bytes%scale,
 		op->usecs/scale, op->usecs%scale);
@@ -845,6 +855,7 @@ static void rpmtsPrintStats(rpmts ts)
     rpmtsPrintStat("dbget:       ", rpmtsOp(ts, RPMTS_OP_DBGET));
     rpmtsPrintStat("dbput:       ", rpmtsOp(ts, RPMTS_OP_DBPUT));
     rpmtsPrintStat("dbdel:       ", rpmtsOp(ts, RPMTS_OP_DBDEL));
+    rpmtsPrintStat("readhdr:     ", rpmtsOp(ts, RPMTS_OP_READHDR));
 }
 
 rpmts rpmtsFree(rpmts ts)
@@ -892,11 +903,6 @@ rpmts rpmtsFree(rpmts ts)
     if (_rpmts_stats)
 	rpmtsPrintStats(ts);
 
-    /* Free up the memory used by the rpmtsScore */
-/*@-kepttrans -onlytrans @*/
-    ts->score = rpmtsScoreFree(ts->score);
-/*@=kepttrans =onlytrans @*/
-
     (void) rpmtsUnlink(ts, "tsCreate");
 
     /*@-refcounttrans -usereleased @*/
@@ -924,34 +930,41 @@ rpmVSFlags rpmtsSetVSFlags(rpmts ts, rpmVSFlags vsflags)
     return ovsflags;
 }
 
-/* 
+/*
  * This allows us to mark transactions as being of a certain type.
  * The three types are:
- * 
+ *
  *     RPM_TRANS_NORMAL 	
  *     RPM_TRANS_ROLLBACK
  *     RPM_TRANS_AUTOROLLBACK
- * 
+ *
  * ROLLBACK and AUTOROLLBACK transactions should always be ran as
- * a best effort.  In particular this is important to the autorollback 
- * feature to avoid rolling back a rollback (otherwise known as 
- * dueling rollbacks (-;).  AUTOROLLBACK's additionally need instance 
+ * a best effort.  In particular this is important to the autorollback
+ * feature to avoid rolling back a rollback (otherwise known as
+ * dueling rollbacks (-;).  AUTOROLLBACK's additionally need instance
  * counts passed to scriptlets to be altered.
  */
-void rpmtsSetType(rpmts ts, rpmtsType type)
+/* Let them know what type of transaction we are */
+rpmTSType rpmtsType(rpmts ts)
 {
-    if (ts != NULL) {
-	ts->type = type;
-    }	 
+    return ((ts != NULL) ? ts->type : 0);
 }
 
-/* Let them know what type of transaction we are */
-rpmtsType rpmtsGetType(rpmts ts) 
+void rpmtsSetType(rpmts ts, rpmTSType type)
 {
-    if (ts != NULL) 
-	return ts->type;
-    else
-	return 0;
+    if (ts != NULL)
+	ts->type = type;
+}
+
+uint_32 rpmtsARBGoal(rpmts ts)
+{
+    return ((ts != NULL) ?  ts->arbgoal : 0);
+}
+
+void rpmtsSetARBGoal(rpmts ts, uint_32 goal)
+{
+    if (ts != NULL)
+	ts->arbgoal = goal;
 }
 
 int rpmtsUnorderedSuccessors(rpmts ts, int first)
@@ -1260,6 +1273,12 @@ int rpmtsInitDSI(const rpmts ts)
 	if (rc)
 	    break;
 	dsi->dev = sb.st_dev;
+/* XXX figger out how to get this info for non-statvfs systems. */
+#if STATFS_IN_SYS_STATVFS
+	dsi->f_fsid = sfb.f_fsid;
+	dsi->f_flag = sfb.f_flag;
+	dsi->f_namemax = sfb.f_namemax;
+#endif
 
 	dsi->bsize = sfb.f_bsize;
 	dsi->bneeded = 0;
@@ -1276,9 +1295,13 @@ int rpmtsInitDSI(const rpmts ts)
 	/* XXX Avoid FAT and other file systems that have not inodes. */
 	dsi->iavail = !(sfb.f_ffree == 0 && sfb.f_files == 0)
 				? sfb.f_ffree : -1;
-	rpmMessage(RPMMESS_DEBUG, _("%5d 0x%08x %8u %12ld %12ld %s\n"),
+#if !defined(ST_RDONLY)
+#define	ST_RDONLY	1
+#endif
+	rpmMessage(RPMMESS_DEBUG, _("%5d 0x%08x %8u %12ld %12ld %s %s\n"),
 		i, (unsigned) dsi->dev, (unsigned) dsi->bsize,
 		(signed long) dsi->bavail, (signed long) dsi->iavail,
+		((dsi->f_flag & ST_RDONLY) ? "ro" : "rw"),
 		ts->filesystems[i]);
     }
     return rc;
@@ -1354,25 +1377,31 @@ void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
     ps = rpmtsProblems(ts);
     for (i = 0; i < ts->filesystemCount; i++, dsi++) {
 
-	if (dsi->bavail >= 0 && adj_fs_blocks(dsi->bneeded) > dsi->bavail) {
+	if (dsi->bavail > 0 && adj_fs_blocks(dsi->bneeded) > dsi->bavail) {
 	    rpmpsAppend(ps, RPMPROB_DISKSPACE,
 			rpmteNEVR(te), rpmteKey(te),
 			ts->filesystems[i], NULL, NULL,
  	   (adj_fs_blocks(dsi->bneeded) - dsi->bavail) * dsi->bsize);
 	}
 
-	if (dsi->iavail >= 0 && adj_fs_blocks(dsi->ineeded) > dsi->iavail) {
+	if (dsi->iavail > 0 && adj_fs_blocks(dsi->ineeded) > dsi->iavail) {
 	    rpmpsAppend(ps, RPMPROB_DISKNODES,
 			rpmteNEVR(te), rpmteKey(te),
 			ts->filesystems[i], NULL, NULL,
  	    (adj_fs_blocks(dsi->ineeded) - dsi->iavail));
 	}
+
+	if ((dsi->bneeded || dsi->ineeded) && (dsi->f_flag & ST_RDONLY)) {
+	    rpmpsAppend(ps, RPMPROB_RDONLY,
+			rpmteNEVR(te), rpmteKey(te),
+			ts->filesystems[i], NULL, NULL, 0);
+	}
     }
     ps = rpmpsFree(ps);
 }
 
 void * rpmtsNotify(rpmts ts, rpmte te,
-		rpmCallbackType what, unsigned long amount, unsigned long total)
+		rpmCallbackType what, unsigned long long amount, unsigned long long total)
 {
     void * ptr = NULL;
     if (ts && ts->notify && te) {
@@ -1475,11 +1504,6 @@ uint_32 rpmtsSetColor(rpmts ts, uint_32 color)
     return ocolor;
 }
 
-uint_32 rpmtsPrefColor(rpmts ts)
-{
-    return (ts != NULL ? ts->prefcolor : 0);
-}
-
 rpmop rpmtsOp(rpmts ts, rpmtsOpX opx)
 {
     rpmop op = NULL;
@@ -1501,36 +1525,6 @@ int rpmtsSetNotifyCallback(rpmts ts,
     return 0;
 }
 
-int rpmtsGetKeys(const rpmts ts, fnpyKey ** ep, int * nep)
-{
-    int rc = 0;
-
-    if (nep) *nep = ts->orderCount;
-    if (ep) {
-	rpmtsi pi;	rpmte p;
-	fnpyKey * e;
-
-	*ep = e = xmalloc(ts->orderCount * sizeof(*e));
-	pi = rpmtsiInit(ts);
-	while ((p = rpmtsiNext(pi, 0)) != NULL) {
-	    switch (rpmteType(p)) {
-	    case TR_ADDED:
-		/*@-dependenttrans@*/
-		*e = rpmteKey(p);
-		/*@=dependenttrans@*/
-		/*@switchbreak@*/ break;
-	    case TR_REMOVED:
-	    default:
-		*e = NULL;
-		/*@switchbreak@*/ break;
-	    }
-	    e++;
-	}
-	pi = rpmtsiFree(pi);
-    }
-    return rc;
-}
-
 rpmts rpmtsCreate(void)
 {
     rpmts ts;
@@ -1559,7 +1553,6 @@ rpmts rpmtsCreate(void)
     ts->delta = 5;
 
     ts->color = rpmExpandNumeric("%{?_transaction_color}");
-    ts->prefcolor = rpmExpandNumeric("%{?_prefer_color}")?:2;
 
     ts->numRemovedPackages = 0;
     ts->allocedRemovedPackages = ts->delta;
@@ -1575,6 +1568,9 @@ rpmts rpmtsCreate(void)
     ts->numAddedPackages = 0;
     ts->addedPackages = NULL;
 
+    ts->numErasedPackages = 0;
+    ts->erasedPackages = NULL;
+
     ts->numAvailablePackages = 0;
     ts->availablePackages = NULL;
 
@@ -1592,193 +1588,10 @@ rpmts rpmtsCreate(void)
     memset(ts->pksignid, 0, sizeof(ts->pksignid));
     ts->dig = NULL;
 
-    /* 
-     * We only use the score in an autorollback.  So set this to
-     * NULL by default.
-     */
-    ts->score = NULL;
+    /* Set autorollback goal to the end of time. */
+    ts->arbgoal = 0xffffffff;
 
     ts->nrefs = 0;
 
     return rpmtsLink(ts, "tsCreate");
 }
-
-/**********************
- * Transaction Scores *
- **********************/
-
-
-rpmRC rpmtsScoreInit(rpmts runningTS, rpmts rollbackTS) 
-{
-    rpmtsScore score;
-    rpmtsi     pi;
-    rpmte      p;
-    int        i;
-    int        tranElements;  /* Number of transaction elements in runningTS */
-    int        found = 0;
-    rpmRC      rc = RPMRC_OK; /* Assume success */
-    rpmtsScoreEntry se;
-
-    rpmMessage(RPMMESS_DEBUG, _("Creating transaction score board(%p, %p)\n"),
-	runningTS, rollbackTS); 
-
-    /* Allocate space for score board */
-    score = xcalloc(1, sizeof(*score));
-    rpmMessage(RPMMESS_DEBUG, _("\tScore board address:  %p\n"), score);
-
-    /* 
-     * Determine the maximum size needed for the entry list.
-     * XXX: Today, I just get the count of rpmts elements, and allocate
-     *      an array that big.  Yes this is guaranteed to waste memory.
-     *      Future updates will hopefully make this more efficient,
-     *      but for now it will work.
-     */
-    tranElements  = rpmtsNElements(runningTS);
-    rpmMessage(RPMMESS_DEBUG, _("\tAllocating space for %d entries\n"), tranElements);
-    score->scores = xcalloc(tranElements, sizeof(score->scores));
-
-    /* Initialize score entry count */
-    score->entries = 0;
-    score->nrefs   = 0;
-
-    /*
-     * Increment through transaction elements and make sure for every 
-     * N there is an rpmtsScoreEntry.
-     */
-    pi = rpmtsiInit(runningTS); 
-    while ((p = rpmtsiNext(pi, TR_ADDED|TR_REMOVED)) != NULL) {
-	found  = 0;
-
-	/* Try to find the entry in the score list */
-	for(i = 0; i < score->entries; i++) {
-	    se = score->scores[i]; 
-  	    if (strcmp(rpmteN(p), se->N) == 0) {
-		found = 1;
-	 	/*@innerbreak@*/ break;
-	    }
-	}
-
-	/* If we did not find the entry then allocate space for it */
-	if (!found) {
-/*@-compdef -usereleased@*/ /* XXX p->fi->te undefined. */
-    	    rpmMessage(RPMMESS_DEBUG, _("\tAdding entry for %s to score board.\n"),
-		rpmteN(p));
-/*@=compdef =usereleased@*/
-    	    se = xcalloc(1, sizeof(*(*(score->scores))));
-    	    rpmMessage(RPMMESS_DEBUG, _("\t\tEntry address:  %p\n"), se);
-	    se->N         = xstrdup(rpmteN(p));
-	    se->te_types  = rpmteType(p); 
-	    se->installed = 0;
-	    se->erased    = 0; 
-	    score->scores[score->entries] = se;
-	    score->entries++;
-	} else {
-	    /* We found this one, so just add the element type to the one 
-	     * already there.
-	     */
-    	    rpmMessage(RPMMESS_DEBUG, _("\tUpdating entry for %s in score board.\n"),
-		rpmteN(p));
-	    score->scores[i]->te_types |= rpmteType(p);
-	}
-	 
-    }
-    pi = rpmtsiFree(pi);
- 
-    /* 
-     * Attach the score to the running transaction and the autorollback
-     * transaction.
-     */
-    runningTS->score  = score;
-    score->nrefs++;
-    rollbackTS->score = score;
-    score->nrefs++;
-
-    return rc;
-}
-
-rpmtsScore rpmtsScoreFree(rpmtsScore score) 
-{
-    rpmtsScoreEntry se = NULL;
-    int i;
-
-    rpmMessage(RPMMESS_DEBUG, _("May free Score board(%p)\n"), score);
-
-    /* If score is not initialized, then just return.
-     * This is likely the case if autorollbacks are not enabled.
-     */
-    if (score == NULL) return NULL;
-
-    /* Decrement the reference count */
-    score->nrefs--;
-
-    /* Do we have any more references?  If so
-     * just return.
-     */
-    if (score->nrefs > 0) return NULL;
-
-    rpmMessage(RPMMESS_DEBUG, _("\tRefcount is zero...will free\n"));
-    /* No more references, lets clean up  */
-    /* First deallocate the score entries */
-/*@-branchstate@*/
-    for(i = 0; i < score->entries; i++) {
-	/* Get the score for the ith entry */
-	se = score->scores[i]; 
-	
-	/* Deallocate the score entries name */
-	se->N = _free(se->N);
-
-	/* Deallocate the score entry itself */
-	se = _free(se);
-    }
-/*@=branchstate@*/
-
-    /* Next deallocate the score entry table */
-    score->scores = _free(score->scores);
-
-    /* Finally deallocate the score itself */
-    score = _free(score);
-
-    return NULL;
-}
-
-/* 
- * XXX: Do not get the score and then store it aside for later use.
- *      we will delete it out from under you.  There is no rpmtsScoreLink()
- *      as this may be a very temporary fix for autorollbacks.
- */
-rpmtsScore rpmtsGetScore(rpmts ts) 
-{
-    if (ts == NULL) return NULL;
-    return ts->score;
-}
-
-/* 
- * XXX: Do not get the score entry and then store it aside for later use.
- *      we will delete it out from under you.  There is no 
- *      rpmtsScoreEntryLink() as this may be a very temporary fix 
- *      for autorollbacks.
- * XXX: The scores are not sorted.  This should be fixed at earliest
- *      opportunity (i.e. when we have the whole autorollback working).
- */
-rpmtsScoreEntry rpmtsScoreGetEntry(rpmtsScore score, const char *N) 
-{
-    int i;
-    rpmtsScoreEntry se;
-    rpmtsScoreEntry ret = NULL; /* Assume we don't find it */
-
-    rpmMessage(RPMMESS_DEBUG, _("Looking in score board(%p) for %s\n"), score, N);
-
-    /* Try to find the entry in the score list */
-    for(i = 0; i < score->entries; i++) {
-	se = score->scores[i]; 
- 	if (strcmp(N, se->N) == 0) {
-    	    rpmMessage(RPMMESS_DEBUG, _("\tFound entry at address:  %p\n"), se);
-	    ret = se;
-	    break;
-	}
-    }
-	
-/*@-compdef@*/ /* XXX score->scores undefined. */
-    return ret;	
-/*@=compdef@*/
-}
