@@ -30,23 +30,23 @@ typedef enum rpmVSFlags_e {
     RPMVSF_NOSHA1HEADER	= (1 <<  8),
     RPMVSF_NOMD5HEADER	= (1 <<  9),	/* unimplemented */
     RPMVSF_NODSAHEADER	= (1 << 10),
-    RPMVSF_NORSAHEADER	= (1 << 11),	/* unimplemented */
+    RPMVSF_NORSAHEADER	= (1 << 11),
     /* bit(s) 12-15 unused */
     RPMVSF_NOSHA1	= (1 << 16),	/* unimplemented */
     RPMVSF_NOMD5	= (1 << 17),
     RPMVSF_NODSA	= (1 << 18),
     RPMVSF_NORSA	= (1 << 19)
-    /* bit(s) 16-31 unused */
+    /* bit(s) 20-31 unused */
 } rpmVSFlags;
 
 /**
  * Transaction Types
  */
-typedef enum rpmtsType_e {
+typedef enum rpmTSType_e {
 	RPMTRANS_TYPE_NORMAL       = 0,
 	RPMTRANS_TYPE_ROLLBACK     = (1 << 0),
 	RPMTRANS_TYPE_AUTOROLLBACK = (1 << 1)
-} rpmtsType;
+} rpmTSType;
 
 #define	_RPMVSF_NODIGESTS	\
   ( RPMVSF_NOSHA1HEADER |	\
@@ -93,7 +93,8 @@ typedef	enum rpmtsOpX_e {
     RPMTS_OP_DBGET		= 14,
     RPMTS_OP_DBPUT		= 15,
     RPMTS_OP_DBDEL		= 16,
-    RPMTS_OP_MAX		= 17
+    RPMTS_OP_READHDR		= 17,
+    RPMTS_OP_MAX		= 18
 } rpmtsOpX;
 
 #if defined(_RPMTS_INTERNAL)
@@ -101,94 +102,6 @@ typedef	enum rpmtsOpX_e {
 #include "rpmhash.h"	/* XXX hashTable */
 #include "rpmal.h"	/* XXX availablePackage/relocateFileList ,*/
 
-/**********************
- * Transaction Scores *
- **********************
- *
- * In order to allow instance counts to be adjusted properly when an
- * autorollback transaction is ran, we keep a list that is indexed
- * by rpm name of whether the rpm has been installed or erased.  This listed
- * is only updated:
- *
- *	iif autorollbacks are enabled.
- *	iif this is not a rollback or autorollback transaction.
- *
- * When creating an autorollback transaction, its rpmts points to the same
- * rpmtsScore object as the running transaction.  So when the autorollback
- * transaction runs it can see where each package was in the running transaction
- * at the point the running transaction failed, and thus on a per package
- * basis make adjustments to the instance counts.
- *
- * XXX: Jeff, I am not convinced that this does not need to be in its own file
- *      (i.e. rpmtsScore.{h,c}), but I first wanted to get it working.
- */
-struct rpmtsScoreEntry_s {
-    char *         N;			/*!<Name of package                */
-    rpmElementType te_types;		/*!<te types this entry represents */
-    int            installed;		/*!<Was the new header installed   */
-    int            erased;		/*!<Was the old header removed     */
-};
-
-typedef /*@abstract@*/ struct rpmtsScoreEntry_s * rpmtsScoreEntry;
-
-struct rpmtsScore_s {
-	int entries;			/*!< Number of scores       */
-	rpmtsScoreEntry * scores;	/*!< Array of score entries */
-    	int nrefs;			/*!< Reference count.       */
-};
-
-typedef /*@abstract@*/ struct rpmtsScore_s * rpmtsScore;
-
-
-/** \ingroup rpmts
- * initialize rpmtsScore for running transaction and autorollback
- * transaction.
- * @param runningTS	Running Transaction.
- * @param rollbackTS	Rollback Transaction.
- * @return		RPMRC_OK
- */
-rpmRC rpmtsScoreInit(rpmts runningTS, rpmts rollbackTS)
-	/*@globals fileSystem @*/
-	/*@modifies runningTS, rollbackTS, fileSystem @*/;
-
-/** \ingroup rpmts
- * Free rpmtsScore provided no more references exist against it.
- * @param score		rpmtsScore to free
- * @return		NULL always
- */
-/*@-exportlocal@*/
-/*@null@*/
-rpmtsScore rpmtsScoreFree(/*@only@*/ /*@null@*/ rpmtsScore score)
-	/*@modifies score @*/;
-/*@=exportlocal@*/
-
-/** \ingroup rpmts
- * Get rpmtsScore from transaction.
- * @param ts	RPM Transaction.
- * @return	rpmtsScore or NULL.
- */
-/*@exposed@*/ /*@null@*/
-rpmtsScore rpmtsGetScore(rpmts ts)
-	/*@*/;
-
-/** \ingroup rpmts
- * Get rpmtsScoreEntry from rpmtsScore.
- * @param score   RPM Transaction Score.
- * @return	  rpmtsScoreEntry or NULL.
- */
-/*@null@*/
-rpmtsScoreEntry rpmtsScoreGetEntry(rpmtsScore score, const char *N)
-	/*@*/;
-
-/** \ingroup rpmts
- * \file lib/rpmts.h
- * Structures and prototypes used for an "rpmts" transaction set.
- */
-
-/**************************
- * END Transaction Scores *
- **************************/
-
 /*@unchecked@*/
 /*@-exportlocal@*/
 extern int _cacheDependsRC;
@@ -202,6 +115,9 @@ typedef	/*@abstract@*/ struct diskspaceInfo_s * rpmDiskSpaceInfo;
  */
 struct diskspaceInfo_s {
     dev_t dev;			/*!< File system device number. */
+    unsigned long f_fsid;	/*!< File system id. */
+    unsigned long f_flag;	/*!< Mount flags. */
+    unsigned long f_namemax;	/*!< Maximum filename length. */
     signed long bneeded;	/*!< No. of blocks needed. */
     signed long ineeded;	/*!< No. of inodes needed. */
     int bsize;			/*!< File system block size. */
@@ -232,7 +148,7 @@ typedef enum tsStage_e {
 struct rpmts_s {
     rpmtransFlags transFlags;	/*!< Bit(s) to control operation. */
     tsmStage goal;		/*!< Transaction goal (i.e. mode) */
-    rpmtsType type;             /*!< default, rollback, autorollback */
+    rpmTSType type;		/*!< default, rollback, autorollback */
 
 /*@refcounted@*/ /*@null@*/
     rpmdb sdb;			/*!< Solve database handle. */
@@ -251,6 +167,8 @@ struct rpmts_s {
 /*@observer@*/ /*@null@*/
     rpmCallbackData notifyData;	/*!< Callback private data. */
 
+    struct rpmPRCO_s PRCO;	/*!< Current transaction dependencies. */
+
 /*@refcounted@*/ /*@null@*/
     rpmps probs;		/*!< Current problems in transaction. */
     rpmprobFilterFlags ignoreSet;
@@ -277,6 +195,10 @@ struct rpmts_s {
     rpmal addedPackages;	/*!< Set of packages being installed. */
     int numAddedPackages;	/*!< No. added package instances. */
 
+/*@only@*/
+    rpmal erasedPackages;	/*!< Set of packages being erased. */
+    int numErasedPackages;	/*!< No. erased package instances. */
+
 #ifndef	DYING
 /*@only@*/
     rpmal availablePackages;	/*!< Universe of available packages. */
@@ -297,6 +219,11 @@ struct rpmts_s {
     int ntrees;			/*!< No. of dependency trees. */
     int maxDepth;		/*!< Maximum depth of dependency tree(s). */
 
+/*@dependent@*/
+    rpmte teInstall;		/*!< current rpmtsAddInstallElement element. */
+/*@dependent@*/
+    rpmte teErase;		/*!< current rpmtsAddEraseElement element. */
+
     int selinuxEnabled;		/*!< Is SE linux enabled? */
     int chrootDone;		/*!< Has chroot(2) been been done? */
 /*@only@*/ /*@null@*/
@@ -309,7 +236,6 @@ struct rpmts_s {
     int_32 tid;			/*!< Transaction id. */
 
     uint_32 color;		/*!< Transaction color bits. */
-    uint_32 prefcolor;		/*!< Preferred file color. */
 
     rpmVSFlags vsflags;		/*!< Signature/digest verification flags. */
 
@@ -334,8 +260,12 @@ struct rpmts_s {
 /*@null@*/
     Spec spec;			/*!< Spec file control structure. */
 
+#ifdef	DYING
 /*@kept@*/ /*@null@*/
-    rpmtsScore score;		/*!< Transaction Score (autorollback). */
+    rpmtsScore score;		/*!< Transaction score (autorollback). */
+#endif
+
+    uint_32 arbgoal;		/*!< Autorollback goal */
 
 /*@refs@*/
     int nrefs;			/*!< Reference count. */
@@ -396,6 +326,21 @@ int rpmtsRun(rpmts ts, rpmps okProbs, rpmprobFilterFlags ignoreSet)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
+/**
+ * Function to perform autorollback goal.  Today is called automatically
+ * from rpmtsCheck(), rpmtsOrder() and rpmtsRun().  May change to be a 
+ * programmer decision.
+ *
+ * @param failedTransaction	Failed transaction.
+ * @param rbts			rollback transaction (can be NULL).
+ * @param ignoreSet		Problems to ignore.
+ * @return			RPMRC_OK, or RPMRC_FAIL
+ */
+rpmRC rpmtsDoARBGoal(rpmts failedTransaction, rpmts rbts,
+    rpmprobFilterFlags ignoreSet)
+	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
+	/*@modifies rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/;
+
 /** \ingroup rpmts
  * Unreference a transaction instance.
  * @param ts		transaction set
@@ -568,23 +513,38 @@ int rpmtsSetSolveCallback(rpmts ts,
 /**
  * Return the type of a transaction.
  * @param ts		transaction set
- * @return		0 it is not, 1 it is.
+ * @return		transaction type, 0 on unknown
  */
-rpmtsType rpmtsGetType(rpmts ts)
+rpmTSType rpmtsType(rpmts ts)
 	/*@*/;
 
 /**
- * Set transaction type.   Allowed types are:
- *
+ * Set transaction type.
+ *   Allowed types are:
  * 	RPMTRANS_TYPE_NORMAL
  *	RPMTRANS_TYPE_ROLLBACK
  * 	RPMTRANS_TYPE_AUTOROLLBACK
  *
  * @param ts		transaction set
  * @param type		transaction type
- * @return		void
  */
-void rpmtsSetType(rpmts ts, rpmtsType type)
+void rpmtsSetType(rpmts ts, rpmTSType type)
+	/*@modifies ts @*/;
+
+/**
+ * Return the autorollback goal. 
+ * @param ts		transaction set
+ * @return		autorollback goal
+ */
+uint_32 rpmtsARBGoal(rpmts ts)
+	/*@*/;
+
+/**
+ * Set autorollback goal.   
+ * @param ts		transaction set
+ * @param goal		autorollback goal
+ */
+void rpmtsSetARBGoal(rpmts ts, uint_32 goal)
 	/*@modifies ts @*/;
 
 /**
@@ -836,9 +796,11 @@ pgpDigParams rpmtsSignature(const rpmts ts)
  * @param ts		transaction set
  * @return		pubkey constants.
  */
+/*@-exportlocal@*/
 /*@exposed@*/ /*@null@*/
 pgpDigParams rpmtsPubkey(const rpmts ts)
 	/*@*/;
+/*@=exportlocal@*/
 
 /** \ingroup rpmts
  * Get transaction set database handle.
@@ -882,6 +844,8 @@ void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
 
 /**
  * Perform transaction progress notify callback.
+ * @warning This function's args have changed, so the function cannot be
+ * used portably
  * @param ts		transaction set
  * @param te		current transaction element
  * @param what		type of call back
@@ -891,7 +855,7 @@ void rpmtsCheckDSIProblems(const rpmts ts, const rpmte te)
  */
 /*@null@*/
 void * rpmtsNotify(rpmts ts, rpmte te,
-                rpmCallbackType what, unsigned long amount, unsigned long total)
+                rpmCallbackType what, unsigned long long amount, unsigned long long total)
 	/*@*/;
 
 /**
@@ -983,14 +947,6 @@ rpmte rpmtsSetRelocateElement(rpmts ts, /*@null@*/ rpmte relocateElement)
 uint_32 rpmtsColor(rpmts ts)
 	/*@*/;
 
-/**
- * Retrieve prefered file color
- * @param ts		transaction set
- * @return		color bits
- */
-uint_32 rpmtsPrefColor(rpmts ts)
-	/*@*/;
-
 /**
  * Set color bits of transaction set.
  * @param ts		transaction set
@@ -1050,7 +1006,7 @@ rpmts rpmtsCreate(void)
  */
 int rpmtsAddInstallElement(rpmts ts, Header h,
 		/*@exposed@*/ /*@null@*/ const fnpyKey key, int upgrade,
-		/*@null@*/ rpmRelocation * relocs)
+		/*@null@*/ rpmRelocation relocs)
 	/*@globals rpmcliPackagesTotal, rpmGlobalMacroContext, h_errno,
 		fileSystem, internalState @*/
 	/*@modifies ts, h, rpmcliPackagesTotal, rpmGlobalMacroContext,
@@ -1067,28 +1023,14 @@ int rpmtsAddEraseElement(rpmts ts, Header h, int dboffset)
 	/*@globals rpmGlobalMacroContext, h_errno, fileSystem, internalState @*/
 	/*@modifies ts, h, rpmGlobalMacroContext, fileSystem, internalState @*/;
 
-/** \ingroup rpmts
- * Retrieve keys from ordered transaction set.
- * @todo Removed packages have no keys, returned as interleaved NULL pointers.
- * @param ts		transaction set
- * @retval ep		address of returned element array pointer (or NULL)
- * @retval nep		address of no. of returned elements (or NULL)
- * @return		0 always
- */
-/*@unused@*/
-int rpmtsGetKeys(rpmts ts,
-		/*@null@*/ /*@out@*/ fnpyKey ** ep,
-		/*@null@*/ /*@out@*/ int * nep)
-	/*@globals fileSystem, internalState @*/
-	/*@modifies ts, ep, nep, fileSystem, internalState @*/;
-
 /**
  * Return (malloc'd) header name-version-release string.
  * @param h		header
  * @retval np		name tag value
  * @return		name-version-release string
  */
-/*@only@*/ char * hGetNEVR(Header h, /*@null@*/ /*@out@*/ const char ** np )
+/*@only@*/
+char * hGetNEVR(Header h, /*@null@*/ /*@out@*/ const char ** np )
 	/*@modifies *np @*/;
 
 /**
@@ -1097,7 +1039,8 @@ int rpmtsGetKeys(rpmts ts,
  * @retval np		name tag value
  * @return		name-version-release string
  */
-/*@only@*/ char * hGetNEVRA(Header h, /*@null@*/ /*@out@*/ const char ** np )
+/*@only@*/
+char * hGetNEVRA(Header h, /*@null@*/ /*@out@*/ const char ** np )
 	/*@modifies *np @*/;
 
 /**
