@@ -9,9 +9,11 @@
 
 #include "psm.h"
 #include "rpmfi.h"
+
+#define	_RPMPS_INTERNAL	/* XXX rpmps needs iterator. */
 #include "rpmts.h"
 
-#include "legacy.h"	/* XXX domd5(), uidToUname(), gnameToGid */
+#include "legacy.h"	/* XXX dodigest(), uidToUname(), gnameToGid */
 #include "ugid.h"
 #include "debug.h"
 
@@ -78,36 +80,38 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
 	return 1;
     }
 
+    flags |= RPMVERIFY_CONTEXTS;	/* no disable from package. */
+
     /*
      * Not all attributes of non-regular files can be verified.
      */
     if (S_ISDIR(sb.st_mode))
-	flags &= ~(RPMVERIFY_MD5 | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME | 
+	flags &= ~(RPMVERIFY_FDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
 			RPMVERIFY_LINKTO);
     else if (S_ISLNK(sb.st_mode)) {
-	flags &= ~(RPMVERIFY_MD5 | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
+	flags &= ~(RPMVERIFY_FDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
 		RPMVERIFY_MODE);
 #if CHOWN_FOLLOWS_SYMLINK
 	    flags &= ~(RPMVERIFY_USER | RPMVERIFY_GROUP);
 #endif
     }
     else if (S_ISFIFO(sb.st_mode))
-	flags &= ~(RPMVERIFY_MD5 | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME | 
+	flags &= ~(RPMVERIFY_FDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
 			RPMVERIFY_LINKTO);
     else if (S_ISCHR(sb.st_mode))
-	flags &= ~(RPMVERIFY_MD5 | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME | 
+	flags &= ~(RPMVERIFY_FDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
 			RPMVERIFY_LINKTO);
     else if (S_ISBLK(sb.st_mode))
-	flags &= ~(RPMVERIFY_MD5 | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME | 
+	flags &= ~(RPMVERIFY_FDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
 			RPMVERIFY_LINKTO);
-    else 
+    else
 	flags &= ~(RPMVERIFY_LINKTO);
 
     /*
      * Content checks of %ghost files are meaningless.
      */
     if (fileAttrs & RPMFILE_GHOST)
-	flags &= ~(RPMVERIFY_MD5 | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME | 
+	flags &= ~(RPMVERIFY_FDIGEST | RPMVERIFY_FILESIZE | RPMVERIFY_MTIME |
 			RPMVERIFY_LINKTO);
 
     /*
@@ -115,23 +119,55 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
      */
     flags &= ~(omitMask | RPMVERIFY_FAILURES);
 
+    /*
+     * Verify file security context.
+     */
+/*@-branchstate@*/
+    if (selinuxEnabled == 1 && (flags & RPMVERIFY_CONTEXTS)) {
+	security_context_t con;
+
+	rc = lgetfilecon(fn, &con);
+	if (rc == -1)
+	    *res |= (RPMVERIFY_LGETFILECONFAIL|RPMVERIFY_CONTEXTS);
+	else {
+	    rpmsx sx = rpmtsREContext(ts);
+	    const char * fcontext;
+
+	    if (sx != NULL) {
+		/* Get file security context from patterns. */
+		fcontext = rpmsxFContext(sx, fn, fmode);
+		sx = rpmsxFree(sx);
+	    } else {
+		/* Get file security context from package. */
+		fcontext = rpmfiFContext(fi);
+	    }
+	    if (fcontext == NULL || strcmp(fcontext, con))
+		*res |= RPMVERIFY_CONTEXTS;
+	    freecon(con);
+	}
+    }
 /*@=branchstate@*/
 
-    if (flags & RPMVERIFY_MD5) {
-	unsigned char md5sum[16];
-	size_t fsize;
+    if (flags & RPMVERIFY_FDIGEST) {
+	int dalgo = 0;
+	size_t dlen = 0;
+	const unsigned char * digest = rpmfiDigest(fi, &dalgo, &dlen);
 
-	/* XXX If --nomd5, then prelinked library sizes are not corrected. */
-	rc = domd5(fn, md5sum, 0, &fsize);
-	sb.st_size = fsize;
-	if (rc)
-	    *res |= (RPMVERIFY_READFAIL|RPMVERIFY_MD5);
+	if (digest == NULL)
+	    *res |= RPMVERIFY_FDIGEST;
 	else {
-	    const unsigned char * MD5 = rpmfiMD5(fi);
-	    if (MD5 == NULL || memcmp(md5sum, MD5, sizeof(md5sum)))
-		*res |= RPMVERIFY_MD5;
+	/* XXX If --nofdigest, then prelinked library sizes fail to verify. */
+	    unsigned char * fdigest = memset(alloca(dlen), 0, dlen);
+	    size_t fsize;
+	    rc = dodigest(dalgo, fn, fdigest, 0, &fsize);
+	    sb.st_size = fsize;
+	    if (rc)
+		*res |= (RPMVERIFY_READFAIL|RPMVERIFY_FDIGEST);
+	    else
+	    if (memcmp(fdigest, digest, dlen))
+		*res |= RPMVERIFY_FDIGEST;
 	}
-    } 
+    }
 
     if (flags & RPMVERIFY_LINKTO) {
 	char linkto[1024+1];
@@ -145,12 +181,12 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
 	    if (flink == NULL || strcmp(linkto, flink))
 		*res |= RPMVERIFY_LINKTO;
 	}
-    } 
+    }
 
     if (flags & RPMVERIFY_FILESIZE) {
 	if (sb.st_size != rpmfiFSize(fi))
 	    *res |= RPMVERIFY_FILESIZE;
-    } 
+    }
 
     if (flags & RPMVERIFY_MODE) {
 	unsigned short metamode = fmode;
@@ -158,7 +194,7 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
 
 	/*
 	 * Platforms (like AIX) where sizeof(unsigned short) != sizeof(mode_t)
-	 * need the (unsigned short) cast here. 
+	 * need the (unsigned short) cast here.
 	 */
 	filemode = (unsigned short)sb.st_mode;
 
@@ -184,7 +220,7 @@ int rpmVerifyFile(const rpmts ts, const rpmfi fi,
 	    uint_16 frdev = (rpmfiFRdev(fi) & 0xffff);
 	    if (st_rdev != frdev)
 		*res |= RPMVERIFY_RDEV;
-	} 
+	}
     }
 
     if (flags & RPMVERIFY_MTIME) {
@@ -274,42 +310,46 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
     fi = rpmfiInit(fi, 0);
     if (fi != NULL)	/* XXX lclint */
     while ((i = rpmfiNext(fi)) >= 0) {
-	rpmfileAttrs fileAttrs;
+	rpmfileAttrs fflags;
 	int rc;
 
-	fileAttrs = rpmfiFFlags(fi);
+	fflags = rpmfiFFlags(fi);
+
+	/* If not querying %config, skip config files. */
+	if ((qva->qva_fflags & RPMFILE_CONFIG) && (fflags & RPMFILE_CONFIG))
+	    continue;
+
+	/* If not querying %doc, skip doc files. */
+	if ((qva->qva_fflags & RPMFILE_DOC) && (fflags & RPMFILE_DOC))
+	    continue;
 
 	/* If not verifying %ghost, skip ghost files. */
-	if (!(qva->qva_fflags & RPMFILE_GHOST)
-	&& (fileAttrs & RPMFILE_GHOST))
+	/* XXX the broken!!! logic disables %ghost queries always. */
+	if (!(qva->qva_fflags & RPMFILE_GHOST) && (fflags & RPMFILE_GHOST))
 	    continue;
 
 /*@-boundswrite@*/
 	rc = rpmVerifyFile(ts, fi, &verifyResult, omitMask);
 /*@=boundswrite@*/
 	if (rc) {
-	    if (!(fileAttrs & (RPMFILE_MISSINGOK|RPMFILE_GHOST)) || rpmIsVerbose()) {
+	    if (!(fflags & (RPMFILE_MISSINGOK|RPMFILE_GHOST)) || rpmIsVerbose()) {
 		sprintf(te, _("missing   %c %s"),
-			((fileAttrs & RPMFILE_CONFIG)	? 'c' :
-			 (fileAttrs & RPMFILE_DOC)	? 'd' :
-			 (fileAttrs & RPMFILE_GHOST)	? 'g' :
-			 (fileAttrs & RPMFILE_LICENSE)	? 'l' :
-			 (fileAttrs & RPMFILE_PUBKEY)	? 'P' :
-			 (fileAttrs & RPMFILE_README)	? 'r' : ' '), 
+			((fflags & RPMFILE_CONFIG)	? 'c' :
+			 (fflags & RPMFILE_DOC)		? 'd' :
+			 (fflags & RPMFILE_GHOST)	? 'g' :
+			 (fflags & RPMFILE_LICENSE)	? 'l' :
+			 (fflags & RPMFILE_PUBKEY)	? 'P' :
+			 (fflags & RPMFILE_README)	? 'r' : ' '),
 			rpmfiFN(fi));
 		te += strlen(te);
-		if ((verifyResult & RPMVERIFY_LSTATFAIL) != 0 &&
-		    errno != ENOENT) {
-		    sprintf(te, " (%s)", strerror(errno));
-		    te += strlen(te);
-		}
 		ec = rc;
 	    }
 	} else if (verifyResult || rpmIsVerbose()) {
-	    const char * size, * MD5, * link, * mtime, * mode;
-	    const char * group, * user, * rdev;
+	    const char * size, * digest, * link, * mtime, * mode;
+	    const char * group, * user, * rdev, *ctxt;
 	    /*@observer@*/ static const char *const aok = ".";
 	    /*@observer@*/ static const char *const unknown = "?";
+	    /*@observer@*/ static const char *const ctxt_ignore = " ";
 
 	    ec = 1;
 
@@ -321,8 +361,12 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 #define	_verifyfile(_RPMVERIFY_F, _C)	\
 	((verifyResult & RPMVERIFY_READFAIL) ? unknown : \
 	 (verifyResult & _RPMVERIFY_F) ? _C : aok)
+#define	_verifyctxt(_RPMVERIFY_F, _C)	\
+	((selinuxEnabled != 1 ? ctxt_ignore : \
+	 (verifyResult & RPMVERIFY_LGETFILECONFAIL) ? unknown : \
+	 (verifyResult & _RPMVERIFY_F) ? _C : aok))
 	
-	    MD5 = _verifyfile(RPMVERIFY_MD5, "5");
+	    digest = _verifyfile(RPMVERIFY_FDIGEST, "5");
 	    size = _verify(RPMVERIFY_FILESIZE, "S");
 	    link = _verifylink(RPMVERIFY_LINKTO, "L");
 	    mtime = _verify(RPMVERIFY_MTIME, "T");
@@ -330,19 +374,21 @@ static int verifyHeader(QVA_t qva, const rpmts ts, rpmfi fi)
 	    user = _verify(RPMVERIFY_USER, "U");
 	    group = _verify(RPMVERIFY_GROUP, "G");
 	    mode = _verify(RPMVERIFY_MODE, "M");
+	    ctxt = _verifyctxt(RPMVERIFY_CONTEXTS, "C");
 
+#undef _verifyctxt
 #undef _verifyfile
 #undef _verifylink
 #undef _verify
 
-	    sprintf(te, "%s%s%s%s%s%s%s%s  %c %s",
-			size, mode, MD5, rdev, link, user, group, mtime,
-			((fileAttrs & RPMFILE_CONFIG)	? 'c' :
-			 (fileAttrs & RPMFILE_DOC)	? 'd' :
-			 (fileAttrs & RPMFILE_GHOST)	? 'g' :
-			 (fileAttrs & RPMFILE_LICENSE)	? 'l' :
-			 (fileAttrs & RPMFILE_PUBKEY)	? 'P' :
-			 (fileAttrs & RPMFILE_README)	? 'r' : ' '), 
+	    sprintf(te, "%s%s%s%s%s%s%s%s%s %c %s",
+		size, mode, digest, rdev, link, user, group, mtime, ctxt,
+			((fflags & RPMFILE_CONFIG)	? 'c' :
+			 (fflags & RPMFILE_DOC)	? 'd' :
+			 (fflags & RPMFILE_GHOST)	? 'g' :
+			 (fflags & RPMFILE_LICENSE)	? 'l' :
+			 (fflags & RPMFILE_PUBKEY)	? 'P' :
+			 (fflags & RPMFILE_README)	? 'r' : ' '),
 			rpmfiFN(fi));
 	    te += strlen(te);
 	}
@@ -397,20 +443,26 @@ static int verifyDependencies(/*@unused@*/ QVA_t qva, rpmts ts,
 	for (i = 0; i < numProblems; i++) {
 	    p = ps->probs + i;
 	    altNEVR = (p->altNEVR ? p->altNEVR : "? ?altNEVR?");
-	    nb += strlen(altNEVR+2) + sizeof(", ") - 1;
+	    if (altNEVR[0] == 'R' && altNEVR[1] == ' ')
+		nb += sizeof("\tRequires: ")-1;
+	    if (altNEVR[0] == 'C' && altNEVR[1] == ' ')
+		nb += sizeof("\tConflicts: ")-1;
+	    nb += strlen(altNEVR+2) + sizeof("\n") - 1;
 	}
 	te = t = alloca(nb);
 /*@-boundswrite@*/
 	*te = '\0';
 	pkgNEVR = (ps->probs->pkgNEVR ? ps->probs->pkgNEVR : "?pkgNEVR?");
-	sprintf(te, _("Unsatisfied dependencies for %s: "), pkgNEVR);
+	sprintf(te, _("Unsatisfied dependencies for %s:\n"), pkgNEVR);
 	te += strlen(te);
 	for (i = 0; i < numProblems; i++) {
 	    p = ps->probs + i;
 	    altNEVR = (p->altNEVR ? p->altNEVR : "? ?altNEVR?");
-	    if (i) te = stpcpy(te, ", ");
-	    /* XXX FIXME: should probably supply the "[R|C] " type prefix */
-	    te = stpcpy(te, altNEVR+2);
+	    if (altNEVR[0] == 'R' && altNEVR[1] == ' ')
+		te = stpcpy(te, "\tRequires: ");
+	    if (altNEVR[0] == 'C' && altNEVR[1] == ' ')
+		te = stpcpy(te, "\tConflicts: ");
+	    te = stpcpy( stpcpy(te, altNEVR+2), "\n");
 	}
 
 	if (te > t) {
@@ -434,14 +486,28 @@ static int verifyDependencies(/*@unused@*/ QVA_t qva, rpmts ts,
 
 int showVerifyPackage(QVA_t qva, rpmts ts, Header h)
 {
+/*
+ * XXX Sick hackery to work around qva being clobbered on CentOS3
+ * XXX using gcc-3.2.3-49.x86_64.
+ */
+#if defined(__x86_64__)
+static QVA_t Qva;
+#endif
     int scareMem = 1;	/* XXX only rpmVerifyScript needs now */
-    rpmfi fi;
+    rpmfi fi = NULL;
     int ec = 0;
     int rc;
 
+#if defined(__x86_64__)
+fi = rpmfiFree(fi);	/* XXX do something to confuse the optimizer. */
+Qva = qva;
+#endif
     fi = rpmfiNew(ts, h, RPMTAG_BASENAMES, scareMem);
     if (fi != NULL) {
 
+#if defined(__x86_64__)
+qva = Qva;
+#endif
 	if (qva->qva_flags & VERIFY_DEPS) {
 	    int save_noise = _rpmds_unspecified_epoch_noise;
 /*@-mods@*/
@@ -491,6 +557,20 @@ int rpmcliVerify(rpmts ts, QVA_t qva, const char ** argv)
 	vsflags |= RPMVSF_NOHDRCHK;
     vsflags &= ~RPMVSF_NEEDPAYLOAD;
 
+    /* Initialize security context patterns (if not already done). */
+    if (qva->qva_flags & VERIFY_CONTEXTS) {
+	rpmsx sx = rpmtsREContext(ts);
+	if (sx == NULL) {
+	    arg = rpmGetPath("%{?_verify_file_context_path}", NULL);
+	    if (arg != NULL && *arg != '\0') {
+		sx = rpmsxNew(arg);
+		(void) rpmtsSetREContext(ts, sx);
+	    }
+	    arg = _free(arg);
+	}
+	sx = rpmsxFree(sx);
+    }
+
     ovsflags = rpmtsSetVSFlags(ts, vsflags);
     ec = rpmcliArgIter(ts, qva, argv);
     vsflags = rpmtsSetVSFlags(ts, ovsflags);
