@@ -9,8 +9,7 @@
 	{"rex", luaopen_rex}, \
 	{"luapath", luapath},
 
-#if 0
-
+#if 0	/* XXX don't depend on readline and history (#147013) */
 #define lua_readline	myreadline
 #define lua_saveline	mysaveline
 
