@@ -5,11 +5,21 @@ test -z "$srcdir" && srcdir=.
 
 THEDIR="`pwd`"
 
+libtoolize=`which glibtoolize 2>/dev/null`
+case $libtoolize in
+/*) ;;
+*)  libtoolize=`which libtoolize 2>/dev/null`
+    case $libtoolize in
+    /*) ;;
+    *)  libtoolize=libtoolize
+    esac
+esac
+
 cd "$srcdir"
-libtoolize --copy --force
+$libtoolize --copy --force
 aclocal
 autoheader
-automake
+automake -a -c
 autoconf
 
 if [ "$1" = "--noconfigure" ]; then 
@@ -19,7 +29,7 @@ fi
 cd "$THEDIR"
 
 if [ X"$@" = X  -a "X`uname -s`" = "XLinux" ]; then
-    $srcdir/configure --prefix=/usr
+    $srcdir/configure --prefix=/usr "$@"
 else
     $srcdir/configure "$@"
 fi
