@@ -72,7 +72,7 @@
 
 /** \ingroup python
  * \class Rpmhdr
- * \brief A python header object represents an RPM package header.
+ * \brief A python rpm.hdr object represents an RPM package header.
  *
  * All RPM packages have headers that provide metadata for the package.
  * Header objects can be returned by database queries or loaded from a
@@ -128,11 +128,6 @@
  * will fail.
  */
 
-/** \ingroup python
- * \name Class: rpm.hdr
- */
-/*@{*/
-
 /** \ingroup py_c
  */
 struct hdrObject_s {
@@ -148,6 +143,8 @@ struct hdrObject_s {
     unsigned short * modes;
 } ;
 
+/**
+ */
 /*@unused@*/ static inline Header headerAllocated(Header h)
 	/*@modifies h @*/
 {
@@ -155,7 +152,260 @@ struct hdrObject_s {
     return 0;
 }
 
-/** \ingroup py_c
+/*@-boundsread@*/
+static int dncmp(const void * a, const void * b)
+	/*@*/
+{
+    const char *const * first = a;
+    const char *const * second = b;
+    return strcmp(*first, *second);
+}
+/*@=boundsread@*/
+
+/**
+ * Convert (dirname,basename,dirindex) tags to absolute path tag.
+ * @param h		header
+ */
+static void expandFilelist(Header h)
+        /*@modifies h @*/
+{
+    HAE_t hae = (HAE_t)headerAddEntry;
+    HRE_t hre = (HRE_t)headerRemoveEntry;
+    const char ** fileNames = NULL;
+    int count = 0;
+    int xx;
+
+    /*@-branchstate@*/
+    if (!headerIsEntry(h, RPMTAG_OLDFILENAMES)) {
+	rpmfiBuildFNames(h, RPMTAG_BASENAMES, &fileNames, &count);
+	if (fileNames == NULL || count <= 0)
+	    return;
+	xx = hae(h, RPMTAG_OLDFILENAMES, RPM_STRING_ARRAY_TYPE,
+			fileNames, count);
+	fileNames = _free(fileNames);
+    }
+    /*@=branchstate@*/
+
+    xx = hre(h, RPMTAG_DIRNAMES);
+    xx = hre(h, RPMTAG_BASENAMES);
+    xx = hre(h, RPMTAG_DIRINDEXES);
+}
+
+/*@-bounds@*/
+/**
+ * Convert absolute path tag to (dirname,basename,dirindex) tags.
+ * @param h             header
+ */
+static void compressFilelist(Header h)
+	/*@modifies h @*/
+{
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HAE_t hae = (HAE_t)headerAddEntry;
+    HRE_t hre = (HRE_t)headerRemoveEntry;
+    HFD_t hfd = headerFreeData;
+    char ** fileNames;
+    const char ** dirNames;
+    const char ** baseNames;
+    int_32 * dirIndexes;
+    rpmTagType fnt;
+    int count;
+    int i, xx;
+    int dirIndex = -1;
+
+    /*
+     * This assumes the file list is already sorted, and begins with a
+     * single '/'. That assumption isn't critical, but it makes things go
+     * a bit faster.
+     */
+
+    if (headerIsEntry(h, RPMTAG_DIRNAMES)) {
+	xx = hre(h, RPMTAG_OLDFILENAMES);
+	return;		/* Already converted. */
+    }
+
+    if (!hge(h, RPMTAG_OLDFILENAMES, &fnt, (void **) &fileNames, &count))
+	return;		/* no file list */
+    if (fileNames == NULL || count <= 0)
+	return;
+
+    dirNames = alloca(sizeof(*dirNames) * count);	/* worst case */
+    baseNames = alloca(sizeof(*dirNames) * count);
+    dirIndexes = alloca(sizeof(*dirIndexes) * count);
+
+    if (fileNames[0][0] != '/') {
+	/* HACK. Source RPM, so just do things differently */
+	dirIndex = 0;
+	dirNames[dirIndex] = "";
+	for (i = 0; i < count; i++) {
+	    dirIndexes[i] = dirIndex;
+	    baseNames[i] = fileNames[i];
+	}
+	goto exit;
+    }
+
+    /*@-branchstate@*/
+    for (i = 0; i < count; i++) {
+	const char ** needle;
+	char savechar;
+	char * baseName;
+	int len;
+
+	if (fileNames[i] == NULL)	/* XXX can't happen */
+	    continue;
+	baseName = strrchr(fileNames[i], '/') + 1;
+	len = baseName - fileNames[i];
+	needle = dirNames;
+	savechar = *baseName;
+	*baseName = '\0';
+/*@-compdef@*/
+	if (dirIndex < 0 ||
+	    (needle = bsearch(&fileNames[i], dirNames, dirIndex + 1, sizeof(dirNames[0]), dncmp)) == NULL) {
+	    char *s = alloca(len + 1);
+	    memcpy(s, fileNames[i], len + 1);
+	    s[len] = '\0';
+	    dirIndexes[i] = ++dirIndex;
+	    dirNames[dirIndex] = s;
+	} else
+	    dirIndexes[i] = needle - dirNames;
+/*@=compdef@*/
+
+	*baseName = savechar;
+	baseNames[i] = baseName;
+    }
+    /*@=branchstate@*/
+
+exit:
+    if (count > 0) {
+	xx = hae(h, RPMTAG_DIRINDEXES, RPM_INT32_TYPE, dirIndexes, count);
+	xx = hae(h, RPMTAG_BASENAMES, RPM_STRING_ARRAY_TYPE,
+			baseNames, count);
+	xx = hae(h, RPMTAG_DIRNAMES, RPM_STRING_ARRAY_TYPE,
+			dirNames, dirIndex + 1);
+    }
+
+    fileNames = hfd(fileNames, fnt);
+
+    xx = hre(h, RPMTAG_OLDFILENAMES);
+}
+/*@=bounds@*/
+/* make a header with _all_ the tags we need */
+/**
+ */
+static void mungeFilelist(Header h)
+	/*@*/
+{
+    const char ** fileNames = NULL;
+    int count = 0;
+
+    if (!headerIsEntry (h, RPMTAG_BASENAMES)
+	|| !headerIsEntry (h, RPMTAG_DIRNAMES)
+	|| !headerIsEntry (h, RPMTAG_DIRINDEXES))
+	compressFilelist(h);
+
+    rpmfiBuildFNames(h, RPMTAG_BASENAMES, &fileNames, &count);
+
+    if (fileNames == NULL || count <= 0)
+	return;
+
+    /* XXX Legacy tag needs to go away. */
+    headerAddEntry(h, RPMTAG_OLDFILENAMES, RPM_STRING_ARRAY_TYPE,
+			fileNames, count);
+
+    fileNames = _free(fileNames);
+}
+
+/**
+ * Retrofit an explicit Provides: N = E:V-R dependency into package headers.
+ * Up to rpm 3.0.4, packages implicitly provided their own name-version-release.
+ * @param h             header
+ */
+static void providePackageNVR(Header h)
+{
+    HGE_t hge = (HGE_t)headerGetEntryMinMemory;
+    HFD_t hfd = headerFreeData;
+    const char *name, *version, *release;
+    int_32 * epoch;
+    const char *pEVR;
+    char *p;
+    int_32 pFlags = RPMSENSE_EQUAL;
+    const char ** provides = NULL;
+    const char ** providesEVR = NULL;
+    rpmTagType pnt, pvt;
+    int_32 * provideFlags = NULL;
+    int providesCount;
+    int i, xx;
+    int bingo = 1;
+
+    /* Generate provides for this package name-version-release. */
+    xx = headerNVR(h, &name, &version, &release);
+    if (!(name && version && release))
+	return;
+    pEVR = p = alloca(21 + strlen(version) + 1 + strlen(release) + 1);
+    *p = '\0';
+    if (hge(h, RPMTAG_EPOCH, NULL, (void **) &epoch, NULL)) {
+	sprintf(p, "%d:", *epoch);
+	while (*p != '\0')
+	    p++;
+    }
+    (void) stpcpy( stpcpy( stpcpy(p, version) , "-") , release);
+
+    /*
+     * Rpm prior to 3.0.3 does not have versioned provides.
+     * If no provides at all are available, we can just add.
+     */
+    if (!hge(h, RPMTAG_PROVIDENAME, &pnt, (void **) &provides, &providesCount))
+	goto exit;
+
+    /*
+     * Otherwise, fill in entries on legacy packages.
+     */
+    if (!hge(h, RPMTAG_PROVIDEVERSION, &pvt, (void **) &providesEVR, NULL)) {
+	for (i = 0; i < providesCount; i++) {
+	    char * vdummy = "";
+	    int_32 fdummy = RPMSENSE_ANY;
+	    xx = headerAddOrAppendEntry(h, RPMTAG_PROVIDEVERSION, RPM_STRING_ARRAY_TYPE,
+			&vdummy, 1);
+	    xx = headerAddOrAppendEntry(h, RPMTAG_PROVIDEFLAGS, RPM_INT32_TYPE,
+			&fdummy, 1);
+	}
+	goto exit;
+    }
+
+    xx = hge(h, RPMTAG_PROVIDEFLAGS, NULL, (void **) &provideFlags, NULL);
+
+    /*@-nullderef@*/	/* LCL: providesEVR is not NULL */
+    if (provides && providesEVR && provideFlags)
+    for (i = 0; i < providesCount; i++) {
+        if (!(provides[i] && providesEVR[i]))
+            continue;
+	if (!(provideFlags[i] == RPMSENSE_EQUAL &&
+	    !strcmp(name, provides[i]) && !strcmp(pEVR, providesEVR[i])))
+	    continue;
+	bingo = 0;
+	break;
+    }
+    /*@=nullderef@*/
+
+exit:
+    provides = hfd(provides, pnt);
+    providesEVR = hfd(providesEVR, pvt);
+
+    if (bingo) {
+	xx = headerAddOrAppendEntry(h, RPMTAG_PROVIDENAME, RPM_STRING_ARRAY_TYPE,
+		&name, 1);
+	xx = headerAddOrAppendEntry(h, RPMTAG_PROVIDEFLAGS, RPM_INT32_TYPE,
+		&pFlags, 1);
+	xx = headerAddOrAppendEntry(h, RPMTAG_PROVIDEVERSION, RPM_STRING_ARRAY_TYPE,
+		&pEVR, 1);
+    }
+}
+
+/** \ingroup python
+ * \name Class: Rpmhdr
+ */
+/*@{*/
+
+/**
  */
 static PyObject * hdrKeyList(hdrObject * s)
 	/*@*/
@@ -171,11 +421,14 @@ static PyObject * hdrKeyList(hdrObject * s)
         if (tag == HEADER_I18NTABLE) continue;
 
 	switch (type) {
+	case RPM_OPENPGP_TYPE:
+	case RPM_ASN1_TYPE:
 	case RPM_BIN_TYPE:
+	case RPM_INT64_TYPE:
 	case RPM_INT32_TYPE:
-	case RPM_CHAR_TYPE:
-	case RPM_INT8_TYPE:
 	case RPM_INT16_TYPE:
+	case RPM_INT8_TYPE:
+	case RPM_CHAR_TYPE:
 	case RPM_STRING_ARRAY_TYPE:
 	case RPM_STRING_TYPE:
 	    PyList_Append(list, o=PyInt_FromLong(tag));
@@ -187,7 +440,7 @@ static PyObject * hdrKeyList(hdrObject * s)
     return list;
 }
 
-/** \ingroup py_c
+/**
  */
 static PyObject * hdrUnload(hdrObject * s, PyObject * args, PyObject *keywords)
 	/*@*/
@@ -222,7 +475,7 @@ static PyObject * hdrUnload(hdrObject * s, PyObject * args, PyObject *keywords)
     return rc;
 }
 
-/** \ingroup py_c
+/**
  */
 static PyObject * hdrExpandFilelist(hdrObject * s)
 	/*@*/
@@ -233,7 +486,7 @@ static PyObject * hdrExpandFilelist(hdrObject * s)
     return Py_None;
 }
 
-/** \ingroup py_c
+/**
  */
 static PyObject * hdrCompressFilelist(hdrObject * s)
 	/*@*/
@@ -244,32 +497,6 @@ static PyObject * hdrCompressFilelist(hdrObject * s)
     return Py_None;
 }
 
-/* make a header with _all_ the tags we need */
-/** \ingroup py_c
- */
-static void mungeFilelist(Header h)
-	/*@*/
-{
-    const char ** fileNames = NULL;
-    int count = 0;
-
-    if (!headerIsEntry (h, RPMTAG_BASENAMES)
-	|| !headerIsEntry (h, RPMTAG_DIRNAMES)
-	|| !headerIsEntry (h, RPMTAG_DIRINDEXES))
-	compressFilelist(h);
-
-    rpmfiBuildFNames(h, RPMTAG_BASENAMES, &fileNames, &count);
-
-    if (fileNames == NULL || count <= 0)
-	return;
-
-    /* XXX Legacy tag needs to go away. */
-    headerAddEntry(h, RPMTAG_OLDFILENAMES, RPM_STRING_ARRAY_TYPE,
-			fileNames, count);
-
-    fileNames = _free(fileNames);
-}
-
 /**
  */
 static PyObject * rhnUnload(hdrObject * s)
@@ -329,7 +556,40 @@ static PyObject * rhnUnload(hdrObject * s)
     return rc;
 }
 
-/** \ingroup py_c
+/**
+ */
+static PyObject * hdrGetOrigin(hdrObject * s)
+	/*@*/
+{
+    const char * origin = NULL;
+    if (s->h != NULL)
+
+	origin = headerGetOrigin(s->h);
+    if (origin != NULL)
+	return Py_BuildValue("s", origin);
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+/**
+ */
+static PyObject * hdrSetOrigin(hdrObject * s, PyObject * args, PyObject * kwds)
+	/*@*/
+{
+    char * kwlist[] = {"origin", NULL};
+    const char * origin = NULL;
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "s:SetOrigin", kwlist, &origin))
+        return NULL;
+
+    if (s->h != NULL && origin != NULL)
+	headerSetOrigin(s->h, origin);
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+/**
  */
 static PyObject * hdrFullFilelist(hdrObject * s)
 	/*@*/
@@ -340,7 +600,7 @@ static PyObject * hdrFullFilelist(hdrObject * s)
     return Py_None;
 }
 
-/** \ingroup py_c
+/**
  */
 static PyObject * hdrSprintf(hdrObject * s, PyObject * args, PyObject * kwds)
 	/*@*/
@@ -366,18 +626,7 @@ static PyObject * hdrSprintf(hdrObject * s, PyObject * args, PyObject * kwds)
     return result;
 }
 
-/**
- */
-static int hdr_compare(hdrObject * a, hdrObject * b)
-	/*@*/
-{
-    return rpmVersionCompare(a->h, b->h);
-}
-
-static long hdr_hash(PyObject * h)
-{
-    return (long) h;
-}
+/*@}*/
 
 /** \ingroup py_c
  */
@@ -395,6 +644,10 @@ static struct PyMethodDef hdr_methods[] = {
 	NULL },
     {"rhnUnload",	(PyCFunction) rhnUnload,	METH_NOARGS,
 	NULL },
+    {"getorigin",	(PyCFunction) hdrGetOrigin,	METH_NOARGS,
+	NULL },
+    {"setorigin",	(PyCFunction) hdrSetOrigin,	METH_NOARGS,
+	NULL },
     {"sprintf",		(PyCFunction) hdrSprintf,	METH_VARARGS|METH_KEYWORDS,
 	NULL },
 
@@ -408,6 +661,19 @@ static struct PyMethodDef hdr_methods[] = {
     {NULL,		NULL}		/* sentinel */
 };
 
+/**
+ */
+static int hdr_compare(hdrObject * a, hdrObject * b)
+	/*@*/
+{
+    return rpmVersionCompare(a->h, b->h);
+}
+
+static long hdr_hash(PyObject * h)
+{
+    return (long) h;
+}
+
 static PyObject * hdr_getattro(PyObject * o, PyObject * n)
 	/*@*/
 {
@@ -451,6 +717,68 @@ long tagNumFromPyObject (PyObject *item)
     return -1;
 }
 
+/** \ingroup py_c
+ * Retrieve tag info from header.
+ * This is a "dressed" entry to headerGetEntry to do:
+ *	1) DIRNAME/BASENAME/DIRINDICES -> FILENAMES tag conversions.
+ *	2) i18n lookaside (if enabled).
+ *
+ * @param h		header
+ * @param tag		tag
+ * @retval type		address of tag value data type
+ * @retval p		address of pointer to tag value(s)
+ * @retval c		address of number of values
+ * @return		0 on success, 1 on bad magic, 2 on error
+ */
+static int rpmHeaderGetEntry(Header h, int_32 tag, /*@out@*/ int_32 *type,
+		/*@out@*/ void **p, /*@out@*/ int_32 *c)
+	/*@modifies *type, *p, *c @*/
+{
+    switch (tag) {
+    case RPMTAG_OLDFILENAMES:
+    {	const char ** fl = NULL;
+	int count;
+	rpmfiBuildFNames(h, RPMTAG_BASENAMES, &fl, &count);
+	if (count > 0) {
+	    *p = fl;
+	    if (c)	*c = count;
+	    if (type)	*type = RPM_STRING_ARRAY_TYPE;
+	    return 1;
+	}
+	if (c)	*c = 0;
+	return 0;
+    }	/*@notreached@*/ break;
+
+    case RPMTAG_GROUP:
+    case RPMTAG_DESCRIPTION:
+    case RPMTAG_SUMMARY:
+    {	char fmt[128];
+	const char * msgstr;
+	const char * errstr;
+
+	fmt[0] = '\0';
+	(void) stpcpy( stpcpy( stpcpy( fmt, "%{"), tagName(tag)), "}\n");
+
+	/* XXX FIXME: memory leak. */
+        msgstr = headerSprintf(h, fmt, rpmTagTable, rpmHeaderFormats, &errstr);
+	if (msgstr) {
+	    *p = (void *) msgstr;
+	    if (type)	*type = RPM_STRING_TYPE;
+	    if (c)	*c = 1;
+	    return 1;
+	} else {
+	    if (c)	*c = 0;
+	    return 0;
+	}
+    }	/*@notreached@*/ break;
+
+    default:
+	return headerGetEntry(h, tag, type, p, c);
+	/*@notreached@*/ break;
+    }
+    /*@notreached@*/
+}
+
 /** \ingroup py_c
  */
 static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
@@ -537,8 +865,6 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
     case RPMTAG_CONFLICTNAME:
     case RPMTAG_CONFLICTFLAGS:
     case RPMTAG_CONFLICTVERSION:
-    case RPMTAG_CHANGELOGTIME:
-    case RPMTAG_FILEVERIFYFLAGS:
 	forceArray = 1;
 	break;
     case RPMTAG_SUMMARY:
@@ -551,10 +877,25 @@ static PyObject * hdr_subscript(hdrObject * s, PyObject * item)
     }
 
     switch (type) {
+    case RPM_OPENPGP_TYPE:
+    case RPM_ASN1_TYPE:
     case RPM_BIN_TYPE:
 	o = PyString_FromStringAndSize(data, count);
 	break;
 
+    case RPM_INT64_TYPE:
+	if (count != 1 || forceArray) {
+	    metao = PyList_New(0);
+	    for (i = 0; i < count; i++) {
+		o = PyInt_FromLong(((long long *) data)[i]);
+		PyList_Append(metao, o);
+		Py_DECREF(o);
+	    }
+	    o = metao;
+	} else {
+	    o = PyInt_FromLong(*((long long *) data));
+	}
+	break;
     case RPM_INT32_TYPE:
 	if (count != 1 || forceArray) {
 	    metao = PyList_New(0);
@@ -1065,9 +1406,6 @@ PyObject * labelCompare (PyObject * self, PyObject * args)
 			&e1, &v1, &r1, &e2, &v2, &r2))
 	return NULL;
 
-    if (e1 == NULL)	e1 = "0";
-    if (e2 == NULL)	e2 = "0";
-
     rc = compare_values(e1, e2);
     if (!rc) {
 	rc = compare_values(v1, v2);
@@ -1076,5 +1414,3 @@ PyObject * labelCompare (PyObject * self, PyObject * args)
     }
     return Py_BuildValue("i", rc);
 }
-
-/*@}*/
