@@ -7,6 +7,9 @@
  * \file python/rpmdb-py.h
  */
 
+/** \name Type: _rpm.db */
+/*@{*/
+
 /** \ingroup py_c
  */
 typedef struct rpmdbObject_s rpmdbObject;
@@ -22,18 +25,30 @@ struct rpmdbObject_s {
     int *offsets;
 } ;
 
+/** \ingroup py_c
+ */
 /*@unchecked@*/
 extern PyTypeObject rpmdb_Type;
 
+__BEGIN_DECLS
 #ifdef  _LEGACY_BINDINGS_TOO
+/** \ingroup py_c
+ */
 rpmdb dbFromDb(rpmdbObject * db)
 	/*@*/;
 
+/** \ingroup py_c
+ */
 rpmdbObject * rpmOpenDB(PyObject * self, PyObject * args, PyObject * kwds)
 	/*@*/;
+/** \ingroup py_c
+ */
 PyObject * rebuildDB (PyObject * self, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext @*/
 	/*@modifies rpmGlobalMacroContext @*/;
 #endif
 
+__END_DECLS
+/*@}*/
+
 #endif
