@@ -62,101 +62,6 @@ void rpmds_ParseEVR(char * evr,
     if (rp) *rp = release;
 }
 
-/*@null@*/
-static PyObject *
-rpmds_Debug(/*@unused@*/ rpmdsObject * s, PyObject * args, PyObject * kwds)
-	/*@globals _Py_NoneStruct @*/
-	/*@modifies _Py_NoneStruct @*/
-{
-    char * kwlist[] = {"debugLevel", NULL};
-
-    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist, &_rpmds_debug))
-	return NULL;
-
-    Py_INCREF(Py_None);
-    return Py_None;
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Count(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsCount(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Ix(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsIx(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_DNEVR(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("s", rpmdsDNEVR(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_N(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("s", rpmdsN(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_EVR(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("s", rpmdsEVR(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Flags(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsFlags(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_BT(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", (int) rpmdsBT(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_TagN(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsTagN(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Color(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsColor(s->ds));
-}
-
-/*@null@*/
-static PyObject *
-rpmds_Refs(rpmdsObject * s)
-	/*@*/
-{
-    return Py_BuildValue("i", rpmdsRefs(s->ds));
-}
-
 /**
  */
 static int compare_values(const char *str1, const char *str2)
@@ -254,13 +159,18 @@ rpmds_iternext(rpmdsObject * s)
 	if (N != NULL) N = xstrdup(N);
 	if (EVR != NULL) EVR = xstrdup(EVR);
 /*@=branchstate@*/
-	result = rpmds_Wrap( rpmdsSingle(tagN, N, EVR, Flags) );
+	result = (PyObject *)rpmds_Wrap( rpmdsSingle(tagN, N, EVR, Flags) );
     } else
 	s->active = 0;
 
     return result;
 }
 
+/** \ingroup python
+ * \name Class: Rpmds
+ */
+/*@{*/
+
 /*@null@*/
 static PyObject *
 rpmds_Next(rpmdsObject * s)
@@ -278,6 +188,108 @@ rpmds_Next(rpmdsObject * s)
     return result;
 }
 
+/*@null@*/
+static PyObject *
+rpmds_Debug(/*@unused@*/ rpmdsObject * s, PyObject * args, PyObject * kwds)
+	/*@globals _Py_NoneStruct @*/
+	/*@modifies _Py_NoneStruct @*/
+{
+    char * kwlist[] = {"debugLevel", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "i", kwlist, &_rpmds_debug))
+	return NULL;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+/*@null@*/
+static PyObject *
+rpmds_Count(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("i", rpmdsCount(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_Ix(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("i", rpmdsIx(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_DNEVR(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("s", rpmdsDNEVR(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_N(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("s", rpmdsN(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_EVR(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("s", rpmdsEVR(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_Flags(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("i", rpmdsFlags(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_BT(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("i", (int) rpmdsBT(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_TagN(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("i", rpmdsTagN(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_Color(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("i", rpmdsColor(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_Refs(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("i", rpmdsRefs(s->ds));
+}
+
+/*@null@*/
+static PyObject *
+rpmds_Result(rpmdsObject * s)
+	/*@*/
+{
+    return Py_BuildValue("i", rpmdsResult(s->ds));
+}
 /*@null@*/
 static PyObject *
 rpmds_SetNoPromote(rpmdsObject * s, PyObject * args, PyObject * kwds)
@@ -319,7 +331,12 @@ rpmds_Sort(rpmdsObject * s)
 	/*@globals _Py_NoneStruct @*/
 	/*@modifies _Py_NoneStruct @*/
 {
-    /* XXX sort on (N,EVR,F) here. */
+    rpmds nds = NULL;
+
+    if (rpmdsMerge(&nds, s->ds) >= 0) {
+	s->ds = rpmdsFree(s->ds);
+	s->ds = nds;
+    }
     Py_INCREF(Py_None);
     return Py_None;
 }
@@ -331,7 +348,6 @@ rpmds_Find(rpmdsObject * s, PyObject * args, PyObject * kwds)
 {
     PyObject * to = NULL;
     rpmdsObject * o;
-    int rc;
     char * kwlist[] = {"element", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:Find", kwlist, &to))
@@ -343,8 +359,7 @@ rpmds_Find(rpmdsObject * s, PyObject * args, PyObject * kwds)
     /* XXX make sure ods index is valid, real fix in lib/rpmds.c. */
     if (rpmdsIx(o->ds) == -1)	rpmdsSetIx(o->ds, 0);
 
-    rc = rpmdsFind(s->ds, o->ds);
-    return Py_BuildValue("i", rc);
+    return Py_BuildValue("i", rpmdsFind(s->ds, o->ds));
 }
 
 /*@null@*/
@@ -364,6 +379,90 @@ rpmds_Merge(rpmdsObject * s, PyObject * args, PyObject * kwds)
     return Py_BuildValue("i", rpmdsMerge(&s->ds, o->ds));
 }
 
+/*@null@*/
+static PyObject *
+rpmds_Search(rpmdsObject * s, PyObject * args, PyObject * kwds)
+	/*@modifies s @*/
+{
+    PyObject * to = NULL;
+    rpmdsObject * o;
+    char * kwlist[] = {"element", NULL};
+
+    if (!PyArg_ParseTupleAndKeywords(args, kwds, "O:Merge", kwlist, &to))
+	return NULL;
+
+    /* XXX ds type check needed. */
+    o = (rpmdsObject *)to;
+    return Py_BuildValue("i", rpmdsSearch(s->ds, o->ds));
+}
+
+static PyObject *
+rpmds_Cpuinfo(rpmdsObject * s)
+	/*@*/
+{
+    rpmds ds = NULL;
+    int xx;
+
+    /* XXX check return code, permit arg (NULL uses system default). */
+    xx = rpmdsCpuinfo(&ds, NULL);
+
+    return (PyObject *) rpmds_Wrap( ds );
+}
+
+static PyObject *
+rpmds_Rpmlib(rpmdsObject * s)
+	/*@*/
+{
+    rpmds ds = NULL;
+    int xx;
+
+    /* XXX check return code, permit arg (NULL uses system default). */
+    xx = rpmdsRpmlib(&ds, NULL);
+
+    return (PyObject *) rpmds_Wrap( ds );
+}
+
+static PyObject *
+rpmds_Sysinfo(rpmdsObject * s)
+	/*@*/
+{
+    rpmds ds = NULL;
+    int xx;
+
+    /* XXX check return code, permit arg (NULL uses system default). */
+    xx = rpmdsSysinfo(&ds, NULL);
+
+    return (PyObject *) rpmds_Wrap( ds );
+}
+
+static PyObject *
+rpmds_Getconf(rpmdsObject * s)
+	/*@*/
+{
+    rpmds ds = NULL;
+    int xx;
+
+    /* XXX check return code, permit arg (NULL uses system default). */
+    xx = rpmdsGetconf(&ds, NULL);
+
+    return (PyObject *) rpmds_Wrap( ds );
+}
+
+static PyObject *
+rpmds_Ldconfig(rpmdsObject * s)
+	/*@*/
+{
+    rpmPRCO PRCO = memset(alloca(sizeof(*PRCO)), 0, sizeof(*PRCO));
+    rpmds P = NULL;
+    int xx;
+
+    PRCO->Pdsp = &P;
+    /* XXX check return code, permit arg (NULL uses system default). */
+    xx = rpmdsLdconfig(PRCO, NULL);
+
+    return (PyObject *) rpmds_Wrap( P );
+}
+
 #ifdef	NOTYET
 static PyObject *
 rpmds_Compare(rpmdsObject * s, PyObject * args, PyObject * kwds)
@@ -393,6 +492,8 @@ rpmds_Problem(rpmdsObject * s)
 }
 #endif
 
+/*@}*/
+
 /*@-fullinitblock@*/
 /*@unchecked@*/ /*@observer@*/
 static struct PyMethodDef rpmds_methods[] = {
@@ -418,6 +519,8 @@ static struct PyMethodDef rpmds_methods[] = {
 	"ds.Color -> Color	- Return current Color.\n" },
  {"Refs",	(PyCFunction)rpmds_Refs,	METH_NOARGS,
 	"ds.Refs -> Refs	- Return current Refs.\n" },
+ {"Result",	(PyCFunction)rpmds_Result,	METH_NOARGS,
+	"ds.Result -> Result	- Return current Result.\n" },
  {"next",	(PyCFunction)rpmds_Next,	METH_NOARGS,
 "ds.next() -> (N, EVR, Flags)\n\
 - Retrieve next dependency triple.\n" },
@@ -426,11 +529,29 @@ static struct PyMethodDef rpmds_methods[] = {
  {"Notify",	(PyCFunction)rpmds_Notify,	METH_VARARGS|METH_KEYWORDS,
 	NULL},
  {"Sort",	(PyCFunction)rpmds_Sort,	METH_NOARGS,
-	NULL},
+"ds.Sort() -> None\n\
+- Sort the (N,EVR,Flags) elements in ds\n" },
  {"Find",	(PyCFunction)rpmds_Find,	METH_VARARGS|METH_KEYWORDS,
-	NULL},
+"ds.Find(element) -> matching ds index (-1 on failure)\n\
+- Check for an exactly matching element in ds.\n\
+The current index in ds is positioned at matching member upon success.\n" },
  {"Merge",	(PyCFunction)rpmds_Merge,	METH_VARARGS|METH_KEYWORDS,
-	NULL},
+"ds.Merge(elements) -> 0 on success\n\
+- Merge elements into ds, maintaining (N,EVR,Flags) sort order.\n" },
+ {"Search",	(PyCFunction)rpmds_Search,	METH_VARARGS|METH_KEYWORDS,
+"ds.Search(element) -> matching ds index (-1 on failure)\n\
+- Check that element dependency range overlaps some member of ds.\n\
+The current index in ds is positioned at overlapping member upon success.\n" },
+ {"Cpuinfo",	(PyCFunction)rpmds_Cpuinfo,	METH_NOARGS,
+	"ds.Cpuinfo -> nds	- Return /proc/cpuinfo dependency set.\n"},
+ {"Rpmlib",	(PyCFunction)rpmds_Rpmlib,	METH_NOARGS,
+	"ds.Rpmlib -> nds	- Return internal rpmlib dependency set.\n"},
+ {"Sysinfo",	(PyCFunction)rpmds_Sysinfo,	METH_NOARGS,
+	"ds.Sysinfo -> nds	- Return /etc/rpm/sysinfo dependency set.\n"},
+ {"Getconf",	(PyCFunction)rpmds_Getconf,	METH_NOARGS,
+	"ds.Getconf -> nds	- Return getconf(1) dependency set.\n"},
+ {"Ldconfig",	(PyCFunction)rpmds_Ldconfig,	METH_NOARGS,
+	"ds.Ldconfig -> nds	- Return /etc/ld.so.cache dependency set.\n"},
 #ifdef	NOTYET
  {"Compare",	(PyCFunction)rpmds_Compare,	METH_VARARGS|METH_KEYWORDS,
 	NULL},
@@ -511,7 +632,7 @@ static PyMappingMethods rpmds_as_mapping = {
         (objobjargproc)0,		/* mp_ass_subscript */
 };
 
-/** \ingroup py_c
+/**
  */
 static int rpmds_init(rpmdsObject * s, PyObject *args, PyObject *kwds)
 	/*@globals rpmGlobalMacroContext @*/
@@ -543,7 +664,7 @@ fprintf(stderr, "*** rpmds_init(%p,%p,%p)\n", s, args, kwds);
     return 0;
 }
 
-/** \ingroup py_c
+/**
  */
 static void rpmds_free(/*@only@*/ rpmdsObject * s)
 	/*@modifies s @*/
@@ -555,7 +676,7 @@ fprintf(stderr, "%p -- ds %p\n", s, s->ds);
     PyObject_Del((PyObject *)s);
 }
 
-/** \ingroup py_c
+/**
  */
 static PyObject * rpmds_alloc(PyTypeObject * subtype, int nitems)
 	/*@*/
@@ -567,7 +688,7 @@ fprintf(stderr, "*** rpmds_alloc(%p,%d) ret %p\n", subtype, nitems, s);
     return s;
 }
 
-/** \ingroup py_c
+/**
  */
 /*@null@*/
 static PyObject * rpmds_new(PyTypeObject * subtype, PyObject *args, PyObject *kwds)
@@ -663,6 +784,7 @@ rpmds_Wrap(rpmds ds)
     return s;
 }
 
+
 rpmdsObject *
 rpmds_Single(/*@unused@*/ PyObject * s, PyObject * args, PyObject * kwds)
 {
