@@ -7,7 +7,10 @@
  * \file python/rpmds-py.h
  */
 
-/**
+/** \name Type: _rpm.ds */
+/*@{*/
+
+/** \ingroup py_c
  */
 typedef struct rpmdsObject_s {
     PyObject_HEAD
@@ -17,39 +20,43 @@ typedef struct rpmdsObject_s {
     rpmds	ds;
 } rpmdsObject;
 
-/**
+/** \ingroup py_c
  */
 /*@unchecked@*/
 extern PyTypeObject rpmds_Type;
 
-/**
+__BEGIN_DECLS
+/** \ingroup py_c
  */
 /*@null@*/
 rpmds dsFromDs(rpmdsObject * ds)
 	/*@*/;
 
-/**
+/** \ingroup py_c
  */
 /*@null@*/
 rpmdsObject * rpmds_Wrap(rpmds ds)
 	/*@*/;
 
-/**
+/** \ingroup py_c
  */
 /*@null@*/
-rpmdsObject * rpmds_Single(PyObject * s, PyObject * args)
+rpmdsObject * rpmds_Single(PyObject * s, PyObject * args, PyObject * kwds)
 	/*@*/;
 
-/**
+/** \ingroup py_c
  */
 /*@null@*/
-rpmdsObject * hdr_dsFromHeader(PyObject * s, PyObject * args)
+rpmdsObject * hdr_dsFromHeader(PyObject * s, PyObject * args, PyObject * kwds)
 	/*@*/;
 
-/**
+/** \ingroup py_c
  */
 /*@null@*/
-rpmdsObject * hdr_dsOfHeader(PyObject * s, PyObject * args)
+rpmdsObject * hdr_dsOfHeader(PyObject * s)
 	/*@*/;
 
+__END_DECLS
+/*@}*/
+
 #endif
