@@ -5,6 +5,9 @@
  * \file python/rpmmi-py.h
  */
 
+/** \name Type: _rpm.mi */
+/*@{*/
+
 /** \ingroup py_c
  */
 typedef struct rpmmiObject_s rpmmiObject;
@@ -14,15 +17,22 @@ typedef struct rpmmiObject_s rpmmiObject;
 struct rpmmiObject_s {
     PyObject_HEAD
     PyObject *md_dict;		/*!< to look like PyModuleObject */
-    PyObject *ref;		/* for db/ts refcounting */
     rpmdbMatchIterator mi;
 } ;
 
+/** \ingroup py_c
+ */
 /*@unchecked@*/
 extern PyTypeObject rpmmi_Type;
 
+__BEGIN_DECLS
+/** \ingroup py_c
+ */
 /*@null@*/
-rpmmiObject * rpmmi_Wrap(rpmdbMatchIterator mi, PyObject *s)
+rpmmiObject * rpmmi_Wrap(rpmdbMatchIterator mi)
 	/*@*/;
 
+__END_DECLS
+/*@}*/
+
 #endif
