@@ -5,6 +5,9 @@
  * \file python/rpmrc-py.h
  */
 
+/** \name Type: _rpm.rc */
+/*@{*/
+
 /** \ingroup py_c
  */
 typedef struct rpmrcObject_s rpmrcObject;
@@ -23,19 +26,30 @@ struct rpmrcObject_s {
 /*@unchecked@*/
 extern PyTypeObject rpmrc_Type;
 
+__BEGIN_DECLS
+/** \ingroup py_c
+ */
 /*@null@*/
 PyObject * rpmrc_AddMacro(PyObject * self, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies rpmGlobalMacroContext, _Py_NoneStruct @*/;
+
+/** \ingroup py_c
+ */
 /*@null@*/
 PyObject * rpmrc_DelMacro(PyObject * self, PyObject * args, PyObject * kwds)
 	/*@globals rpmGlobalMacroContext, _Py_NoneStruct @*/
 	/*@modifies rpmGlobalMacroContext, _Py_NoneStruct @*/;
 
 #if Py_TPFLAGS_HAVE_ITER	/* XXX backport to python-1.5.2 */
+/** \ingroup py_c
+ */
 /*@null@*/
 PyObject * rpmrc_Create(PyObject * self, PyObject * args, PyObject * kwds)
 	/*@*/;
 #endif
 
+__END_DECLS
+/*@}*/
+
 #endif
