@@ -12,6 +12,7 @@
 
 #define	__HEADER_PROTOTYPES__
 
+#include <rpmio_internal.h>	/* XXX for fdGetOPath() */
 #include <header_internal.h>
 
 #include "debug.h"
@@ -53,8 +54,8 @@ static int typeAlign[16] =  {
     1,	/*!< RPM_BIN_TYPE */
     1,	/*!< RPM_STRING_ARRAY_TYPE */
     1,	/*!< RPM_I18NSTRING_TYPE */
-    0,
-    0,
+    1,  /*!< RPM_ASN1_TYPE */
+    1,  /*!< RPM_OPENPGP_TYPE */
     0,
     0,
     0,
@@ -71,13 +72,13 @@ static int typeSizes[16] =  {
     1,	/*!< RPM_INT8_TYPE */
     2,	/*!< RPM_INT16_TYPE */
     4,	/*!< RPM_INT32_TYPE */
-    -1,	/*!< RPM_INT64_TYPE */
+    8,	/*!< RPM_INT64_TYPE */
     -1,	/*!< RPM_STRING_TYPE */
     1,	/*!< RPM_BIN_TYPE */
     -1,	/*!< RPM_STRING_ARRAY_TYPE */
     -1,	/*!< RPM_I18NSTRING_TYPE */
-    0,
-    0,
+    1,  /*!< RPM_ASN1_TYPE */
+    1,  /*!< RPM_OPENPGP_TYPE */
     0,
     0,
     0,
@@ -206,6 +207,7 @@ Header headerFree(/*@killref@*/ /*@null@*/ Header h)
 	}
 	h->index = _free(h->index);
     }
+    h->origin = _free(h->origin);
 
     /*@-refcounttrans@*/ h = _free(h); /*@=refcounttrans@*/
     return h;
@@ -228,6 +230,8 @@ Header headerNew(void)
     /*@=assignexpose@*/
 /*@=boundsread@*/
     h->blob = NULL;
+    h->origin = NULL;
+    h->instance = 0;
     h->indexAlloced = INDEX_MALLOC_SIZE;
     h->indexUsed = 0;
     h->flags |= HEADERFLAG_SORTED;
@@ -555,6 +559,19 @@ static int regionSwab(/*@null@*/ indexEntry entry, int il, int dl,
 	/* Perform endian conversions */
 	switch (ntohl(pe->type)) {
 /*@-bounds@*/
+	case RPM_INT64_TYPE:
+	{   int_64 * it = (int_64 *)t;
+	    int_32 b[2];
+	    for (; ie.info.count > 0; ie.info.count--, it += 1) {
+		if (dataEnd && ((unsigned char *)it) >= dataEnd)
+		    return -1;
+		b[1] = htonl(((int_32 *)it)[0]);
+		b[0] = htonl(((int_32 *)it)[1]);
+		if (b[1] != ((int_32 *)it)[0])
+		    memcpy(it, b, sizeof(b));
+	    }
+	    t = (char *) it;
+	}   /*@switchbreak@*/ break;
 	case RPM_INT32_TYPE:
 	{   int_32 * it = (int_32 *)t;
 	    for (; ie.info.count > 0; ie.info.count--, it += 1) {
@@ -813,6 +830,22 @@ t = te;
 	/* copy data w/ endian conversions */
 /*@-boundswrite@*/
 	switch (entry->info.type) {
+	case RPM_INT64_TYPE:
+	{   int_32 b[2];
+	    count = entry->info.count;
+	    src = entry->data;
+	    while (count--) {
+		b[1] = htonl(((int_32 *)src)[0]);
+		b[0] = htonl(((int_32 *)src)[1]);
+		if (b[1] == ((int_32 *)src)[0])
+		    memcpy(te, src, sizeof(b));
+		else
+		    memcpy(te, b, sizeof(b));
+		te += sizeof(b);
+		src += sizeof(b);
+	    }
+	}   /*@switchbreak@*/ break;
+
 	case RPM_INT32_TYPE:
 	    count = entry->info.count;
 	    src = entry->data;
@@ -1169,6 +1202,62 @@ errxit:
     /*@=refcounttrans =globstate@*/
 }
 
+/** \ingroup header
+ * Return header origin (e.g path or URL).
+ * @param h		header
+ * @return		header origin
+ */
+static /*@observer@*/ /*@null@*/
+const char * headerGetOrigin(/*@null@*/ Header h)
+	/*@*/
+{
+    return (h != NULL ? h->origin : NULL);
+}
+
+/** \ingroup header
+ * Store header origin (e.g path or URL).
+ * @param h		header
+ * @param origin	new header origin
+ * @return		0 always
+ */
+static
+int headerSetOrigin(/*@null@*/ Header h, const char * origin)
+	/*@modifies h @*/
+{
+    if (h != NULL) {
+	h->origin = _free(h->origin);
+	h->origin = xstrdup(origin);
+    }
+    return 0;
+}
+
+/** \ingroup header
+ * Return header instance (if from rpmdb).
+ * @param h		header
+ * @return		header instance
+ */
+static
+int headerGetInstance(/*@null@*/ Header h)
+	/*@*/
+{
+    return (h != NULL ? h->instance : 0);
+}
+
+/** \ingroup header
+ * Store header instance (e.g path or URL).
+ * @param h		header
+ * @param origin	new header instance
+ * @return		0 always
+ */
+static
+int headerSetInstance(/*@null@*/ Header h, int instance)
+	/*@modifies h @*/
+{
+    if (h != NULL)
+	h->instance = instance;
+    return 0;
+}
+
 /** \ingroup header
  * Convert header to on-disk representation, and then reload.
  * This is used to insure that all header data is in one chunk.
@@ -1186,7 +1275,11 @@ Header headerReload(/*@only@*/ Header h, int tag)
 /*@-boundswrite@*/
     void * uh = doHeaderUnload(h, &length);
 /*@=boundswrite@*/
+    const char * origin;
+    int_32 instance = h->instance;
+    int xx;
 
+    origin = (h->origin != NULL ? xstrdup(h->origin) : NULL);
     h = headerFree(h);
     /*@=onlytrans@*/
     if (uh == NULL)
@@ -1205,6 +1298,11 @@ Header headerReload(/*@only@*/ Header h, int tag)
 	    nh->index[0].info.tag = tag;
 /*@=boundswrite@*/
     }
+    if (origin != NULL) {
+	xx = headerSetOrigin(nh, origin);
+	origin = _free(origin);
+    }
+    xx = headerSetInstance(nh, instance);
     return nh;
 }
 
@@ -1314,6 +1412,11 @@ Header headerRead(FD_t fd, enum hMagic magicp)
     
     h = headerLoad(ei);
 
+    {   const char * origin = fdGetOPath(fd);
+	if (origin != NULL)
+            (void) headerSetOrigin(h, origin);
+    }
+
 exit:
     if (h) {
 	if (h->flags & HEADERFLAG_ALLOCATED)
@@ -1453,7 +1556,7 @@ static int copyEntry(const indexEntry entry,
 	    /*@=sizeoftype@*/
 /*@=bounds@*/
 
-	    rc = regionSwab(NULL, ril, 0, pe, dataStart, dataStart + rdl, 0);
+	    rc = regionSwab(NULL, ril, 0, pe, dataStart, NULL, 0);
 	    /* XXX 1 on success. */
 	    rc = (rc < 0) ? 0 : 1;
 	} else {
@@ -1502,6 +1605,8 @@ static int copyEntry(const indexEntry entry,
 	}
     }	break;
 
+    case RPM_OPENPGP_TYPE:	/* XXX W2DO? */
+    case RPM_ASN1_TYPE:		/* XXX W2DO? */
     default:
 	*p = entry->data;
 	break;
@@ -1527,7 +1632,7 @@ static int copyEntry(const indexEntry entry,
  * @param td		header i18n table data, NUL terminated
  * @param l		start of locale	to match
  * @param le		end of locale to match
- * @return		1 on good match, 2 on weak match, 0 on no match
+ * @return		1 on match, 0 on no match
  */
 static int headerMatchLocale(const char *td, const char *l, const char *le)
 	/*@*/
@@ -1590,7 +1695,7 @@ static int headerMatchLocale(const char *td, const char *l, const char *le)
     for (fe = l; fe < le && *fe != '_'; fe++)
 	{};
     if (fe < le && !strncmp(td, l, (fe - l)))
-	return 2;
+	return 1;
 
     return 0;
 }
@@ -1623,7 +1728,7 @@ headerFindI18NString(Header h, indexEntry entry)
 /*@-boundsread@*/
     for (l = lang; *l != '\0'; l = le) {
 	const char *td;
-	char *ed, *ed_weak = NULL;
+	char *ed;
 	int langNum;
 
 	while (*l && *l == ':')			/* skip leading colons */
@@ -1638,12 +1743,10 @@ headerFindI18NString(Header h, indexEntry entry)
 	     langNum < entry->info.count;
 	     langNum++, td += strlen(td) + 1, ed += strlen(ed) + 1) {
 
-	           int match = headerMatchLocale(td, l, le);
-		   if (match == 1) return ed;
-		   else if (match == 2) ed_weak = ed;
+		if (headerMatchLocale(td, l, le))
+		    return ed;
 
 	}
-	if (ed_weak) return ed_weak;
     }
 /*@=boundsread@*/
 
@@ -1716,7 +1819,9 @@ static /*@null@*/ void * headerFreeTag(/*@unused@*/ Header h,
 	if (type == -1 ||
 	    type == RPM_STRING_ARRAY_TYPE ||
 	    type == RPM_I18NSTRING_TYPE ||
-	    type == RPM_BIN_TYPE)
+	    type == RPM_BIN_TYPE ||
+	    type == RPM_ASN1_TYPE ||
+	    type == RPM_OPENPGP_TYPE)
 		data = _free(data);
 	/*@=branchstate@*/
     }
@@ -1975,7 +2080,6 @@ int headerAppendEntry(Header h, int_32 tag, int_32 type,
 
 /** \ingroup header
  * Add or append element to tag array in header.
- * @todo Arg "p" should have const.
  * @param h		header
  * @param tag		tag
  * @param type		tag value data type
@@ -2731,8 +2835,6 @@ static int parseFormat(headerSprintfArgs hsa, /*@null@*/ char * str,
 		if (xisdigit(*start)) {
 		    i = strtoul(start, &start, 10);
 		    token->u.tag.pad += i;
-		    start = chptr;
-		    break;
 		} else {
 		    start++;
 		}
@@ -2747,7 +2849,7 @@ static int parseFormat(headerSprintfArgs hsa, /*@null@*/ char * str,
 		start++;
 	    }
 
-	    dst = next = start;
+	    next = start;
 	    while (*next && *next != '}') next++;
 	    if (!*next) {
 		hsa->errmsg = _("missing } after %{");
@@ -3041,6 +3143,7 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
     int_32 count, type;
     hPTR_t data;
     unsigned int intVal;
+    uint_64 llVal;
     const char ** strarray;
     int datafree = 0;
     int countBuf;
@@ -3076,6 +3179,8 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
 		return NULL;
 	    }
 	    break;
+	case RPM_OPENPGP_TYPE:
+	case RPM_ASN1_TYPE:
 	case RPM_BIN_TYPE:
 	case RPM_STRING_TYPE:
 	    break;
@@ -3106,7 +3211,7 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
 	strarray = (const char **)data;
 
 	if (tag->fmt)
-	    val = tag->fmt(RPM_STRING_TYPE, strarray[element], buf, tag->pad, element);
+	    val = tag->fmt(RPM_STRING_TYPE, strarray[element], buf, tag->pad, (count > 1 ? element : -1));
 
 	if (val) {
 	    need = strlen(val);
@@ -3123,7 +3228,7 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
 
     case RPM_STRING_TYPE:
 	if (tag->fmt)
-	    val = tag->fmt(RPM_STRING_TYPE, data, buf, tag->pad,  0);
+	    val = tag->fmt(RPM_STRING_TYPE, data, buf, tag->pad,  -1);
 
 	if (val) {
 	    need = strlen(val);
@@ -3137,6 +3242,22 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
 	}
 	break;
 
+    case RPM_INT64_TYPE:
+	llVal = *(((int_64 *) data) + element);
+	if (tag->fmt)
+	    val = tag->fmt(RPM_INT64_TYPE, &llVal, buf, tag->pad, (count > 1 ? element : -1));
+	if (val) {
+	    need = strlen(val);
+	} else {
+	    need = 10 + tag->pad + 40;
+	    val = xmalloc(need+1);
+	    strcat(buf, "lld");
+	    /*@-formatconst@*/
+	    sprintf(val, buf, llVal);
+	    /*@=formatconst@*/
+	}
+	break;
+
     case RPM_CHAR_TYPE:
     case RPM_INT8_TYPE:
     case RPM_INT16_TYPE:
@@ -3156,7 +3277,7 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
 	}
 
 	if (tag->fmt)
-	    val = tag->fmt(RPM_INT32_TYPE, &intVal, buf, tag->pad, element);
+	    val = tag->fmt(RPM_INT32_TYPE, &intVal, buf, tag->pad, (count > 1 ? element : -1));
 
 	if (val) {
 	    need = strlen(val);
@@ -3170,6 +3291,8 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
 	}
 	break;
 
+    case RPM_OPENPGP_TYPE:	/* XXX W2DO? */
+    case RPM_ASN1_TYPE:		/* XXX W2DO? */
     case RPM_BIN_TYPE:
 	/* XXX HACK ALERT: element field abused as no. bytes of binary data. */
 	if (tag->fmt)
@@ -3178,8 +3301,25 @@ static char * formatValue(headerSprintfArgs hsa, sprintfTag tag, int element)
 	if (val) {
 	    need = strlen(val);
 	} else {
-	    val = bin2hex(data, count);
-	    need = strlen(val) + tag->pad;
+#ifdef	NOTYET
+	    val = memcpy(xmalloc(count), data, count);
+#else
+	    /* XXX format string not used */
+	    static char hex[] = "0123456789abcdef";
+	    const char * s = data;
+
+/*@-boundswrite@*/
+	    need = 2*count + tag->pad;
+	    val = t = xmalloc(need+1);
+	    while (count-- > 0) {
+		unsigned int i;
+		i = *s++;
+		*t++ = hex[ (i >> 4) & 0xf ];
+		*t++ = hex[ (i     ) & 0xf ];
+	    }
+	    *t = '\0';
+/*@=boundswrite@*/
+#endif
 	}
 	break;
 
@@ -3295,7 +3435,7 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 /*@=boundswrite@*/
 	    } 
 
-	    if (type == RPM_BIN_TYPE)
+	    if (type == RPM_BIN_TYPE || type == RPM_ASN1_TYPE || type == RPM_OPENPGP_TYPE)
 		count = 1;	/* XXX count abused as no. of bytes. */
 
 	    if (numElements > 1 && count != numElements)
@@ -3305,6 +3445,8 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 			_("array iterator used with different sized arrays");
 		return NULL;
 		/*@notreached@*/ /*@switchbreak@*/ break;
+	    case RPM_OPENPGP_TYPE:
+	    case RPM_ASN1_TYPE:
 	    case RPM_BIN_TYPE:
 	    case RPM_STRING_TYPE:
 		/*@switchbreak@*/ break;
@@ -3314,38 +3456,65 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 	}
 
 	if (numElements == -1) {
-	    need = sizeof("(none)") - 1;
+#ifdef	DYING	/* XXX lots of pugly "(none)" lines with --conflicts. */
+	    need = sizeof("(none)\n") - 1;
 	    t = hsaReserve(hsa, need);
 /*@-boundswrite@*/
-	    te = stpcpy(t, "(none)");
+	    te = stpcpy(t, "(none)\n");
 /*@=boundswrite@*/
 	    hsa->vallen += (te - t);
+#endif
 	} else {
 	    int isxml;
+	    int isyaml;
 
-	    need = numElements * token->u.array.numTokens * 10;
+	    need = numElements * token->u.array.numTokens;
 	    if (need == 0) break;
 
 	    spft = token->u.array.format;
 	    isxml = (spft->type == PTOK_TAG && spft->u.tag.type != NULL &&
 		!strcmp(spft->u.tag.type, "xml"));
+	    isyaml = (spft->type == PTOK_TAG && spft->u.tag.type != NULL &&
+		!strcmp(spft->u.tag.type, "yaml"));
 
 	    if (isxml) {
 		const char * tagN = myTagName(hsa->tags, spft->u.tag.tag);
 
+assert(tagN != NULL);	/* XXX splats pick up old or unknown tags. */
 		need = sizeof("  <rpmTag name=\"\">\n") - 1;
 		if (tagN != NULL)
 		    need += strlen(tagN);
-		t = hsaReserve(hsa, need);
+		te = t = hsaReserve(hsa, need);
 /*@-boundswrite@*/
-		te = stpcpy(t, "  <rpmTag name=\"");
+		te = stpcpy(te, "  <rpmTag name=\"");
 		if (tagN != NULL)
 		    te = stpcpy(te, tagN);
 		te = stpcpy(te, "\">\n");
 /*@=boundswrite@*/
 		hsa->vallen += (te - t);
 	    }
+	    if (isyaml) {
+		const char * tagN = myTagName(hsa->tags, spft->u.tag.tag);
 
+assert(tagN != NULL);	/* XXX splats pick up old or unknown tags. */
+		need = sizeof("  : ") - 1;
+		if (tagN != NULL)
+		    need += strlen(tagN);
+		if (numElements > 1)
+		    need += sizeof("\n") - 1;
+		te = t = hsaReserve(hsa, need);
+/*@-boundswrite@*/
+		te = stpcpy(te, "  ");
+		if (tagN != NULL)
+		    te = stpcpy(te, tagN);
+		te = stpcpy(te, ": ");
+		if (numElements > 1)
+		    te = stpcpy(te, "\n");
+/*@=boundswrite@*/
+		hsa->vallen += (te - t);
+	    }
+
+	    need = numElements * token->u.array.numTokens * 10;
 	    t = hsaReserve(hsa, need);
 	    for (j = 0; j < numElements; j++) {
 		spft = token->u.array.format;
@@ -3358,12 +3527,22 @@ static char * singleSprintf(headerSprintfArgs hsa, sprintfToken token,
 
 	    if (isxml) {
 		need = sizeof("  </rpmTag>\n") - 1;
-		t = hsaReserve(hsa, need);
+		te = t = hsaReserve(hsa, need);
 /*@-boundswrite@*/
-		te = stpcpy(t, "  </rpmTag>\n");
+		te = stpcpy(te, "  </rpmTag>\n");
 /*@=boundswrite@*/
 		hsa->vallen += (te - t);
 	    }
+	    if (isyaml) {
+#if 0
+		need = sizeof("\n") - 1;
+		te = t = hsaReserve(hsa, need);
+/*@-boundswrite@*/
+		te = stpcpy(te, "\n");
+/*@=boundswrite@*/
+		hsa->vallen += (te - t);
+#endif
+	    }
 
 	}
 	break;
@@ -3445,6 +3624,7 @@ char * headerSprintf(Header h, const char * fmt,
     sprintfTag tag;
     char * t, * te;
     int isxml;
+    int isyaml;
     int need;
  
     hsa->h = headerLink(h);
@@ -3470,6 +3650,7 @@ char * headerSprintf(Header h, const char * fmt,
 	    ? &hsa->format->u.array.format->u.tag :
 	NULL));
     isxml = (tag != NULL && tag->tag == -2 && tag->type != NULL && !strcmp(tag->type, "xml"));
+    isyaml = (tag != NULL && tag->tag == -2 && tag->type != NULL && !strcmp(tag->type, "yaml"));
 
     if (isxml) {
 	need = sizeof("<rpmHeader>\n") - 1;
@@ -3479,6 +3660,14 @@ char * headerSprintf(Header h, const char * fmt,
 /*@=boundswrite@*/
 	hsa->vallen += (te - t);
     }
+    if (isyaml) {
+	need = sizeof("- !!omap\n") - 1;
+	t = hsaReserve(hsa, need);
+/*@-boundswrite@*/
+	te = stpcpy(t, "- !!omap\n");
+/*@=boundswrite@*/
+	hsa->vallen += (te - t);
+    }
 
     hsa = hsaInit(hsa);
     while ((nextfmt = hsaNext(hsa)) != NULL) {
@@ -3498,6 +3687,14 @@ char * headerSprintf(Header h, const char * fmt,
 /*@=boundswrite@*/
 	hsa->vallen += (te - t);
     }
+    if (isyaml) {
+	need = sizeof("\n") - 1;
+	t = hsaReserve(hsa, need);
+/*@-boundswrite@*/
+	te = stpcpy(t, "\n");
+/*@=boundswrite@*/
+	hsa->vallen += (te - t);
+    }
 
     if (hsa->val != NULL && hsa->vallen < hsa->alloced)
 	hsa->val = xrealloc(hsa->val, hsa->vallen+1);	
@@ -3529,9 +3726,7 @@ static char * octalFormat(int_32 type, hPTR_t data,
 {
     char * val;
 
-    if (type != RPM_INT32_TYPE) {
-	val = xstrdup(_("(not a number)"));
-    } else {
+    if (type == RPM_INT32_TYPE) {
 	val = xmalloc(20 + padding);
 /*@-boundswrite@*/
 	strcat(formatPrefix, "o");
@@ -3539,7 +3734,16 @@ static char * octalFormat(int_32 type, hPTR_t data,
 	/*@-formatconst@*/
 	sprintf(val, formatPrefix, *((int_32 *) data));
 	/*@=formatconst@*/
-    }
+    } else if (type == RPM_INT64_TYPE) {
+	val = xmalloc(40 + padding);
+/*@-boundswrite@*/
+	strcat(formatPrefix, "llo");
+/*@=boundswrite@*/
+	/*@-formatconst@*/
+	sprintf(val, formatPrefix, *((int_64 *) data));
+	/*@=formatconst@*/
+    } else
+	val = xstrdup(_("(not a number)"));
 
     return val;
 }
@@ -3558,9 +3762,7 @@ static char * hexFormat(int_32 type, hPTR_t data,
 {
     char * val;
 
-    if (type != RPM_INT32_TYPE) {
-	val = xstrdup(_("(not a number)"));
-    } else {
+    if (type == RPM_INT32_TYPE) {
 	val = xmalloc(20 + padding);
 /*@-boundswrite@*/
 	strcat(formatPrefix, "x");
@@ -3568,7 +3770,16 @@ static char * hexFormat(int_32 type, hPTR_t data,
 	/*@-formatconst@*/
 	sprintf(val, formatPrefix, *((int_32 *) data));
 	/*@=formatconst@*/
-    }
+    } else if (type == RPM_INT64_TYPE) {
+	val = xmalloc(40 + padding);
+/*@-boundswrite@*/
+	strcat(formatPrefix, "llx");
+/*@=boundswrite@*/
+	/*@-formatconst@*/
+	sprintf(val, formatPrefix, *((int_64 *) data));
+	/*@=formatconst@*/
+    } else
+	val = xstrdup(_("(not a number)"));
 
     return val;
 }
@@ -3662,6 +3873,14 @@ static char * shescapeFormat(int_32 type, hPTR_t data,
 	/*@-formatconst@*/
 	sprintf(result, formatPrefix, *((int_32 *) data));
 	/*@=formatconst@*/
+    } else if (type == RPM_INT64_TYPE) {
+	result = xmalloc(padding + 40);
+/*@-boundswrite@*/
+	strcat(formatPrefix, "lld");
+/*@=boundswrite@*/
+	/*@-formatconst@*/
+	sprintf(result, formatPrefix, *((int_64 *) data));
+	/*@=formatconst@*/
     } else {
 	buf = alloca(strlen(data) + padding + 2);
 /*@-boundswrite@*/
@@ -3766,6 +3985,10 @@ static struct HV_s hdrVec1 = {
     headerFreeIterator,
     headerInitIterator,
     headerNextIterator,
+    headerGetOrigin,
+    headerSetOrigin,
+    headerGetInstance,
+    headerSetInstance,
     NULL, NULL,
     1
 };
