@@ -48,8 +48,8 @@
  *	  changes to the original header.
  *	- (rpm-4.0.3) Private methods (e.g. headerLoad(), headerUnload(), etc.)
  *	  to permit header data to be manipulated opaquely through vectors.
- *	- (rpm-4.0.3) Sanity checks on header data to limit #tags to 65K,
- *	  #bytes to 16Mb, and total metadata size to 32Mb added.
+ *	- (rpm-4.0.3) Sanity checks on header data to limit no. of tags to 65K,
+ *	  no. of bytes to 16Mb, and total metadata size to 32Mb added.
  *	- with addition of tracking dependencies, the package version has been
  *	  reverted back to 3.
  * .
@@ -86,16 +86,14 @@
 extern "C" {
 #endif
 
-#if 0	/* XXX hpux needs -Ae in CFLAGS to grok this */
+/* XXX hpux needs -Ae in CFLAGS to grok this */
 typedef long long int int_64;
-#endif
 typedef int int_32;
 typedef short int int_16;
 typedef char int_8;
 
-#if 0	/* XXX hpux needs -Ae in CFLAGS to grok this */
+/* XXX hpux needs -Ae in CFLAGS to grok this */
 typedef unsigned long long int uint_64;
-#endif
 typedef unsigned int uint_32;
 typedef unsigned short uint_16;
 typedef unsigned char uint_8;
@@ -124,12 +122,41 @@ typedef /*@abstract@*/ struct headerIterator_s * HeaderIterator;
  * Associate tag names with numeric values.
  */
 typedef /*@abstract@*/ struct headerTagTableEntry_s * headerTagTableEntry;
+#if !defined(SWIG)
 struct headerTagTableEntry_s {
-/*@observer@*/ /*@null@*/
+/*@observer@*/ /*@relnull@*/
     const char * name;		/*!< Tag name. */
     int val;			/*!< Tag numeric value. */
     int type;			/*!< Tag type. */
 };
+#endif
+
+/**
+ */
+typedef /*@abstract@*/ struct headerTagIndices_s * headerTagIndices;
+#if !defined(SWIG)
+struct headerTagIndices_s {
+    int (*loadIndex) (headerTagTableEntry ** ipp, int * np,
+                int (*cmp) (const void * avp, const void * bvp))
+        /*@ modifies *ipp, *np */;	/*!< load sorted tag index. */
+/*@relnull@*/
+    headerTagTableEntry * byName;	/*!< header tags sorted by name. */
+    int byNameSize;			/*!< no. of entries. */
+    int (*byNameCmp) (const void * avp, const void * bvp)
+        /*@*/;				/*!< compare entries by name. */
+    int (*tagValue) (const char * name)
+	/*@*/;				/* return value from name. */
+/*@relnull@*/
+    headerTagTableEntry * byValue;	/*!< header tags sorted by value. */
+    int byValueSize;			/*!< no. of entries. */
+    int (*byValueCmp) (const void * avp, const void * bvp)
+        /*@*/;				/*!< compare entries by value. */
+    const char * (*tagName) (int value)
+	/*@*/;				/* Return name from value. */
+    int (*tagType) (int value)
+	/*@*/;				/* Return type from value. */
+};
+#endif
 
 /** \ingroup header
  */
@@ -180,6 +207,7 @@ typedef int (*headerTagTagFunction) (Header h,
  * Define header tag output formats.
  */
 typedef /*@abstract@*/ struct headerSprintfExtension_s * headerSprintfExtension;
+#if !defined(SWIG)
 struct headerSprintfExtension_s {
     enum headerSprintfExtensionType type;	/*!< Type of extension. */
 /*@observer@*/ /*@null@*/
@@ -192,6 +220,7 @@ struct headerSprintfExtension_s {
 	struct headerSprintfExtension_s * more;	/*!< Chained table extension. */
     } u;
 };
+#endif
 
 /** \ingroup header
  * Supported default header tag output formats.
@@ -213,19 +242,22 @@ enum hMagic {
  * The basic types of data in tags from headers.
  */
 typedef enum rpmTagType_e {
-#define	RPM_MIN_TYPE		0
     RPM_NULL_TYPE		=  0,
     RPM_CHAR_TYPE		=  1,
     RPM_INT8_TYPE		=  2,
     RPM_INT16_TYPE		=  3,
     RPM_INT32_TYPE		=  4,
-/*    RPM_INT64_TYPE	= 5,   ---- These aren't supported (yet) */
+    RPM_INT64_TYPE		=  5,
     RPM_STRING_TYPE		=  6,
     RPM_BIN_TYPE		=  7,
     RPM_STRING_ARRAY_TYPE	=  8,
-    RPM_I18NSTRING_TYPE		=  9
-#define	RPM_MAX_TYPE		9
+    RPM_I18NSTRING_TYPE		=  9,
+    RPM_ASN1_TYPE		= 10,
+    RPM_OPENPGP_TYPE		= 11,
+    RPM_MASK_TYPE		= 0x0000ffff
 } rpmTagType;
+#define	RPM_MIN_TYPE		0
+#define	RPM_MAX_TYPE		11
 
 /** \ingroup header
  * New rpm data types under consideration/development.
@@ -247,6 +279,19 @@ typedef enum rpmSubTagType_e {
 } rpmSubTagType;
 /*@=enummemuse =typeuse @*/
 
+/** \ingroup header
+ * Identify how to return the header data type.
+ */
+/*@-enummemuse -typeuse @*/
+typedef enum rpmTagReturnType_e {
+    RPM_ANY_RETURN_TYPE		= 0,
+    RPM_SCALAR_RETURN_TYPE	= 0x00010000,
+    RPM_ARRAY_RETURN_TYPE	= 0x00020000,
+    RPM_MAPPING_RETURN_TYPE	= 0x00040000,
+    RPM_MASK_RETURN_TYPE	= 0xffff0000
+} rpmTagReturnType;
+/*@=enummemuse =typeuse @*/
+
 /**
  * Header private tags.
  * @note General use tags should start at 1000 (RPM's tag space starts there).
@@ -262,7 +307,9 @@ typedef enum rpmSubTagType_e {
 /**
  */
 /*@-typeuse -fielduse@*/
-typedef union hRET_s {
+typedef union hRET_s * hRET_t;
+#if !defined(SWIG)
+union hRET_s {
     const void * ptr;
     const char ** argv;
     const char * str;
@@ -271,13 +318,16 @@ typedef union hRET_s {
     int_32 * i32p;
     int_16 * i16p;
     int_8 * i8p;
-} * hRET_t;
+};
+#endif
 /*@=typeuse =fielduse@*/
 
 /**
  */
 /*@-typeuse -fielduse@*/
-typedef struct HE_s {
+typedef struct HE_s * HE_t;
+#if !defined(SWIG)
+struct HE_s {
     int_32 tag;
 /*@null@*/
     hTYP_t typ;
@@ -289,7 +339,8 @@ typedef struct HE_s {
     } u;
 /*@null@*/
     hCNT_t cnt;
-} * HE_t;
+};
+#endif
 /*@=typeuse =fielduse@*/
 
 /** \ingroup header
@@ -650,10 +701,49 @@ int (*HDRnextiter) (HeaderIterator hi,
 		/*@null@*/ /*@out@*/ hCNT_t c)
 	/*@modifies hi, *tag, *type, *p, *c @*/;
 
+/** \ingroup header
+ * Return header origin (e.g path or URL).
+ * @param h		header
+ * @return		header origin
+ */
+typedef /*@observer@*/ /*@null@*/
+const char * (*HDRgetorigin) (/*@null@*/ Header h)
+	/*@*/;
+
+/** \ingroup header
+ * Store header origin (e.g path or URL).
+ * @param h		header
+ * @param origin	new header origin
+ * @return		0 always
+ */
+typedef
+int (*HDRsetorigin) (/*@null@*/ Header h, const char * origin)
+	/*@modifies h @*/;
+
+/** \ingroup header
+ * Return header instance (if from rpmdb).
+ * @param h		header
+ * @return		header instance
+ */
+typedef
+int (*HDRgetinstance) (/*@null@*/ Header h)
+	/*@*/;
+
+/** \ingroup header
+ * Store header instance (e.g path or URL).
+ * @param h		header
+ * @param origin	new header instance
+ * @return		0 always
+ */
+typedef
+int (*HDRsetinstance) (/*@null@*/ Header h, int instance)
+	/*@modifies h @*/;
+
 /** \ingroup header
  * Header method vectors.
  */
 typedef /*@abstract@*/ struct HV_s * HV_t;
+#if !defined(SWIG)
 struct HV_s {
     HDRlink	hdrlink;
     HDRunlink	hdrunlink;
@@ -684,13 +774,19 @@ struct HV_s {
     HDRfreeiter	hdrfreeiter;
     HDRinititer	hdrinititer;
     HDRnextiter	hdrnextiter;
+    HDRgetorigin hdrgetorigin;
+    HDRsetorigin hdrsetorigin;
+    HDRgetinstance hdrgetinstance;
+    HDRsetinstance hdrsetinstance;
 /*@null@*/
     void *	hdrvecs;
 /*@null@*/
     void *	hdrdata;
     int		hdrversion;
 };
+#endif
 
+#if !defined(SWIG)
 /** \ingroup header
  * Free data allocated when retrieved from header.
  * @deprecated Use headerFreeTag() instead.
@@ -715,6 +811,7 @@ void * headerFreeData( /*@only@*/ /*@null@*/ const void * data, rpmTagType type)
     }
     return NULL;
 }
+#endif
 
 #if !defined(__HEADER_PROTOTYPES__)
 #include "hdrinline.h"
