@@ -28,10 +28,13 @@ extern void regfree (/*@only@*/ regex_t *preg)
 /*@=declundef =exportheader @*/
 #endif
 
-#include <rpmio_internal.h>
+#include <rpmio.h>
+#include <rpmpgp.h>
+#include <rpmurl.h>
 #include <rpmmacro.h>
 #include <rpmsq.h>
 
+#define	_RPMDB_INTERNAL
 #include "rpmdb.h"
 #include "fprint.h"
 #include "legacy.h"
@@ -43,7 +46,6 @@ extern void regfree (/*@only@*/ regex_t *preg)
 /*@access rpmts@*/		/* XXX compared with NULL */
 /*@access Header@*/		/* XXX compared with NULL */
 /*@access rpmdbMatchIterator@*/
-/*@access pgpDig@*/
 
 /*@unchecked@*/
 int _rpmdb_debug = 0;
@@ -62,12 +64,6 @@ static int _db_filter_dups = 0;
 /*@unchecked@*/
 int dbiTagsMax = 0;
 
-/* We use this to comunicate back to the the rpm transaction
- *  what their install instance was on a rpmdbAdd().
- */ 
-/*@unchecked@*/
-unsigned int myinstall_instance = 0;
-
 /* Bit mask macros. */
 /*@-exporttype@*/
 typedef	unsigned int __pbm_bits;
@@ -175,7 +171,8 @@ static int dbiTagToDbix(int rpmtag)
 /**
  * Initialize database (index, tag) tuple from configuration.
  */
-static void dbiTagsInit(void)
+/*@-exportheader@*/
+void dbiTagsInit(void)
 	/*@globals dbiTags, dbiTagsMax, rpmGlobalMacroContext, h_errno @*/
 	/*@modifies dbiTags, dbiTagsMax, rpmGlobalMacroContext @*/
 {
@@ -228,6 +225,7 @@ static void dbiTagsInit(void)
 
     dbiTagStr = _free(dbiTagStr);
 }
+/*@=exportheader@*/
 
 /*@-redecl@*/
 #define	DB1vec		NULL
@@ -269,6 +267,11 @@ dbiIndex dbiOpen(rpmdb db, rpmTag rpmtag, /*@unused@*/ unsigned int flags)
     int _dbapi, _dbapi_rebuild, _dbapi_wanted;
     int rc = 0;
 
+/*@-modfilesys@*/
+if (_rpmdb_debug)
+fprintf(stderr, "==> %s(%p, %s, 0x%x)\n", __FUNCTION__, db, tagName(rpmtag), flags);
+/*@=modfilesys@*/
+
     if (db == NULL)
 	return NULL;
 
@@ -795,22 +798,6 @@ static struct rpmdb_s dbTemplate = {
 };
 /*@=fullinitblock@*/
 
-static int isTemporaryDB(int rpmtag) 
-{
-    int rc = 0;
-    switch (rpmtag) {
-    case RPMDBI_AVAILABLE:
-    case RPMDBI_ADDED:
-    case RPMDBI_REMOVED:
-    case RPMDBI_DEPENDS:
-	rc = 1;
-	break;
-    default:
-	break;
-    }
-    return rc;
-}
-
 int rpmdbOpenAll(rpmdb db)
 {
     int dbix;
@@ -822,9 +809,6 @@ int rpmdbOpenAll(rpmdb db)
     for (dbix = 0; dbix < dbiTagsMax; dbix++) {
 	if (db->_dbi[dbix] != NULL)
 	    continue;
-	/* Filter out temporary databases */
-	if (isTemporaryDB(dbiTags[dbix])) 
-	    continue;
 	(void) dbiOpen(db, dbiTags[dbix], db->db_flags);
     }
     return rc;
@@ -925,17 +909,16 @@ int rpmdbSync(rpmdb db)
 	int xx;
 	if (db->_dbi[dbix] == NULL)
 	    continue;
-	if (db->_dbi[dbix]->dbi_no_dbsync)
-	    continue;
     	xx = dbiSync(db->_dbi[dbix], 0);
 	if (xx && rc == 0) rc = xx;
     }
     return rc;
 }
 
+/*@-exportheader@*/
 /*@-mods@*/	/* FIX: dbTemplate structure assignment */
-static /*@only@*/ /*@null@*/
-rpmdb newRpmdb(/*@kept@*/ /*@null@*/ const char * root,
+/*@only@*/ /*@null@*/
+rpmdb rpmdbNew(/*@kept@*/ /*@null@*/ const char * root,
 		/*@kept@*/ /*@null@*/ const char * home,
 		int mode, int perms, int flags)
 	/*@globals _db_filter_dups, rpmGlobalMacroContext, h_errno @*/
@@ -945,6 +928,11 @@ rpmdb newRpmdb(/*@kept@*/ /*@null@*/ const char * root,
     const char * epfx = _DB_ERRPFX;
     static int _initialized = 0;
 
+/*@-modfilesys@*/ /*@-nullpass@*/
+if (_rpmdb_debug)
+fprintf(stderr, "==> %s(%s, %s, 0x%x, 0%o, 0x%x) db %p\n", __FUNCTION__, root, home, mode, perms, flags, db);
+/*@=modfilesys@*/ /*@=nullpass@*/
+
     if (!_initialized) {
 	_db_filter_dups = rpmExpandNumeric("%{_filterdbdups}");
 	_initialized = 1;
@@ -1005,8 +993,10 @@ rpmdb newRpmdb(/*@kept@*/ /*@null@*/ const char * root,
     /*@=globstate@*/
 }
 /*@=mods@*/
+/*@=exportheader@*/
 
-static int openDatabase(/*@null@*/ const char * prefix,
+/*@-exportheader@*/
+int rpmdbOpenDatabase(/*@null@*/ const char * prefix,
 		/*@null@*/ const char * dbpath,
 		int _dbapi, /*@null@*/ /*@out@*/ rpmdb *dbp,
 		int mode, int perms, int flags)
@@ -1038,7 +1028,7 @@ static int openDatabase(/*@null@*/ const char * prefix,
     if (mode & O_WRONLY) 
 	return 1;
 
-    db = newRpmdb(prefix, dbpath, mode, perms, flags);
+    db = rpmdbNew(prefix, dbpath, mode, perms, flags);
     if (db == NULL)
 	return 1;
 
@@ -1059,8 +1049,16 @@ static int openDatabase(/*@null@*/ const char * prefix,
 	    int rpmtag;
 
 	    /* Filter out temporary databases */
-	    if (isTemporaryDB((rpmtag = dbiTags[dbix])))
+	    switch ((rpmtag = dbiTags[dbix])) {
+	    case RPMDBI_AVAILABLE:
+	    case RPMDBI_ADDED:
+	    case RPMDBI_REMOVED:
+	    case RPMDBI_DEPENDS:
 		continue;
+		/*@notreached@*/ /*@switchbreak@*/ break;
+	    default:
+		/*@switchbreak@*/ break;
+	    }
 
 	    dbi = dbiOpen(db, rpmtag, 0);
 	    if (dbi == NULL) {
@@ -1101,6 +1099,7 @@ exit:
 
     return rc;
 }
+/*@=exportheader@*/
 
 rpmdb XrpmdbUnlink(rpmdb db, const char * msg, const char * fn, unsigned ln)
 {
@@ -1127,7 +1126,7 @@ int rpmdbOpen (const char * prefix, rpmdb *dbp, int mode, int perms)
 {
     int _dbapi = rpmExpandNumeric("%{_dbapi}");
 /*@-boundswrite@*/
-    return openDatabase(prefix, NULL, _dbapi, dbp, mode, perms, 0);
+    return rpmdbOpenDatabase(prefix, NULL, _dbapi, dbp, mode, perms, 0);
 /*@=boundswrite@*/
 }
 
@@ -1138,7 +1137,7 @@ int rpmdbInit (const char * prefix, int perms)
     int rc;
 
 /*@-boundswrite@*/
-    rc = openDatabase(prefix, NULL, _dbapi, &db, (O_CREAT | O_RDWR),
+    rc = rpmdbOpenDatabase(prefix, NULL, _dbapi, &db, (O_CREAT | O_RDWR),
 		perms, RPMDB_FLAG_JUSTCHECK);
 /*@=boundswrite@*/
     if (db != NULL) {
@@ -1159,7 +1158,7 @@ int rpmdbVerify(const char * prefix)
     int rc = 0;
 
 /*@-boundswrite@*/
-    rc = openDatabase(prefix, NULL, _dbapi, &db, O_RDONLY, 0644, 0);
+    rc = rpmdbOpenDatabase(prefix, NULL, _dbapi, &db, O_RDONLY, 0644, 0);
 /*@=boundswrite@*/
 
     if (db != NULL) {
@@ -1602,6 +1601,24 @@ static rpmRC dbiFindByLabel(dbiIndex dbi, DBC * dbcursor, DBT * key, DBT * data,
     /*@=nullstate@*/
 }
 
+void * dbiStatsAccumulator(dbiIndex dbi, int opx)
+{
+    void * sw = NULL;
+    switch (opx) {
+    case 14:	/* RPMTS_OP_DBGET */
+	sw = &dbi->dbi_rpmdb->db_getops;
+	break;
+    case 15:	/* RPMTS_OP_DBPUT */
+	sw = &dbi->dbi_rpmdb->db_putops;
+	break;
+    default:	/* XXX wrong, but let's not return NULL. */
+    case 16:	/* RPMTS_OP_DBDEL */
+	sw = &dbi->dbi_rpmdb->db_delops;
+	break;
+    }
+    return sw;
+}
+
 /**
  * Rewrite a header into packages (if necessary) and free the header.
  *   Note: this is called from a markReplacedFiles iteration, and *must*
@@ -2024,14 +2041,16 @@ static int mireSkip (const rpmdbMatchIterator mi)
     int rc;
 
     if (mi->mi_h == NULL)	/* XXX can't happen */
-	return 0;
+	return 1;
 
     /*
      * Apply tag tests, implicitly "||" for multiple patterns/values of a
      * single tag, implicitly "&&" between multiple tag patterns.
      */
 /*@-boundsread@*/
-    if ((mire = mi->mi_re) != NULL)
+    if ((mire = mi->mi_re) == NULL)
+	return 0;
+
     for (i = 0; i < mi->mi_nre; i++, mire++) {
 	int anymatch;
 
@@ -2103,7 +2122,7 @@ static int mireSkip (const rpmdbMatchIterator mi)
 	    nmatches++;
     }
 
-    return (ntags == nmatches ? 0 : 1);
+    return (ntags > 0 && ntags == nmatches ? 0 : 1);
 }
 
 int rpmdbSetIteratorRewrite(rpmdbMatchIterator mi, int rewrite)
@@ -2340,6 +2359,13 @@ if (dbiByteSwapped(dbi) == 1)
 	return NULL;
     }
 
+    /* Mark header with its instance number. */
+    {	char origin[32];
+	sprintf(origin, "rpmdb (h#%u)", mi->mi_offset);
+	(void) headerSetOrigin(mi->mi_h, origin);
+	(void) headerSetInstance(mi->mi_h, mi->mi_offset);
+    }
+
     mi->mi_prevoffset = mi->mi_offset;
     mi->mi_modified = 0;
 
@@ -2681,11 +2707,14 @@ memset(data, 0, sizeof(*data));
 /*@=boundsread@*/
 
 	    /*@-branchstate@*/
-	    /* Filter out temporary databases */
-	    if (isTemporaryDB(rpmtag)) 
-		continue;
-
 	    switch (rpmtag) {
+	    /* Filter out temporary databases */
+	    case RPMDBI_AVAILABLE:
+	    case RPMDBI_ADDED:
+	    case RPMDBI_REMOVED:
+	    case RPMDBI_DEPENDS:
+		continue;
+		/*@notreached@*/ /*@switchbreak@*/ break;
 	    case RPMDBI_PACKAGES:
 		dbi = dbiOpen(db, rpmtag, 0);
 		if (dbi == NULL)	/* XXX shouldn't happen */
@@ -2792,18 +2821,11 @@ if (dbiByteSwapped(dbi) == 1)
 		    }
 		    /* Extract the pubkey id from the base64 blob. */
 		    if (dbi->dbi_rpmtag == RPMTAG_PUBKEYS) {
-			pgpDig dig = pgpNewDig();
-			const byte * pkt;
-			ssize_t pktlen;
-
-			if (b64decode(rpmvals[i], (void **)&pkt, &pktlen))
+			int nbin = pgpExtractPubkeyFingerprint(rpmvals[i], bin);
+			if (nbin <= 0)
 			    /*@innercontinue@*/ continue;
-			(void) pgpPrtPkts(pkt, pktlen, dig, 0);
-			memcpy(bin, dig->pubkey.signid, 8);
-			pkt = _free(pkt);
-			dig = _free(dig);
 			key->data = bin;
-			key->size = 8;
+			key->size = nbin;
 			/*@switchbreak@*/ break;
 		    }
 /*@=boundsread@*/
@@ -2938,13 +2960,8 @@ DBT * data = alloca(sizeof(*data));
     int rc;
     int xx;
 
-    /* Reinitialize to zero, so in the event the add fails
-     * we won't have bogus information (i.e. the last succesful
-     * add).
-     */
-/*@-mods@*/
-    myinstall_instance = 0;
-/*@=mods@*/
+    /* Initialize the header instance */
+    (void) headerSetInstance(h, 0);
 
     if (db == NULL)
 	return 0;
@@ -2969,9 +2986,6 @@ memset(data, 0, sizeof(*data));
 
     xx = hge(h, RPMTAG_BASENAMES, &bnt, (void **) &baseNames, &count);
 
-    if (_noDirTokens)
-	expandFilelist(h);
-
     (void) blockSignals(db, &signalMask);
 
     {
@@ -3056,10 +3070,8 @@ memset(data, 0, sizeof(*data));
     {	
 	dbiIndexItem rec = dbiIndexNewItem(hdrNum, 0);
 
-	/* Save the header number for the current transaction. */
-/*@-mods@*/
-	myinstall_instance = hdrNum;
-/*@=mods@*/
+	/* Save the header instance. */
+	(void) headerSetInstance(h, hdrNum);
 	
 	if (dbiTags != NULL)
 	for (dbix = 0; dbix < dbiTagsMax; dbix++) {
@@ -3079,11 +3091,14 @@ memset(data, 0, sizeof(*data));
 	    rpmtag = dbiTags[dbix];
 /*@=boundsread@*/
 
-	    /* Filter out temporary databases */
-	    if (isTemporaryDB(rpmtag)) 
-		continue;
-
 	    switch (rpmtag) {
+	    /* Filter out temporary databases */
+	    case RPMDBI_AVAILABLE:
+	    case RPMDBI_ADDED:
+	    case RPMDBI_REMOVED:
+	    case RPMDBI_DEPENDS:
+		continue;
+		/*@notreached@*/ /*@switchbreak@*/ break;
 	    case RPMDBI_PACKAGES:
 		dbi = dbiOpen(db, rpmtag, 0);
 		if (dbi == NULL)	/* XXX shouldn't happen */
@@ -3253,18 +3268,11 @@ data->size = 0;
 		    }
 		    /* Extract the pubkey id from the base64 blob. */
 		    if (dbi->dbi_rpmtag == RPMTAG_PUBKEYS) {
-			pgpDig dig = pgpNewDig();
-			const byte * pkt;
-			ssize_t pktlen;
-
-			if (b64decode(rpmvals[i], (void **)&pkt, &pktlen))
+			int nbin = pgpExtractPubkeyFingerprint(rpmvals[i], bin);
+			if (nbin <= 0)
 			    /*@innercontinue@*/ continue;
-			(void) pgpPrtPkts(pkt, pktlen, dig, 0);
-			memcpy(bin, dig->pubkey.signid, 8);
-			pkt = _free(pkt);
-			dig = _free(dig);
 			key->data = bin;
-			key->size = 8;
+			key->size = nbin;
 			/*@switchbreak@*/ break;
 		    }
 /*@=boundsread@*/
@@ -3360,34 +3368,6 @@ exit:
     return ret;
 }
 
-#define _skip(_dn)	{ sizeof(_dn)-1, (_dn) }
-
-/*@unchecked@*/ /*@observer@*/
-static struct skipDir_s {
-    int dnlen;
-/*@observer@*/ /*@null@*/
-    const char * dn;
-} skipDirs[] = {
-    { 0, NULL }
-};
-
-static int skipDir(const char * dn)
-	/*@*/
-{
-    struct skipDir_s * sd = skipDirs;
-    int dnlen;
-
-    dnlen = strlen(dn);
-    for (sd = skipDirs; sd->dn != NULL; sd++) {
-	if (dnlen < sd->dnlen)
-	    continue;
-	if (strncmp(dn, sd->dn, sd->dnlen))
-	    continue;
-	return 1;
-    }
-    return 0;
-}
-
 /* XXX transaction.c */
 /*@-compmempass@*/
 int rpmdbFindFpList(rpmdb db, fingerPrint * fpList, dbiIndexSet * matchList, 
@@ -3402,11 +3382,11 @@ DBT * data;
     Header h;
     int i, xx;
 
-    if (db == NULL) return 1;
+    if (db == NULL) return 0;
 
     mi = rpmdbInitIterator(db, RPMTAG_BASENAMES, NULL, 0);
     if (mi == NULL)	/* XXX should  never happen */
-	return 1;
+	return 0;
 
 key = &mi->mi_key;
 data = &mi->mi_data;
@@ -3424,9 +3404,6 @@ key->data = (void *) fpList[i].baseName;
 key->size = strlen((char *)key->data);
 if (key->size == 0) key->size++;	/* XXX "/" fixup. */
 
-	if (skipDir(fpList[i].entry->dirName))
-	    continue;
-
 	xx = rpmdbGrowIterator(mi, i);
 
     }
@@ -3658,8 +3635,16 @@ static int rpmdbMoveDatabase(const char * prefix,
 	    int rpmtag;
 
 	    /* Filter out temporary databases */
-	    if (isTemporaryDB((rpmtag = dbiTags[i])))
+	    switch ((rpmtag = dbiTags[i])) {
+	    case RPMDBI_AVAILABLE:
+	    case RPMDBI_ADDED:
+	    case RPMDBI_REMOVED:
+	    case RPMDBI_DEPENDS:
 		continue;
+		/*@notreached@*/ /*@switchbreak@*/ break;
+	    default:
+		/*@switchbreak@*/ break;
+	    }
 
 	    base = tagName(rpmtag);
 	    sprintf(ofilename, "%s/%s/%s", prefix, olddbpath, base);
@@ -3763,7 +3748,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     }
     dbpath = rootdbpath = rpmGetPath(prefix, tfn, NULL);
     if (!(prefix[0] == '/' && prefix[1] == '\0'))
-	dbpath += strlen(prefix) - 1;
+	dbpath += strlen(prefix);
     tfn = _free(tfn);
 
     /*@-nullpass@*/
@@ -3786,7 +3771,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     }
     newdbpath = newrootdbpath = rpmGetPath(prefix, tfn, NULL);
     if (!(prefix[0] == '/' && prefix[1] == '\0'))
-	newdbpath += strlen(prefix) - 1;
+	newdbpath += strlen(prefix);
     tfn = _free(tfn);
 
     rpmMessage(RPMMESS_DEBUG, _("rebuilding database %s into %s\n"),
@@ -3813,7 +3798,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
     rpmMessage(RPMMESS_DEBUG, _("opening old database with dbapi %d\n"),
 		_dbapi);
 /*@-boundswrite@*/
-    if (openDatabase(prefix, dbpath, _dbapi, &olddb, O_RDONLY, 0644, 
+    if (rpmdbOpenDatabase(prefix, dbpath, _dbapi, &olddb, O_RDONLY, 0644, 
 		     RPMDB_FLAG_MINIMAL)) {
 	rc = 1;
 	goto exit;
@@ -3825,7 +3810,7 @@ int rpmdbRebuild(const char * prefix, rpmts ts,
 		_dbapi_rebuild);
     (void) rpmDefineMacro(NULL, "_rpmdb_rebuild %{nil}", -1);
 /*@-boundswrite@*/
-    if (openDatabase(prefix, newdbpath, _dbapi_rebuild, &newdb, O_RDWR | O_CREAT, 0644, 0)) {
+    if (rpmdbOpenDatabase(prefix, newdbpath, _dbapi_rebuild, &newdb, O_RDWR | O_CREAT, 0644, 0)) {
 	rc = 1;
 	goto exit;
     }
