@@ -52,6 +52,7 @@ typedef /*@abstract@*/ struct _dbiIndexSet * dbiIndexSet;
  */
 typedef /*@abstract@*/ struct _dbiIndex * dbiIndex;
 
+#if !defined(SWIG)	/* XXX inline dbiFoo() need */
 /* this will break if sizeof(int) != 4 */
 /** \ingroup dbi
  * A single item from an index database (i.e. the "data returned").
@@ -269,7 +270,7 @@ struct _dbiVec {
  * Describes an index database (implemented on Berkeley db3 functionality).
  */
 struct _dbiIndex {
-/*@null@*/
+/*@relnull@*/
     const char * dbi_root;	/*!< chroot(2) component of path */
 /*@null@*/
     const char * dbi_home;	/*!< directory component of path */
@@ -377,7 +378,7 @@ struct _dbiIndex {
     rpmTag dbi_rpmtag;		/*!< rpm tag used for index */
     int	dbi_jlen;		/*!< size of join key */
 
-/*@only@*//*@null@*/
+/*@only@*//*@relnull@*/
     DB * dbi_db;		/*!< Berkeley DB * handle */
 /*@only@*//*@null@*/
     DB_TXN * dbi_txnid;		/*!< Bekerley DB_TXN * transaction id */
@@ -388,7 +389,9 @@ struct _dbiIndex {
     const struct _dbiVec * dbi_vec;	/*!< private methods */
 
 };
+#endif	/* !defined(SWIG) */
 
+#if defined(_RPMDB_INTERNAL)
 /** \ingroup rpmdb
  * Describes the collection of index databases used by rpm.
  */
@@ -436,6 +439,7 @@ struct rpmdb_s {
 /*@refs@*/
     int nrefs;			/*!< Reference count. */
 };
+#endif	/* !defined(SWIG) */
 
 /* for RPM's internal use only */
 
@@ -482,7 +486,7 @@ dbiIndex db3Free( /*@only@*/ /*@null@*/ dbiIndex dbi)
  */
 /*@-redecl@*/
 /*@exposed@*/
-extern const char *const prDbiOpenFlags(int dbflags, int print_dbenv_flags)
+extern const char * prDbiOpenFlags(int dbflags, int print_dbenv_flags)
 	/*@*/;
 /*@=redecl@*/
 
@@ -498,6 +502,16 @@ extern const char *const prDbiOpenFlags(int dbflags, int print_dbenv_flags)
 	/*@globals rpmGlobalMacroContext, errno, h_errno @*/
 	/*@modifies db, rpmGlobalMacroContext, errno @*/;
 
+/**
+ * Return dbiStats accumulator structure.
+ * @param dbi		index database handle
+ * @param opx		per-rpmdb accumulator index (aka rpmtsOpX)
+ * @return		per-rpmdb accumulator pointer
+ */
+void * dbiStatsAccumulator(dbiIndex dbi, int opx)
+        /*@*/;
+
+#if !defined(SWIG)
 /*@-globuse -mustmod @*/ /* FIX: vector annotations */
 /** \ingroup dbi
  * Open a database cursor.
@@ -562,11 +576,12 @@ int dbiDel(dbiIndex dbi, /*@null@*/ DBC * dbcursor, DBT * key, DBT * data,
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, fileSystem, internalState @*/
 {
+    void * sw = dbiStatsAccumulator(dbi, 16);	/* RPMTS_OP_DBDEL */
     int rc;
     assert(key->data != NULL && key->size > 0);
-    (void) rpmswEnter(&dbi->dbi_rpmdb->db_delops, 0);
+    (void) rpmswEnter(sw, 0);
     rc = (dbi->dbi_vec->cdel) (dbi, dbcursor, key, data, flags);
-    (void) rpmswExit(&dbi->dbi_rpmdb->db_delops, data->size);
+    (void) rpmswExit(sw, data->size);
     return rc;
 }
 
@@ -585,11 +600,12 @@ int dbiGet(dbiIndex dbi, /*@null@*/ DBC * dbcursor, DBT * key, DBT * data,
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, *key, *data, fileSystem, internalState @*/
 {
+    void * sw = dbiStatsAccumulator(dbi, 14);	/* RPMTS_OP_DBGET */
     int rc;
     assert((flags == DB_NEXT) || (key->data != NULL && key->size > 0));
-    (void) rpmswEnter(&dbi->dbi_rpmdb->db_getops, 0);
+    (void) rpmswEnter(sw, 0);
     rc = (dbi->dbi_vec->cget) (dbi, dbcursor, key, data, flags);
-    (void) rpmswExit(&dbi->dbi_rpmdb->db_getops, data->size);
+    (void) rpmswExit(sw, data->size);
     return rc;
 }
 
@@ -609,11 +625,12 @@ int dbiPget(dbiIndex dbi, /*@null@*/ DBC * dbcursor,
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, *key, *pkey, *data, fileSystem, internalState @*/
 {
+    void * sw = dbiStatsAccumulator(dbi, 14);	/* RPMTS_OP_DBGET */
     int rc;
     assert((flags == DB_NEXT) || (key->data != NULL && key->size > 0));
-    (void) rpmswEnter(&dbi->dbi_rpmdb->db_getops, 0);
+    (void) rpmswEnter(sw, 0);
     rc = (dbi->dbi_vec->cpget) (dbi, dbcursor, key, pkey, data, flags);
-    (void) rpmswExit(&dbi->dbi_rpmdb->db_getops, data->size);
+    (void) rpmswExit(sw, data->size);
     return rc;
 }
 
@@ -632,11 +649,12 @@ int dbiPut(dbiIndex dbi, /*@null@*/ DBC * dbcursor, DBT * key, DBT * data,
 	/*@globals fileSystem, internalState @*/
 	/*@modifies dbi, *dbcursor, *key, fileSystem, internalState @*/
 {
+    void * sw = dbiStatsAccumulator(dbi, 15);	/* RPMTS_OP_DBPUT */
     int rc;
     assert(key->data != NULL && key->size > 0 && data->data != NULL && data->size > 0);
-    (void) rpmswEnter(&dbi->dbi_rpmdb->db_putops, 0);
+    (void) rpmswEnter(sw, 0);
     rc = (dbi->dbi_vec->cput) (dbi, dbcursor, key, data, flags);
-    (void) rpmswExit(&dbi->dbi_rpmdb->db_putops, data->size);
+    (void) rpmswExit(sw, data->size);
     return rc;
 }
 
@@ -748,8 +766,9 @@ int dbiByteSwapped(dbiIndex dbi)
         dbi->dbi_byteswapped = (*dbi->dbi_vec->byteswapped) (dbi);
     return dbi->dbi_byteswapped;
 }
+
 /** \ingroup dbi
- * Is database byte swapped?
+ * Return dbi statistics.
  * @param dbi		index database handle
  * @param flags		DB_FAST_STAT or 0
  * @return		0 on success
@@ -760,7 +779,20 @@ int dbiStat(dbiIndex dbi, unsigned int flags)
 {
     return (*dbi->dbi_vec->stat) (dbi, flags);
 }
+
+/** \ingroup dbi
+ * Return dbi transaction id.
+ * @param dbi		index database handle
+ * @return		transaction id
+ */
+/*@unused@*/ static inline /*@observer@*/ /*@null@*/
+DB_TXN * dbiTxnid(dbiIndex dbi)
+	/*@*/
+{
+    return dbi->dbi_txnid;
+}
 /*@=globuse =mustmod @*/
+#endif	/* !defined(SWIG) */
 
 /*@=exportlocal@*/
 
@@ -809,6 +841,7 @@ unsigned int dbiIndexRecordFileNumber(dbiIndexSet set, int recno)
 /*@-exportlocal@*/
 /*@unchecked@*/
 /*@only@*/ /*@null@*/ extern int * dbiTags;
+
 /*@unchecked@*/
 extern int dbiTagsMax;
 /*@=exportlocal@*/
