@@ -4,6 +4,8 @@
 
 #include "system.h"
 #include "rpmio_internal.h"
+#include "rmd128.h"
+#include "rmd160.h"
 #include "debug.h"
 
 #ifdef	SHA_DEBUG
@@ -12,6 +14,65 @@
 #define	DPRINTF(_a)
 #endif
 
+typedef struct {
+	uint32_t crc;
+	uint32_t polynomial;
+	uint32_t xorout;
+	uint32_t table[256];
+} crc32Param;
+
+static int crc32Reset(register crc32Param* mp)
+	/*@modifies *mp @*/
+{
+	mp->crc = 0xffffffff;
+	mp->polynomial = 0xedb88320;	/* reflected 0x04c11db7 */
+	mp->xorout = 0xffffffff;
+
+	/* generate the table of CRC remainders for all possible bytes */
+	{   uint32_t c;
+	    uint32_t i, j;
+	    for (i = 0;  i < 256;  i++) {
+		c = i;
+		for (j = 0;  j < 8;  j++) {
+	             if (c & 1)
+	                c = mp->polynomial ^ (c >> 1);
+	             else
+	                c = (c >> 1);
+		}
+	        mp->table[i] = c;
+	    }
+	}
+	return 0;
+}
+
+static int crc32Update(crc32Param* mp, const byte* data, size_t size)
+	/*@modifies *mp @*/
+{
+	uint32_t c = mp->crc;
+
+	while (size) {
+		c = mp->table[(c ^ *data) & 0xff] ^ (c >> 8);
+		size--;
+		data++;
+	}
+	mp->crc = c;
+	return 0;
+}
+
+static int crc32Digest(crc32Param* mp, byte* data)
+	/*@modifies *mp, data @*/
+{
+	uint32_t c = mp->crc ^ mp->xorout;
+
+	data[ 0] = (byte)(c >> 24);
+	data[ 1] = (byte)(c >> 16);
+	data[ 2] = (byte)(c >>  8);
+	data[ 3] = (byte)(c      );
+
+	(void) crc32Reset(mp);
+	return 0;
+}
+
 /*@access DIGEST_CTX@*/
 
 /**
@@ -52,7 +113,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 
     switch (hashalgo) {
     case PGPHASHALGO_MD5:
-	ctx->digestlen = 16;
+	ctx->digestlen = 128/8;
 	ctx->datalen = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
 	ctx->paramlen = sizeof(md5Param);
@@ -65,7 +126,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_SHA1:
-	ctx->digestlen = 20;
+	ctx->digestlen = 160/8;
 	ctx->datalen = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
 	ctx->paramlen = sizeof(sha1Param);
@@ -75,11 +136,50 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 	ctx->Reset = (void *) sha1Reset;
 	ctx->Update = (void *) sha1Update;
 	ctx->Digest = (void *) sha1Digest;
+/*@=type@*/
+	break;
+    case PGPHASHALGO_RIPEMD160:
+	ctx->digestlen = 160/8;
+	ctx->datalen = 64;
+/*@-sizeoftype@*/ /* FIX: union, not void pointer */
+	ctx->paramlen = sizeof(rmd160Param);
+/*@=sizeoftype@*/
+	ctx->param = xcalloc(1, ctx->paramlen);
+/*@-type@*/ /* FIX: cast? */
+	ctx->Reset = (void *) rmd160Reset;
+	ctx->Update = (void *) rmd160Update;
+	ctx->Digest = (void *) rmd160Digest;
+/*@=type@*/
+	break;
+    case PGPHASHALGO_RIPEMD128:
+	ctx->digestlen = 128/8;
+	ctx->datalen = 64;
+/*@-sizeoftype@*/ /* FIX: union, not void pointer */
+	ctx->paramlen = sizeof(rmd128Param);
+/*@=sizeoftype@*/
+	ctx->param = xcalloc(1, ctx->paramlen);
+/*@-type@*/ /* FIX: cast? */
+	ctx->Reset = (void *) rmd128Reset;
+	ctx->Update = (void *) rmd128Update;
+	ctx->Digest = (void *) rmd128Digest;
+/*@=type@*/
+	break;
+    case PGPHASHALGO_CRC32:
+	ctx->digestlen = 32/8;
+	ctx->datalen = 8;
+/*@-sizeoftype@*/ /* FIX: union, not void pointer */
+	ctx->paramlen = sizeof(crc32Param);
+/*@=sizeoftype@*/
+	ctx->param = xcalloc(1, ctx->paramlen);
+/*@-type@*/ /* FIX: cast? */
+	ctx->Reset = (void *) crc32Reset;
+	ctx->Update = (void *) crc32Update;
+	ctx->Digest = (void *) crc32Digest;
 /*@=type@*/
 	break;
 #if HAVE_BEECRYPT_API_H
     case PGPHASHALGO_SHA256:
-	ctx->digestlen = 32;
+	ctx->digestlen = 256/8;
 	ctx->datalen = 64;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
 	ctx->paramlen = sizeof(sha256Param);
@@ -92,7 +192,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_SHA384:
-	ctx->digestlen = 48;
+	ctx->digestlen = 384/8;
 	ctx->datalen = 128;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
 	ctx->paramlen = sizeof(sha384Param);
@@ -105,7 +205,7 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
     case PGPHASHALGO_SHA512:
-	ctx->digestlen = 64;
+	ctx->digestlen = 512/8;
 	ctx->datalen = 128;
 /*@-sizeoftype@*/ /* FIX: union, not void pointer */
 	ctx->paramlen = sizeof(sha512Param);
@@ -118,7 +218,6 @@ rpmDigestInit(pgpHashAlgo hashalgo, rpmDigestFlags flags)
 /*@=type@*/
 	break;
 #endif
-    case PGPHASHALGO_RIPEMD160:
     case PGPHASHALGO_MD2:
     case PGPHASHALGO_TIGER192:
     case PGPHASHALGO_HAVAL_5_160:
