@@ -253,7 +253,7 @@ findEntry(MacroContext mc, const char * name, size_t namelen)
 {
     MacroEntry key, *ret;
     struct MacroEntry_s keybuf;
-    char *namebuf = NULL;
+    char namebuf[1024];
 
 /*@-globs@*/
     if (mc == NULL) mc = rpmGlobalMacroContext;
@@ -263,8 +263,6 @@ findEntry(MacroContext mc, const char * name, size_t namelen)
 
 /*@-branchstate@*/
     if (namelen > 0) {
-	namebuf = alloca(namelen + 1);
-	memset(namebuf, 0, (namelen + 1));
 	strncpy(namebuf, name, namelen);
 	namebuf[namelen] = '\0';
 	name = namebuf;
@@ -604,11 +602,6 @@ doShellEscape(MacroBuf mb, const char * cmd, size_t clen)
     int rc;
     int c;
 
-    if (clen >= sizeof(pcmd)) {
-	rpmError(RPMERR_BADSPEC, _("Target buffer overflow\n"));
-	return 1;
-    }
-
     strncpy(pcmd, cmd, clen);
     pcmd[clen] = '\0';
     rc = expandU(mb, pcmd, sizeof(pcmd));
@@ -617,11 +610,8 @@ doShellEscape(MacroBuf mb, const char * cmd, size_t clen)
 
     if ((shf = popen(pcmd, "r")) == NULL)
 	return 1;
-    while((c = fgetc(shf)) != EOF) {
-	if (mb->nb > 1) {
-	    SAVECHAR(mb, c);
-	} 
-    }
+    while(mb->nb > 0 && (c = fgetc(shf)) != EOF)
+	SAVECHAR(mb, c);
     (void) pclose(shf);
 
     /* XXX delete trailing \r \n */
@@ -1089,10 +1079,6 @@ doOutput(MacroBuf mb, int waserror, const char * msg, size_t msglen)
 {
     char buf[BUFSIZ];
 
-    if (msglen >= sizeof(buf)) {
-	rpmError(RPMERR_BADSPEC, _("Target buffer overflow\n"));
-	msglen = sizeof(buf) - 1;
-    }
     strncpy(buf, msg, msglen);
     buf[msglen] = '\0';
     (void) expandU(mb, buf, sizeof(buf));
@@ -1122,10 +1108,6 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 
     buf[0] = '\0';
     if (g != NULL) {
-	if (gn >= sizeof(buf)) {
-	    rpmError(RPMERR_BADSPEC, _("Target buffer overflow\n"));
-	    gn = sizeof(buf) - 1;
-        }
 	strncpy(buf, g, gn);
 	buf[gn] = '\0';
 	(void) expandU(mb, buf, sizeof(buf));
@@ -1166,24 +1148,23 @@ doFoo(MacroBuf mb, int negate, const char * f, size_t fn,
 	    be++;
 /*@=globs@*/
 	*be++ = '\0';
+#ifndef	DEBUG_MACROS
 	(void) isCompressed(b, &compressed);
+#endif
 	switch(compressed) {
 	default:
-	case COMPRESSED_NOT:
+	case 0:	/* COMPRESSED_NOT */
 	    sprintf(be, "%%_cat %s", b);
 	    break;
-	case COMPRESSED_OTHER:
+	case 1:	/* COMPRESSED_OTHER */
 	    sprintf(be, "%%_gzip -dc %s", b);
 	    break;
-	case COMPRESSED_BZIP2:
+	case 2:	/* COMPRESSED_BZIP2 */
 	    sprintf(be, "%%_bzip2 %s", b);
 	    break;
-	case COMPRESSED_ZIP:
+	case 3:	/* COMPRESSED_ZIP */
 	    sprintf(be, "%%_unzip %s", b);
 	    break;
-        case COMPRESSED_LZMA:
-            sprintf(be, "%%_lzma -dc %s", b);
-            break;
 	}
 	b = be;
     } else if (STREQ("S", f, fn)) {
@@ -1702,8 +1683,6 @@ int rpmGlob(const char * patterns, int * argcPtr, const char *** argvPtr)
     int argc = 0;
     const char ** argv = NULL;
     char * globRoot = NULL;
-    const char *home = getenv("HOME");
-    int gflags = 0;
 #ifdef ENABLE_NLS
     const char * old_collate = NULL;
     const char * old_ctype = NULL;
@@ -1713,9 +1692,6 @@ int rpmGlob(const char * patterns, int * argcPtr, const char *** argvPtr)
     int i, j;
     int rc;
 
-    if (home != NULL && strlen(home) > 0) 
-	gflags |= GLOB_TILDE;
-
     rc = XpoptParseArgvString(patterns, &ac, &av);
     if (rc)
 	return rc;
@@ -1750,7 +1726,7 @@ fprintf(stderr, "*** rpmGlob argv[%d] \"%s\"\n", argc, argv[argc]);
 	
 	gl.gl_pathc = 0;
 	gl.gl_pathv = NULL;
-	rc = Glob(av[j], gflags, Glob_error, &gl);
+	rc = Glob(av[j], GLOB_TILDE, Glob_error, &gl);
 	if (rc)
 	    goto exit;
 
@@ -2022,15 +1998,8 @@ rpmInitMacros(MacroContext mc, const char * macrofiles)
 	    continue;
 
 	/* Read macros from each file. */
-	for (i = 0; i < ac; i++) {
-	    if (strstr(av[i], ".rpmnew") || 
-		strstr(av[i], ".rpmsave") ||
-		strstr(av[i], ".rpmorig")) {
-		continue;
-	    }
+	for (i = 0; i < ac; i++)
 	    (void) rpmLoadMacroFile(mc, av[i]);
-	    av[i] = _free(av[i]);
-	}
 	av = _free(av);
     }
     mfiles = _free(mfiles);
@@ -2069,22 +2038,13 @@ rpmFreeMacros(MacroContext mc)
 }
 /*@=globstate@*/
 
-static int rpmFileHasSuffix(const char *path, const char *suffix)
-{
-    size_t plen = strlen(path);
-    size_t slen = strlen(suffix);
-    return (plen >= slen &&
-            strcmp(path+plen-slen, suffix) == 0);
-}
-
-
 /* =============================================================== */
 int isCompressed(const char * file, rpmCompressedMagic * compressed)
 {
     FD_t fd;
     ssize_t nb;
     int rc = -1;
-    unsigned char magic[13];
+    unsigned char magic[4];
 
     *compressed = COMPRESSED_NOT;
 
@@ -2115,11 +2075,6 @@ int isCompressed(const char * file, rpmCompressedMagic * compressed)
     } else if ((magic[0] == 0120) && (magic[1] == 0113) &&
 	 (magic[2] == 0003) && (magic[3] == 0004)) {	/* pkzip */
 	*compressed = COMPRESSED_ZIP;
-    } else if ((magic[0] == 0xff) && (magic[1] == 0x4c) &&
-	       (magic[2] == 0x5a) && (magic[3] == 0x4d) &&
-	       (magic[4] == 0x41) && (magic[5] == 0x00)) {
-	/* new style lzma with magic */
-	*compressed = COMPRESSED_LZMA;
     } else if (((magic[0] == 0037) && (magic[1] == 0213)) || /* gzip */
 	((magic[0] == 0037) && (magic[1] == 0236)) ||	/* old gzip */
 	((magic[0] == 0037) && (magic[1] == 0036)) ||	/* pack */
@@ -2127,8 +2082,6 @@ int isCompressed(const char * file, rpmCompressedMagic * compressed)
 	((magic[0] == 0037) && (magic[1] == 0235))	/* compress */
 	) {
 	*compressed = COMPRESSED_OTHER;
-    } else if (rpmFileHasSuffix(file, ".lzma")) {
-	*compressed = COMPRESSED_LZMA;
     }
 
     return rc;
@@ -2206,6 +2159,9 @@ char *rpmCleanPath(char * path)
 	    if (s[1] == '/' && s[2] == '/') {
 		*t++ = *s++;
 		*t++ = *s++;
+		/* XXX handle "file:///" */
+		if (s[0] == '/') *t++ = *s++;
+		te = t;
 		/*@switchbreak@*/ break;
 	    }
 	    begin=1;
@@ -2220,7 +2176,7 @@ char *rpmCleanPath(char * path)
 	    }
 	    while (s[1] == '/')
 		s++;
-	    while (t > path && t[-1] == '/')
+	    while (t > te && t[-1] == '/')
 		t--;
 	    /*@switchbreak@*/ break;
 	case '.':
