@@ -9,8 +9,6 @@
 #include <pthread.h>
 #endif
 
-#ifdef WITH_NEON
-
 #include "ne_alloc.h"
 #include "ne_auth.h"
 #include "ne_basic.h"
@@ -29,13 +27,18 @@
 #include "ne_string.h"
 #include "ne_utils.h"
 
-#endif /* WITH_NEON */
+/* XXX API changes for neon-0.26.0 */
+#if !defined(NE_FREE)
+#define	ne_set_persist(_sess, _flag)
+#define	ne_propfind_set_private(_pfh, _create_item, NULL) \
+	ne_propfind_set_private(_pfh, _create_item, NULL, NULL)
+#endif
 
 #include <rpmio_internal.h>
 
 #define _RPMDAV_INTERNAL
 #include <rpmdav.h>
-                                                                                
+
 #include "argv.h"
 #include "ugid.h"
 #include "debug.h"
@@ -65,8 +68,6 @@ _free(/*@only@*/ /*@null@*/ /*@out@*/ const void * p)
     return NULL;
 }
 
-#ifdef WITH_NEON
-
 /* =============================================================== */
 static int davFree(urlinfo u)
 	/*@globals internalState @*/
@@ -293,7 +294,6 @@ static int davConnect(urlinfo u)
 if (_dav_debug)
 fprintf(stderr, "*** Connect to %s:%d failed(%d):\n\t%s\n",
 		   u->host, u->port, rc, ne_get_error(u->sess));
-	u = urlLink(u, __FUNCTION__);   /* XXX error exit refcount adjustment */
 	break;
     }
 
@@ -359,7 +359,7 @@ static int davInit(const char * url, urlinfo * uret)
 	/* XXX check that neon is ssl enabled. */
 	if (!strcasecmp(u->scheme, "https"))
 	    ne_ssl_set_verify(u->sess, davVerifyCert, (char *)u->host);
-                                                                                
+
 	ne_set_session_private(u->sess, "urlinfo", u);
 
 	ne_hook_destroy_session(u->sess, davDestroySession, u);
@@ -393,8 +393,9 @@ enum fetch_rtype_e {
     resr_reference,
     resr_error
 };
-                                                                                
+
 struct fetch_resource_s {
+/*@dependent@*/
     struct fetch_resource_s *next;
     char *uri;
 /*@unused@*/
@@ -412,8 +413,8 @@ struct fetch_resource_s {
 static void *fetch_destroy_item(/*@only@*/ struct fetch_resource_s *res)
 	/*@modifies res @*/
 {
-    NE_FREE(res->uri);
-    NE_FREE(res->error_reason);
+    ne_free(res->uri);
+    ne_free(res->error_reason);
     res = _free(res);
     return NULL;
 }
@@ -443,7 +444,7 @@ static void *fetch_create_item(/*@unused@*/ void *userdata, /*@unused@*/ const c
 
 /* =============================================================== */
 struct fetch_context_s {
-/*@relnull@*/
+/*@relnull@*/ /*@dependent@*/
     struct fetch_resource_s **resrock;
     const char *uri;
     unsigned int include_target; /* Include resource at href */
@@ -517,7 +518,7 @@ static const struct ne_xml_idmap fetch_idmap[] = {
     { "DAV:", "collection", ELM_collection }
 };
 
-static int fetch_startelm(void *userdata, int parent, 
+static int fetch_startelm(void *userdata, int parent,
 		const char *nspace, const char *name,
 		/*@unused@*/ const char **atts)
 	/*@*/
@@ -527,7 +528,7 @@ static int fetch_startelm(void *userdata, int parent,
     int state = ne_xml_mapid(fetch_idmap, NE_XML_MAPLEN(fetch_idmap),
                              nspace, name);
 
-    if (r == NULL || 
+    if (r == NULL ||
         !((parent == NE_207_STATE_PROP && state == ELM_resourcetype) ||
           (parent == ELM_resourcetype && state == ELM_collection)))
         return NE_XML_DECLINE;
@@ -539,7 +540,7 @@ static int fetch_startelm(void *userdata, int parent,
     return state;
 }
 
-static int fetch_compare(const struct fetch_resource_s *r1, 
+static int fetch_compare(const struct fetch_resource_s *r1,
 			    const struct fetch_resource_s *r2)
 	/*@*/
 {
@@ -563,7 +564,7 @@ static int fetch_compare(const struct fetch_resource_s *r1,
     }
 }
 
-static void fetch_results(void *userdata, const char *uri,
+static void fetch_results(void *userdata, void *uarg,
 		    const ne_prop_result_set *set)
 	/*@*/
 {
@@ -574,7 +575,13 @@ static void fetch_results(void *userdata, const char *uri,
     const ne_status *status = NULL;
     const char * path = NULL;
 
+#if !defined(NE_FREE)
+    const ne_uri * uri = uarg;
+    (void) urlPath(uri->path, &path);
+#else
+    const char * uri = uarg;
     (void) urlPath(uri, &path);
+#endif
     if (path == NULL)
 	return;
 
@@ -582,26 +589,28 @@ static void fetch_results(void *userdata, const char *uri,
 
 if (_dav_debug < 0)
 fprintf(stderr, "==> %s in uri %s\n", path, ctx->uri);
-    
+
     if (ne_path_compare(ctx->uri, path) == 0 && !ctx->include_target) {
 	/* This is the target URI */
 if (_dav_debug < 0)
 fprintf(stderr, "==> %s skipping target resource.\n", path);
 	/* Free the private structure. */
+/*@-dependenttrans -exposetrans@*/
 	free(newres);
+/*@=dependenttrans =exposetrans@*/
 	return;
     }
 
     newres->uri = ne_strdup(path);
 
 /*@-boundsread@*/
-    clength = ne_propset_value(set, &fetch_props[0]);    
+    clength = ne_propset_value(set, &fetch_props[0]);
     modtime = ne_propset_value(set, &fetch_props[1]);
     isexec = ne_propset_value(set, &fetch_props[2]);
     checkin = ne_propset_value(set, &fetch_props[4]);
     checkout = ne_propset_value(set, &fetch_props[5]);
 /*@=boundsread@*/
-    
+
 /*@-branchstate@*/
     if (clength == NULL)
 	status = ne_propset_status(set, &fetch_props[0]);
@@ -653,7 +662,7 @@ fprintf(stderr, "==> %s skipping target resource.\n", path);
 	newres->is_vcr = 0;
     }
 
-    for (current = *ctx->resrock, previous = NULL; current != NULL; 
+    for (current = *ctx->resrock, previous = NULL; current != NULL;
 	previous = current, current = current->next)
     {
 	if (fetch_compare(current, newres) >= 0) {
@@ -663,9 +672,9 @@ fprintf(stderr, "==> %s skipping target resource.\n", path);
     if (previous) {
 	previous->next = newres;
     } else {
-/*@-boundswrite@*/
+/*@-boundswrite -dependenttrans @*/
 	*ctx->resrock = newres;
-/*@=boundswrite@*/
+/*@=boundswrite =dependenttrans @*/
     }
     newres->next = current;
 }
@@ -693,7 +702,7 @@ static int davFetch(const urlinfo u, struct fetch_context_s * ctx)
     ctx->resrock = resrock;
     ctx->include_target = include_target;
 
-    ne_xml_push_handler(ne_propfind_get_parser(pfh), 
+    ne_xml_push_handler(ne_propfind_get_parser(pfh),
                         fetch_startelm, NULL, NULL, pfh);
 
     ne_propfind_set_private(pfh, fetch_create_item, NULL);
@@ -731,7 +740,7 @@ static int davFetch(const urlinfo u, struct fetch_context_s * ctx)
 	xx = argvAdd(&ctx->av, val);
 if (_dav_debug < 0)
 fprintf(stderr, "*** argvAdd(%p,\"%s\")\n", &ctx->av, val);
-	NE_FREE(val);
+	ne_free(val);
 
 	while (ctx->ac >= ctx->nalloced) {
 	    if (ctx->nalloced <= 0)
@@ -843,12 +852,13 @@ static void hexdump(const unsigned char * buf, ssize_t len)
 }
 #endif
 
+/*@-mustmod@*/
 static void davAcceptRanges(void * userdata, /*@null@*/ const char * value)
 	/*@modifies userdata @*/
 {
     urlinfo u = userdata;
 
-    if (!(u && value)) return;
+    if (!(u != NULL && value != NULL)) return;
 if (_dav_debug < 0)
 fprintf(stderr, "*** u %p Accept-Ranges: %s\n", u, value);
     if (!strcmp(value, "bytes"))
@@ -856,37 +866,41 @@ fprintf(stderr, "*** u %p Accept-Ranges: %s\n", u, value);
     if (!strcmp(value, "none"))
 	u->httpHasRange = 0;
 }
+/*@=mustmod@*/
 
 #if !defined(HAVE_NEON_NE_GET_RESPONSE_HEADER)
 static void davAllHeaders(void * userdata, const char * value)
 {
     FD_t ctrl = userdata;
 
-    if (!(ctrl && value)) return;
+    if (!(ctrl != NULL && value != NULL)) return;
 if (_dav_debug)
 fprintf(stderr, "<- %s\n", value);
 }
 #endif
 
+/*@-mustmod@*/
 static void davContentLength(void * userdata, /*@null@*/ const char * value)
 	/*@modifies userdata @*/
 {
     FD_t ctrl = userdata;
 
-    if (!(ctrl && value)) return;
+    if (!(ctrl != NULL && value != NULL)) return;
 if (_dav_debug < 0)
 fprintf(stderr, "*** fd %p Content-Length: %s\n", ctrl, value);
 /*@-unrecog@*/
    ctrl->contentLength = strtoll(value, NULL, 10);
 /*@=unrecog@*/
 }
+/*@=mustmod@*/
 
+/*@-mustmod@*/
 static void davConnection(void * userdata, /*@null@*/ const char * value)
 	/*@modifies userdata @*/
 {
     FD_t ctrl = userdata;
 
-    if (!(ctrl && value)) return;
+    if (!(ctrl != NULL && value != NULL)) return;
 if (_dav_debug < 0)
 fprintf(stderr, "*** fd %p Connection: %s\n", ctrl, value);
     if (!strcasecmp(value, "close"))
@@ -894,6 +908,7 @@ fprintf(stderr, "*** fd %p Connection: %s\n", ctrl, value);
     else if (!strcasecmp(value, "Keep-Alive"))
 	ctrl->persist = 1;
 }
+/*@=mustmod@*/
 
 /*@-mustmod@*/ /* HACK: stash error in *str. */
 int davResp(urlinfo u, FD_t ctrl, /*@unused@*/ char *const * str)
@@ -1037,7 +1052,9 @@ fprintf(stderr, "*** davOpen(%s,0x%x,0%o,%p)\n", url, flags, mode, uret);
 	fd = fdNew("grab ctrl (davOpen)");
 
     if (fd) {
+	fdSetOpen(fd, url, flags, mode);
 	fdSetIo(fd, ufdio);
+
 	fd->ftpFileDoneNeeded = 0;
 	fd->rd_timeoutsecs = httpTimeoutSecs;
 	fd->contentLength = fd->bytesRemain = -1;
@@ -1068,7 +1085,7 @@ assert(count >= 128);	/* HACK: see ne_request.h comment */
     rc = ne_read_response_block(fd->req, buf, count);
 
 if (_dav_debug < 0) {
-fprintf(stderr, "*** davRead(%p,%p,0x%x) rc 0x%x\n", cookie, buf, count, (unsigned)rc);
+fprintf(stderr, "*** davRead(%p,%p,0x%x) rc 0x%x\n", cookie, buf, (unsigned)count, (unsigned)rc);
 #ifdef	DYING
 hexdump(buf, rc);
 #endif
@@ -1083,7 +1100,7 @@ ssize_t davWrite(void * cookie, const char * buf, size_t count)
     ssize_t rc;
     int xx;
 
-#ifndef	NEONBLOWSCHUNKS
+#if !defined(NEONBLOWSCHUNKS)
     ne_session * sess;
 
 assert(fd->req != NULL);
@@ -1093,7 +1110,7 @@ assert(sess != NULL);
     /* HACK: include ne_private.h to access sess->socket for now. */
     xx = ne_sock_fullwrite(sess->socket, buf, count);
 #else
-#if defined(HAVE_NEON_NE_SEND_REQUEST_CHUNK)
+#if defined(HAVE_NEON_NE_SEND_REQUEST_CHUNK) || defined(__LCLINT__)
 assert(fd->req != NULL);
     xx = ne_send_request_chunk(fd->req, buf, count);
 #else
@@ -1106,7 +1123,7 @@ assert(fd->req != NULL);
     rc = (xx == 0 ? count : -1);
 
 if (_dav_debug < 0)
-fprintf(stderr, "*** davWrite(%p,%p,0x%x) rc 0x%x\n", cookie, buf, count, rc);
+fprintf(stderr, "*** davWrite(%p,%p,0x%x) rc 0x%x\n", cookie, buf, (unsigned)count, (unsigned)rc);
 #ifdef	DYING
 if (count > 0)
 hexdump(buf, count);
@@ -1306,7 +1323,7 @@ int davStat(const char * path, /*@out@*/ struct stat *st)
     if (S_ISREG(st->st_mode)) {
 	st->st_nlink = 1;
 	st->st_mode |= 0644;
-    } 
+    }
 
     /* XXX fts(3) needs/uses st_ino, make something up for now. */
     if (st->st_ino == 0)
@@ -1351,7 +1368,7 @@ int davLstat(const char * path, /*@out@*/ struct stat *st)
     if (S_ISREG(st->st_mode)) {
 	st->st_nlink = 1;
 	st->st_mode |= 0644;
-    } 
+    }
 
     /* XXX fts(3) needs/uses st_ino, make something up for now. */
     if (st->st_ino == 0)
@@ -1377,8 +1394,6 @@ fprintf(stderr, "*** davReadlink(%s) rc %d\n", path, rc);
 }
 #endif	/* NOTYET */
 
-#endif /* WITH_NEON */
-
 /* =============================================================== */
 /*@unchecked@*/
 int avmagicdir = 0x3607113;
@@ -1432,7 +1447,7 @@ struct dirent * avReaddir(DIR * dir)
     dp->d_ino = i + 1;		/* W2DO? */
     dp->d_reclen = 0;		/* W2DO? */
 
-#if !defined(hpux) && !defined(sun)
+#if !(defined(hpux) || defined(__hpux) || defined(sun))
 #if !defined(__APPLE__)
     dp->d_off = 0;		/* W2DO? */
 #endif
@@ -1445,7 +1460,7 @@ struct dirent * avReaddir(DIR * dir)
     strncpy(dp->d_name, av[i], sizeof(dp->d_name));
 if (_av_debug)
 fprintf(stderr, "*** avReaddir(%p) %p \"%s\"\n", (void *)avdir, dp, dp->d_name);
-    
+
     return dp;
 }
 
@@ -1503,8 +1518,6 @@ fprintf(stderr, "*** avOpendir(%s)\n", path);
 }
 /*@=boundswrite@*/
 
-#ifdef WITH_NEON
-
 /* =============================================================== */
 /*@unchecked@*/
 int davmagicdir = 0x8440291;
@@ -1558,7 +1571,7 @@ struct dirent * davReaddir(DIR * dir)
     dp->d_ino = i + 1;		/* W2DO? */
     dp->d_reclen = 0;		/* W2DO? */
 
-#if !defined(hpux) && !defined(sun)
+#if !(defined(hpux) || defined(__hpux) || defined(sun))
 #if !defined(__APPLE__)
     dp->d_off = 0;		/* W2DO? */
 #endif
@@ -1571,7 +1584,7 @@ struct dirent * davReaddir(DIR * dir)
     strncpy(dp->d_name, av[i], sizeof(dp->d_name));
 if (_dav_debug < 0)
 fprintf(stderr, "*** davReaddir(%p) %p \"%s\"\n", (void *)avdir, dp, dp->d_name);
-    
+
     return dp;
 }
 
@@ -1672,6 +1685,4 @@ fprintf(stderr, "*** davOpendir(%s)\n", path);
     return (DIR *) avdir;
 /*@=kepttrans@*/
 }
-
-#endif /* WITH_NEON */
 /*@=modfilesys@*/
