@@ -9,10 +9,6 @@
 # include <machine/types.h>
 #endif
 
-#if HAVE_SYS_SOCKET_H
-# include <sys/socket.h>
-#endif
-
 #include <netinet/in.h>
 #include <arpa/inet.h>		/* XXX for inet_aton and HP-UX */
 
@@ -21,12 +17,14 @@
 # include <netinet/in_systm.h>
 #endif
 
+#include <rpmmacro.h>		/* XXX rpmioAccess needs rpmCleanPath() */
+
 #if HAVE_LIBIO_H && defined(_G_IO_IO_FILE_VERSION)
 #define	_USE_LIBIO	1
 #endif
 
 /* XXX HP-UX w/o -D_XOPEN_SOURCE needs */
-#if !defined(HAVE_HERRNO) && (defined(__hpux) || defined(__LCLINT__))
+#if !defined(HAVE_HERRNO) && (defined(hpux) || defined(__hpux) || defined(__LCLINT__))
 /*@unchecked@*/
 extern int h_errno;
 #endif
@@ -105,11 +103,6 @@ int noLibio = 1;
 /*@unchecked@*/
 static int ftpTimeoutSecs = TIMEOUT_SECS;
 
-/**
- */
-/*@unchecked@*/
-static int httpTimeoutSecs = TIMEOUT_SECS;
-
 /**
  */
 /*@unchecked@*/
@@ -242,6 +235,7 @@ FD_t fdDup(int fdno)
     if ((nfdno = dup(fdno)) < 0)
 	return NULL;
     fd = fdNew("open (fdDup)");
+    fdSetOpen(fd, "fdDup", nfdno, 0);	/* XXX bogus */
     fdSetFdno(fd, nfdno);
 DBGIO(fd, (stderr, "==> fdDup(%d) fd %p %s\n", fdno, (fd ? fd : NULL), fdbg(fd)));
     /*@-refcounttrans@*/ return fd; /*@=refcounttrans@*/
@@ -307,6 +301,7 @@ DBGREFS(0, (stderr, "--> fd  %p -- %d %s at %s:%u\n", fd, FDNREFS(fd), msg, file
 DBGREFS(fd, (stderr, "--> fd  %p -- %d %s at %s:%u %s\n", fd, fd->nrefs, msg, file, line, fdbg(fd)));
 	if (--fd->nrefs > 0)
 	    /*@-refcounttrans -retalias@*/ return fd; /*@=refcounttrans =retalias@*/
+	fd->opath = _free(fd->opath);
 	fd->stats = _free(fd->stats);
 	for (i = fd->ndigests - 1; i >= 0; i--) {
 	    FDDIGEST_t fddig = fd->digests + i;
@@ -341,8 +336,11 @@ FD_t XfdNew(const char * msg, const char * file, unsigned line)
     fd->fps[0].fp = NULL;
     fd->fps[0].fdno = -1;
 
+    fd->opath = NULL;
+    fd->oflags = 0;
+    fd->omode = 0;
     fd->url = NULL;
-    fd->rd_timeoutsecs = 60;	/* XXX default value used to be -1 */
+    fd->rd_timeoutsecs = 1;	/* XXX default value used to be -1 */
     fd->contentLength = fd->bytesRemain = -1;
     fd->wr_chunked = 0;
     fd->syserrno = 0;
@@ -353,8 +351,6 @@ FD_t XfdNew(const char * msg, const char * file, unsigned line)
     memset(fd->digests, 0, sizeof(fd->digests));
 
     fd->ftpFileDoneNeeded = 0;
-    fd->firstFree = 0;
-    fd->fileSize = 0;
     fd->fd_cpioPos = 0;
 
     return XfdLink(fd, msg, file, line);
@@ -375,11 +371,7 @@ static ssize_t fdRead(void * cookie, /*@out@*/ char * buf, size_t count)
 /*@-boundswrite@*/
     /* HACK: flimsy wiring for davRead */
     if (fd->req != NULL) {
-#ifdef WITH_NEON
 	rc = davRead(fd, buf, (count > fd->bytesRemain ? fd->bytesRemain : count));
-#else
-	rc = -1;
-#endif
 	/* XXX Chunked davRead EOF. */
 	if (rc == 0)
 	    fd->bytesRemain = 0;
@@ -388,7 +380,7 @@ static ssize_t fdRead(void * cookie, /*@out@*/ char * buf, size_t count)
 /*@=boundswrite@*/
     fdstat_exit(fd, FDSTAT_READ, rc);
 
-    if (fd->ndigests && rc > 0) fdUpdateDigests(fd, buf, rc);
+    if (fd->ndigests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
 
 DBGIO(fd, (stderr, "==>\tfdRead(%p,%p,%ld) rc %ld %s\n", cookie, buf, (long)count, (long)rc, fdbg(fd)));
 
@@ -405,20 +397,16 @@ static ssize_t fdWrite(void * cookie, const char * buf, size_t count)
 
     if (fd->bytesRemain == 0) return 0;	/* XXX simulate EOF */
 
-    if (fd->ndigests && count > 0) fdUpdateDigests(fd, buf, count);
+    if (fd->ndigests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
 
     if (count == 0) return 0;
 
     fdstat_enter(fd, FDSTAT_WRITE);
 /*@-boundsread@*/
     /* HACK: flimsy wiring for davWrite */
-    if (fd->req != NULL) {
-#ifdef WITH_NEON
+    if (fd->req != NULL)
 	rc = davWrite(fd, buf, (count > fd->bytesRemain ? fd->bytesRemain : count));
-#else
-	return -1;
-#endif
-    } else
+    else
 	rc = write(fdno, buf, (count > fd->bytesRemain ? fd->bytesRemain : count));
 /*@=boundsread@*/
     fdstat_exit(fd, FDSTAT_WRITE, rc);
@@ -467,13 +455,9 @@ static int fdClose( /*@only@*/ void * cookie)
     fdstat_enter(fd, FDSTAT_CLOSE);
     /* HACK: flimsy wiring for davClose */
 /*@-branchstate@*/
-    if (fd->req != NULL) {
-#ifdef WITH_NEON
+    if (fd->req != NULL)
 	rc = davClose(fd);
-#else
-	return -1;
-#endif
-    } else
+    else
 	rc = ((fdno >= 0) ? close(fdno) : -2);
 /*@=branchstate@*/
     fdstat_exit(fd, FDSTAT_CLOSE, rc);
@@ -498,6 +482,7 @@ static /*@null@*/ FD_t fdOpen(const char *path, int flags, mode_t mode)
 	return NULL;
     }
     fd = fdNew("open (fdOpen)");
+    fdSetOpen(fd, path, flags, mode);
     fdSetFdno(fd, fdno);
     fd->flags = flags;
 DBGIO(fd, (stderr, "==>\tfdOpen(\"%s\",%x,0%o) %s\n", path, (unsigned)flags, (unsigned)mode, fdbg(fd)));
@@ -687,7 +672,7 @@ fprintf(stderr, "*** read: fd %p rc %d EOF errno %d %s \"%s\"\n", fd, rc, errno,
 /* =============================================================== */
 /* Support for FTP/HTTP I/O.
  */
-const char *const ftpStrerror(int errorNumber)
+const char * ftpStrerror(int errorNumber)
 {
     switch (errorNumber) {
     case 0:
@@ -766,7 +751,6 @@ const char *urlStrerror(const char *url)
     return retstr;
 }
 
-#if !defined(HAVE_GETADDRINFO)
 #if !defined(USE_ALT_DNS) || !USE_ALT_DNS 
 static int mygethostbyname(const char * host,
 		/*@out@*/ struct in_addr * address)
@@ -817,50 +801,14 @@ static int getHostAddress(const char * host, /*@out@*/ struct in_addr * address)
 }
 /*@=compdef@*/
 /*@=boundsread@*/
-#endif
 
 static int tcpConnect(FD_t ctrl, const char * host, int port)
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies ctrl, fileSystem, internalState @*/
 {
+    struct sockaddr_in sin;
     int fdno = -1;
     int rc;
-#ifdef	HAVE_GETADDRINFO
-    struct addrinfo hints, *res, *res0;
-    char pbuf[NI_MAXSERV];
-
-    memset(&hints, 0, sizeof(hints));
-    hints.ai_family = AF_UNSPEC;
-    hints.ai_socktype = SOCK_STREAM;
-    sprintf(pbuf, "%d", port);
-    pbuf[sizeof(pbuf)-1] = '\0';
-    rc = FTPERR_FAILED_CONNECT;
-    if (getaddrinfo(host, pbuf, &hints, &res0) == 0) {
-	for (res = res0; res != NULL; res= res->ai_next) {
-	    if ((fdno = socket(res->ai_family, res->ai_socktype, res->ai_protocol)) < 0)
-		continue;
-	    if (connect(fdno, res->ai_addr, res->ai_addrlen) < 0) {
-		close(fdno);
-		continue;
-	    }
-	    /* success */
-	    rc = 0;
-	    if (_ftp_debug) {
-		char hbuf[NI_MAXHOST];
-		getnameinfo(res->ai_addr, res->ai_addrlen, hbuf, sizeof(hbuf),
-				NULL, 0, NI_NUMERICHOST);
-		fprintf(stderr,"++ connect [%s]:%d on fdno %d\n",
-				/*@-unrecog@*/ hbuf /*@=unrecog@*/, port, fdno);
-	    }
-	    break;
-	}
-	freeaddrinfo(res0);
-    }
-    if (rc < 0)
-	goto errxit;
-
-#else	/* HAVE_GETADDRINFO */			    
-    struct sockaddr_in sin;
 
 /*@-boundswrite@*/
     memset(&sin, 0, sizeof(sin));
@@ -895,7 +843,6 @@ fprintf(stderr,"++ connect %s:%d on fdno %d\n",
 inet_ntoa(sin.sin_addr)
 /*@=unrecog =moduncon =evalorderuncon @*/ ,
 (int)ntohs(sin.sin_port), fdno);
-#endif	/* HAVE_GETADDRINFO */
 
     fdSetFdno(ctrl, (fdno >= 0 ? fdno : -1));
     return 0;
@@ -1228,17 +1175,12 @@ errxit2:
 int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
 {
     urlinfo u = data->url;
-#if !defined(HAVE_GETADDRINFO)
     struct sockaddr_in dataAddress;
-#endif	/* HAVE_GETADDRINFO */
-    char remoteIP[NI_MAXHOST];
     char * cmd;
     int cmdlen;
     char * passReply;
     char * chptr;
     int rc;
-    int epsv;
-    int port;
 
 /*@-boundswrite@*/
     URLSANE(u);
@@ -1273,35 +1215,8 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
 	data->contentLength = cl;
     }
 
-    epsv = 0;
     passReply = NULL;
-#ifdef HAVE_GETNAMEINFO
-    rc = ftpCommand(u, &passReply, "EPSV", NULL);
-    if (rc==0) {
-#ifdef HAVE_GETADDRINFO
-	struct sockaddr_storage ss;
-#else /* HAVE_GETADDRINFO */
-	struct sockaddr_in ss;
-#endif /* HAVE_GETADDRINFO */
-	int size;
-	/* we need to know IP of remote host */
-	size=sizeof(ss);
-	if ((getpeername(fdFileno(c2f(u->ctrl)), (struct sockaddr *)&ss, &size) == 0) &&
-			(getnameinfo((struct sockaddr *)&ss, size, remoteIP, sizeof(remoteIP),
-				NULL, 0, NI_NUMERICHOST) == 0))
-		epsv++;
-	else {
-		/* abort EPSV and fall back to PASV */
-		rc = ftpCommand(u, &passReply, "ABOR", NULL);
-		if (rc) {
-		    rc = FTPERR_PASSIVE_ERROR;
-		    goto errxit;
-		}
-	}
-    }
-    if (epsv==0)
-#endif /* HAVE_GETNAMEINFO */
-        rc = ftpCommand(u, &passReply, "PASV", NULL);
+    rc = ftpCommand(u, &passReply, "PASV", NULL);
     if (rc) {
 	rc = FTPERR_PASSIVE_ERROR;
 	goto errxit;
@@ -1316,15 +1231,6 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
     if (*chptr != ')') return FTPERR_PASSIVE_ERROR;
     *chptr-- = '\0';
 
-    if (epsv) {
-	int i;
-        if(sscanf(passReply,"%*c%*c%*c%d%*c",&i) != 1) {
-	   rc = FTPERR_PASSIVE_ERROR;
-	   goto errxit;
-	}
-	port = i;
-    } else {
- 
     while (*chptr && *chptr != ',') chptr--;
     if (*chptr != ',') return FTPERR_PASSIVE_ERROR;
     chptr--;
@@ -1336,11 +1242,13 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
        port number portion */
 
     {	int i, j;
+	memset(&dataAddress, 0, sizeof(dataAddress));
+	dataAddress.sin_family = AF_INET;
 	if (sscanf(chptr, "%d,%d", &i, &j) != 2) {
 	    rc = FTPERR_PASSIVE_ERROR;
 	    goto errxit;
 	}
-	port = (((unsigned)i) << 8) + j;
+	dataAddress.sin_port = htons((((unsigned)i) << 8) + j);
     }
 
     chptr = passReply;
@@ -1348,75 +1256,9 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
 	if (*chptr == ',') *chptr = '.';
     }
 /*@=boundswrite@*/
-    sprintf(remoteIP, "%s", passReply);
-    } /* if (epsv) */
-
-#ifdef HAVE_GETADDRINFO
-    {
-        struct addrinfo hints, *res, *res0;
-	char pbuf[NI_MAXSERV];
-
-	memset(&hints, 0, sizeof(hints));
-	hints.ai_family = AF_UNSPEC;
-	hints.ai_socktype = SOCK_STREAM;
-	hints.ai_flags = AI_NUMERICHOST;
-	sprintf(pbuf, "%d", port);
-	pbuf[sizeof(pbuf)-1] = '\0';
-	if (getaddrinfo(remoteIP, pbuf, &hints, &res0)) {
-	    rc = FTPERR_PASSIVE_ERROR;
-	    goto errxit;
-	}
-
-	for (res = res0; res != NULL; res = res->ai_next) {
-	    rc = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
-	    fdSetFdno(data, (rc >= 0 ? rc : -1));
-	    if (rc < 0) {
-	        if (res->ai_next)
-		    continue;
-		else {
-		    rc = FTPERR_FAILED_CONNECT;
-		    freeaddrinfo(res0);
-		    goto errxit;
-		}
-	    }
-	    data = fdLink(data, "open data (ftpReq)");
-
-	    /* XXX setsockopt SO_LINGER */
-	    /* XXX setsockopt SO_KEEPALIVE */
-	    /* XXX setsockopt SO_TOS IPTOS_THROUGHPUT */
-	    
-	    {
-		int criterr = 0;
-	        while (connect(fdFileno(data), res->ai_addr, res->ai_addrlen) < 0) {
-	            if (errno == EINTR)
-		        continue;
-		    criterr++;
-		}
-		if (criterr) {
-		    if (res->ai_addr) {
-		        fdClose(data);
-		        continue;
-		    } else {
-		        rc = FTPERR_PASSIVE_ERROR;
-		        freeaddrinfo(res0);
-		        goto errxit;
-		    }
-		}
-	    }
-	    /* success */
-	    rc = 0;
-	    break;
-	}
-	freeaddrinfo(res0);
-    }
-	    
-#else /* HAVE_GETADDRINFO */
-    memset(&dataAddress, 0, sizeof(dataAddress));
-    dataAddress.sin_family = AF_INET;
-    dataAddress.sin_port = htons(port);
 
     /*@-moduncon@*/
-    if (!inet_aton(remoteIP, &dataAddress.sin_addr)) {
+    if (!inet_aton(passReply, &dataAddress.sin_addr)) {
 	rc = FTPERR_PASSIVE_ERROR;
 	goto errxit;
     }
@@ -1444,7 +1286,6 @@ int ftpReq(FD_t data, const char * ftpCmd, const char * ftpArg)
 	goto errxit;
     }
     /*@=internalglobs@*/
-#endif /* HAVE_GETADDRINFO */
 
 if (_ftp_debug)
 fprintf(stderr, "-> %s", cmd);
@@ -1567,6 +1408,7 @@ static int urlConnect(const char * url, /*@out@*/ urlinfo * uret)
 
 	if ((fd = u->ctrl) == NULL) {
 	    fd = u->ctrl = fdNew("persist ctrl (urlConnect FTP)");
+	    fdSetOpen(u->ctrl, url, 0, 0);
 	    fdSetIo(u->ctrl, ufdio);
 	}
 	
@@ -1722,6 +1564,7 @@ static int ftpFileDone(urlinfo u, FD_t data)
     return rc;
 }
 
+#ifdef DEAD
 static int httpResp(urlinfo u, FD_t ctrl, /*@out@*/ char ** str)
 	/*@globals fileSystem @*/
 	/*@modifies ctrl, *str, fileSystem @*/
@@ -1758,7 +1601,6 @@ static int httpReq(FD_t ctrl, const char * httpCmd, const char * httpArg)
     urlinfo u;
     const char * host;
     const char * path;
-    char hthost[NI_MAXHOST];
     int port;
     int rc;
     char * req;
@@ -1771,10 +1613,6 @@ assert(ctrl != NULL);
 
     if (((host = (u->proxyh ? u->proxyh : u->host)) == NULL))
 	return FTPERR_BAD_HOSTNAME;
-    if (strchr(host, ':'))
-	sprintf(hthost, "[%s]", host);
-    else
-	strcpy(hthost, host);
 
     if ((port = (u->proxyp > 0 ? u->proxyp : u->port)) < 0) port = 80;
     path = (u->proxyh || u->proxyp > 0) ? u->url : httpArg;
@@ -1804,7 +1642,7 @@ Host: y:z\r\n\
 Accept: text/plain\r\n\
 Transfer-Encoding: chunked\r\n\
 \r\n\
-") + strlen(httpCmd) + strlen(path) + sizeof(VERSION) + strlen(hthost) + 20;
+") + strlen(httpCmd) + strlen(path) + sizeof(VERSION) + strlen(host) + 20;
 
 /*@-boundswrite@*/
     req = alloca(len);
@@ -1818,7 +1656,7 @@ Host: %s:%d\r\n\
 Accept: text/plain\r\n\
 Transfer-Encoding: chunked\r\n\
 \r\n\
-",	httpCmd, path, (u->httpVersion ? 1 : 0), VERSION, hthost, port);
+",	httpCmd, path, (u->httpVersion ? 1 : 0), VERSION, host, port);
 } else {
     sprintf(req, "\
 %s %s HTTP/1.%d\r\n\
@@ -1826,7 +1664,7 @@ User-Agent: rpm/%s\r\n\
 Host: %s:%d\r\n\
 Accept: text/plain\r\n\
 \r\n\
-",	httpCmd, path, (u->httpVersion ? 1 : 0), VERSION, hthost, port);
+",	httpCmd, path, (u->httpVersion ? 1 : 0), VERSION, host, port);
 }
 /*@=boundswrite@*/
 
@@ -1872,6 +1710,7 @@ errxit2:
     return rc;
 /*@=usereleased@*/
 }
+#endif
 
 /* XXX DYING: unused */
 void * ufdGetUrlinfo(FD_t fd)
@@ -2175,6 +2014,7 @@ int ufdClose( /*@only@*/ void * cookie)
 	fd = fdNew("grab data (ftpOpen)");
 
     if (fd) {
+	fdSetOpen(fd, url, flags, mode);
 	fdSetIo(fd, ufdio);
 	fd->ftpFileDoneNeeded = 0;
 	fd->rd_timeoutsecs = ftpTimeoutSecs;
@@ -2194,56 +2034,6 @@ exit:
 }
 /*@=nullstate@*/
 
-#ifndef WITH_NEON
-/*@-nullstate@*/        /* FIX: u->{ctrl,data}->url undef after XurlLink. */
-static /*@null@*/ FD_t httpOpen(const char * url, /*@unused@*/ int flags,
-                /*@unused@*/ mode_t mode, /*@out@*/ urlinfo * uret)
-        /*@globals internalState @*/
-        /*@modifies *uret, internalState @*/
-{
-    urlinfo u = NULL;
-    FD_t fd = NULL;
-
-#if 0   /* XXX makeTempFile() heartburn */
-    assert(!(flags & O_RDWR));
-#endif
-    if (urlSplit(url, &u))
-        goto exit;
-
-    if (u->ctrl == NULL)
-        u->ctrl = fdNew("persist ctrl (httpOpen)");
-    if (u->ctrl->nrefs > 2 && u->data == NULL)
-        u->data = fdNew("persist data (httpOpen)");
-
-    if (u->ctrl->url == NULL)
-        fd = fdLink(u->ctrl, "grab ctrl (httpOpen persist ctrl)");
-    else if (u->data->url == NULL)
-        fd = fdLink(u->data, "grab ctrl (httpOpen persist data)");
-    else
-        fd = fdNew("grab ctrl (httpOpen)");
-
-    if (fd) {
-        fdSetIo(fd, ufdio);
-        fd->ftpFileDoneNeeded = 0;
-        fd->rd_timeoutsecs = httpTimeoutSecs;
-        fd->contentLength = fd->bytesRemain = -1;
-        fd->url = urlLink(u, "url (httpOpen)");
-        fd = fdLink(fd, "grab data (httpOpen)");
-        fd->urlType = URL_IS_HTTP;
-    }
-
-exit:
-/*@-boundswrite@*/
-    if (uret)
-        *uret = u;
-/*@=boundswrite@*/
-    /*@-refcounttrans@*/
-    return fd;
-    /*@=refcounttrans@*/
-}
-/*@=nullstate@*/
-#endif
-
 static /*@null@*/ FD_t ufdOpen(const char * url, int flags, mode_t mode)
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/
@@ -2282,11 +2072,7 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
     case URL_IS_HTTPS:
     case URL_IS_HTTP:
     case URL_IS_HKP:
-#ifdef WITH_NEON
 	fd = davOpen(url, flags, mode, &u);
-#else
-	fd = httpOpen(url, flags, mode, &u);
-#endif
 	if (fd == NULL || u == NULL)
 	    break;
 
@@ -2294,11 +2080,7 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
 		?  ((flags & O_APPEND) ? "PUT" :
 		   ((flags & O_CREAT) ? "PUT" : "PUT"))
 		: "GET");
-#ifdef WITH_NEON
 	u->openError = davReq(fd, cmd, path);
-#else
-	u->openError = httpReq(fd, cmd, path);
-#endif
 	if (u->openError < 0) {
 	    /* XXX make sure that we can exit through ufdClose */
 	    fd = fdLink(fd, "error ctrl (ufdOpen HTTP)");
@@ -2314,6 +2096,7 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
 	assert(!(flags & O_RDWR));
 	fd = fdDup( ((flags & O_WRONLY) ? STDOUT_FILENO : STDIN_FILENO) );
 	if (fd) {
+	    fdSetOpen(fd, url, flags, mode);
 	    fdSetIo(fd, ufdio);
 	    fd->rd_timeoutsecs = 600;	/* XXX W2DO? 10 mins? */
 	    fd->contentLength = fd->bytesRemain = -1;
@@ -2325,7 +2108,7 @@ fprintf(stderr, "*** ufdOpen(%s,0x%x,0%o)\n", url, (unsigned)flags, (unsigned)mo
 	fd = fdOpen(path, flags, mode);
 	if (fd) {
 	    fdSetIo(fd, ufdio);
-	    fd->rd_timeoutsecs = 60;
+	    fd->rd_timeoutsecs = 1;
 	    fd->contentLength = fd->bytesRemain = -1;
 	}
 	break;
@@ -2452,7 +2235,7 @@ DBGIO(fd, (stderr, "==>\tgzdRead(%p,%p,%u) rc %lx %s\n", cookie, buf, (unsigned)
 	}
     } else if (rc >= 0) {
 	fdstat_exit(fd, FDSTAT_READ, rc);
-	if (fd->ndigests && rc > 0) fdUpdateDigests(fd, buf, rc);
+	if (fd->ndigests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
     }
     return rc;
 }
@@ -2467,7 +2250,7 @@ static ssize_t gzdWrite(void * cookie, const char * buf, size_t count)
 
     if (fd == NULL || fd->bytesRemain == 0) return 0;	/* XXX simulate EOF */
 
-    if (fd->ndigests && count > 0) fdUpdateDigests(fd, buf, count);
+    if (fd->ndigests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
 
     gzfile = gzdFileno(fd);
     if (gzfile == NULL) return -2;	/* XXX can't happen */
@@ -2689,7 +2472,7 @@ static ssize_t bzdRead(void * cookie, /*@out@*/ char * buf, size_t count)
     } else if (rc >= 0) {
 	fdstat_exit(fd, FDSTAT_READ, rc);
 	/*@-compdef@*/
-	if (fd->ndigests && rc > 0) fdUpdateDigests(fd, buf, rc);
+	if (fd->ndigests && rc > 0) fdUpdateDigests(fd, (void *)buf, rc);
 	/*@=compdef@*/
     }
     return rc;
@@ -2708,7 +2491,7 @@ static ssize_t bzdWrite(void * cookie, const char * buf, size_t count)
 
     if (fd->bytesRemain == 0) return 0;	/* XXX simulate EOF */
 
-    if (fd->ndigests && count > 0) fdUpdateDigests(fd, buf, count);
+    if (fd->ndigests && count > 0) fdUpdateDigests(fd, (void *)buf, count);
 
     bzfile = bzdFileno(fd);
     fdstat_enter(fd, FDSTAT_WRITE);
@@ -2994,6 +2777,10 @@ DBGIO(fd, (stderr, "==> Fclose(%p) %s\n", (fd ? fd : NULL), fdbg(fd)));
  * Convert stdio fmode to open(2) mode, filtering out zlib/bzlib flags.
  *	returns stdio[0] = NUL on error.
  *
+ * @todo glibc also supports ",ccs="
+ *
+ * - glibc:	m use mmap'd input
+ * - glibc:	c no cancel
  * - gzopen:	[0-9] is compession level
  * - gzopen:	'f' is filtered (Z_FILTERED)
  * - gzopen:	'h' is Huffman encoding (Z_HUFFMAN_ONLY)
@@ -3041,6 +2828,8 @@ static inline void cvtfmode (const char *m,
 	    if (--nstdio > 0) *stdio++ = c;
 	    continue;
 	    /*@notreached@*/ /*@switchbreak@*/ break;
+	case 'm':	/* glibc: mmap'd reads */
+	case 'c':	/* glibc: no cancel */
 	case 'b':
 	    if (--nstdio > 0) *stdio++ = c;
 	    continue;
@@ -3186,7 +2975,7 @@ FD_t Fopen(const char *path, const char *fmode)
     char stdio[20], other[20];
     const char *end = NULL;
     mode_t perms = 0666;
-    int flags;
+    int flags = 0;
     FD_t fd;
 
     if (path == NULL || fmode == NULL)
@@ -3317,7 +3106,6 @@ int Fileno(FD_t fd)
 {
     int i, rc = -1;
 
-    if (fd == NULL) return -1;
     if (fd->req != NULL)
 	rc = 123456789;	/* HACK: https has no steenkin fileno. */
     else
@@ -3340,31 +3128,6 @@ int Fcntl(FD_t fd, int op, void *lip)
 /* =============================================================== */
 /* Helper routines that may be generally useful.
  */
-
-char * rpmEscapeSpaces(const char * s)
-{
-    const char * se;
-    char * t;
-    char * te;
-    size_t nb = 0;
-
-    for (se = s; *se; se++) {
-        if (isspace(*se))
-            nb++;
-        nb++;
-    }
-    nb++;
-
-    t = te = xmalloc(nb);
-    for (se = s; *se; se++) {
-        if (isspace(*se))
-            *te++ = '\\';
-        *te++ = *se;
-    }
-    *te = '\0';
-    return t;
-}
-
 /*@-bounds@*/
 int rpmioMkpath(const char * path, mode_t mode, uid_t uid, gid_t gid)
 {
@@ -3416,6 +3179,147 @@ int rpmioMkpath(const char * path, mode_t mode, uid_t uid, gid_t gid)
 }
 /*@=bounds@*/
 
+
+#define	_PATH	"/usr/kerberos/sbin:/usr/kerberos/bin:/usr/lib/ccache/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin:/usr/X11R6/bin:~//bin"
+/*@unchecked@*/ /*@observer@*/
+static const char * _path = _PATH;
+
+#define alloca_strdup(_s)       strcpy(alloca(strlen(_s)+1), (_s))
+
+int rpmioAccess(const char * FN, const char * path, int mode)
+{
+    char fn[4096];
+    char * bn;
+    char * r, * re;
+    char * t, * te;
+    int negate = 0;
+    int rc = 0;
+
+    /* Empty paths are always accessible. */
+    if (FN == NULL || *FN == '\0')
+	return 0;
+
+    if (mode == 0)
+	mode = X_OK;
+
+    /* Strip filename out of its name space wrapper. */
+    bn = alloca_strdup(FN);
+    for (t = bn; t && *t; t++) {
+	if (*t != '(')
+	    continue;
+	*t++ = '\0';
+
+	/* Permit negation on name space tests. */
+	if (*bn == '!') {
+	    negate = 1;
+	    bn++;
+	}
+
+	/* Set access flags from name space marker. */
+	if (strlen(bn) == 3
+	 && strchr("Rr_", bn[0]) != NULL
+	 && strchr("Ww_", bn[1]) != NULL
+	 && strchr("Xx_", bn[2]) != NULL) {
+	    mode = 0;
+	    if (strchr("Rr", bn[0]) != NULL)
+		mode |= R_OK;
+	    if (strchr("Ww", bn[1]) != NULL)
+		mode |= W_OK;
+	    if (strchr("Xx", bn[2]) != NULL)
+		mode |= X_OK;
+	    if (mode == 0)
+		mode = F_OK;
+	} else if (!strcmp(bn, "exists"))
+	    mode = F_OK;
+	else if (!strcmp(bn, "executable"))
+	    mode = X_OK;
+	else if (!strcmp(bn, "readable"))
+	    mode = R_OK;
+	else if (!strcmp(bn, "writable"))
+	    mode = W_OK;
+
+	bn = t;
+	te = bn + strlen(t) - 1;
+	if (*te != ')')		/* XXX syntax error, never exists */
+	    return 1;
+	*te = '\0';
+	break;
+    }
+
+    /* Empty paths are always accessible. */
+    if (*bn == '\0')
+	goto exit;
+
+    /* Check absolute path for access. */
+    if (*bn == '/') {
+	rc = (Access(bn, mode) != 0 ? 1 : 0);
+if (_rpmio_debug)
+fprintf(stderr, "*** rpmioAccess(\"%s\", 0x%x) rc %d\n", bn, mode, rc);
+	goto exit;
+    }
+
+    /* Find path to search. */
+    if (path == NULL)
+	path = getenv("PATH");
+    if (path == NULL)
+	path = _path;
+    if (path == NULL) {
+	rc = 1;
+	goto exit;
+    }
+
+    /* Look for relative basename on PATH. */
+/*@-branchstate@*/
+    for (r = alloca_strdup(path); r != NULL && *r != '\0'; r = re) {
+
+	/* Find next element, terminate current element. */
+        for (re = r; (re = strchr(re, ':')) != NULL; re++) {
+            if (!(re[1] == '/' && re[2] == '/'))
+                /*@innerbreak@*/ break;
+        }
+        if (re && *re == ':')
+            *re++ = '\0';
+	else
+	    re = r + strlen(r);
+
+	/* Expand ~/ to $HOME/ */
+	fn[0] = '\0';
+	t = fn;
+	*t = '\0';	/* XXX redundant. */
+	if (r[0] == '~' && r[1] == '/') {
+	    const char * home = getenv("HOME");
+	    if (home == NULL)	/* XXX No HOME? */
+		continue;
+	    if (strlen(home) > (sizeof(fn) - strlen(r))) /* XXX too big */
+		continue;
+	    t = stpcpy(t, home);
+	    r++;	/* skip ~ */
+	}
+	t = stpcpy(t, r);
+	if (t[-1] != '/' && *bn != '/')
+	    *t++ = '/';
+	t = stpcpy(t, bn);
+	t = rpmCleanPath(fn);
+	if (t == NULL)	/* XXX can't happen */
+	    continue;
+
+	/* Check absolute path for access. */
+	rc = (Access(t, mode) != 0 ? 1 : 0);
+if (_rpmio_debug)
+fprintf(stderr, "*** rpmioAccess(\"%s\", 0x%x) rc %d\n", t, mode, rc);
+	if (rc == 0)
+	    goto exit;
+    }
+/*@=branchstate@*/
+
+    rc = 1;
+
+exit:
+    if (negate)
+	rc ^= 1;
+    return rc;
+}
+
 /*@-boundswrite@*/
 int rpmioSlurp(const char * fn, const byte ** bp, ssize_t * blenp)
 {
