@@ -556,7 +556,7 @@ int fdReadable(FD_t fd, int secs)
 
 /**
  * Insure that directories in path exist, creating as needed.
- * @param path		diretory path
+ * @param path		directory path
  * @param mode		directory mode (if created)
  * @param uid		directory uid (if created), or -1 to skip
  * @param gid		directory uid (if created), or -1 to skip
@@ -566,6 +566,17 @@ int rpmioMkpath(const char * path, mode_t mode, uid_t uid, gid_t gid)
 	/*@globals h_errno, fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
 
+/**
+ * Check FN access, expanding relative paths and twiddles.
+ * @param FN		file path to check
+ * @param path		colon separated search path (NULL uses $PATH)
+ * @param mode		type of access(2) to check (0 uses X_OK)
+ * @return		0 if accessible
+ */
+int rpmioAccess(const char *FN, /*@null@*/ const char * path, int mode)
+	/*@globals fileSystem, internalState @*/
+	/*@modifies fileSystem, internalState @*/;
+
 /**
  * FTP and HTTP error codes.
  */
@@ -599,7 +610,7 @@ typedef enum ftperrCode_e {
 /**
  */
 /*@-redecl@*/
-/*@observer@*/ const char *const ftpStrerror(int errorNumber)
+/*@observer@*/ const char * ftpStrerror(int errorNumber)
 	/*@*/;
 /*@=redecl@*/
 
@@ -705,6 +716,12 @@ int xstrcasecmp(const char * s1, const char * s2)		/*@*/;
  */
 int xstrncasecmp(const char *s1, const char * s2, size_t n)	/*@*/;
 
+/** \ingroup rpmio
+ * Force encoding of string.
+ */
+/*@only@*/ /*@null@*/
+const char * xstrtolocale(/*@only@*/ const char *str)
+	/*@modifies *str @*/;
 #ifdef __cplusplus
 }
 #endif
