@@ -167,6 +167,11 @@ struct _FD_s {
 /*@observer@*/
     const void *errcookie;	/* gzdio/bzdio/ufdio: */
 
+/*null@*/
+    const char *opath;		/* open(2) args. */
+    int		oflags;
+    mode_t	omode;
+
     FDSTAT_t	stats;		/* I/O statistics */
 
     int		ndigests;
@@ -174,9 +179,7 @@ struct _FD_s {
     struct _FDDIGEST_s	digests[FDDIGEST_MAX];
 
     int		ftpFileDoneNeeded; /* ufdio: (FTP) */
-    unsigned int firstFree;	/* fadio: */
-    long int	fileSize;	/* fadio: */
-    long int	fd_cpioPos;	/* cpio: */
+    unsigned long long	fd_cpioPos;	/* cpio: */
 };
 /*@access FD_t@*/
 
@@ -250,6 +253,52 @@ int ufdClose( /*@only@*/ void * cookie)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies cookie, fileSystem, internalState @*/;
 
+/** \ingroup rpmio
+ */
+/*@unused@*/ static inline
+void fdSetOpen(FD_t fd, const char * path, int flags, mode_t mode)
+	/*@modifies fd @*/
+{
+    FDSANE(fd);
+    if (fd->opath != NULL) {
+	free((void *)fd->opath);
+	fd->opath = NULL;
+    }
+    fd->opath = xstrdup(path);
+    fd->oflags = flags;
+    fd->omode = mode;
+}
+
+/** \ingroup rpmio
+ */
+/*@unused@*/ static inline
+/*@null@*/ /*@observer@*/ const char * fdGetOPath(FD_t fd)
+	/*@*/
+{
+    FDSANE(fd);
+    return fd->opath;
+}
+
+/** \ingroup rpmio
+ */
+/*@unused@*/ static inline
+int fdGetOFlags(FD_t fd)
+	/*@*/
+{
+    FDSANE(fd);
+    return fd->oflags;
+}
+
+/** \ingroup rpmio
+ */
+/*@unused@*/ static inline
+mode_t fdGetOMode(FD_t fd)
+	/*@*/
+{
+    FDSANE(fd);
+    return fd->omode;
+}
+
 /** \ingroup rpmio
  */
 /*@unused@*/ static inline
@@ -451,14 +500,14 @@ void fdstat_print(/*@null@*/ FD_t fd, const char * msg, FILE * fp)
 	switch (opx) {
 	case FDSTAT_READ:
 	    if (msg) fprintf(fp, "%s:", msg);
-	    fprintf(fp, "%8d reads, %8ld total bytes in %d.%06d secs\n",
-		op->count, (long)op->bytes,
+	    fprintf(fp, "%8d reads, %8lu total bytes in %d.%06d secs\n",
+		op->count, (unsigned long)op->bytes,
 		(int)(op->usecs/usec_scale), (int)(op->usecs%usec_scale));
 	    /*@switchbreak@*/ break;
 	case FDSTAT_WRITE:
 	    if (msg) fprintf(fp, "%s:", msg);
-	    fprintf(fp, "%8d writes, %8ld total bytes in %d.%06d secs\n",
-		op->count, (long)op->bytes,
+	    fprintf(fp, "%8d writes, %8lu total bytes in %d.%06d secs\n",
+		op->count, (unsigned long)op->bytes,
 		(int)(op->usecs/usec_scale), (int)(op->usecs%usec_scale));
 	    /*@switchbreak@*/ break;
 	case FDSTAT_SEEK:
@@ -496,7 +545,7 @@ int fdGetRdTimeoutSecs(FD_t fd)
 /** \ingroup rpmio
  */
 /*@unused@*/ static inline
-long int fdGetCpioPos(FD_t fd)
+unsigned long long fdGetCpioPos(FD_t fd)
 	/*@*/
 {
     FDSANE(fd);
