@@ -102,7 +102,7 @@ typedef	enum rpmlogFac_e {
 
 
 #ifdef RPMLOG_NAMES
-CODE facilitynames[] =
+RPMCODE facilitynames[] =
   {
     { "auth",	RPMLOG_AUTH },
     { "authpriv",RPMLOG_AUTHPRIV },
@@ -133,8 +133,8 @@ CODE facilitynames[] =
 /*
  * arguments to setlogmask.
  */
-#define	RPMLOG_MASK(pri) (1 << (pri))		/*!< mask for one priority */
-#define	RPMLOG_UPTO(pri) ((1 << ((pri)+1)) - 1)	/*!< all priorities through pri */
+#define	RPMLOG_MASK(pri) (1 << ((unsigned)(pri)))	/*!< mask for one priority */
+#define	RPMLOG_UPTO(pri) ((1 << (((unsigned)(pri))+1)) - 1)	/*!< all priorities through pri */
 
 /*
  * Option flags for openlog.
@@ -158,7 +158,8 @@ typedef void (*rpmlogCallback) (void);
  */
 typedef /*@abstract@*/ struct rpmlogRec_s {
     int		code;
-/*@owned@*/ /*@null@*/ const char * message;
+/*@owned@*/ /*@null@*/
+    const char * message;
 } * rpmlogRec;
 
 #ifdef __cplusplus
@@ -175,7 +176,7 @@ int rpmlogGetNrecs(void)	/*@*/;
  * Print all rpmError() messages.
  * @param f		file handle (NULL uses stderr)
  */
-void rpmlogPrint(FILE *f)
+void rpmlogPrint(/*@null@*/ FILE *f)
 	/*@modifies *f @*/;
 
 /**
@@ -184,6 +185,7 @@ void rpmlogPrint(FILE *f)
  */
 /*@unused@*/
 void rpmlogClose (void)
+	/*@globals internalState@*/
 	/*@modifies internalState @*/;
 
 /**
@@ -192,19 +194,23 @@ void rpmlogClose (void)
  */
 /*@unused@*/
 void rpmlogOpen (const char * ident, int option, int facility)
+	/*@globals internalState@*/
 	/*@modifies internalState @*/;
 
 /**
  * Set the log mask level.
+ * @param mask		log mask (0 is no operation)
+ * @return		previous log mask
  */
 int rpmlogSetMask (int mask)
+	/*@globals internalState@*/
 	/*@modifies internalState @*/;
 
 /**
  * Generate a log message using FMT string and option arguments.
  */
-/*@mayexit@*/ /*@printflike@*/ void rpmlog (int pri, const char *fmt, ...)
-	/*@modifies internalState @*/;
+/*@mayexit@*/ /*@printflike@*/ void rpmlog (int code, const char *fmt, ...)
+	/*@*/;
 
 /*@-exportlocal@*/
 /**
@@ -212,7 +218,8 @@ int rpmlogSetMask (int mask)
  * @return		text of last message
  */
 /*@-redecl@*/
-/*@observer@*/ /*@null@*/ const char * rpmlogMessage(void)	/*@*/;
+/*@observer@*/ /*@null@*/ const char * rpmlogMessage(void)
+	/*@*/;
 /*@=redecl@*/
 
 /**
@@ -222,12 +229,26 @@ int rpmlogSetMask (int mask)
  *	and parsed IMHO.
  * @return		code from last message
  */
-int rpmlogCode(void)	/*@*/;
+int rpmlogCode(void)
+	/*@*/;
 
 /**
  * Set rpmlog callback function.
+ * @param cb		rpmlog callback function
+ * @return		previous rpmlog callback function
  */
 rpmlogCallback rpmlogSetCallback(rpmlogCallback cb)
+	/*@globals internalState@*/
+	/*@modifies internalState @*/;
+
+/**
+ * Set rpmlog file handle.
+ * @param fp		rpmlog file handle (NULL uses stdout/stderr)
+ * @return		previous rpmlog file handle
+ */
+/*@null@*/
+FILE * rpmlogSetFile(/*@null@*/ FILE * fp)
+	/*@globals internalState@*/
 	/*@modifies internalState @*/;
 /*@=exportlocal@*/
 
@@ -236,6 +257,7 @@ rpmlogCallback rpmlogSetCallback(rpmlogCallback cb)
  * @deprecated gnorpm needs, use rpmlogSetCallback() instead.
  */
 extern rpmlogCallback rpmErrorSetCallback(rpmlogCallback cb)
+	/*@globals internalState@*/
 	/*@modifies internalState @*/;
 
 /**
@@ -243,7 +265,8 @@ extern rpmlogCallback rpmErrorSetCallback(rpmlogCallback cb)
  * @deprecated Perl-RPM needs, use rpmlogCode() instead.
  * @return		code from last message
  */
-extern int rpmErrorCode(void)	/*@*/;
+extern int rpmErrorCode(void)
+	/*@*/;
 
 /**
  * Return text of last rpmError() message.
