@@ -36,18 +36,25 @@ typedef /*@abstract@*/ const void * fnpyKey;
 /*@=redef@*/
 
 /**
+ * Bit(s) to identify progress callbacks.
  */
 typedef enum rpmCallbackType_e {
-    RPMCALLBACK_INST_PROGRESS,
-    RPMCALLBACK_INST_START,
-    RPMCALLBACK_INST_OPEN_FILE,
-    RPMCALLBACK_INST_CLOSE_FILE,
-    RPMCALLBACK_TRANS_PROGRESS,
-    RPMCALLBACK_TRANS_START,
-    RPMCALLBACK_TRANS_STOP,
-    RPMCALLBACK_UNINST_PROGRESS,
-    RPMCALLBACK_UNINST_START,
-    RPMCALLBACK_UNINST_STOP
+    RPMCALLBACK_UNKNOWN		= 0,
+    RPMCALLBACK_INST_PROGRESS	= (1 <<  0),
+    RPMCALLBACK_INST_START	= (1 <<  1),
+    RPMCALLBACK_INST_OPEN_FILE	= (1 <<  2),
+    RPMCALLBACK_INST_CLOSE_FILE	= (1 <<  3),
+    RPMCALLBACK_TRANS_PROGRESS	= (1 <<  4),
+    RPMCALLBACK_TRANS_START	= (1 <<  5),
+    RPMCALLBACK_TRANS_STOP	= (1 <<  6),
+    RPMCALLBACK_UNINST_PROGRESS	= (1 <<  7),
+    RPMCALLBACK_UNINST_START	= (1 <<  8),
+    RPMCALLBACK_UNINST_STOP	= (1 <<  9),
+    RPMCALLBACK_REPACKAGE_PROGRESS = (1 << 10),
+    RPMCALLBACK_REPACKAGE_START	= (1 << 11),
+    RPMCALLBACK_REPACKAGE_STOP	= (1 << 12),
+    RPMCALLBACK_UNPACK_ERROR	= (1 << 13),
+    RPMCALLBACK_CPIO_ERROR	= (1 << 14)
 } rpmCallbackType;
 
 /**
@@ -63,8 +70,8 @@ extern "C" {
 typedef void * (*rpmCallbackFunction)
 		(/*@null@*/ const void * h, 
 		const rpmCallbackType what, 
-		const unsigned long amount, 
-		const unsigned long total,
+		const unsigned long long amount, 
+		const unsigned long long total,
 		/*@null@*/ fnpyKey key,
 		/*@null@*/ rpmCallbackData data)
 	/*@globals internalState@*/
