@@ -10,8 +10,6 @@
  *	Copyright (C) The Internet Society (1998).  All Rights Reserved.
  */
 
-#include <string.h>
-
 #if !defined(_BEECRYPT_API_H)
 /*@-redef@*/
 typedef unsigned char byte;
@@ -284,13 +282,16 @@ extern struct pgpValTbl_s pgpCompressionTbl[];
 typedef enum pgpHashAlgo_e {
     PGPHASHALGO_MD5		=  1,	/*!< MD5 */
     PGPHASHALGO_SHA1		=  2,	/*!< SHA1 */
-    PGPHASHALGO_RIPEMD160	=  3,	/*!< RIPEMD160 */
+    PGPHASHALGO_RIPEMD160	=  3,	/*!< RIPEMD-160 */
     PGPHASHALGO_MD2		=  5,	/*!< MD2 */
-    PGPHASHALGO_TIGER192	=  6,	/*!< TIGER192 */
+    PGPHASHALGO_TIGER192	=  6,	/*!< TIGER-192 */
     PGPHASHALGO_HAVAL_5_160	=  7,	/*!< HAVAL-5-160 */
     PGPHASHALGO_SHA256		=  8,	/*!< SHA256 */
     PGPHASHALGO_SHA384		=  9,	/*!< SHA384 */
     PGPHASHALGO_SHA512		= 10,	/*!< SHA512 */
+
+    PGPHASHALGO_RIPEMD128	= 105,	/*!< (private) RIPEMD-128 */
+    PGPHASHALGO_CRC32		= 106,	/*!< (private) CRC-32 */
 } pgpHashAlgo;
 
 /**
@@ -1149,16 +1150,17 @@ int pgpValTok(pgpValTbl vs, const char * s, const char * se)
     return vs->val;
 }
 
-/*@-exportlocal@*/
 /**
  * Print an OpenPGP value.
  * @param pre		output prefix
  * @param vs		table of (string,value) pairs
  * @param val		byte value to print
  */
+/*@-exportlocal@*/
 void pgpPrtVal(const char * pre, pgpValTbl vs, byte val)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/;
+/*@=exportlocal@*/
 
 /**
  * Print/parse an OpenPGP subtype packet.
@@ -1167,9 +1169,11 @@ void pgpPrtVal(const char * pre, pgpValTbl vs, byte val)
  * @param sigtype	signature type
  * @return		0 on success
  */
+/*@-exportlocal@*/
 int pgpPrtSubType(const byte *h, unsigned int hlen, pgpSigType sigtype)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/;
+/*@=exportlocal@*/
 
 /**
  * Print/parse an OpenPGP signature packet.
@@ -1178,9 +1182,11 @@ int pgpPrtSubType(const byte *h, unsigned int hlen, pgpSigType sigtype)
  * @param hlen		packet length (no. of bytes)
  * @return		0 on success
  */
+/*@-exportlocal@*/
 int pgpPrtSig(pgpTag tag, const byte *h, unsigned int hlen)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
+/*@=exportlocal@*/
 
 /**
  * Print/parse an OpenPGP key packet.
@@ -1200,9 +1206,11 @@ int pgpPrtKey(pgpTag tag, const byte *h, unsigned int hlen)
  * @param hlen		packet length (no. of bytes)
  * @return		0 on success
  */
+/*@-exportlocal@*/
 int pgpPrtUserID(pgpTag tag, const byte *h, unsigned int hlen)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
+/*@=exportlocal@*/
 
 /**
  * Print/parse an OpenPGP comment packet.
@@ -1211,9 +1219,11 @@ int pgpPrtUserID(pgpTag tag, const byte *h, unsigned int hlen)
  * @param hlen		packet length (no. of bytes)
  * @return		0 on success
  */
+/*@-exportlocal@*/
 int pgpPrtComment(pgpTag tag, const byte *h, unsigned int hlen)
 	/*@globals fileSystem @*/
 	/*@modifies fileSystem @*/;
+/*@=exportlocal@*/
 
 /**
  * Calculate OpenPGP public key fingerprint.
@@ -1223,9 +1233,21 @@ int pgpPrtComment(pgpTag tag, const byte *h, unsigned int hlen)
  * @retval keyid	publick key fingerprint
  * @return		0 on sucess, else -1
  */
+/*@-exportlocal@*/
 int pgpPubkeyFingerprint(const byte * pkt, unsigned int pktlen,
 		/*@out@*/ byte * keyid)
 	/*@modifies *keyid @*/;
+/*@=exportlocal@*/
+
+/**
+ * Extract OpenPGP public key fingerprint from base64 encoded packet.
+ * @todo V3 non-RSA public keys not implemented.
+ * @param b64pkt	base64 encoded openpgp packet
+ * @retval keyid[8]	public key fingerprint
+ * @return		8 (no. of bytes) on success, < 0 on error
+ */
+int pgpExtractPubkeyFingerprint(const char * b64pkt, /*@out@*/ byte * keyid)
+	/*@modifies *keyid @*/;
 
 /**
  * Print/parse next OpenPGP packet.
@@ -1233,6 +1255,7 @@ int pgpPubkeyFingerprint(const byte * pkt, unsigned int pktlen,
  * @param pleft		no. bytes remaining
  * @return		-1 on error, otherwise this packet length
  */
+/*@-exportlocal@*/
 int pgpPrtPkt(const byte *pkt, unsigned int pleft)
 	/*@globals fileSystem, internalState @*/
 	/*@modifies fileSystem, internalState @*/;
@@ -1316,9 +1339,9 @@ int pgpIsPkt(const byte * p)
 	return 0;
 
     if (val & 0x40)
-	tag = (pgpTag)(val & 0x3f);
+	tag = (val & 0x3f);
     else
-	tag = (pgpTag)((val >> 2) & 0xf);
+	tag = (val >> 2) & 0xf;
 
     switch (tag) {
     case PGPTAG_MARKER:
