@@ -246,8 +246,8 @@ fprintf(stderr, "    Remove(%p): %p\n", ME(), sq);
 
 	    sq->id = NULL;
 /*@-bounds@*/
-	    if (sq->pipes[1])	ret = close(sq->pipes[1]);
-	    if (sq->pipes[0])	ret = close(sq->pipes[0]);
+	    if (sq->pipes[1] > 0)	ret = close(sq->pipes[1]);
+	    if (sq->pipes[0] > 0)	ret = close(sq->pipes[0]);
 	    sq->pipes[0] = sq->pipes[1] = -1;
 /*@=bounds@*/
 #ifdef	NOTYET	/* rpmpsmWait debugging message needs */
@@ -437,7 +437,6 @@ fprintf(stderr, "    Enable(%p): %p\n", ME(), sq);
 
     pid = fork();
     if (pid < (pid_t) 0) {		/* fork failed.  */
-	sq->child = (pid_t)-1;
 /*@-bounds@*/
 	xx = close(sq->pipes[0]);
 	xx = close(sq->pipes[1]);
@@ -515,7 +514,7 @@ static int rpmsqWaitUnregister(rpmsq sq)
 	    xx = sigrelse(SIGCHLD);
 	    
 	    /* 
-	     * We start before the fork with this mutex locked;
+	     * We start of before the fork with this mutex locked;
 	     * The only one that unlocks this the signal handler.
 	     * So if we get the lock the child has been reaped.
 	     */
