@@ -1,5 +1,5 @@
 #include "system.h"
-const char *__progname;
+extern const char *__progname;
 
 #define	_AUTOHELP
 
@@ -18,6 +18,10 @@ const char *__progname;
 #include "rpmps.h"
 #include "rpmts.h"
 
+#include "rpmlua.h"	/* XXX for rpmluaFree() */
+
+#include "fs.h"		/* XXX for rpmFreeFilesystems() */
+
 #ifdef	IAM_RPMBT
 #include "build.h"
 #define GETOPT_REBUILD		1003
@@ -64,8 +68,6 @@ enum modes {
 #define	MODES_FOR_TEST		(MODES_BT | MODES_IE)
 #define	MODES_FOR_ROOT		(MODES_BT | MODES_IE | MODES_QV | MODES_DB | MODES_K)
 
-static int quiet;
-
 /* the structure describing the options we take and the defaults */
 /*@unchecked@*/
 static struct poptOption optionsTable[] = {
@@ -111,8 +113,6 @@ static struct poptOption optionsTable[] = {
 	NULL },
 #endif	/* IAM_RPMEIU */
 
- { "quiet", '\0', 0, &quiet, 0,			NULL, NULL},
-
  { NULL, '\0', POPT_ARG_INCLUDE_TABLE, rpmcliAllPoptTable, 0,
 	N_("Common options for all rpm modes and executables:"),
 	NULL },
@@ -472,7 +472,7 @@ int main(int argc, const char ** argv)
 	argerror(_("--ignoreos may only be specified during package "
 		   "installation"));
 
-    if (bigMode != MODE_INSTALL && bigMode != MODE_ERASE &&
+    if (bigMode != MODE_INSTALL &&
 	(ia->probFilter & (RPMPROB_FILTER_DISKSPACE|RPMPROB_FILTER_DISKNODES)))
 	argerror(_("--ignoresize may only be specified during package "
 		   "installation"));
@@ -528,9 +528,6 @@ int main(int argc, const char ** argv)
 	}
     }
 
-    if (quiet)
-	rpmSetVerbosity(RPMLOG_WARNING);
-
 #if defined(IAM_RPMBT) || defined(IAM_RPMK)
     if (0
 #if defined(IAM_RPMBT)
@@ -631,7 +628,7 @@ int main(int argc, const char ** argv)
     switch (bigMode) {
 #ifdef	IAM_RPMDB
     case MODE_INITDB:
-	ec = rpmtsInitDB(ts, 0644);
+	(void) rpmtsInitDB(ts, 0644);
 	break;
 
     case MODE_REBUILDDB:
@@ -688,7 +685,7 @@ int main(int argc, const char ** argv)
     case MODE_BUILD:
     case MODE_TARBUILD:
     {	const char * pkg;
-        if (!quiet) while (!rpmIsVerbose())
+        while (!rpmIsVerbose())
 	    rpmIncreaseVerbosity();
        
 	switch (ba->buildChar) {
@@ -749,7 +746,7 @@ int main(int argc, const char ** argv)
 	if (!poptPeekArg(optCon)) {
 	    if (ia->rbtid == 0)
 		argerror(_("no packages given for erase"));
-ia->transFlags |= RPMTRANS_FLAG_NOMD5;
+ia->transFlags |= RPMTRANS_FLAG_NOFDIGESTS;
 ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
 	    ec += rpmRollback(ts, ia, NULL);
 	} else {
@@ -789,7 +786,7 @@ ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
 	if (!poptPeekArg(optCon)) {
 	    if (ia->rbtid == 0)
 		argerror(_("no packages given for install"));
-ia->transFlags |= RPMTRANS_FLAG_NOMD5;
+ia->transFlags |= RPMTRANS_FLAG_NOFDIGESTS;
 ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
 /*@i@*/	    ec += rpmRollback(ts, ia, NULL);
 	} else {
@@ -828,7 +825,7 @@ ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
 #ifdef IAM_RPMK
     case MODE_CHECKSIG:
     {	rpmVerifyFlags verifyFlags =
-		(VERIFY_MD5|VERIFY_DIGEST|VERIFY_SIGNATURE);
+		(VERIFY_FDIGEST|VERIFY_HDRCHK|VERIFY_DIGEST|VERIFY_SIGNATURE);
 
 	verifyFlags &= ~ka->qva_flags;
 	ka->qva_flags = (rpmQueryFlags) verifyFlags;
@@ -866,7 +863,7 @@ ia->probFilter |= RPMPROB_FILTER_OLDPACKAGE;
 #endif
     case MODE_UNKNOWN:
 	if (poptPeekArg(optCon) != NULL || argc <= 1 || rpmIsVerbose()) {
-	    printUsage(optCon, stderr, 0);
+	    printUsage(optCon, stdout, 0);
 	    ec = argc;
 	}
 	break;
@@ -882,6 +879,7 @@ exit:
     rpmFreeMacros(NULL);
 /*@i@*/	rpmFreeMacros(rpmCLIMacroContext);
     rpmFreeRpmrc();
+    (void) rpmluaFree(NULL);
 
     if (pipeChild) {
 	(void) fclose(stdout);
@@ -889,7 +887,7 @@ exit:
     }
 
     /* keeps memory leak checkers quiet */
-    freeFilesystems();
+    rpmFreeFilesystems();
 /*@i@*/	urlFreeCache();
     rpmlogClose();
     dbiTags = _free(dbiTags);
@@ -917,8 +915,8 @@ exit:
     /*@=noeffect@*/
 #endif
 
-    /* XXX Avoid exit status overflow. Status 255 is special to xargs(1) */
-    if (ec > 254) ec = 254;
+    /* XXX don't overflow single byte exit status */
+    if (ec > 255) ec = 255;
 
     /*@-globstate@*/
     return ec;
