@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2003, 2005, 2007 Red Hat, Inc.
+/* Copyright (C) 2001, 2002, 2003, 2005 Red Hat, Inc.
    Written by Alexander Larsson <alexl@redhat.com>, 2002
    Based on code by Jakub Jelinek <jakub@redhat.com>, 2001.
 
@@ -36,8 +36,6 @@
 #include <gelf.h>
 #include <dwarf.h>
 
-#include <beecrypt/beecrypt.h>
-
 #include "hashtab.h"
 
 #define DW_TAG_partial_unit 0x3c
@@ -45,8 +43,7 @@
 char *base_dir = NULL;
 char *dest_dir = NULL;
 char *list_file = NULL;
-int list_file_fd = -1;
-int do_build_id = 0;
+int list_file_fd = -1; 
 
 typedef unsigned int uint_32;
 typedef unsigned short uint_16;
@@ -374,45 +371,42 @@ canonicalize_path (const char *s, char *d)
 
       if (s[0] == '.' && (s[1] == 0 || IS_DIR_SEPARATOR (s[1])))
 	{
-	  s++;
+	  s ++;
 	  if (*s)
-	    while (IS_DIR_SEPARATOR (*s))
-	      ++s;
+	    s++;
+	  else if (d > droot)
+	    d--;
 	}
 
       else if (s[0] == '.' && s[1] == '.'
 	       && (s[2] == 0 || IS_DIR_SEPARATOR (s[2])))
 	{
-	  char *pre = d - 1; /* includes slash */
+	  char *pre = d-1; /* includes slash */
 	  while (droot < pre && IS_DIR_SEPARATOR (*pre))
 	    pre--;
 	  if (droot <= pre && ! IS_DIR_SEPARATOR (*pre))
 	    {
-	      while (droot < pre && ! IS_DIR_SEPARATOR (*pre))
-		pre--;
-	      /* pre now points to the slash */
-	      if (droot < pre)
-		pre++;
-	      if (pre + 3 == d && pre[0] == '.' && pre[1] == '.')
-		{
-		  *d++ = *s++;
-		  *d++ = *s++;
-		}
-	      else
-		{
-		  d = pre;
-		  s += 2;
-		  if (*s)
-		    while (IS_DIR_SEPARATOR (*s))
-		      s++;
-		}
+	      d = pre;
+	      while (droot < d && ! IS_DIR_SEPARATOR (*d))
+		d--;
+	      /* d now points to the slash */
+	      if (droot < d)
+		d++;
+	      s += 2;
+	      if (*s)
+		s++;
+	      else if (d > droot)
+		d--;
 	    }
 	  else
 	    {
 	      *d++ = *s++;
 	      *d++ = *s++;
+	      if (*s)
+		*d++ = *s++;
 	    }
 	}
+
       else
 	{
 	  while (*s && ! IS_DIR_SEPARATOR (*s))
@@ -567,22 +561,23 @@ edit_dwarf2_line (DSO *dso, uint_32 off, char *comp_dir, int phase)
 	  memcpy (s + comp_dir_len + 1 + dir_len + 1, file, file_len + 1);
 	}
       canonicalize_path (s, s);
-      if (list_file_fd != -1)
+      if (base_dir == NULL ||
+	  has_prefix (s, base_dir))
 	{
-	  char *p = NULL;
-	  if (base_dir == NULL)
-	    p = s;
-	  else if (has_prefix (s, base_dir))
+	  char *p;
+	  size_t size;
+	  ssize_t ret;
+	  if (base_dir)
 	    p = s + strlen (base_dir);
-	  else if (has_prefix (s, dest_dir))
-	    p = s + strlen (dest_dir);
-
-	  if (p)
+	  else
+	    p = s;
+	  
+	  if (list_file_fd != -1)
 	    {
-	      size_t size = strlen (p) + 1;
+	      size = strlen (p) + 1;
 	      while (size > 0)
 		{
-		  ssize_t ret = write (list_file_fd, p, size);
+		  ret = write (list_file_fd, p, size);
 		  if (ret == -1)
 		    break;
 		  size -= ret;
@@ -615,7 +610,6 @@ edit_dwarf2_line (DSO *dso, uint_32 off, char *comp_dir, int phase)
 	}
       else
 	ptr = srcptr = dir;
-      unsigned char *srcstart=srcptr;
       while (*srcptr != 0)
 	{
 	  size_t len = strlen (srcptr) + 1;
@@ -729,57 +723,37 @@ edit_attributes (DSO *dso, unsigned char *ptr, struct abbrev_tag *t, int phase)
 		}
 	    }
 
-	  if (t->attr[i].attr == DW_AT_comp_dir)
-	  {
-	      if ( form == DW_FORM_string )
-	      {
-		  free (comp_dir);
-		  comp_dir = strdup (ptr);
+	    if (t->attr[i].attr == DW_AT_comp_dir) {
+		if (form == DW_FORM_strp &&
+	      debug_sections[DEBUG_STR].data)
+	    {
+	      char *dir;
+	      
+	      dir = debug_sections[DEBUG_STR].data
+		    + do_read_32_relocated (ptr);
+	      free (comp_dir);
+	      comp_dir = strdup (dir);
+
+	      if (phase == 1 && dest_dir && has_prefix (dir, base_dir))
+		{
+		  base_len = strlen (base_dir);
+		  dest_len = strlen (dest_dir);
 		  
-		  if (phase == 1 && dest_dir && has_prefix (ptr, base_dir))
-		  {
-		      base_len = strlen (base_dir);
-		      dest_len = strlen (dest_dir);
-		      
-		      memcpy (ptr, dest_dir, dest_len);
-		      if (dest_len < base_len)
-		      {
-			  memset(ptr + dest_len, '/',
-				 base_len - dest_len);
-			  
-		      }
-		      elf_flagdata (debug_sections[DEBUG_INFO].elf_data,
-				    ELF_C_SET, ELF_F_DIRTY);
-		  }
-	      }
-	  
-	      else if (form == DW_FORM_strp &&
-		       debug_sections[DEBUG_STR].data)
-	      {
-		  char *dir;
-
-		  dir = debug_sections[DEBUG_STR].data
-		      + do_read_32_relocated (ptr);
-
-		  free (comp_dir);
-		  comp_dir = strdup (dir);
-
-		  if (phase == 1 && dest_dir && has_prefix (dir, base_dir))
-		  {
-		      base_len = strlen (base_dir);
-		      dest_len = strlen (dest_dir);
-		  
-		      memcpy (dir, dest_dir, dest_len);
-		      if (dest_len < base_len)
-		      {
-			  memmove (dir + dest_len, dir + base_len,
-				   strlen (dir + base_len) + 1);
-		      }
-		      elf_flagdata (debug_sections[DEBUG_STR].elf_data,
-				    ELF_C_SET, ELF_F_DIRTY);
-		  }
-	      }
-	  }
+		  memcpy (dir, dest_dir, dest_len);
+		  if (dest_len < base_len)
+		    {
+		      memmove (dir + dest_len, dir + base_len,
+			       strlen (dir + base_len) + 1);
+		    }
+		  elf_flagdata (debug_sections[DEBUG_STR].elf_data,
+				ELF_C_SET, ELF_F_DIRTY);
+		}
+	    }
+		else if (form == DW_FORM_string) {
+			free(comp_dir);
+			comp_dir = strdup (ptr);			
+		}
+	    }
 	  else if ((t->tag == DW_TAG_compile_unit
 		    || t->tag == DW_TAG_partial_unit)
 		   && t->attr[i].attr == DW_AT_name
@@ -1076,7 +1050,7 @@ edit_dwarf2 (DSO *dso)
 		  break;
 		case EM_PPC:
 		case EM_PPC64:
-		  if (rtype != R_PPC_ADDR32 && rtype != R_PPC_UADDR32)
+		  if (rtype != R_PPC_ADDR32 || rtype != R_PPC_UADDR32)
 		    goto fail;
 		  break;
 		case EM_S390:
@@ -1219,8 +1193,6 @@ static struct poptOption optionsTable[] = {
       "directory to rewrite base-dir into", NULL },
     { "list-file",  'l', POPT_ARG_STRING, &list_file, 0,
       "file where to put list of source and header file names", NULL },
-    { "build-id",  'i', POPT_ARG_NONE, &do_build_id, 0,
-      "recompute build ID note and print ID on stdout", NULL },
       POPT_AUTOHELP
     { NULL, 0, 0, NULL, 0, NULL, NULL }
 };
@@ -1299,126 +1271,6 @@ error_out:
   return NULL;
 }
 
-/* Compute a fresh build ID bit-string from the editted file contents.  */
-static void
-handle_build_id (DSO *dso, Elf_Data *build_id,
-		 size_t build_id_offset, size_t build_id_size)
-{
-  hashFunctionContext ctx;
-  const hashFunction *hf = NULL;
-  int i = hashFunctionCount ();
-
-  while (i-- > 0)
-    {
-      hf = hashFunctionGet (i);
-      if (hf != NULL && hf->digestsize == build_id_size)
-	break;
-    }
-  if (hf == NULL)
-    {
-      fprintf (stderr, "Cannot handle %Zu-byte build ID\n", build_id_size);
-      exit (1);
-    }
-
-  if (elf_update (dso->elf, ELF_C_NULL) < 0)
-    {
-      fprintf (stderr, "Failed to update file: %s\n",
-	       elf_errmsg (elf_errno ()));
-      exit (1);
-    }
-
-  /* Clear the old bits so they do not affect the new hash.  */
-  memset ((char *) build_id->d_buf + build_id_offset, 0, build_id_size);
-
-  hashFunctionContextInit (&ctx, hf);
-
-  /* Slurp the relevant header bits and section contents and feed them
-     into the hash function.  The only bits we ignore are the offset
-     fields in ehdr and shdrs, since the semantically identical ELF file
-     could be written differently if it doesn't change the phdr layout.
-     We always use the GElf (i.e. Elf64) formats for the bits to hash
-     since it is convenient.  It doesn't matter whether this is an Elf32
-     or Elf64 object, only that we are consistent in what bits feed the
-     hash so it comes out the same for the same file contents.  */
-  {
-    inline void process (const void *data, size_t size);
-    inline void process (const void *data, size_t size)
-    {
-      memchunk chunk = { .data = (void *) data, .size = size };
-      hashFunctionContextUpdateMC (&ctx, &chunk);
-    }
-
-    union
-    {
-      GElf_Ehdr ehdr;
-      GElf_Phdr phdr;
-      GElf_Shdr shdr;
-    } u;
-    Elf_Data x = { .d_version = EV_CURRENT, .d_buf = &u };
-
-    x.d_type = ELF_T_EHDR;
-    x.d_size = sizeof u.ehdr;
-    u.ehdr = dso->ehdr;
-    u.ehdr.e_phoff = u.ehdr.e_shoff = 0;
-    if (elf64_xlatetom (&x, &x, dso->ehdr.e_ident[EI_DATA]) == NULL)
-      {
-      bad:
-	fprintf (stderr, "Failed to compute header checksum: %s\n",
-		 elf_errmsg (elf_errno ()));
-	exit (1);
-      }
-
-    x.d_type = ELF_T_PHDR;
-    x.d_size = sizeof u.phdr;
-    for (i = 0; i < dso->ehdr.e_phnum; ++i)
-      {
-	if (gelf_getphdr (dso->elf, i, &u.phdr) == NULL)
-	  goto bad;
-	if (elf64_xlatetom (&x, &x, dso->ehdr.e_ident[EI_DATA]) == NULL)
-	  goto bad;
-	process (x.d_buf, x.d_size);
-      }
-
-    x.d_type = ELF_T_SHDR;
-    x.d_size = sizeof u.shdr;
-    for (i = 0; i < dso->ehdr.e_shnum; ++i)
-      if (dso->scn[i] != NULL)
-	{
-	  u.shdr = dso->shdr[i];
-	  u.shdr.sh_offset = 0;
-	  if (elf64_xlatetom (&x, &x, dso->ehdr.e_ident[EI_DATA]) == NULL)
-	    goto bad;
-	  process (x.d_buf, x.d_size);
-
-	  if (u.shdr.sh_type != SHT_NOBITS)
-	    {
-	      Elf_Data *d = elf_rawdata (dso->scn[i], NULL);
-	      if (d == NULL)
-		goto bad;
-	      process (d->d_buf, d->d_size);
-	    }
-	}
-  }
-
-  hashFunctionContextDigest (&ctx, (byte *) build_id->d_buf + build_id_offset);
-  hashFunctionContextFree (&ctx);
-
-  elf_flagdata (build_id, ELF_C_SET, ELF_F_DIRTY);
-
-  /* Now format the build ID bits in hex to print out.  */
-  {
-    const unsigned char * id = build_id->d_buf + build_id_offset;
-    char hex[build_id_size * 2 + 1];
-    int n = snprintf (hex, 3, "%02" PRIx8, id[0]);
-    assert (n == 2);
-    for (i = 1; i < build_id_size; ++i)
-      {
-	n = snprintf (&hex[i * 2], 3, "%02" PRIx8, id[i]);
-	assert (n == 2);
-      }
-    puts (hex);
-  }
-}
 
 int
 main (int argc, char *argv[])
@@ -1431,9 +1283,7 @@ main (int argc, char *argv[])
   const char **args;
   struct stat stat_buf;
   char *p;
-  Elf_Data *build_id = NULL;
-  size_t build_id_offset = 0, build_id_size = 0;
-
+  
   optCon = poptGetContext("debugedit", argc, (const char **)argv,
 			  optionsTable, 0);
   
@@ -1536,52 +1386,13 @@ main (int argc, char *argv[])
 #endif
 	  if (strcmp (name, ".debug_info") == 0)
 	    edit_dwarf2 (dso);
-
-	  break;
-	case SHT_NOTE:
-	  if (do_build_id
-	      && build_id == NULL && (dso->shdr[i].sh_flags & SHF_ALLOC))
-	    {
-	      /* Look for a build-ID note here.  */
-	      Elf_Data *data = elf_rawdata (elf_getscn (dso->elf, i), NULL);
-	      Elf32_Nhdr nh;
-	      Elf_Data dst =
-		{
-		  .d_version = EV_CURRENT, .d_type = ELF_T_NHDR,
-		  .d_buf = &nh, .d_size = sizeof nh
-		};
-	      Elf_Data src = dst;
-	      src.d_buf = data->d_buf;
-	      assert (sizeof (Elf32_Nhdr) == sizeof (Elf64_Nhdr));
-	      while (data->d_buf + data->d_size - src.d_buf > (int) sizeof nh
-		     && elf32_xlatetom (&dst, &src, dso->ehdr.e_ident[EI_DATA]))
-		{
-		  Elf32_Word len = sizeof nh + nh.n_namesz;
-		  len = (len + 3) & ~3;
-
-		  if (nh.n_namesz == sizeof "GNU" && nh.n_type == 3
-		      && !memcmp (src.d_buf + sizeof nh, "GNU", sizeof "GNU"))
-		    {
-		      build_id = data;
-		      build_id_offset = src.d_buf + len - data->d_buf;
-		      build_id_size = nh.n_descsz;
-		      break;
-		    }
-
-		  len += nh.n_descsz;
-		  len = (len + 3) & ~3;
-		  src.d_buf += len;
-		}
-	    }
+	  
 	  break;
 	default:
 	  break;
 	}
     }
 
-  if (do_build_id && build_id != NULL)
-    handle_build_id (dso, build_id, build_id_offset, build_id_size);
-
   if (elf_update (dso->elf, ELF_C_WRITE) < 0)
     {
       fprintf (stderr, "Failed to write file: %s\n", elf_errmsg (elf_errno()));
