@@ -3,13 +3,14 @@ const char *__progname;
 
 #include <rpmcli.h>
 
-#include "rpmdb.h"
-#include "rpmps.h"
+#include <rpmdb.h>
+#include <rpmps.h>
 
-#include "rpmte.h"
+#include <rpmds.h>
+#include <rpmte.h>
 
 #define _RPMTS_INTERNAL         /* ts->goal, ts->dbmode, ts->suggests */
-#include "rpmts.h"
+#include <rpmts.h>
 
 #include "manifest.h"
 #include "misc.h"		/* rpmGlob */
@@ -19,7 +20,7 @@ static int noDeps = 1;
 
 static rpmVSFlags vsflags = 0;
 
-static inline /*@observer@*/ const char * const identifyDepend(int_32 f)
+static inline /*@observer@*/ const char * identifyDepend(int_32 f)
 	/*@*/
 {
     if (isLegacyPreReq(f))
